--
-- Created by IntelliJ IDEA.
-- User: rioukinnei
-- Date: 13-12-18
-- Time: 上午10:00
-- 数据管理类
--
GameState=require(cc.PACKAGE_NAME .. ".api.GameState")

-- 包含普通数据
requires(ROOT_PATH, "app/Data/DataNormal")
requires(ROOT_PATH, "app/Data/DataItemInfo")
requires(ROOT_PATH, "app/Data/DataTask")
requires(ROOT_PATH, "app/Data/DataUserInfo")
requires(ROOT_PATH, "app/Data/DataActivity")
requires(ROOT_PATH, "app/Data/DataUpAndMain")

DATAMGR= {}

-- 数据版本号
DATAMGR.nDataVer    = 1

-- 登陆的数据保存
DATAMGR.pLoginData = {
    strUser = nil,
    strPsw  = nil,
    strName = nil,
    nServIdx = -1,
}

-- 初始化数据
function DATAMGR:InitDataMgr()
	-- init GameState
	GameState.init(function(param)
		local returnValue
		if param.errorCode then
			-- crypto
			if param.name=="save" then
				CCLuaLog("Save Data error:" .. param.errorCode)
			elseif param.name=="load" then
				CCLuaLog("Load Data error:" .. param.errorCode)
			else
				CCLuaLog("Data error:" .. param.errorCode)
			end

		else
			-- crypto
			if param.name=="save" then
				local str=json.encode(param.values)
				str=crypto.encryptXXTEA(str, "carrace1234")
				returnValue={data=str}
			elseif param.name=="load" then
				local str=crypto.decryptXXTEA(param.values.data, "carrace1234")
				returnValue=json.decode(str)
			else
				CCLuaLog("Data error")
			end
			-- returnValue=param.values
		end
		return returnValue
	end, "GameInfo.info","clt_carrace")

	self.tSaveInfo = {}
	self.tSaveInfo.bIsMusicOpen = true
	self.tSaveInfo.bIsSoundOpen = true

	-- 读取玩家数据
	self:LoadUserData()

	-- 初始化物品信息数据管理
	self:GetItemInfoMgr():InitItemInfo()

	-- 玩家背包和阵容等信息
	self:GetUserInfoMgr():InitUserInfo(self)

	-- 任务信息
	self:GetTaskInfoMgr():InitDataTask(self)
	-- 初始化活动管理
	self:GetActivityMgr():InitDataActivity(self)

	-- 当前的服务器列表
	self.pArrServList = {}

	-- 玩家数据信息
	self.tPlayerLocalInfo = {}
	-- 订单的本地处理
	self.tAllDealInfo = {}

	-- 登陆服务器的key和Uid
	self.strLoginKey = ""
	self.nLoginUid = 0

	-- 声音开关
	SOUNDMGR:SetEnablePlayMusic(self:isMusicOpen())
	SOUNDMGR:SetEnablePlayEffect(self:isSoundOpen())

	-- 获取要保护数据的MD5值
	-------------- 读取本地的文件 ------------------
	-- 被保护文件列表的md5值对应
	self.DicProFileMd5 = {}

	local strLocalFile = Public_GetResPathByKey("Images/StaticDatas/res.xxx", true)
	local bTargetLocal
	local strVerTextLocal
	bTargetLocal, strVerTextLocal = getFileString(strVerTextLocal, strLocalFile, "res.xxx", nil);
	local tLocalVerList = json.decode(strVerTextLocal)

	-- 遍历所有要特殊检测的文件
	for i,strFile in ipairs(g_DicProtectFiles) do
		-- 如果当前文件没有需要更新，则开始判断md5值
		local tVerInfo = tLocalVerList[strFile];
		if tVerInfo ~= nil then
			self.DicProFileMd5[ strFile ] = tVerInfo[2];
		end
	end
	-- 不被保护的文件
	for i,strFile in ipairs(g_DicNotProtectFiles) do
		self.DicProFileMd5[ strFile ] = "1";
	end
	----------------------------------------------
end
-- mark: ========================= 版本文件处理 ============================
-- 保护的数据文件列表是否发生了改变
function DATAMGR:IsProtectFileHadChange()
	-- 遍历所有要特殊检测的文件
	for i,strFile in ipairs(g_DicProtectFiles) do
		-- 如果当前文件没有需要更新，则开始判断md5值
		local strOldMd5 = self.DicProFileMd5[strFile];
		if strOldMd5 ~= nil then
			-- 获取对应文件的md5值
			local strVerFile = Public_GetResPathByKey(strFile, true);
			local strVerFileText = io.readfile(strVerFile);
			local strMd5 = crypto.md5(strVerFileText, false);
			if strMd5 ~= nil then
				strMd5 = strMd5..G_CLIENT_FILE_PWS
				strMd5 = crypto.md5(strMd5, false)
			end

			-- 判断和现有的md5值是否一致，不一至，则反正文件发生了改变
			if strMd5 ~= strOldMd5 then
				if DEBUG == 0 then
					return true;
				else
					return false;
				end
			end
		else
			-- 如果受保护的数据没有找到MD5值，则表明也被修改了
			if DEBUG == 0 then
				return true;
			else
				return false;
			end
		end
	end

	return false;
end

-- 读取和判断单个文件
function DATAMGR:LoadOneDataFileAndJudgeIsChange(strFile, bIsJudge)
	if bIsJudge == nil then
		bIsJudge = true;
	end
	-- 获取当前文件的文本
	local strFilePath = Public_GetResPathByKey(strFile, true);
	local strVerFileText = io.readfile(strFilePath);

	local bIsChange = false;

	-- 如果需要判断
	if bIsJudge == true then

		repeat
			-- 判断是否发生了修改
			local strOldMd5 = self.DicProFileMd5[strFile];
			if strOldMd5 == nil then
				bIsChange = true;
				break;
			-- 不在保护中，默认为不修改
			elseif strOldMd5 == "1" then
				break;
			end

			local strMd5 = crypto.md5(strVerFileText, false);
			if strMd5 ~= nil then
				strMd5 = strMd5..G_CLIENT_FILE_PWS
				strMd5 = crypto.md5(strMd5, false)
			end
			-- 判断和现有的md5值是否一致
			if strMd5 == strOldMd5 then
				break;
			end

			bIsChange = true;

		until true
	end

	-- 如果发生了变化。。且不在DEBUG阶段，则显示更新
	if bIsChange == true and DIALOGMGR and DEBUG == 0 then
		DIALOGMGR:ShowDialog(DLGTYPE.DataFileHadChange);
	end


	return strVerFileText, bIsChange;
end

-- mark: ========================= 循环数据 ============================
-- 
function DATAMGR:DataLoop( dt )
	-- 循环处理数据
	self:GetUserInfoMgr():DataUserLoop(dt)

	self:GetActivityMgr():DataActLoop(dt)
end

-- 后台回来的时间差处理
function DATAMGR:BackGroundDeltaTime( dt )
	self:GetUserInfoMgr():DataUserLoop(dt)
	self:GetActivityMgr():DataBackActLoop(dt)
end

-- 清除所有信息登陆之后的临时数据
function DATAMGR:ClearAllTempInfo()
	-- 清除任务信息标记
	self:GetTaskInfoMgr():ClearAllTaskMask()

	-- 清除所有玩家标记
	self:GetUserInfoMgr():ClearAllMask()

	-- 清除活动标记
	self:GetActivityMgr():ClearActMarks()
end

-- 断线重连时，清除一些断线重连的数据
function DATAMGR:ClearAllTempInfoByReconnect()
	self:GetUserInfoMgr():ClearAllMaskByReconnect()
end

-- mark: ======================= 获取各种数据的管理 ==========================
-- 获取物品信息管理
function DATAMGR:GetItemInfoMgr()
	return DATA_ITEM
end
-- 获取任务信息的管理
function DATAMGR:GetTaskInfoMgr()
	return DATA_TASK
end
-- 获取玩家数据管理
function DATAMGR:GetUserInfoMgr()
	return DATA_USER
end
-- 活动数据管理
function DATAMGR:GetActivityMgr()
	return DATA_ACTIVITY
end

-- mark: ========================= 登陆信息处理 ==============================

-- 保存服务器列表
function DATAMGR:SetServList( tServList )
	if tServList == nil then
		tServList = {}
	end

	self.pArrServList = tServList
end
function DATAMGR:GetServList()
	return self.pArrServList
end

-- 设置选择服务器的索引
function DATAMGR:SetSelServIdx( nIdx )
	DATAMGR.pLoginData.nServIdx = nIdx
	-- 保存数据
	self:SaveUserData()
end

-- 获取选择服务器的索引
function DATAMGR:GetSelServIdx()
	return DATAMGR.pLoginData.nServIdx
end

-- 获取当前登陆服务器的名字
function DATAMGR:GetCurServName()
	return NetWorkTypeName[ CT_SERV_TYPE ];
end
-- 获取区服全称
function DATAMGR:GetCurServAllNameInfo()
	local strServ = self:GetCurServName()
	if strServ == nil then
		strServ = "未知";
	end
	local tLoginData = self:GetSelServInfo()
    -- 登陆服
    if tLoginData then
        local strServName = string.format("[%s][%d区]%s", strServ, tLoginData.nIdx+1, tLoginData.strName)
        return strServName;
    end
    return nil;
end

-- 获取选择信息
function DATAMGR:GetSelServInfo()
	local tOneServ
	if self.pArrServList then
		-- 获取记录的服或者最后的服
		local tLastOne
		for i,v in ipairs(self.pArrServList) do
			if tLastOne == nil then
				tLastOne = v;
			elseif v.nIdx > tLastOne.nIdx then
				tLastOne = v;
			end
			if v.nIdx == DATAMGR.pLoginData.nServIdx then
				tOneServ = v;
				break;
			end
		end

		if tOneServ == nil and tLastOne ~= nil then
			tOneServ = tLastOne
			self:SetSelServIdx( tOneServ.nIdx );
		end
	end

	if tOneServ == nil then
		CCLuaLog("服务器ip地址获取失败！")
		tOneServ = clone(CTSertListUnit);
	end

	return tOneServ
end

-- 登陆游戏服务器的具体信息
function DATAMGR:SetLoginGameServInfo( strLoginKey, nUid )
	strLoginKey = strLoginKey or ""
	nUid = nUid or 0
	self.strLoginKey = strLoginKey
	self.nLoginUid = nUid
end
function DATAMGR:GetLoginGameServKey()
	print("key33::" .. self.strLoginKey)
	return self.strLoginKey
end
function DATAMGR:GetLoginGameServUid()
	return self.nLoginUid
end

-- 设置密码
function DATAMGR:SetLoginInfo(strUser, strPsw, strShowName)
	DATAMGR.pLoginData.strUser = strUser
	DATAMGR.pLoginData.strPsw = strPsw
	DATAMGR.pLoginData.strName = strShowName

	-- 保存数据
	self:SaveUserData()
end


-- 保存玩家信息
function DATAMGR:SaveUserData()
	-- 要保存的数据
	local tGameData = {}
	tGameData.nVer = DATAMGR.nDataVer
	tGameData.pLoginInfo = DATAMGR.pLoginData
	tGameData.pSaveInfo = DATAMGR.tSaveInfo

	-- 开始保存
	GameState.save(tGameData)
end


function DATAMGR:LoadUserData()
	local tGameData = GameState.load()

	if tGameData then
		DATAMGR.nDataVer = tGameData.nVer
		for key, value in pairs(tGameData.pLoginInfo) do  
    		DATAMGR.pLoginData[key] = value
		end
		if tGameData.pSaveInfo then
			for key, value in pairs(tGameData.pSaveInfo) do  
	    		DATAMGR.tSaveInfo[key] = value
			end 
		end
		
		
	else
		CCLuaLog("Load Data error11")
	end
end

-- 保存和读取特定用户的数据
function DATAMGR:SaveOneUserInfo(nUserId, tInfo)
	if nUserId == nil or tInfo == nil then
		return 
	end
	local strFile = CCFileUtils:sharedFileUtils():getWritablePath() .. nUserId .. ".u";
	local strText = json.encode(tInfo);
	if strText == nil then
		return
	end
	io.writefile(strFile, strText, "wb+");
end
function DATAMGR:LoadOneUserInfo(nUserId)
	if nUserId==nil then
		return nil;
	end
	local strFile = CCFileUtils:sharedFileUtils():getWritablePath() .. nUserId .. ".u";
	local strText = io.readfile(strFile)
	if strText then
		local tLocal = json.decode(strText);
		return tLocal;
	end
	return nil;
end

-- 获取当前玩家数据结构
function DATAMGR:getUserSaveInfoById(nUid, nServIdx, bIsJustGet)
	-- 保存信息
	local tSaveInfo = self.tPlayerLocalInfo[ nUid.."" ]
	if tSaveInfo == nil then
		-- 再读取一次
		tSaveInfo = self:LoadOneUserInfo( nUid )
		-- 是否只是获取
		if bIsJustGet == true then
			if tSaveInfo == nil then
				return nil;
			else
				self.tPlayerLocalInfo[ nUid.."" ] = tSaveInfo;
			end
		else
			if tSaveInfo == nil then
				tSaveInfo = {}
				self.tPlayerLocalInfo[ nUid.."" ] = tSaveInfo;
			end
		end
	end


	-- 对应服务器信息
	local tServInfo = tSaveInfo[nServIdx .. ""]
	-- 只是获取
	if bIsJustGet == true then
		if tServInfo == nil then
			return nil;
		end
	else
		if tServInfo == nil then
			tServInfo = {}
			tSaveInfo[nServIdx .. ""] = tServInfo;
		end
	end

	return tSaveInfo, tServInfo;
end

-- mark ====设置信息====

-- 获取是否已经显示解锁关卡效果
function DATAMGR:getIsShowUnlockEff(nStageIdx)
	local tUserInfo = self:GetUserInfoMgr():GetUserBaseInfo();
	-- 保存信息
	local tSaveInfo, tServInfo = self:getUserSaveInfoById( tUserInfo.nUserId, DATAMGR.pLoginData.nServIdx, true )
	if tServInfo == nil then
		return true;
	end

	if tServInfo.tStage == nil then
		return true;
	end
	local bIsShow = tServInfo.tStage[nStageIdx..""]
	if bIsShow ~= true then
		return true;
	end

	return false
end
-- 设置已经显示解锁关卡效果
function DATAMGR:setIsShowUnlockEff(nStageIdx)
	local tUserInfo = self:GetUserInfoMgr():GetUserBaseInfo();
	-- 保存信息
	local tSaveInfo, tServInfo = self:getUserSaveInfoById( tUserInfo.nUserId, DATAMGR.pLoginData.nServIdx, false )
	if tServInfo == nil then
		return;
	end

	if tServInfo.tStage == nil then
		tServInfo.tStage = {}
	end
	tServInfo.tStage[nStageIdx..""] = true;
	-- 保存数据
	self:SaveOneUserInfo( tUserInfo.nUserId, tSaveInfo )
end

-- 是否是首次启动
function DATAMGR:getIsFistStart( str )
	 self.tPlayerFisrtStart = { }
	local strFileName = "firstStart"
	-- 保存信息
	local tSaveInfo = self.tPlayerFisrtStart[ strFileName.."" ]
	if tSaveInfo == nil then
		tSaveInfo = self:LoadOneUserInfo( strFileName )
		if tSaveInfo == nil then
			return true;
		else
			self.tPlayerFisrtStart[ strFileName.."" ] = tSaveInfo;
		end
	end
	return false
end

-- 设置首次启动
function DATAMGR:setIsFistStart(str)
	local strFileName = "firstStart"
	-- 保存信息
	local tSaveInfo = self.tPlayerFisrtStart[ strFileName.."" ]
	if tSaveInfo == nil then
		tSaveInfo = {}
		self.tPlayerFisrtStart[ strFileName.."" ] = tSaveInfo;
	end
	-- 保存数据
	self:SaveOneUserInfo( strFileName, str )
end

function DATAMGR:isMusicOpen()
	return self.tSaveInfo.bIsMusicOpen
end
function DATAMGR:isSoundOpen()
	return self.tSaveInfo.bIsSoundOpen
end
function DATAMGR:setIsMusicOpen(b)
	self.tSaveInfo.bIsMusicOpen = b 
	SOUNDMGR:SetEnablePlayMusic(b)
	self:SaveUserData()
end
function DATAMGR:setIsSoundOpen(b)
	self.tSaveInfo.bIsSoundOpen = b
	SOUNDMGR:SetEnablePlayEffect(b)
	self:SaveUserData()
end