--
-- Author: Link
-- Date: 2014-02-25 19:42:06
--

DATA_TASK= {}

-- 数据管理的处理
DATA_TASK.pDataMgr = nil

-- 当前章节列表，TaskBigChapter数组
DATA_TASK.tChapterList = {}
-- 任务进度信息
DATA_TASK.tTaskProgress = clone(ChapterProgress)
-- 物品刷物品所在地
DATA_TASK.tCardStoryInfo = {--[==[   [ID] = TaskItemAddress,  ]==]}

-- 初始化任务数据
function DATA_TASK:InitDataTask(pDataMgr)
	self.pDataMgr = pDataMgr

	-- 读取新本任务信息
	self:LoadChapterInfo()
end

-- 清除玩家任务标记
function DATA_TASK:ClearAllTaskMask()
	print("清除玩家任务标记")
	-- 取消所有获取信息
	self.tTaskProgress.bIsGetBig = false
	-- 大章节
	self.tTaskProgress.nBigID 	= 0
	-- 大章节ID所在索引
	self.tTaskProgress.nBigIdx 	= 0
	-- 解锁到的中章节ID
	self.tTaskProgress.nCurMidID 	= 0
	-- 中章节ID所在索引
	self.tTaskProgress.nCurMidIdx 	= 0
	-- 当前任务ID
	self.tTaskProgress.nCurTaskID	= 0
	-- 当前任务ID所在索引
	self.tTaskProgress.nCurTaskIdx	= 0
	-- 当前是否已经获取大章节完成情况

	-- 章节信息处理
	for i,tOneChapter in ipairs(self.tChapterList) do
		tOneChapter.bIsGetMid = false
	end
end

-- mark: ========================= 任务信息处理 ==============================

-- 获取是否获取过章节进度
function DATA_TASK:GetIsHadGotTaskProgress()
	return self.tTaskProgress.bIsGetBig
end
-- 获取是否获取过对应大章节索引的信息
function DATA_TASK:GetIsHadGotTaskInfoByBigIdx( nBigIdx )
	local tChapter = self:GetMidChapterListByBigChapterIdx(nBigIdx)
	-- 
	if tChapter ~= nil then
		return tChapter.bIsGetMid
	end
	return true
end

-- mark: ========================== 任务的读取 =================================
-- 读取章节信息
function DATA_TASK:LoadChapterInfo()
	local strTaskInfo = DATAMGR:LoadOneDataFileAndJudgeIsChange("Images/StaticDatas/TaskData.json", false)

	local tTaskInfo = {}
	local tTempInfo = json.decode(strTaskInfo)

	-- 遍历读取的任务信息
	for strID,tBigInfo in pairs(tTempInfo) do
		local tChapterInfo = clone(TaskBigChapter)
		-- 大章节信息
		tChapterInfo.nID = toint(strID)
		tChapterInfo.strName = tBigInfo.name


		-- 按顺序插入大章节信息，先获取插入的索引
		local nBigInsertIdx = #tTaskInfo+1
		for m = #tTaskInfo,1, -1 do
			local tOneCha = tTaskInfo[m]
			nBigInsertIdx = m
			if tOneCha.nID < tChapterInfo.nID then
				nBigInsertIdx = m+1
				break
			end
		end

		-- 
		-- 中章节任务列表
		for strMidID,tMidInfo in pairs(tBigInfo.page) do
			local tMidTask = clone(TaskMidChapter)
			--  中章节信息
			tMidTask.nID = toint(strMidID)
			tMidTask.strName = tMidInfo.name

			tMidTask.tBossInfo.nItemID = tMidInfo.boss.cardId
			tMidTask.tBossInfo.nTotalFValue = math.floor(tMidInfo.boss.totalV);
			-- 
			tMidTask.tBossInfo.nJingYingItemID = tMidInfo.pickBoss.cardId
			tMidTask.tBossInfo.nJingYingFValue = tMidInfo.pickBoss.totalV

			local nStartID = tMidInfo.fid
			local nBossTaskID = nStartID
			-- 获取任务信息
			while nStartID ~= 0 do
				for nIdx,tOneTask in pairs(tMidInfo.task) do
					if nStartID == tOneTask[1] then
						local tTaskTmp = clone(TaskBaseInfo)
						tTaskTmp.nTaskID 		= tOneTask[1]
						tTaskTmp.strTitle 		= tOneTask[2]
						tTaskTmp.strContent 	= tOneTask[3]
						tTaskTmp.nNeedEnergy 	= tOneTask[4]
						tTaskTmp.nTotalCount 	= tOneTask[5]
						tTaskTmp.nGetGold 		= tOneTask[6]
						tTaskTmp.nGetExp 		= tOneTask[7]
						local nNextID = tOneTask[8]
						if nNextID ~= 0 then
							tTaskTmp.bIsBoss = false
						else
							tTaskTmp.bIsBoss = true
							-- 记录BOSS的ID
							nBossTaskID = tTaskTmp.nTaskID;
						end
						tTaskTmp.nCurCount		= 0

						nStartID = nNextID
						tMidInfo.task[nIdx] = nil
						-- 添加信息
						tMidTask.tTaskList[#tMidTask.tTaskList+1] = tTaskTmp
						break
					end
				end
				-- 如果没有数据了，则跳出
				if table.nums(tMidInfo.task) <= 0 then
					break
				end
			end

			-- 按顺序插入中章节信息
			local nIdx = #tChapterInfo.tChapterList + 1
			for m = #tChapterInfo.tChapterList,1, -1 do
				local tOneCha = tChapterInfo.tChapterList[m]
				nIdx = m
				if tOneCha.nID < tMidTask.nID then
					nIdx = m+1
					break
				end
			end
			table.insert(tChapterInfo.tChapterList, nIdx, tMidTask)

			-- 普通物品ID
			self:AddOneStoryItemInfo(tMidTask.tBossInfo.nItemID, tChapterInfo.nID,
										tMidTask.nID, nBossTaskID, false, false)
			-- 精英物品ID
			self:AddOneStoryItemInfo(tMidTask.tBossInfo.nJingYingItemID, tChapterInfo.nID,
										tMidTask.nID, nBossTaskID, true, true)
		end

		table.insert(tTaskInfo, nBigInsertIdx, tChapterInfo)
	end

	self.tChapterList = tTaskInfo
end

-- 添加一个剧情物品的位置信息
function DATA_TASK:AddOneStoryItemInfo(nItemID, nBigID, nMidID, nTaskID, bIsPiece, bIsJingYing)
	-- 车子获取得到的数据
	local tGetInfoTmp = clone(TaskItemAddress)
	-- 大章节
	tGetInfoTmp.nBigID 		= nBigID
	-- 中章节ID
	tGetInfoTmp.nMidID 		= nMidID
	-- 小章节ID
	tGetInfoTmp.nTaskID 	= nTaskID
	-- 物品所在的地址位置
	tGetInfoTmp.nItemID		= nItemID
	-- 是否是碎片
	tGetInfoTmp.bIsJingYing	= bIsJingYing

	-- 是否是碎片
	tGetInfoTmp.bIsPiece	= bIsPiece

	-- 获取数组
	local tItemList = self.tCardStoryInfo[ nItemID ]
	if tItemList == nil then
		tItemList = {}
		self.tCardStoryInfo[ nItemID ] = tItemList
	end


	-- 按顺序插入中章节信息
	local nIdx = #tItemList + 1
	for m = #tItemList, 1, -1 do
		local tOneCha = tItemList[m]
		nIdx = m
		if tOneCha.nBigID <= tGetInfoTmp.nBigID and tOneCha.nMidID <= tGetInfoTmp.nMidID and 
			tOneCha.nTaskID <= tGetInfoTmp.nTaskID then

			nIdx = m+1
			break
		end
	end
	table.insert(tItemList, nIdx, tGetInfoTmp)
end

-- mark: ========================== 任务的获取信息 =================================
-- 获取所有任务列表信息
function DATA_TASK:GetBigChapterList()
	return self.tChapterList
end
-- 获取对应大章节的中章节信息任务列表信息
function DATA_TASK:GetMidChapterListByBigChapterIdx(nIdx)
	if nIdx > #self.tChapterList or nIdx <= 0 then
		-- CTAssert(0, "获取中章节列表错误："..nIdx)
		return nil
	end
	return self.tChapterList[nIdx]
end
-- 获取对应大章节的中章节信息任务列表信息
function DATA_TASK:GetMidChapterListByBigID(nBigID)
	local nGetIdx = -1
	for i,v in ipairs(self.tChapterList) do
		if v.nID == nBigID then
			nGetIdx = i
			break
		end
	end
	-- 获取任务信息
	return self:GetMidChapterListByBigChapterIdx(nGetIdx)
end
-- 获取任务的完成进行
function DATA_TASK:GetTaskCurProgress()
	return self.tTaskProgress
end

-- 获取任务信息信息
function DATA_TASK:GetOneTaskInfoByIdx(nBigIdx, nMidIdx, nTaskIdx)
	local tMidChapter = self.tChapterList[nBigIdx].tChapterList[nMidIdx]
	local tTaskInfo = tMidChapter.tTaskList[nTaskIdx]
	return tTaskInfo, tMidChapter
end
-- 获取任务信息信息
function DATA_TASK:GetOneTaskInfoByID(nBigID, nMidID, nTaskID)
	local tMidChapter = self:GetMidChapterInfoByInfo(nBigID, nMidID)
	local tTaskInfo
	for nIdx,tOneTask in ipairs(tMidChapter.tTaskList) do
		-- 如果是小任务列表中到达的进度，则解开信息
		if tOneTask.nTaskID == nTaskID then
			tTaskInfo = tOneTask
		end
	end
	return tTaskInfo, tMidChapter
end

-- 获取对应大章节是否已经获取过
function DATA_TASK:GetBigChapterHadGetInfo(nBigIdx)
	local tBigInfo = self:GetMidChapterListByBigChapterIdx(nBigIdx)
	if tBigInfo == nil then
		return true
	end
	return tBigInfo.bIsGetMid
end
-- 获取对应的小章节列表信息
function DATA_TASK:GetMidChapterInfoByInfo(nBigID, nMidID)
	local tBigInfo = self:GetMidChapterListByBigID(nBigID)

	if tBigInfo == nil then
		return nil
	end
	-- 获取对应小章节列表
	for m,tOneChapter in ipairs(tBigInfo.tChapterList) do
		if tOneChapter.nID == nMidID then
			return tOneChapter
		end
	end
	return nil
end
-- 获取中章的任务列表信息
function DATA_TASK:GetMidChapterInfoByIndex(nBigIdx, nMidIdx)
	local tMidChapter = self.tChapterList[nBigIdx].tChapterList[nMidIdx]
	return tMidChapter
end

-- 改变刷BOSS次数
function DATA_TASK:ChangeBossCount(nBigID, nMidID, nBossTimes)
	local tChapterInfo = self:GetMidChapterInfoByInfo(nBigID, nMidID)
	tChapterInfo.tBossInfo.nRestCount = nBossTimes
end

-- 根据3种层次的ID，获取三种层次的索引
function DATA_TASK:GetTaskAllIdxByAllID( nBigID, nMidID, nTaskID )
	local nBigIdx = 1
	local nMidIdx = 1
	local nTaskIdx = 1
	-- 获取大章节索引
	local tBigChapter
	for i,tOneBig in ipairs(self.tChapterList) do
		if tOneBig.nID == nBigID then
			nBigIdx = i
			tBigChapter = tOneBig
			break
		end
	end

	repeat
		-- 获取全部
		if tBigChapter == nil then
			nBigIdx = #self.tChapterList+1;
			break;
		end
		-- 获取中章节索引
		local tMidTask
		if tBigChapter then
			for m,tOneChapter in ipairs(tBigChapter.tChapterList) do
				if tOneChapter.nID == nMidID then
					nMidIdx = m
					tMidTask = tOneChapter
					break
				end
			end
		end

		-- 获取全部
		if tMidTask == nil then
			nMidIdx = #tBigChapter.tChapterList+1;
			break;
		end

		-- 获取中章节索引
		if tMidTask then
			for m,tOneList in ipairs(tMidTask.tTaskList) do
				if tOneList.nTaskID == nTaskID then
					nTaskIdx = m
					break
				end
			end
		end
	until true

	return nBigIdx, nMidIdx, nTaskIdx
end
-- 获取最新的三个索引
function DATA_TASK:GetTaskProgressAllIdx()
	return self.tTaskProgress.nBigIdx, self.tTaskProgress.nCurMidIdx, self.tTaskProgress.nCurTaskIdx
end

-- 判断当前精英索引进度情况
function DATA_TASK:GetJingYingTaskOpenStatus(nBigIdx, nMidIdx)
	-- 章节完成情况
	if self.tTaskProgress.nJYBigIdx > nBigIdx then
		return StoryProgressType.HadPast;
	elseif self.tTaskProgress.nJYBigIdx < nBigIdx then
		return StoryProgressType.NotReach;
	end

	if self.tTaskProgress.nJYMidIdx > nMidIdx then
		return StoryProgressType.HadPast;
	elseif self.tTaskProgress.nJYMidIdx == nMidIdx then
		return StoryProgressType.Progressing;
	end

	return StoryProgressType.NotReach;
end
-- mark: ========================== 从网络获取的相关信息 =================================

-- 设置当前剧情的进度
function DATA_TASK:SetAllTaskProgressInfo(tProgress)
	-- 精英副本进度
	self.tTaskProgress.nJYBigID = tProgress.nJYBigID
	self.tTaskProgress.nJYMidID = tProgress.nJYMidID

	local nBigIdx, nMidIdx, nTaskIdx = self:GetTaskAllIdxByAllID( tProgress.nJYBigID, tProgress.nJYMidID, -1 )

	self.tTaskProgress.nJYBigIdx = nBigIdx
	self.tTaskProgress.nJYMidIdx = nMidIdx

	-- 普通剧情的进度设置
	return self:SetAllTaskFinishedProgressInfo( tProgress.nBigID, tProgress.nCurMidID, tProgress.nCurTaskID )
end

-- 设置目前的任务进度，反正值，0是没有变化，1是大章节变化，2是中章节变化，3是小节变化
function DATA_TASK:SetAllTaskFinishedProgressInfo(nBigID, nMidID, nTaskID)
	if self.tTaskProgress.nBigID == nBigID and
		self.tTaskProgress.nCurMidID == nMidID and 
		self.tTaskProgress.nCurTaskID == nTaskID then
		return 0
	end
	-- 变化类型
	local nChangeType = 0
	if self.tTaskProgress.nBigID ~= nBigID then
		nChangeType = 1
	elseif self.tTaskProgress.nCurMidID ~= nMidID then
		nChangeType = 2
	else
		nChangeType = 3
	end

	local bIsChangeAgain = false
	-- 如果不是第一次初始化
	if self.tTaskProgress.nBigID > 0 then
		bIsChangeAgain = true
	end

	-- 章节完成情况
	self.tTaskProgress.nBigID		= nBigID
	self.tTaskProgress.nCurMidID	= nMidID
	self.tTaskProgress.nCurTaskID	= nTaskID
	-- 标记已经完成
	self.tTaskProgress.bIsGetBig	= true
	-- 设置索引
	local bIsHad = false
	for i,v in ipairs(self.tChapterList) do
		if v.nID == nBigID then
			self.tTaskProgress.nBigIdx = i
			bIsHad = true
			break
		end
	end


	-- 如果不是当前拥有的章节
	if bIsHad == false then
		self.tTaskProgress.nBigIdx = nBigID
		self.tTaskProgress.nCurMidIdx = nMidID
		self.tTaskProgress.nCurTaskIdx = nTaskID
		-- 当前信息
		for i,tOneBig in ipairs(self.tChapterList) do
			tOneBig.bIsUnlock = true
			-- 所有章节都打开了
			for m,tOneChapter in ipairs(tOneBig.tChapterList) do
				tOneChapter.bIsUnlock = true
			end
		end
		
	else
		-- 当前信息
		for i,tOneBig in ipairs(self.tChapterList) do
			if i <= self.tTaskProgress.nBigIdx then
				tOneBig.bIsUnlock = true
			else
				tOneBig.bIsUnlock = false
			end
		end
		-- 当前信息
		local tBigChapter = self.tChapterList[ self.tTaskProgress.nBigIdx ]
		local tMidChapter
		-- 获取对应小章节列表
		for m,tOneChapter in ipairs(tBigChapter.tChapterList) do
			if tOneChapter.nID == nMidID then
				tMidChapter = tOneChapter
				-- 设置中章节的索引
				self.tTaskProgress.nCurMidIdx = m
				tOneChapter.bIsUnlock = true

			elseif tOneChapter.nID < nMidID then
				tOneChapter.bIsUnlock = true
			else
				tOneChapter.bIsUnlock = false
			end
		end
		-- 获取当前任务的索引
		local bIsEnd = true
		for nIdx,tOneTask in ipairs(tMidChapter.tTaskList) do
			-- 如果是再次修改进度
			if bIsChangeAgain == true then
				-- 如果是小任务列表中到达的进度，则解开信息
				if tOneTask.nTaskID == nTaskID then
					self.tTaskProgress.nCurTaskIdx = nIdx
					tOneTask.nCurCount = 0
					tOneTask.bIsUnlock = true
					bIsEnd = false
				else
					tOneTask.bIsUnlock = false
					if bIsEnd == true then
						tOneTask.nCurCount = tOneTask.nTotalCount
					else
						tOneTask.nCurCount = 0
					end
				end

			-- 如果没有再次修改进度
			else
				if tOneTask.nTaskID == nTaskID then
					-- 设置小任务索引
					self.tTaskProgress.nCurTaskIdx = nIdx
					break
				end
			end
			
		end
	end
	-- print("nStageId: , nPageId: , nCopyId: ", self.tTaskProgress.nBigIdx, self.tTaskProgress.nCurMidIdx, self.tTaskProgress.nCurTaskIdx)

	return nChangeType
end
-- 设置一个大章节信息获取
function DATA_TASK:SetOneBigChapterFinishedInfo(nBigID, tTaskInfo)
	local tBigInfo = self:GetMidChapterListByBigID(nBigID)

	if tBigInfo == nil then
		return
	end

	local bIsCurProgress = false
	-- 如果当前的ID信息小于最大的解锁信息
	if self.tTaskProgress.nBigID == nBigID then
		bIsCurProgress = true
	end

	-- 标记已经获取进度信息
	tBigInfo.bIsGetMid = true
	-- -- 章节数目不正确
	-- if #tTaskInfo.tTasks ~= tBigInfo.tChapterList then
	-- 	CTAssert(0, "章节数目不正确")
	-- end
	-- 章节信息
	for i,tOneInfo in ipairs(tTaskInfo.tTasks) do
		for m,tOneChapter in ipairs(tBigInfo.tChapterList) do
			local nChapterID = tOneChapter.nID
			-- 找到对应章节信息
			if nChapterID == tOneInfo.nChapterID then



				-- 是否是进度的中章节
				local bIsInMidProgress = false

				-- 如果是进度中的大章节，则进度章节之后的属于未解琐
				if bIsCurProgress == true then
					-- 如果是进度章节之前，则已经解锁，boss关肯定解锁
					if nChapterID < self.tTaskProgress.nCurMidID then
						tOneChapter.bIsUnlock = true
					-- 如果是进度章节，则解锁，且现将boss关闭，如果下一层判断到达boss进度，再重新打开
					elseif nChapterID == self.tTaskProgress.nCurMidID then
						tOneChapter.bIsUnlock = true
						tOneChapter.tTaskList[#tOneChapter.tTaskList].bIsUnlock = false
						bIsInMidProgress = true
					-- 如果是进度章节之后，则没有解锁
					else
						tOneChapter.bIsUnlock = false
					end
				-- 不在进度大章节，即就是在之前，则全部解锁了
				else
					tOneChapter.bIsUnlock = true
				end

				-- BOSS剩余次数
				tOneChapter.tBossInfo.nRestCount = tOneInfo.nBossTimes

				-- 精英的剩余次数
				tOneChapter.tBossInfo.nJingYingTimes = tOneInfo.nPieceTimes

				-- 精英的已购买次数
				tOneChapter.tBossInfo.nJingYingBuyCnt = tOneInfo.nPieceBuyCnt

				-- 遍历任务过去，处理小任务完成情况
				local bIsEnd = true
				for nIdx,tOneTask in ipairs(tOneChapter.tTaskList) do
					-- 如果是进度章节之前，则已经解锁
					if self.tTaskProgress.nBigID > nBigID or nChapterID <= self.tTaskProgress.nCurMidID then
						-- 如果是小任务列表中到达的进度，则解开信息
						if tOneTask.nTaskID == tOneInfo.nTaskID then
							tOneTask.nCurCount = tOneInfo.nFinishCnt
							tOneTask.bIsUnlock = true
							bIsEnd = false
						else
							tOneTask.bIsUnlock = false
							if bIsEnd == true then
								tOneTask.nCurCount = tOneTask.nTotalCount
							else
								tOneTask.nCurCount = 0
							end
						end
					-- 如果不在在当前解锁的中章节中
					else
						tOneTask.nCurCount = 0
						tOneTask.bIsUnlock = false
					end
				end

				-- 如果是在解锁大章节中
				if bIsCurProgress == true then
					-- 如果在当前解锁的中章节中
					if bIsInMidProgress == false then
						-- 如果是进度之前的，则解锁boss关卡
						if nChapterID < self.tTaskProgress.nCurMidID then
							tOneChapter.tTaskList[#tOneChapter.tTaskList].bIsUnlock = true
						end
					end
				-- 不在进度大章节，即就是在之前，则解锁boss关卡
				else
					tOneChapter.tTaskList[#tOneChapter.tTaskList].bIsUnlock = true
				end


			end
		end
	end
end

-- 根据任务ID来判断当前关卡是否是当前进度的关卡，返回-1表明还没有玩到，0表示处在当前中，或者1表示已经玩过
function DATA_TASK:JudgeTaskIsCurProgressByID( nBigID, nMidID, nTaskID )
	-- 章节完成情况
	local nBigIdx, nMidIdx, nTaskIdx = self:GetTaskAllIdxByAllID(nBigID, nMidID, nTaskID)

	return self:JudgeTaskIsCurProgressByTaskIndex(nBigIdx, nMidIdx, nTaskIdx)
end
-- 根据任务的数组索引来判断当前关卡是否是当前进度的关卡，返回-1表明还没有玩到，0表示处在当前中，或者1表示已经玩过
function DATA_TASK:JudgeTaskIsCurProgressByTaskIndex( nBigIdx, nMidIdx, nTaskIdx )
	-- 章节完成情况
	if self.tTaskProgress.nBigIdx > nBigIdx then
		return StoryProgressType.HadPast;
	elseif self.tTaskProgress.nBigIdx < nBigIdx then
		return StoryProgressType.NotReach;
	end

	if self.tTaskProgress.nCurMidIdx > nMidIdx then
		return StoryProgressType.HadPast;
	elseif self.tTaskProgress.nCurMidIdx < nMidIdx then
		return StoryProgressType.NotReach;
	end
	
	if self.tTaskProgress.nCurTaskIdx > nTaskIdx then
		return StoryProgressType.HadPast;
	elseif self.tTaskProgress.nCurTaskIdx == nTaskIdx then
		return StoryProgressType.Progressing;
	end

	return StoryProgressType.NotReach;
end
-- 当前任务是否是最后一个任务，返回true表明是最后一个任务
function DATA_TASK:CurProgressIsLastTask()
	-- 先判断大章节是否是已经是最后一章
	local nTotal = #self.tChapterList
	if self.tTaskProgress.nBigIdx < nTotal then
		return false
	elseif self.tTaskProgress.nBigIdx < nTotal then
		return true
	end

	-- 再判断中章节是否是最后一章
	local tLastBigChapter = self.tChapterList[nTotal]
	nTotal = #tLastBigChapter.tChapterList
	if self.tTaskProgress.nCurMidIdx < nTotal then
		return false
	elseif self.tTaskProgress.nCurMidIdx < nTotal then
		return true
	end

	-- 再判断任务列表信息
	local tLastTask = tLastBigChapter.tChapterList[nTotal]
	nTotal = #tLastTask.tTaskList
	if self.tTaskProgress.nCurTaskIdx < nTotal then
		return false
	elseif self.tTaskProgress.nCurTaskIdx < nTotal then
		return true
	end

	return true
end

-- 设置任务完成情况,返回true表明，所有任务完成结束,nChapter
function DATA_TASK:SetOneTaskFinishInfo(nBigID, nMidID, nTaskID, nCnt, bIsLose)
	if type(nBigID) ~= "number" or type(nMidID) ~= "number" or type(nTaskID) ~= "number" or type(nCnt) ~= "number" then
		CTAssert(0, "设置数据章节进度错误!!")
		return false,1
	end

	if bIsLose == nil then
		bIsLose = false;
	end

	-- 获取当前小章节列表
	local tMidList = self:GetMidChapterInfoByInfo(nBigID, nMidID)
	if tMidList == nil then
		CTAssert(0, "获取任务信息错误！")
		return false,1
	end

	local bRet = false
	local nTIdx = 1
	local bHadUnlock = false
	local bHadChange = false
	-- 遍历任务信息
	for i,tOneTask in ipairs(tMidList.tTaskList) do
		-- 如果是要处理的任务信息
		if tOneTask.nTaskID == nTaskID then
			-- 如果是BOSS关卡
			if tOneTask.bIsBoss == true then
				-- boss关卡解锁，可以刷
				tOneTask.bIsUnlock = true
				-- 次数
				tOneTask.nCurCount = 1
				tMidList.tBossInfo.nRestCount = nCnt
				-- 如果是第一次完成
				if bIsLose == false then
					if self:JudgeTaskIsCurProgressByID(nBigID, nMidID, nTaskID) == StoryProgressType.Progressing then
						if self:CurProgressIsLastTask() == true then
							bRet = true
						end
					end

					-- 如果前面没有解锁，则解锁显示第一个，清除所有进度
					if bHadUnlock == false then
						tMidList.tTaskList[1].bIsUnlock = true
						for j,tOther in ipairs(tMidList.tTaskList) do
							tOther.nCurCount = 0
						end
					end
				end
			else
				-- 次数
				tOneTask.nCurCount = nCnt
				-- 如果次数使用完成，则解锁下一个
				if tOneTask.nCurCount >= tOneTask.nTotalCount then
					tOneTask.bIsUnlock = false
					tMidList.tTaskList[i+1].bIsUnlock = true
					-- 如果是第一次完成，则将解锁指向下一个
					if self:JudgeTaskIsCurProgressByID(nBigID, nMidID, nTaskID) == StoryProgressType.Progressing then
						self.tTaskProgress.nCurTaskIdx = i
						self.tTaskProgress.nCurTaskID = nTaskID
					end
				end
			end
			-- 返回
			bHadChange = true
			nTIdx = i
		else
			-- 如果前面已经有已经解锁的
			if tOneTask.bIsUnlock == true then
				bHadUnlock = true
			end
		end
	end

	CTAssert(bHadChange, "获取任务信息错误！")
	return bRet,nTIdx
end

-- 强制更新对应索引为当前进度关卡
function DATA_TASK:ForceToTask(nBigIdx, nMidIdx, nTaskIdx)
	local tBigInfo = self:GetMidChapterListByBigChapterIdx(nBigIdx)
	local tTaskLisk, tMidList = self:GetOneTaskInfoByIdx(nBigIdx, nMidIdx, nTaskIdx)

	self:SetAllTaskFinishedProgressInfo( tBigInfo.nID, tMidList.nID, tTaskLisk.nTaskID );
end

-- 根据ID获取获取得到的地址
function DATA_TASK:GetCardStoryAddress(nItemID)
	return self.tCardStoryInfo[ nItemID ];
end

-- 更新某个大章节的
