--
-- Author: Link
-- Date: 2014-02-28 10:28:44
--

-- cell上面的ui标签
local MultiCellUiTag = {
	LyTopTitle		= 5,	-- 顶部标签
	LyMidContent	= 0,	-- 中间内容标签
	LyBottom 		= 8,	-- 顶部分页标签
	LyAllSel		= 6,	-- 全选层

	SpOneLineBg		= 1,	-- 一行物品的底部信息
	SpAllSel		= 11,	-- 全选上面的勾

	LbTitle 		= 5,	-- cell类型的标题
	LbPageInfo		= 22,	-- 当前页数
	PSpCellBg 		= 15,   -- cell背景

	
	BtnPrevPage		= 9,	-- 上一页
	BtnNextPage		= 18,	-- 下一页
	BtnSelAll 		= 6,	-- 全选按钮
}
-- cell上面的ui标签
local MultiTuJianUiTag = {
	LyOneCellInfo	= 28,	-- 信息层
	LyOneItem 		= 0,	-- 一个物品层的信息

	LyLevel 		= 27, 	-- 等级层
	SpLevelBg 		= 35,	-- 等级背景
	SpLevelTitle	= 52,	-- 等级类型

	LbPageInfo		= 50,	-- 当前页数
	LyBtn 			= 1, 	-- 页数层
	BtnPrevPage		= 42,	-- 上一页
	BtnNextPage		= 46,	-- 下一页
	CellBg 			= 15, 	-- cell的背景
}

-- 物品列表的自定义Tag
local MultiListDelfaultCtTag1 = {
	SpIcon 			= 59,	-- 图标
	SpLvBot			= 64,	-- 等级的底图
	SpHadEquiped	= 65,	-- 是否已经装备

	SpMergeStar 	= 5,	-- 合成的星星底
	LbMergeLevel	= 6,	-- 合成的等级

	LbItemName 		= 67,	-- 物品名字
	LbItemLv 		= 12,	-- 物品的等级

	SpCurLv 		= 33, 	-- 物品的当前等级...S.A
	SpDaoJuBg 		= 40, 	-- 道具item背景
	SpHadUnlock		= 41,	-- 是否已经解锁

}
local MultiListDelfaultCtTag2 = {
	SpIcon 			= 39,	-- 图标
	SpBottom 		= 37,	-- 底下的托盘
	SpHadUnlock		= 41,	-- 是否已经解锁

	LbItemName 		= 40,	-- 物品名字
}

-- 表格类型
local MultiListType = {
	Single 		= 1,	-- 单选
	MultiSel 	= 2,	-- 多选
}

-- 表格的显示类型
local MultiListShowType = {
	SelItem 	= 1,	-- 选择物品
	TuJian 		= 2,	-- 图鉴
}

-- 每行的个数
local MultiListEvLineCnt = {
	Three = 3,	-- 每行3个
	Four  = 4, 	-- 每行4个
}

-- mark: ================================ CTMultiListCell ==========================================
local CTMultiListCell = class("CTMultiListCell", function()
	return display.newNode()
end)

-- 每行显示的个数
local MAX_COUNT_PER_LINE = 3

-- 自身单独的tag
local MultiListTag = {
	SelCircle 		= -234,	-- 选中的光圈
	MulIsSelected 	= -235,	-- 打勾选中的信息
}

-- 初始化
function CTMultiListCell:ctor()
	-- 列表管理的回调
	self.pDelegate = nil
	self.nCellIdx = 0
	self.nCurPage = 0
	self.nTotalPage = 0
	self.nCurType = MultiListType.Single
	self.nCurShowType = MultiListShowType.SelItem;
	self.bIsSelAll = true
	self.bHadTitle = true
	self.fTitleHeight = 00.0

	-- 按钮管理
	self.pBtnMgr = CTMenuManager:new()
	self.pBtnMgr:init()
	self:addChild(self.pBtnMgr, 1)
	self.pBtnMgr:release()

	-- 所有按钮
	self.pArrItems = {}

	-- 统计等级个数
	self.nCountLv = 0
	-- 记录当前行数
	self.nLineCount = 0

end

-- 初始化一些数据
function CTMultiListCell:InitMultiCellInfo( pCallBack, nType, nShowType)
	-- 类型
	nType = nType or MultiListType.Single
	nShowType = nShowType or MultiListShowType.SelItem;
	self.nCurShowType = nShowType;

	self.nCurType = nType 
	-- 回调
	self.pDelegate = pCallBack

	self.pCtuiLayer = CTUILayer:new();
	-- 如果是图鉴
	if self.nCurShowType == MultiListShowType.TuJian then
		-- 当前界面
		self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/TuJian.ctui"))
	-- 如果不是图鉴类型
	else
		self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/DuLiBeiBao.ctui"))
	end

	self.pCtuiLayer:setAnchorPoint(ccp(0.5, 1))
	self.pCtuiLayer:setPosition(ctprccp(0.5, 1))
	self.pCtuiLayer:ignoreAnchorPointForPosition(false)
	self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	self:addChild(self.pCtuiLayer)
	self.pCtuiLayer:release()

	self.pCtuiLayer:setDelegateLua(function(event)
	-- 如果是按钮点击事件
		if event.cbType == "btnCb" then
			self:CTUIButtonCallBack(event.Layer, event.Btn, event.Tag)
		end
	end)
	-- 设置是否为刷新
	self:SetReFreshFlag( false )
	-- 重新处理size
	self:DealCurSizeAndShow()
	
end

-- 刷新改变的信息显示
function CTMultiListCell:RefreshChange()
	self.pCtuiLayer:RemoveAllChildUi()
	self.pBtnMgr:removeAllChildren()
	-- 设置是否为刷新
	self:SetReFreshFlag( false )
	-- 所有按钮
	self.pArrItems = {}
	-- 重新处理size
	self:DealCurSizeAndShow()
	
end

-- 设置是否是刷新当前页面的标记
function CTMultiListCell:SetReFreshFlag( isBool )
	self.bRefresh = isBool
end

-- 获取当前刷新的标记
-- 是否是刷新当前页面的标记
function CTMultiListCell:GetReFreshFlag( )
	 return self.bRefresh 
end

-- 上下页的处理
function CTMultiListCell:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)

	-- 如果是图鉴
	if self.nCurShowType == MultiListShowType.TuJian then
		-- 上一页
		if nTag == MultiTuJianUiTag.BtnPrevPage then
			
			self.nCurPage = self.nCurPage - 1
			-- 如果页码数目达到最大
			if self.nCurPage < 0 then
				self.nCurPage = 0
			else
				self:ChangePageShowInfo()
			end

		-- 下一页
		elseif nTag == MultiTuJianUiTag.BtnNextPage then
			self.nCurPage = self.nCurPage + 1
			-- 如果页码数目达到最大
			if self.nCurPage >= self.nTotalPage then
				self.nCurPage = self.nCurPage - 1
			else
				self:ChangePageShowInfo()
			end
		end

	-- 其他类型
	else
		-- 上一页
		if nTag == MultiCellUiTag.BtnPrevPage then
			
			self.nCurPage = self.nCurPage - 1
			-- 如果页码数目达到最大
			if self.nCurPage < 0 then
				self.nCurPage = 0
			else
				self:ChangePageShowInfo()
			end

		-- 下一页
		elseif nTag == MultiCellUiTag.BtnNextPage then
			self.nCurPage = self.nCurPage + 1
			-- 如果页码数目达到最大
			if self.nCurPage >= self.nTotalPage then
				self.nCurPage = self.nCurPage - 1
			else
				self:ChangePageShowInfo()
			end

		-- 全选
		elseif nTag == MultiCellUiTag.BtnSelAll then
			-- 如果之前已经是全选了
			if self.bIsSelAll == true then
				self.bIsSelAll = false
			else
				self.bIsSelAll = true
			end
			-- 根据情况选不选择勾
			self.pCtuiLayer:SetCTUINodeVisiable(MultiCellUiTag.SpAllSel, self.bIsSelAll)

			local tArrIdx = {}
			local tArrItems = {}
			-- 遍历过去，看哪个发生了改变
			for i,pOneItem in ipairs(self.pArrItems) do
				local bIsSel, bHadChange = self:ChangeItemMultiItemSelStatus(pOneItem, self.bIsSelAll)
				if bHadChange == true and pOneItem:isVisible() then
					tArrIdx[ #tArrIdx +1 ] = pOneItem:getTag()
					tArrItems[ #tArrItems + 1 ] = pOneItem
				end
			end
			-- 如果有发生了改变，则传递通知
			if (#tArrIdx) > 0 then
				-- 如果是全选，则判断是否能选中
				if self.bIsSelAll == true then

					tArrIdx = self.pDelegate:CallBackIsCellCanMultiSelItems(self:getCellIdx(), tArrIdx, tArrItems);

					for i,pOneItem in ipairs(tArrItems) do
						if i > #tArrIdx then
							self:ChangeItemMultiItemSelStatus(pOneItem, false);
						end
					end
				end
				-- 通知发生了变化
				self.pDelegate:CallBackCellMultiSelItems(self:getCellIdx(), tArrIdx, tArrItems, self.bIsSelAll)
			end
		end
	end
end

-- 创建同一个的一种类型
function CTMultiListCell:CreateDefaultButton1()
	local pOneItem = CTMenuItemSprite:new()
	pOneItem:initWithFiles("Images/GX-058.png", "Images/GX-058.png", "Images/GX-058.png")
	pOneItem:SetTouchEndedLuaCallBack(function ( pBtnNode )
		self:CellItemClick(pBtnNode)
	end)

	-- 姓名背景 .png
	local pNameBg = CCSprite:create("Images/GX-061.png")
	tolua.cast(pNameBg, "CCSprite")
	pNameBg:setScale(1.11)
	pNameBg:setPosition(ctspccp(pOneItem, 0.5, -0.2))
	-- pOneItem:addChild(pNameBg)

	-- 随便创建一个图标
	local pSpIcon = CCSprite:create("Images/GX-034.png")
	pSpIcon:setPosition(ctspccp(pOneItem, 0.5, 0.5))
	pOneItem:addChild(pSpIcon, 1, MultiListDelfaultCtTag1.SpIcon)

	-- 随便创建一个图标的名字
	local pLbName = CTLabel:create("暂无", LABEL_BOLD_FONT_NAME, 24)
	pLbName:setPosition(ctspccp(pOneItem, 0.5, -0.15))
	pLbName:setColor(ccc3(0, 14, 57))
	pLbName:setShadowOffset(0.1, 0.1)
	pOneItem:addChild(pLbName, 1, MultiListDelfaultCtTag1.LbItemName)

	-- 等级信息底
	local pSpLvBot =  CCSprite:create("Images/GX-021.png") 
	pSpLvBot:setAnchorPoint(ccp(1.1, 1.1))
	pSpLvBot:setPosition(ctspccp(pOneItem, 1.1, 1.1))
	pOneItem:addChild(pSpLvBot, 1, MultiListDelfaultCtTag1.SpLvBot)

	-- 等级
	local pLbLevel =   CTLabel:create("1", LABEL_BOLD_FONT_NAME, 20)
	pLbLevel:setPosition(ctspccp(pSpLvBot, 0.5, 0.5))
	pSpLvBot:addChild(pLbLevel, 1, MultiListDelfaultCtTag1.LbItemLv)

	-- 等级信息底
	local pSpMergeLvBot = CCSprite:create("Images/GX-023.png")
	pSpMergeLvBot:setAnchorPoint(ccp(0, 1))
	pSpMergeLvBot:setPosition(ctspccp(pOneItem, 0, 1))
	pOneItem:addChild(pSpMergeLvBot, 1, MultiListDelfaultCtTag1.SpMergeStar)
	pSpMergeLvBot:setVisible(false)

	-- 等级
	local pLbMergeLevel = CTLabel:create("1", LABEL_BOLD_FONT_NAME, 20)
	pLbMergeLevel:setPosition(ctspccp(pSpMergeLvBot, 0.5, 0.5))
	pLbMergeLevel:setColor(ccc3(128, 0, 0))
	pSpMergeLvBot:addChild(pLbMergeLevel, 1, MultiListDelfaultCtTag1.LbMergeLevel)

	-- 已经装备 
	local pSpEquiped = CCSprite:create("Images/GX-060.png")
	pSpEquiped:setAnchorPoint(ccp(0.5, 0.5))
	pSpEquiped:setPosition(ctspccp(pOneItem, 0.7, 0.32))
	pOneItem:addChild(pSpEquiped, 1, MultiListDelfaultCtTag1.SpHadEquiped)
	pSpEquiped:setVisible(false)


	return pOneItem;
end

-- 更新页码数目
function CTMultiListCell:ChangePageShowInfo()
	self.nLineCount = 0
	-- 获取总数
	local nTotalCount = self.pDelegate:CallBackGetCellCount( self:getCellIdx() )
	-- 获取每页的行数
	local nLineCount = self.pDelegate:CallBackGetCellLineCountPerPage( self:getCellIdx() )
	-- 单独处理道具的问题
	-- print("nTotalCount ", nTotalCount, )
	-- 每页的总数
	local nPageCount = nLineCount*MAX_COUNT_PER_LINE
	-- print("nLineCount ", nTotalCount, nLineCount, MAX_COUNT_PER_LINE, nPageCount)
	-- print("self:getCellIdx() ", self:getCellIdx(), nLineCount, nTotalCount)
	-- 如果有多页，则更新显示页面信息
	if nTotalCount > nPageCount then
		-- 如果是图鉴
		if self.nCurShowType == MultiListShowType.TuJian then
			-- 更新页码数目
			local pLbPage = self.pCtuiLayer:GetNodeByTag(MultiTuJianUiTag.LbPageInfo)
			tolua.cast(pLbPage, "CCLabelTTF")
			pLbPage:setString("页码: " .. (self.nCurPage+1) .. " / " .. self.nTotalPage)

			-- 按钮的显示颜色处理
			local pBtnNext = self.pCtuiLayer:GetNodeByTag(MultiTuJianUiTag.BtnNextPage)
			tolua.cast(pBtnNext, "CTMenuItem")
			local pBtnPrev = self.pCtuiLayer:GetNodeByTag(MultiTuJianUiTag.BtnPrevPage)
			tolua.cast(pBtnPrev, "CTMenuItem")

			if self.nCurPage == 0 then
				pBtnPrev:SetEnable(false)
			else
				pBtnPrev:SetEnable(true)
			end

			if self.nCurPage >= self.nTotalPage - 1 then
				pBtnNext:SetEnable(false)
			else
				pBtnNext:SetEnable(true)
			end

		-- 其他类型
		else
			-- 更新页码数目
			local pLbPage = self.pCtuiLayer:GetNodeByTag(MultiCellUiTag.LbPageInfo)
			tolua.cast(pLbPage, "CCLabelTTF")
			pLbPage:setString("页码: " .. (self.nCurPage+1) .. " / " .. self.nTotalPage)

			-- 按钮的显示颜色处理
			local pBtnNext = self.pCtuiLayer:GetNodeByTag(MultiCellUiTag.BtnNextPage)
			-- tolua.cast(pBtnNext, "CTMenuItem")
			local pBtnPrev = self.pCtuiLayer:GetNodeByTag(MultiCellUiTag.BtnPrevPage)
			-- tolua.cast(pBtnPrev, "CTMenuItem")

			if self.nCurPage == 0 then
				pBtnPrev:setColor(ccc3(100, 100, 100))
			else
				pBtnPrev:setColor(ccc3(255, 255, 255))
			end

			if self.nCurPage >= self.nTotalPage - 1 then
				pBtnNext:setColor(ccc3(100, 100, 100))
			else
				pBtnNext:setColor(ccc3(255, 255, 255))
			end
		end
	end
	-- 获取当前页码开始索引
	local nStartIdx = self.nCurPage*nPageCount
	local nEndIdx = nStartIdx + nPageCount-1
	if nEndIdx >= nTotalCount then
		nEndIdx = nTotalCount-1
	end

	-- 多选的发生选项
	local tArrIdx = {}
	local tArrItems = {}

	-- 清除所有原来的信息
	for i=1,#self.pArrItems do
		local pOneItem = self.pArrItems[i]
		-- 如果原来是单选，则要去除信息
		if self.nCurType == MultiListType.Single then
			-- 如果光环在，则删除原本的原则
			local pSpCirCle = pOneItem:getChildByTag(MultiListTag.SelCircle)
			if pSpCirCle ~= nil then
				self.pDelegate:CallBackCellSingleClickOneItem(self:getCellIdx(), pOneItem:getTag(), pOneItem, false, false)
			end
		-- 如果是多选
		else
			local bIsSel, bHadChange = self:ChangeItemMultiItemSelStatus(pOneItem, false)
			if bHadChange == true then
				tArrIdx[ #tArrIdx +1 ] = pOneItem:getTag()
				tArrItems[ #tArrItems + 1 ] = pOneItem
			end
		end
		pOneItem:setVisible(false)
	end

	-- 如果是图鉴
	if self.nCurShowType == MultiListShowType.TuJian then
	-- 其他类型
	else
		-- 如果是多选
		if self.nCurType ~= MultiListType.Single then
			-- 去掉全选
			self.pCtuiLayer:SetCTUINodeVisiable(MultiCellUiTag.SpAllSel, false)
			self.bIsSelAll = false
			-- 如果有发生了改变，则传递通知
			if (#tArrIdx) > 0 then
				-- 通知发生了变化
				self.pDelegate:CallBackCellMultiSelItems(self:getCellIdx(), tArrIdx, tArrItems, self.bIsSelAll)
			end
		end
	end
	
	local nOffsePos = 260
	if MAX_COUNT_PER_LINE ~= 3 then 
		nOffsePos = 180
	end


	local fCurScale = ctipwidth(1)
	-- 一行的高度
	local fOneLineHeight = 210.0
	-- 一个一个的生成 这个可以算出几个item
	-- print("*****************************")
	for i=nStartIdx,nEndIdx do
		local nIdx = i-nStartIdx
		-- print("i = ....... ", i)
		local pOneItem = self.pArrItems[nIdx+1]
		-- 如果没有按钮，则处理好
		if pOneItem == nil then
			
			local nVIdx = math.floor(nIdx % MAX_COUNT_PER_LINE)
			local nHIdx = math.floor(nIdx / MAX_COUNT_PER_LINE)

			local fPosX = 0
			-- 有没有标题，高度不一样
			local fPosY = 0
			-- 如果是图鉴
			if self.nCurShowType == MultiListShowType.TuJian then
				fPosX = ctipwidth(150) + ctipwidth(210)*nVIdx
				fPosY = self:getContentSize().height - ctipwidth( 210 + 150*nHIdx + 180);
				
				pOneItem = CTMenuItem:new()
				pOneItem:init()
				pOneItem:setContentSize( CCSizeMake(ctipwidth(160), ctipwidth(160)) )
				
				pOneItem:setPosition(ccp(fPosX + 30, fPosY + 210))
				pOneItem:setScale(fCurScale*1.1)
				pOneItem:SetTouchEndedLuaCallBack(function ( pBtnNode )
					self:CellItemClick(pBtnNode)
				end)
				-- 修正位置
				local pAddNode = self.pCtuiLayer:ReCreateUiNode(MultiTuJianUiTag.LyOneItem, false, MultiTuJianUiTag.LyOneItem, 0, UiNotShowEnd)
				pAddNode:setScale( fCurScale)
				pOneItem:addChild(pAddNode)
				local pSpCar = pOneItem:GetChildNodeByTag(MultiListDelfaultCtTag2.SpIcon)
				pAddNode:setPosition( ccp(-(pSpCar:getPositionX()- 66)*fCurScale, -(pSpCar:getPositionY()-80)*fCurScale) )

			-- 其他类型
			else
				local pSpItmeBg = nil 
				local fScale = 1.2
				-- 控制cell直接的间隔
				fPosX = ctipwidth(130) + ctipwidth(210)*nVIdx
				if self.bHadTitle == true then
					fScale = 1.25
					fPosX = ctipwidth(75) + ctipwidth(nOffsePos)*nVIdx
					fPosY = self:getContentSize().height - ((fOneLineHeight - 50 )*(nHIdx* 1.05+0.52))*fCurScale - ctipwidth(75)
					-- 每行单独添加背景
					local nTmpLine = math.floor(i / MAX_COUNT_PER_LINE) + 1
					if nTmpLine ~= self.nLineCount then 
						self.nLineCount = nTmpLine
						pSpItmeBg =  self.pCtuiLayer:ReCreateUiNode(MultiCellUiTag.PSpCellBg, false, MultiCellUiTag.PSpCellBg, 0, UiNotShowEnd)
						pSpItmeBg:setAnchorPoint( ccp(0, 0.5) )
						pSpItmeBg:setPositionY(38)
						pSpItmeBg:setPositionX(-10)
					end
				else
					fPosY = self:getContentSize().height - (fOneLineHeight*(nHIdx+0.5))*fCurScale -  ctipwidth(5)
				end 
				-- 在这里处理添加背景条问题
				pOneItem = self:CreateDefaultButton1();
				pOneItem:setAnchorPoint(ccp(0.2, 0.5))
				pOneItem:setPosition(ccp(fPosX, fPosY + 12))
				pOneItem:setScale(fCurScale*fScale)

				if pSpItmeBg ~= nil then 
					pOneItem:addChild(pSpItmeBg, -1)
					pSpItmeBg:setScaleY(1.17)
					pSpItmeBg:setScaleX(1.17)
				end

			end

			-- 设置tag信息
			self.pBtnMgr:AddMenuItem(pOneItem, i)
			pOneItem:release()

			self.pArrItems[nIdx+1] = pOneItem

			-- 创建通用显示信息
			-- 如果是图鉴
			if self.nCurShowType == MultiListShowType.TuJian then
				self.pDelegate:CallBackCreateCellShowInfo(self:getCellIdx(), i, pOneItem, MultiListDelfaultCtTag2)
			else
				self.pDelegate:CallBackCreateCellShowInfo(self:getCellIdx(), i, pOneItem, MultiListDelfaultCtTag1)
			end
		else
			pOneItem:setVisible(true)
			pOneItem:setTag(i)
		end

		-- 如果是图鉴
		if self.nCurShowType == MultiListShowType.TuJian then
			-- 回调处理显示信息
			self.pDelegate:CallBackCellShowInfo(self:getCellIdx(), i, pOneItem, MultiListDelfaultCtTag2)
		else
			-- 回调处理显示信息
			self.pDelegate:CallBackCellShowInfo(self:getCellIdx(), i, pOneItem, MultiListDelfaultCtTag1)
		end
	end

end

-- 根据情况获取显示和显示当前标题
function CTMultiListCell:DealCurSizeAndShow()
	-- 如果是图鉴
	if self.nCurShowType == MultiListShowType.TuJian then
		self:DealCurSizeAndShowTuJianType()
	-- 其他类型
	else
		self:DealCurSizeAndShowSelItemType()

	end
end

-- 显示创建默认选中的信息
function CTMultiListCell:DealCurSizeAndShowSelItemType()
	-- 获取总数
	local nTotalCount = self.pDelegate:CallBackGetCellCount( self:getCellIdx() )
	-- 获取每页的行数
	local nLineCount = self.pDelegate:CallBackGetCellLineCountPerPage( self:getCellIdx() )
	-- 每页的总数
	local nPageCount = nLineCount*MAX_COUNT_PER_LINE
	-- 显示的总页数
	self.nTotalPage = math.ceil( (nTotalCount*1.0) / nPageCount )
	-- 不可能没有任何一页
	if self.nTotalPage <= 0 then
		-- CTAssert(0, "分页显示处理时出错了，页数为："..self.nTotalPage)
		return
	end
	-- 不能超过最后一页
	if self.nCurPage >= self.nTotalPage then
		self.nCurPage = self.nTotalPage-1
	end

	-- 获取显示区域
	local tFrame = self.pDelegate:CallBackGetShowFrame()

	-- -- 显示所有信息
	self.pCtuiLayer:ShowAllUiInfo()
	-- -- 修改标题
	local strTitle = self.pDelegate:CallBackGetTypeTitle( self:getCellIdx() )
	-- print("self:getCellIdx() ", self:getCellIdx(), strTitle, nTotalCount, self.nTotalPage, nPageCount)
	-- 如果标题不存在，则要做不同的处理
	if strTitle == nil then
		self.bHadTitle = false
		-- 移除几个层
		self.pCtuiLayer:RemoveChildUiByTag(0)
		self.pCtuiLayer:RemoveChildUiByTag(2)
		self.pCtuiLayer:RemoveChildUiByTag(8)
	else 
		self.bHadTitle = true
		self:ChangeHeadTitle(strTitle, MultiCellUiTag.LbTitle)
		-- cell 背景
		self.pCtuiLayer:SetCTUINodeVisiable(MultiCellUiTag.PSpCellBg, false)
	end
	-- 如果是单选
	if self.nCurType == MultiListType.Single then
		-- 将全选层跟删除掉
		self.pCtuiLayer:RemoveChildUiByTag(MultiCellUiTag.LyAllSel)
	else
		-- 不显示勾
		self.pCtuiLayer:SetCTUINodeVisiable(MultiCellUiTag.SpAllSel, false)
	end
	-- 用与控制CellSize的高度 默认值为多页中的2行
	local nCellSziey = 18
	-- 根据不同的行数来控制上下页按钮层的位置
	local nPageLy = 205
	-- 如果只有一页，则需要特殊处理
	local bIsMulPage = true
	if self.nTotalPage == 1 then
		nLineCount = math.ceil( (nTotalCount*1.0) / MAX_COUNT_PER_LINE )
		bIsMulPage = false
		-- 没有页数，则删除分页层
		self.pCtuiLayer:RemoveChildUiByTag(MultiCellUiTag.LyBottom)
		-- 没有页数区分是一行还是2行 3行
		if nTotalCount > MAX_COUNT_PER_LINE  then 
			-- 3 行
			if   nTotalCount >  (nLineCount-1)*MAX_COUNT_PER_LINE  and nLineCount == 3 then 
				nCellSziey = -35
				if self:GetReFreshFlag() == true then 
					-- nCellSziey = 0
				end
			-- 2 行
			else
				nCellSziey = 0
				if self:GetReFreshFlag() == true then 
					 -- nCellSziey = 0
				end
			end
		-- 1行	
		else
			nCellSziey = 48
			if self:GetReFreshFlag() == true then 
				-- nCellSziey = 0
			end
		end
	else
		if self.bHadTitle == true then
			-- 3行
			if nLineCount == 3 then 
				nPageLy = 240
				nCellSziey = -18
			end
			local pLyBottom = self.pCtuiLayer:GetNodeByTag(MultiCellUiTag.LyBottom)
			pLyBottom:setPositionY( pLyBottom:getPositionY() + nPageLy)
		end
	end

	-- 一行的高度
	local fOneLineHeight = 210.0
	-- 有没有标题，高度不一样
	local fHeight = fOneLineHeight * nLineCount
	if self.bHadTitle == true then
		fHeight = fHeight + nCellSziey
	end
	if bIsMulPage == true then
		-- 页数，重新处理页数层的位置
		local pLyPage = self.pCtuiLayer:GetNodeByTag(MultiCellUiTag.LyBottom)
		pLyPage:setPositionY( pLyPage:getPositionY() - fOneLineHeight * (nLineCount-1))
		fHeight = fHeight + 60
	end

	-- 获取真实高度
	local tHeightPos1 = self.pCtuiLayer:convertToWorldSpace(ccp(0, fHeight))
	local tHeightPos2 = self.pCtuiLayer:convertToWorldSpace(ccp(0, 0))
	local fOrigHeight = tHeightPos1.y-tHeightPos2.y
	
	if self:GetReFreshFlag() == false then 
		self:setContentSize(CCSizeMake(tFrame.size.width, fOrigHeight + ctipwidth(0)))
		-- 各层次位置重新布局 控制背景层
		self.pCtuiLayer:setPosition(ccp(tFrame.size.width*0.5, fOrigHeight + ctipwidth(190)))
	end

	local nTimes = 0
	-- 创建多行信息  B方案  只创建每个等级的东西 只需要创建一次  统计好等级个数之后直接创建即可
   local i = self:getCellIdx() + 1
   -- 有标题
   if strTitle ~= nil then 
	   	-- 根据每页的个数进行计算
	   	local fOffsety = 0
	   	local fHeight = 0
	   	local pLyMid = self.pCtuiLayer:GetNodeByTag(20)
	   	local fScaley = 0
	   	tolua.cast(pLyMid, "CCSprite")
	   	fHeight = pLyMid:getContentSize().height
	   	-- pLyMid:setVisible(false)


	   	local pLyDown = self.pCtuiLayer:GetNodeByTag(21)
	   	-- pLyDown:setVisible(false)

	   	-- local pOtherCell = self.pCtuiLayer:ReCreateUiNode(MultiCellUiTag.SpOneLineBg, true, (MultiCellUiTag.SpOneLineBg+100*(i-1)), 100*(i-1))
	   	-- tolua.cast(pOtherCell, "CCSprite")
	   	-- 每页个数为6个
	   	if  nPageCount == MAX_COUNT_PER_LINE*2 then
	   		-- 当前拥有个数 只有一行
	   		if nTotalCount < MAX_COUNT_PER_LINE or nTotalCount == MAX_COUNT_PER_LINE then 
	   			fScaley = 0.17
	   			pLyMid:setScaleY(fScaley)
	   			fOffsety = fHeight*(1-fScaley)
	   			pLyDown:setPositionY(pLyDown:getPositionY() + fOffsety)
	   			-- pOtherCell:setScaleY(0.5)
	   		elseif nTotalCount >MAX_COUNT_PER_LINE and nTotalCount < MAX_COUNT_PER_LINE*2 + 1 then
	   			-- pOtherCell:setScaleY(0.95)
	   		end

	   	elseif nPageCount == MAX_COUNT_PER_LINE*3 then
	   		-- 只有一行
	   		if nTotalCount < MAX_COUNT_PER_LINE or nTotalCount == MAX_COUNT_PER_LINE then
	   			fScaley = 0.17
	   		-- 2行 
	   		elseif nTotalCount >MAX_COUNT_PER_LINE and nTotalCount < MAX_COUNT_PER_LINE*2 + 1 then
	   			fScaley = 1
	   		-- 3行
	   		else
	   			fScaley = 1.9
	   		end
	   		pLyMid:setScaleY(fScaley)
	   		-- if fScaley > 1 then 
	   		-- 	fOffsety = fHeight*(fScaley-1)
	   		-- else
	   			fOffsety = fHeight*(1-fScaley)
	   		-- end
	   		pLyDown:setPositionY(pLyDown:getPositionY() + fOffsety)

	   end
	end

	-- 删除多余的底图
	self.pCtuiLayer:RemoveChildUiByTag(MultiCellUiTag.SpOneLineBg)

	-- 更新页面
	self:ChangePageShowInfo()
end 

-- 显示创建默认选中的图鉴类型
function CTMultiListCell:DealCurSizeAndShowTuJianType()
	
	-- 获取总数
	local nTotalCount = self.pDelegate:CallBackGetCellCount( self:getCellIdx() )

	-- 获取每页的行数
	local nLineCount = self.pDelegate:CallBackGetCellLineCountPerPage( self:getCellIdx() )

	-- 每页的总数
	local nPageCount = nLineCount*MAX_COUNT_PER_LINE
	-- 显示的总页数
	self.nTotalPage = math.ceil( (nTotalCount*1.0) / nPageCount )
	-- 不可能没有任何一页
	if self.nTotalPage <= 0 then
		-- CTAssert(0, "分页显示处理时出错了，页数为："..self.nTotalPage)
		return
	end
	-- 不能超过最后一页
	if self.nCurPage >= self.nTotalPage then
		self.nCurPage = self.nTotalPage-1
	end

	-- 获取显示区域
	local tFrame = self.pDelegate:CallBackGetShowFrame()

	-- 显示所有信息
	
	-- self.pCtuiLayer:ShowAllUiInfo()
	self.pCtuiLayer:ShowChildUiByTag(MultiTuJianUiTag.LyOneCellInfo);
	self.pCtuiLayer:SetCTUINodeVisiable(MultiTuJianUiTag.LyOneItem, false)
	self.pCtuiLayer:SetCTUINodeVisiable(MultiTuJianUiTag.CellBg, false)

	-- -- 修改标题
	-- local strTitle = self.pDelegate:CallBackGetTypeTitle( self:getCellIdx() )

	-- -- 如果标题不存在，则要做不同的处理
	-- local pSpTitle = self.pCtuiLayer:GetNodeByTag( MultiTuJianUiTag.SpLevelTitle )
	-- self:ChangeHeadTitle(strTitle, MultiTuJianUiTag.SpLevelTitle)


	local nTitleH = 210
	-- 如果只有一页，则需要特殊处理
	local bIsMulPage = true
	if self.nTotalPage == 1 then
		nLineCount = math.ceil( (nTotalCount*1.0) / MAX_COUNT_PER_LINE )
		bIsMulPage = false
		-- 没有页数，则删除分页层 
		self.pCtuiLayer:SetCTUINodeVisiable(MultiTuJianUiTag.LbPageInfo, false)
		-- self.pCtuiLayer:SetCTUINodeVisiable(MultiTuJianUiTag.BtnPrevPage, false)
		-- self.pCtuiLayer:SetCTUINodeVisiable(MultiTuJianUiTag.BtnNextPage, false)
		self.pCtuiLayer:SetCTUINodeVisiable(MultiTuJianUiTag.LyBtn, false)
		nTitleH = 140
	end
	self.pCtuiLayer:SetCTUINodeVisiable(MultiTuJianUiTag.LyLevel, false)

	-- 有没有页数，高度不一样 控制cell的高度
	local fHeight = 600.0
	if bIsMulPage == false then
		fHeight = fHeight - 40
	end

	-- 获取真实高度
	local tHeightPos1 = self.pCtuiLayer:convertToWorldSpace(ccp(0, fHeight))
	local tHeightPos2 = self.pCtuiLayer:convertToWorldSpace(ccp(0, 0))
	local fOrigHeight = tHeightPos1.y-tHeightPos2.y

	-- 设置cell的尺寸大小 ctipwidth(280)
	self:setContentSize(CCSizeMake(tFrame.size.width , fOrigHeight - 110))

	-- 各层次位置重新布局
	self.pCtuiLayer:setPosition(ccp(tFrame.size.width*0.5, fOrigHeight + 110))

	-- 等级层
	local pLyTitle =  self.pCtuiLayer:ReCreateUiNode(MultiTuJianUiTag.LyLevel, false, MultiTuJianUiTag.LyLevel, 0, UiNotShowEnd)
	pLyTitle:setScale(0.9)
	pLyTitle:setPositionY(-fOrigHeight + nTitleH)	
	pLyTitle:setVisible(true)
	
	local lyTitle = CCLayer:create()
	self:addChild(lyTitle, 10)
	lyTitle:addChild(pLyTitle , 20)
	-- 等级背景
	local pSpTitleBg = pLyTitle:getChildByTag(MultiTuJianUiTag.SpLevelBg)
	-- 等级
	local pSpTitle = pSpTitleBg:getChildByTag(MultiTuJianUiTag.SpLevelTitle)
	-- 修改标题
	local strTitle = self.pDelegate:CallBackGetTypeTitle( self:getCellIdx() )
	-- 如果标题不存在，则要做不同的处理
	self:ChangeHeadTitle(strTitle, MultiTuJianUiTag.SpLevelTitle, pSpTitle)

	-- 更新页面
	self:ChangePageShowInfo()
end 


-- 改变标题
function CTMultiListCell:ChangeHeadTitle( strTitle, nTitleTag , pObject)
	-- 如果标题不存在，则要做不同的处理
	if strTitle == nil then
		strTitle = "E";
	end

	local pSpTitle = pObject
	-- 不同等级的信息显示不同等级的标题
	strTitle = string.upper(strTitle);
	-- 处理图鉴问题
	if pSpTitle ~= nil then 
		tolua.cast(pObject, "CCSprite")
		pObject:setTexture(pTitleTex)
		
	else
		pSpTitle = self.pCtuiLayer:GetNodeByTag( nTitleTag )
		tolua.cast(pSpTitle, "CCSprite")
		pSpTitle:setTexture(pTitleTex)

	end
	local pLetterTex  = SHARE_FUNC:GetCardBigStarTextureByStar(strTitle)
	pSpTitle:setTexture( pLetterTex )
	
	
end

-- 按钮点击处理
function CTMultiListCell:CellItemClick( pCellItem )
	-- 如果是图鉴
	if self.nCurShowType == MultiListShowType.TuJian then
		self.pDelegate:CallBackCellSingleClickOneItem(self:getCellIdx(), pCellItem:getTag(), pCellItem, true, true)
	-- 其他类型
	else
		-- 如果是单选
		if self.nCurType == MultiListType.Single then
			-- 如果之前没有选中，则开始提示选中信息
			if self.pDelegate:CallBackIsSigleSelected(self:getCellIdx(), pCellItem:getTag()) == false then
				
				self.pDelegate:CallBackCellSingleClickOneItem(self:getCellIdx(), pCellItem:getTag(), pCellItem, true, true)
				-- 显示选中的光环
				self:ShowOrCancelSelCircle(pCellItem:getTag(), true)
				
				-- 对背包按钮选择单独
			end
		else
			-- 如果是全选，则去掉全选属性
			if self.bIsSelAll == true then
				self.pCtuiLayer:SetCTUINodeVisiable(MultiCellUiTag.SpAllSel, false)
				self.bIsSelAll = false
			end
			-- 切换到另一种状态
			local bIsSel, bIsChange = self:ChangeItemMultiItemSelStatus(pCellItem, nil)

			local tArrIdx = {}
			tArrIdx[1] = pCellItem:getTag()
			local tArrItems = {}
			tArrItems[1] = pCellItem
			-- 如果是全选，则判断是否能选中
			if bIsSel == true then
				tArrIdx = self.pDelegate:CallBackIsCellCanMultiSelItems(self:getCellIdx(), tArrIdx, tArrItems);
				if #tArrIdx <= 0 then
					self:ChangeItemMultiItemSelStatus(pCellItem, false)
				end
			end
			-- 通知发生了变化
			self.pDelegate:CallBackCellMultiSelItems(self:getCellIdx(), tArrIdx, tArrItems, bIsSel)
		end
	end
end

-- 设置和获取cell的idx索引
function CTMultiListCell:setCellIdx( nIdx )
	self.nCellIdx = nIdx
end
function CTMultiListCell:getCellIdx()
	return self.nCellIdx
end

-- 强制性单向选择
function CTMultiListCell:ForceSingleSelected( nCellIdx, bIsCallBack)
	local pSelItem
	for i,pOneItem in ipairs(self.pArrItems) do
		if pOneItem:getTag() == nCellIdx then
			pSelItem = pOneItem
			break
		end
	end
	-- 我的信息
	if pSelItem ~= nil then
		if bIsCallBack == true then
			self:CellItemClick( pSelItem )
		else
			-- 如果之前没有选中，则开始提示选中信息
			if self.pDelegate:CallBackIsSigleSelected(self:getCellIdx(), pSelItem:getTag()) == false then
				self.pDelegate:CallBackCellSingleClickOneItem(self:getCellIdx(), pSelItem:getTag(), pSelItem, true, false)
			end
			-- 显示选中的光环
			self:ShowOrCancelSelCircle(pSelItem:getTag(), true)
		end
		return true
	end
	return false
end

-- 显示或者取消对应单选的item的光环
function CTMultiListCell:ShowOrCancelSelCircle( nCellIdx, bIsShow )
	if self.nCurShowType == MultiListShowType.TuJian then 
		return 
	end

	local pSelItem
	for i,pOneItem in ipairs(self.pArrItems) do
		if pOneItem:getTag() == nCellIdx then
			pSelItem = pOneItem
			break
		end
	end

	-- 找到按钮，则取消掉显示
	if pSelItem ~= nil then
		local pSpTemImg 
		-- 删除东西
		if bIsShow == true then
			--现实光环
			
			local pSpSelCircle = CCSprite:create("Images/GX-017-9.png")
			pSpSelCircle:setAnchorPoint(ccp(0.5, 0.5))
			pSpSelCircle:setPosition(ctspccp(pSelItem, 0.5, 0.5))
			pSpSelCircle:setTag(pSelItem:getTag()*111)
			pSelItem:addChild(pSpSelCircle)
		else
			local pSpReCircle = pSelItem:getChildByTag(pSelItem:getTag()*111)
			if pSpReCircle then 
				pSpReCircle:removeFromParent()
			end
		end
	end
end

-- 改变当前Item的多选中的选中情况,bIsSel为nil时，表示选择相反的
function CTMultiListCell:ChangeItemMultiItemSelStatus( pCellItem, bIsSel )
	-- 获取信息
	local pSpHadSel = pCellItem:getChildByTag(MultiListTag.MulIsSelected)

	-- 是否发生了改变
	local bIsHadChange = false

	-- 如果是要选择相反的信息
	if bIsSel == nil then
		if pSpHadSel == nil then
			bIsSel = true
		else
			bIsSel = false
		end
		bIsHadChange = true
	end

	-- 如果是要选中
	if bIsSel == true then
		-- 如果打勾为空，则创建一个
		if pSpHadSel == nil then
			local pSpCirCle = CCSprite:create("Images/GouXuan.png")
			pSpCirCle:setPosition(ctspccp(pCellItem, 0.75, 0.25))
			pCellItem:addChild(pSpCirCle, 9999, MultiListTag.MulIsSelected)

			bIsHadChange = true
		end
	else
		-- 如果之前存在，则删除掉
		if pSpHadSel ~= nil then
			pSpHadSel:removeFromParent()

			bIsHadChange = true
		end
	end

	return bIsSel, bIsHadChange
end

-- mark: ===== 触控 ======
-- 开始触控
function CTMultiListCell:MultiCellTouchBegan(tLocation, nCount)
	local tLocPos = self.pBtnMgr:convertToNodeSpace(tLocation)
	self.pBtnMgr:touchMenuItemBeganByPoint(tLocPos)
	return self.pCtuiLayer:CTUITouchBegan(tLocation)
end

-- 触控移动
function CTMultiListCell:MultiCellTouchMoved(tLocation, tPrevPos)
	-- 偏差的距离够
	if ccpDistance(tLocation, tPrevPos) > 3.0 then
		self.pBtnMgr:touchMenuItemEndedByPoint(ccp(-1000, -1000))
	end
	
	return self.pCtuiLayer:CTUITouchMoved(tLocation)
end

-- 触控结束
function CTMultiListCell:MultiCellTouchEnded(tLocation, tPrevPos)
	local tLocPos = self.pBtnMgr:convertToNodeSpace(tLocation)
	self.pBtnMgr:touchMenuItemEndedByPoint(tLocPos)
	return self.pCtuiLayer:CTUITouchEnded(tLocation)
end


-- mark: ================================== CTMultiList ============================================
local CTMultiList = class("CTMultiList", function()
	return CTScrollView:new():autorelease()
end)

-- 初始化
function CTMultiList:ctor()
	-- 列表的回调
	self.pMlDelegate = nil
	self.nCurType = MultiListType.Single
	self.nShowType = MultiListShowType.SelItem;

	-- 显示的区域大小
	self.tFrameSize = CCRectMake(0, 0, 0, 0)

	-- 当前所有cell的数组
	self.pArrCells = {}

	-- 当前单选选中的框框
	self.tSinSelInfo = {
		nType = -1,
		nCell = -1,

		nShowType = -1,
		nShowCell = -1,
	}
end
-- nEvLineCnt 每行的个数
function CTMultiList:InitList( tFrame, nType, nShowType)
	if nEvLineCnt ~= nil then 
		MAX_COUNT_PER_LINE =  nEvLineCnt
		
	end
	
	nType = nType or MultiListType.Single
	self.nCurType = nType
	self.nShowType = nShowType or MultiListShowType.SelItem;
	-- print("........... ", self.nShowType)
	local bRet = self:initView(ctScrollViewVAlignLeft, ctScrollViewHAlignTop, ctScrollDirectionH, tFrame)
	self:setDelSpeedRate(SHARE_FUNC.fScorllDelSpeedRate)
	return bRet
end

-- 设置回调
function CTMultiList:SetMlDelegate( pCallBack )
	self.pMlDelegate = pCallBack
end

-- 重新加载显示
function CTMultiList:ReloadMultiList()
	-- 取消原来选中的
	self:CancelSingleSelected()
	-- 将旧的删除掉
	for i,pOneCell in ipairs(self.pArrCells) do
		pOneCell:removeFromParent()
	end
	self.pArrCells = {}
	-- 回调原点
	self:SetToOffsetPosRate(ccp(0, 0))

	-- 创建所有新的 
	local fHeight = 0
	local nTypeCount = self:CallBackGetTypeCount()
	for i=1,nTypeCount do
		local pOneCell = CTMultiListCell.new()
		pOneCell:setCellIdx(i-1)
		pOneCell:InitMultiCellInfo(self, self.nCurType, self.nShowType)
		self:addChild(pOneCell)

		self.pArrCells[i] = pOneCell
		-- 获取高度
		fHeight = fHeight + pOneCell:getContentSize().height
	end

	-- 重新设置大小
	local tFrame = self:CallBackGetShowFrame()
	self:setContentSize( CCSizeMake(tFrame.size.width, fHeight) )
	-- 重新设置每个cell的位置
	local fDelHeight = fHeight
	for i=1,nTypeCount do
		local pOneCell = self.pArrCells[i]
		pOneCell:ignoreAnchorPointForPosition(false)
		pOneCell:setAnchorPoint(ccp(0.5, 1))
		pOneCell:setPosition(ccp(tFrame.size.width*0.5, fDelHeight))
		fDelHeight = fDelHeight - pOneCell:getContentSize().height
	end

	-- -- 红色底部标记
	-- local pLyColor= CCLayerColor:create(ccc4(255, 0, 0, 120), tFrame.size.width, fHeight)
	-- pLyColor:setAnchorPoint(ccp(0, 0))
	-- self:addChild(pLyColor)
end

-- 重新加载显示
function CTMultiList:ReloadChange()
	local nTypeCount = self:CallBackGetTypeCount()
	if nTypeCount ~= #(self.pArrCells) then
		self:ReloadMultiList()
		return
	end
	-- 如果数目没变，则刷新显示即可
	local fHeight = 0
	for i=1,nTypeCount do
		local pOneCell = self.pArrCells[i]
		pOneCell:RefreshChange()

		-- 获取高度
		fHeight = fHeight + pOneCell:getContentSize().height
	end

	local nType = self.tSinSelInfo.nType
	local nItem = self.tSinSelInfo.nCell

	-- 重新设置过
	self.tSinSelInfo.nType = -1
	self.tSinSelInfo.nCell = -1
	self.tSinSelInfo.nShowType = -1
	self.tSinSelInfo.nShowCell = -1

	if self:SingleSelected(nType, nItem, true) == true then
		self.tSinSelInfo.nType = nType
		self.tSinSelInfo.nCell = nItem
		self.tSinSelInfo.nShowType = nType
		self.tSinSelInfo.nShowCell = nItem
	end
	

	-- 重新设置大小
	local tFrame = self:CallBackGetShowFrame()
	self:setContentSize( CCSizeMake(tFrame.size.width, fHeight) )
	-- 重新设置每个cell的位置
	local fDelHeight = fHeight
	for i=1,nTypeCount do
		local pOneCell = self.pArrCells[i]
		pOneCell:ignoreAnchorPointForPosition(false)
		pOneCell:setAnchorPoint(ccp(0.5, 1))
		pOneCell:setPosition(ccp(tFrame.size.width*0.5, fDelHeight))
		fDelHeight = fDelHeight - pOneCell:getContentSize().height
	end
end

-- 单向选中一条信息
function CTMultiList:SingleSelected(nTypeIdx, nCellIdx, bIsCallBack)
	local pOneCell = self.pArrCells[nTypeIdx+1]
	if pOneCell == nil then
		return false
	end
	return pOneCell:ForceSingleSelected(nCellIdx, bIsCallBack)
end

-- 取消单向选择
function CTMultiList:CancelSingleSelected(bIsStillShow)
	bIsStillShow = bIsStillShow or false
	-- 取消原来选中的
	if self.tSinSelInfo.nShowType >= 0 then
		-- 不继续显示，则删除掉选择框
		if bIsStillShow == false then
			local pOneCell = self.pArrCells[self.tSinSelInfo.nShowType+1]
			pOneCell:ShowOrCancelSelCircle(self.tSinSelInfo.nShowCell, false)

			self.tSinSelInfo.nShowType = -1
			self.tSinSelInfo.nShowCell = -1
		end
	end

	-- 取消选择
	self.tSinSelInfo.nType = -1
	self.tSinSelInfo.nCell = -1
end

-- mark: ======= List对外的回调 ======

-- 获取类型的总数
function CTMultiList:CallBackGetTypeCount()
	if self.pMlDelegate then
		return self.pMlDelegate:MultiListGetTypeCount()
	end
	return 0
end

-- 获取每种类型的标题
function CTMultiList:CallBackGetTypeTitle( nTypeIdx )
	local strTitle = ""

	if self.pMlDelegate then
		strTitle = self.pMlDelegate:MultiListGettTypeTitle(nTypeIdx)
	end

	return strTitle
end

-- 获取对应类型索引的总数
function CTMultiList:CallBackGetCellCount(nTypeIdx)
	if self.pMlDelegate then
		return self.pMlDelegate:MultiListGetCellCount(nTypeIdx)
	end

	return 0
end
-- 获取对应类型的每页行数
function CTMultiList:CallBackGetCellLineCountPerPage(nTypeIdx)
	if self.pMlDelegate then
		return self.pMlDelegate:MultiListGetCellLineCountPerPage(nTypeIdx)
	end

	return 0
end
-- 创建对应的信息
function CTMultiList:CallBackCreateCellShowInfo(nTypeIdx, nCellIdx, pCellItemBtn, tUiTags)
	if self.pMlDelegate then
		-- print("MAX_COUNT_PER_LINE ", MAX_COUNT_PER_LINE)
		self.pMlDelegate:MultiListCreateCellShowInfo(nTypeIdx, nCellIdx, pCellItemBtn, tUiTags)
	end
end
-- 信息的具体显示
function CTMultiList:CallBackCellShowInfo(nTypeIdx, nCellIdx, pCellItemBtn, tItemTagInfo)
	if self.pMlDelegate then
		self.pMlDelegate:MultiListCellShowInfo(nTypeIdx, nCellIdx, pCellItemBtn, tItemTagInfo)
	end
end
-- 单体的选中的回调,bIsSelu表明是否是选中，false表示取消选中
function CTMultiList:CallBackCellSingleClickOneItem(nTypeIdx, nCellIdx, pCellItemBtn, bIsSel, bIsCallBack)

	if self.pMlDelegate then
		-- 如果是传近选中状态，且之前已经有选中一个，则取消之前的选中信息
		if bIsSel == true then
			if self.tSinSelInfo.nType ~= -1 and self.tSinSelInfo.nCell ~= -1 then
				self.pMlDelegate:MultiListCellSingleClickOneItem(self.tSinSelInfo.nType, self.tSinSelInfo.nCell, pCellItemBtn, false, bIsCallBack)
			end
		end
		-- 取消原来选中的
		self:CancelSingleSelected()
		if bIsSel == true and bIsCallBack == true then
			-- 记录选中的信息
			self.tSinSelInfo.nType = nTypeIdx
			self.tSinSelInfo.nCell = nCellIdx
		end
		self.tSinSelInfo.nShowType = nTypeIdx
		self.tSinSelInfo.nShowCell = nCellIdx

		self.pMlDelegate:MultiListCellSingleClickOneItem(nTypeIdx, nCellIdx, pCellItemBtn, bIsSel, bIsCallBack)
	end
end
-- 多选的时候选中信息的回调,bIsSelu表明是否是选中，false表示取消选中,tArrCellIdx,tArrCellItemBtn都是数组，一一对应
function CTMultiList:CallBackCellMultiSelItems(nTypeIdx, tArrCellIdx, tArrCellItemBtn, bIsSel)
	-- 如果没有通知的数目，则不处理
	if #tArrCellIdx <= 0 then
		return;
	end
	if self.pMlDelegate then
		self.pMlDelegate:MultiListCellMultiSelItems(nTypeIdx, tArrCellIdx, tArrCellItemBtn, bIsSel)
	end
end

-- 多选的时候选中信息的回调,判断能多选的数组,tArrCellIdx,tArrCellItemBtn都是数组，一一对应
-- 返回值是从 tArrCellIdx 中，能够选中的数组列表
function CTMultiList:CallBackIsCellCanMultiSelItems(nTypeIdx, tArrCellIdx, tArrCellItemBtn)
	-- 没有对应的回调就是全部选中
	if self.pMlDelegate and self.pMlDelegate.MultiListIsCellCanMultiSelItems then
		return self.pMlDelegate:MultiListIsCellCanMultiSelItems(nTypeIdx, tArrCellIdx, tArrCellItemBtn)
	end
	return tArrCellIdx;
end

-- 设置每行的个数
function CTMultiList:SetEvLineCount(nCount)
	MAX_COUNT_PER_LINE = nCount
end



-- 获取每行的个数
function CTMultiList:GetEvLineCount()
	return MAX_COUNT_PER_LINE
end

-- 表格类型
function CTMultiList:MultiListType()
	return MultiListType
end
-- 表格显示类型
function CTMultiList:MultiListShowType()

	return MultiListShowType
end

-- mark: ======= List对内的回调 ======
-- 获取对应类型的每页行数
function CTMultiList:CallBackGetShowFrame()
	return self:getShowFrame()
end
-- 当前的单选是否已经选中
function CTMultiList:CallBackIsSigleSelected(nTypeIdx, nCellIdx)
	if self.tSinSelInfo.nType == nTypeIdx and self.tSinSelInfo.nCell == nCellIdx then
		return true
	end

	return false
end

-- mark: ======= 触控 =========

-- 开始触控
function CTMultiList:MultiListTouchBegan(tLocation, nCount)
	local tFrame = self:CallBackGetShowFrame()
	if tFrame:containsPoint(tLocation) == false then
		return
	end

	local bTouch = false
	-- cell的处理
	for i,pOneCell in ipairs(self.pArrCells) do
		bTouch = pOneCell:MultiCellTouchBegan(tLocation, nCount)
		if bTouch == true then
			break
		end
	end

	if bTouch == false then
		self:CtScrollViewTouchBegan(tLocation)
	end
end

-- 触控移动
function CTMultiList:MultiListTouchMoved(tLocation, tPrevPos)
	self:CtScrollViewTouchMoved(tLocation, tPrevPos)
	-- cell的处理
	for i,pOneCell in ipairs(self.pArrCells) do
		pOneCell:MultiCellTouchMoved(tLocation, tPrevPos)
	end
end

-- 触控结束
function CTMultiList:MultiListTouchEnded(tLocation, tPrevPos)
	self:CtScrollViewTouchEnded(tLocation, tPrevPos)
	-- cell的处理
	for i,pOneCell in ipairs(self.pArrCells) do
		pOneCell:MultiCellTouchEnded(tLocation, tPrevPos)
	end
end

return CTMultiList
