--
-- Author: Link
-- Date: 2014-06-25 15:05:04
--
local CTScrollItems = class("CTScrollItems", function ()
	local pLyOne = CTLayerClip:new()
	pLyOne:init();
	pLyOne:autorelease();
	return pLyOne
end)

function CTScrollItems:ctor()
	self.tShowOneSize 	= CCSizeMake(0, 0);
	self.fScrollSpeed 	= 1000;
	self.tItemList 		= {};
	self.tKeepItemList 	= {};

	self.nStartIdx 		= 0;
	self.nEndIdx 		= 0;
	-- 是否在滚动中
	self.bIsScroll 		= false;

	self.CBack = {}
	setmetatable(self.CBack, {__mode="v"})
end

function CTScrollItems:InitScrollItems( tShowFrame, tCellSize, pDelegate)
	self:SetClipRegion( tShowFrame )
	self:SetIsClipRegion(true)
	self:SetIsMoveFollow(true)

	self.tShowOneSize 	= tCellSize
	-- 设置回调
	self.CBack.delegate = pDelegate;
end

-- 开始显示位置
function CTScrollItems:StartShowByIndex(nStart)
	if nStart < 0 then
		nStart = 0;
	end
	-- 删除所有item
	self:removeAllChildren()
	self.tItemList 		= {};
	self.tKeepItemList 	= {};

	-- 获取总数
	local nCount = self:GetTotalCount()
	if nCount <= 0 or nStart > nCount then
		return;
	end

	local tShowFrame = self:GetClipRect();
	local tMidPos = ccp( tShowFrame:getMidX(), tShowFrame:getMidY() )
	-- 向下增加
	for i=nStart,nCount-1 do
		local pOneNode = self:GetOneScrollItem(i, true)
		pOneNode:setPosition( ccpAdd(tMidPos, ccp(0, -self.tShowOneSize.height*(i-nStart) )) )
		-- 创建结束
		local fLine = pOneNode:getPositionY() - self.tShowOneSize.height*0.5
		if fLine < tShowFrame:getMinY() then
			break
		end
	end
	-- 向上增加
	for i=nStart-1,0,-1 do
		local pOneNode = self:GetOneScrollItem(i, false)
		pOneNode:setPosition( ccpAdd(tMidPos, ccp(0, self.tShowOneSize.height*(nStart-i) )) )
		-- 创建结束
		local fLine = pOneNode:getPositionY() + self.tShowOneSize.height*0.5
		if fLine > tShowFrame:getMaxY() then
			break
		end
	end
	-- 记录开始的索引
	self.nStartIdx 		= nStart;
end

-- 设置滑动的速度
function CTScrollItems:SetScrollSpeed(fSpeed)
	self.fScrollSpeed = fSpeed;
end

-- 开始混动
function CTScrollItems:ScrollToIdx(nEndIdx)
	-- 获取总数
	local nCount = self:GetTotalCount()
	if nCount <= 0 or nEndIdx > nCount or nEndIdx < 0 then
		return;
	end

	self.nEndIdx 		= nEndIdx;

	if self.nStartIdx == nEndIdx then
		self:ShowNodeEnd(nEndIdx)
		return;
	end

	-- 开始滚动
	self.bIsScroll 		= true;
	-- 更新滑动
	self:scheduleUpdate(function(dt)
		if self:UpdateSroll(dt) == true then
			self:unscheduleUpdate();
			-- 滚动结束
			self:ShowNodeEnd(nEndIdx)
		end
	end)
end

-- 
function AddOneItemToHadShowedList(bIsAdd)
	-- body
end

-- 更新显示
function CTScrollItems:UpdateSroll( dt )

	local fDelSpeed = self.fScrollSpeed * dt;
	if self.nEndIdx < self.nStartIdx then
		fDelSpeed = -fDelSpeed;

	end

	local bIsEnd = false;
	-- 中间位置
	local fMidPosY = self:GetClipRect():getMidY();

	for i,v in ipairs(self.tItemList) do
		if v:getTag() == self.nEndIdx then
			if fDelSpeed > 0 then
				if v:getPositionY()+fDelSpeed >= fMidPosY then
					fDelSpeed = fMidPosY - v:getPositionY()
					bIsEnd = true;
				end
			else
				if v:getPositionY()+fDelSpeed <= fMidPosY then
					fDelSpeed = fMidPosY - v:getPositionY()
					bIsEnd = true;
				end
			end
			break;
		end
	end


	-- 上下两条线
	local fUpLine = self:GetClipRect():getMaxY()+self.tShowOneSize.height*0.6;
	local fDownLine = self:GetClipRect():getMinY()-self.tShowOneSize.height*0.6;

	-- 删除超出显示的
	for i=#self.tItemList,1,-1 do
		local v = self.tItemList[i]
		v:setPositionY( v:getPositionY()+fDelSpeed )
		if fDelSpeed > 0 then
			if v:getPositionY() > fUpLine and #self.tItemList > 1 then
				self:SwapOneScrollItemToHide( i )
			end
		else
			if v:getPositionY() < fDownLine and #self.tItemList > 1 then
				self:SwapOneScrollItemToHide( i )
			end
		end
	end

	repeat
		local pTopAdd, pBottomAdd

		-- 最顶部分
		local pTmpOne = self.tItemList[1]
		if pTmpOne:getPositionY() < fUpLine then
			pTopAdd = self:GetOneScrollItem(pTmpOne:getTag()-1, false)
			if pTopAdd then
				pTopAdd:setPosition( ccpAdd(pTmpOne:getPositionInCCPoint(), ccp(0, self.tShowOneSize.height)) )
			end
		end

		-- 最底部分
		local pTmpOne1 = self.tItemList[#self.tItemList]
		if pTmpOne1:getPositionY() > fDownLine then
			pBottomAdd = self:GetOneScrollItem(pTmpOne1:getTag()+1, true)
			if pBottomAdd then
				pBottomAdd:setPosition( ccpAdd(pTmpOne1:getPositionInCCPoint(), ccp(0, -self.tShowOneSize.height)) )
			end
		end

		if pTopAdd == nil and pBottomAdd == nil then
			break;
		end
	until false
	-- 提示更新
	self:ShowMoveScrollUpdate()

	return bIsEnd;
end

-- 从一个节点上面获取一个Node
function CTScrollItems:GetChildNodeByItemAndTag(pItemNode, nTag)
	if pItemNode == nil then
		return nil;
	end
	local pChildren = pItemNode:getChildren();
	if pChildren ~= nil then
		for i=1,pChildren:count() do
			local pOneChild = pChildren:objectAtIndex(i-1)
			tolua.cast(pOneChild, "CCNode")
			if pOneChild:getTag() == nTag then
				return pOneChild;
			end
			local pGet = self:GetChildNodeByItemAndTag(pOneChild, nTag)
			if pGet ~= nil then
				return pGet;
			end
		end
	end

	return nil;
end


-- 获取一个item
function CTScrollItems:GetOneScrollItem(nIdx, bIsAsc)
	local pGetNode
	local nCount = self:GetTotalCount()
	if nIdx < 0 or nIdx >= nCount then
		return nil
	end

	repeat
		if #self.tKeepItemList > 0 then
			pGetNode = self.tKeepItemList[1]
			table.remove(self.tKeepItemList, 1)
			break;
		end
		-- 创建没有找到旧的，在创建新的
		pGetNode = self:CreateOneNode(nIdx)

		if bIsAsc == true then
			self.tItemList[ #self.tItemList +1 ] = pGetNode
		else
			table.insert(self.tItemList, 1, pGetNode)
		end
		
	until true
	pGetNode:setVisible(true)
	self:ShowOneNodeInfo(pGetNode, nIdx)

	return pGetNode
end
-- 删除一个Item到Keep数组中
function CTScrollItems:SwapOneScrollItemToHide( nIdx)
	local tItem = self.tItemList[nIdx]
	if tItem == nil then
		return;
	end

	tItem:setVisible(false)

	self.tKeepItemList[#self.tKeepItemList+1] = v;
	table.remove(self.tItemList, nIdx)

	-- for i,v in ipairs(self.tItemList) do
	-- 	if v:getTag() == nIdx then
	-- 		v:setVisible(false)

	-- 		self.tKeepItemList[#self.tKeepItemList+1] = v;
	-- 		table.remove(self.tItemList, i)

	-- 		break;
	-- 	end
	-- end
end

-- 获取当前拥有的总数
function CTScrollItems:GetTotalCount()
	local nCnt = 0;
	if self.CBack.delegate then
		nCnt = self.CBack.delegate:ScrollItemsGetCount(self)
	end
	return nCnt;
end

-- 创建Items
function CTScrollItems:CreateOneNode(nIdx)
	local pOneNode = CCNode:create();

	pOneNode:setAnchorPoint(ccp(0.5,0.5))
	pOneNode:ignoreAnchorPointForPosition(false)
	pOneNode:setContentSize( self.tShowOneSize )
	pOneNode:setTag(nIdx)

	if self.CBack.delegate then
		self.CBack.delegate:ScrollItemsCreateOneItem(self, pOneNode, nIdx)
	end
	pOneNode:setVisible(false)
	self:addChild(pOneNode)
	return pOneNode;
end

-- 显示Items
function CTScrollItems:ShowOneNodeInfo(pOneNode, nIdx)
	pOneNode:setTag(nIdx)
	if self.CBack.delegate then
		self.CBack.delegate:ScrollItemsShowOneItem(self, pOneNode, nIdx)
	end
end

-- 通知滑动结束
function CTScrollItems:ShowNodeEnd(nIdx)
	self.bIsScroll 		= false;
	if self.CBack.delegate then
		self.CBack.delegate:ScrollItemsShowSrollEnd(self, nIdx)
	end 
end

-- 通知滑动过程
function CTScrollItems:ShowMoveScrollUpdate()
	if self.CBack.delegate and self.CBack.delegate.ScrollItemsMoveUpdate then
		self.CBack.delegate:ScrollItemsMoveUpdate(self, self.tItemList)
	end 
end

return CTScrollItems;