--
-- Author: Link
-- Date: 2014-02-11 11:17:16
--
-- 声音模块管理
SOUNDMGR = {}

-- 是否播放音乐
SOUNDMGR.bIsPlayMusic = true
-- 是否播放音效
SOUNDMGR.bIsPlayEffect = true

-- 背景乐声音小一点
SimpleAudioEngine:sharedEngine():setBackgroundMusicVolume(0.7)

-- mark: ======================= 功能设置 ==========================
-- 设置和获取音乐情况
function SOUNDMGR:SetEnablePlayMusic(bIsEnabled)
	-- 如果没有变化，则不处理
	if SOUNDMGR.bIsPlayMusic == bIsEnabled then
		return
	end

	SOUNDMGR.bIsPlayMusic = bIsEnabled

	-- 如果是开启播放
	if bIsEnabled == true then
		local nMainStatus = GAME_STATUS.GetStatus(1)

		-- 如果在登陆界面
		if nMainStatus <= STATUS_MAIN.LOGIN then
			self:PlayLoginBackgroundMusic()

		elseif nMainStatus == STATUS_MAIN.LOBBY then
			self:PlayLobbyBackgroundMusic()

		elseif nMainStatus == STATUS_MAIN.RACE then
			self:PlayRaceBackgroundMusic()
		end
	-- 如果是停止播放音乐
	else
		SimpleAudioEngine:sharedEngine():stopBackgroundMusic(true)
	end
end
function SOUNDMGR:GetEnablePlayMusic()
	return SOUNDMGR.bIsPlayMusic
end
-- 设置和获取音效信息
function SOUNDMGR:SetEnablePlayEffect(bIsEnabled)
	-- 如果没有变化，则不处理
	if SOUNDMGR.bIsPlayEffect == bIsEnabled then
		return
	end
	SOUNDMGR.bIsPlayEffect = bIsEnabled

	-- 如果是停止播放音效，则立刻停止所有音效播放
	if bIsEnabled == false then
		SimpleAudioEngine:sharedEngine():stopAllEffects()
	end
end
function SOUNDMGR:GetEnablePlayEffect()
	return SOUNDMGR.bIsPlayEffect
end

-- 播放音乐的接口
function SOUNDMGR:PlayBackgroundMusic(strMusic, bLoop)
	if bLoop == nil then
		bLoop = true;
	end
	-- 是否能播放背景音乐
	if SOUNDMGR:GetEnablePlayMusic() ~= true then
		return
	end
	self:StopBackgroundMusic()
	SimpleAudioEngine:sharedEngine():playBackgroundMusic(Public_GetResPathByKey(strMusic), bLoop)
end

-- 播放音效的接口
function SOUNDMGR:PlayGameEffect(strEffect, bLoop)
	if bLoop == nil then
		bLoop = false;
	end
	-- 是否能播放音效
	if SOUNDMGR:GetEnablePlayEffect() ~= true then
		return
	end
	SimpleAudioEngine:sharedEngine():playEffect(Public_GetResPathByKey(strEffect), bLoop)
end

-- mark: ======================= 播放音乐 ==========================
-- 停止播放背景音乐
function SOUNDMGR:StopBackgroundMusic()
	SimpleAudioEngine:sharedEngine():stopBackgroundMusic(false)
end

-- flush的背景音乐
function SOUNDMGR:PlayFlushMusicOnce()
	self:PlayBackgroundMusic("Sound/Music_Flush.mp3", false)
end

-- 登陆界面的背景音乐
function SOUNDMGR:PlayLoginBackgroundMusic()
	self:PlayBackgroundMusic("Sound/Music_Start.mp3", true)
end

-- 大厅界面的背景音乐
function SOUNDMGR:PlayLobbyBackgroundMusic()
	self:PlayBackgroundMusic("Sound/Music_Menu.mp3", true)
end

-- 赛车界面的背景音乐
function SOUNDMGR:PlayRaceBackgroundMusic(nRoadType)
	-- 是否能播放背景音乐
	if SOUNDMGR:GetEnablePlayMusic() ~= true then
		return
	end
	self:StopBackgroundMusic()
	-- 冰路
	if nRoadType == RaceRoadType.BingLu then
		self:PlayBackgroundMusic("Sound/Music_Battle_Snow.mp3", true)
	-- 公路
	elseif nRoadType == RaceRoadType.GongLu then
		self:PlayBackgroundMusic("Sound/Music_Battle_Road.mp3", true)
	-- 越野路
	elseif nRoadType == RaceRoadType.YueYeLu then
		self:PlayBackgroundMusic("Sound/Music_Battle_Wild.mp3", true)
	-- 沙地
	elseif nRoadType == RaceRoadType.ShangDi then
		self:PlayBackgroundMusic("Sound/Music_Battle_Dirty.mp3", true)
	-- 火山
	elseif nRoadType == RaceRoadType.HuoShan then
		self:PlayBackgroundMusic("Sound/Music_Battle_Volcano.mp3", true)
	-- 石路
	elseif nRoadType == RaceRoadType.ShiLu then
		self:PlayBackgroundMusic("Sound/Music_Battle_Stone.mp3", true)
	-- 草地
	else
		self:PlayBackgroundMusic("Sound/Music_Battle_Grass.mp3", true)
	end
end

-- mark: ======================= 播放音效 ==========================
-- 播放按钮信息
function SOUNDMGR:PlayEffClickButton()
	self:PlayGameEffect("Sound/Sound_Click.wav", false)
end

-- 用户升级音效
function SOUNDMGR:PlayEffUserLvUp()
	self:PlayGameEffect("Sound/Sound_UserLvlup.mp3", false)
end

-- 汽车升级的音效
function SOUNDMGR:PlayEffCarLvUp()
	self:PlayGameEffect("Sound/Sound_CarUpgrade.mp3", false)
end

-- 装备升级的音效
function SOUNDMGR:PlayEffEquipLvUp()
	self:PlayGameEffect("Sound/Sound_PartUpgrade.mp3", false)
end

-- mark ========== 大厅界面 ==========
-- 转盘的声音
function SOUNDMGR:PlayEffTurnTable()
	self:PlayGameEffect("Sound/Sound_Roulette.mp3", false)
end
-- 显示获取新物品
function SOUNDMGR:PlayEffGetNewItem()
	self:PlayGameEffect("Sound/Sound_GetCar&Part.mp3", false)
end

-- 汽车钻石开和钥匙开的音效
function SOUNDMGR:PlayEffScrollCarOrKey()
	self:PlayGameEffect("Sound/Sound_SlotCard.mp3", false)
end

-- 活动大转盘滚盘
function SOUNDMGR:PlayEffSpinTable()
	self:PlayGameEffect("Sound/Sound_SpinTable.mp3", false)
end

-- 播放建筑物建造和升级的音效
function SOUNDMGR:PlayEffBuildingBuildOrUpgrade()
	self:PlayGameEffect("Sound/Sound_BuildUpgrade.mp3", false)
end

-- 播放建筑物获得金币的音效
function SOUNDMGR:PlayEffBuildingGetGold()
	self:PlayGameEffect("Sound/Sound_GetGold.mp3", false)
end

-- 播放建筑物获得精力和体力的音效
function SOUNDMGR:PlayEffBuildingGetEngeryOrActivity()
	self:PlayGameEffect("Sound/Sound_P&DGet.mp3", false)
end

-- 播放合成效果音乐
function SOUNDMGR:PlayEffMergeCar()
	self:PlayGameEffect("Sound/Sound_Compose.mp3", false)
end

-- mark ========== 赛车界面 ==========

-- 开始比赛阵容对比的音效
function SOUNDMGR:PlayEffRaceStartVs()
	self:PlayGameEffect("Sound/Sound_VS.mp3", false)
end

-- 播放释放技能的音效
function SOUNDMGR:PlayEffRaceUseSkill()
	self:PlayGameEffect("Sound/Sound_PaySkill.mp3", false)
end

-- 播放车子倒计时
function SOUNDMGR:PlayEffRaceCountDown(nTime)
	-- 是否能播放音效
	if SOUNDMGR:GetEnablePlayEffect() ~= true then
		return
	end

	-- 倒数
	-- if nTime == 3 then
	-- 	self:PlayGameEffect("Sound/Sound_Ready1.mp3", false)
	-- elseif nTime == 2 then
	-- 	self:PlayGameEffect("Sound/Sound_Ready2.mp3", false)
	-- elseif nTime == 1 then
	-- 	self:PlayGameEffect("Sound/Sound_Ready3.mp3", false)
	-- elseif nTime == 0 then
	-- end
	self:PlayGameEffect("Sound/Sound_Ready.mp3", false)
end

-- 第一辆车子通过终点
function SOUNDMGR:PlayEffRaceFirstCarAcrossEnd()
	self:PlayGameEffect("Sound/Sound_Cheer.mp3", false)
end

-- 结局是胜利的音效
function SOUNDMGR:PlayEffRaceResultWin()
	self:PlayGameEffect("Sound/Sound_Win.mp3", false)
end

-- 结局是失败的音效
function SOUNDMGR:PlayEffRaceResultLose()
	self:PlayGameEffect("Sound/Sound_Lose.mp3", false)
end

-- 砖石加年华
function SOUNDMGR:PlayEffDialmondYears()
	self:PlayGameEffect("Sound/ZhuanShiJIaNIanHua.mp3", false)
end
