--
-- Author: Link
-- Date: 2014-08-20 16:38:49
--
local BaseBackground = class("BaseBackground", function()
	return display.newLayer()
end)
-- 
BASE_BG_DATA = {}

-- 
local BgItemType = {
	Road 		= 0,
	RoadItem 	= 1,
	RoadTree 	= 2,
	RoadGrass 	= 3,
	FrontBuild	= 4,
	MidBuild	= 5,
	BackBuild	= 6,
	Cloud 		= 7,
	Sun 		= 8,
}

-- 背景的初始化
function BaseBackground:ctor()
	
	-- 部件的情况
	self.tArrRoad = {}
	-- 路面小部件
	self.tArrRoadItems = {}
	-- 路面上的树
	self.tArrRoadTree = {}
	-- 中间的建筑
	self.tArrMidBuild = {}
	-- 后面的建筑
	self.tArrBackBuild = {}
	-- 云朵
	self.tArrClond = {}
	-- 太阳
	self.pSpSun = {}

	-- 所有部件的处理
	self.pArrKeepItems = {}
	-- 物品对应的key
	self.pArrItemToKey = {}
	-- 对应类型的偏差位置距离
	self.fArrDis = {}

	-- 
	self.pSpBatch = CCSpriteBatchNode:create("Images/BG-000.png")
	self:addChild(self.pSpBatch)

	-- 第二层背景
	self.pSpBatch2 = CCSpriteBatchNode:create("Images/BG-000.png")
	self:addChild(self.pSpBatch2, -2)

	self:CreateBackage()
	
	-- self:scheduleUpdate(function (dt)
	-- 	self:ShowBgScrollDeltaLen(9)
	-- end)
	

	-- 最底层
	self.pLyBg =  CCSprite:create("Images/BG-004.png")    
	self.pLyBg:setScaleX(display.width / self.pLyBg:getContentSize().width)
	self.pLyBg:setScaleY(display.height / self.pLyBg:getContentSize().height)
	self.pLyBg:setPosition(ctprccp(0.5, 0.5))
	self:addChild(self.pLyBg, -4)


   	self.pLyLeaft = CCLayer:create()
    self:addChild(self.pLyLeaft,20)
    -- 透明云彩动画 
	self:CreateCloundAct()
	
end
-- 云彩
function BaseBackground:CreateCloundAct()
	local fTime = math.random(60, 70)
	local fDelTiem = math.random(3, 5)
	local x = math.random(0, display.width/2)
	local y = math.random(display.height/ 3, display.height/ 2 + 170)
	local pSpLeaf = self:GetBgSprite("BG-024.png" )
	pSpLeaf:setAnchorPoint(ccp(0, 0))
	pSpLeaf:setPosition( ccp( x,  y))
	self.pSpBatch2:addChild(pSpLeaf, -6)


	local tArryAct = CCArray:create()
	tArryAct:addObject( CCMoveTo:create(fTime, ccp( display.width, y) )) 
	tArryAct:addObject( CCDelayTime:create(fDelTiem) )
	tArryAct:addObject( CCHide:create()) 
	tArryAct:addObject( CCPlace:create( ccp(-300, y) ))
	tArryAct:addObject( CCShow:create()) 
	pSpLeaf:runAction( CCRepeatForever:create(CCSequence:create(tArryAct)) )
end

-- 根据Key名获取
function BaseBackground:GetBgSpriteFrame(strKey)
	return SHARE_FUNC:GetSpriteFrame(strKey, Public_GetResPathByKey("Images/BG-000.plist"))
end
-- 根据Key名创建Sprite
function BaseBackground:GetBgSprite(strKey)
	local pFrame = self:GetBgSpriteFrame(strKey)
	return CCSprite:createWithSpriteFrame(pFrame)
end

-- 获取一个已经存在的组建
function BaseBackground:GetKeepItemSprite(strKey, nType)
	-- 获取文件Keep列表
	local tItemList = self.pArrKeepItems[strKey];
	if tItemList == nil then
		tItemList = {}
		self.pArrKeepItems[strKey] = tItemList;
	end
	local pItem = nil
	if #tItemList > 0 then
		pItem = tItemList[1];
		table.remove(tItemList, 1)
	else
		-- 后背
		if nType == BgItemType.BackBuild then
			-- pItem = CCSprite:create(strKey)
			pItem = self:GetBgSprite(strKey)
			pItem:setScale( ctipwidth(1.4) )
			self:addChild(pItem, -1)
		elseif nType == BgItemType.Cloud or nType == BgItemType.Sun then
			pItem = self:GetBgSprite(strKey)
			pItem:setScale( ctipwidth(1.27) )
			self.pSpBatch2:addChild(pItem)
		else
			pItem = self:GetBgSprite(strKey)
			pItem:setScale( ctipwidth(1.27) )
			self.pSpBatch:addChild(pItem)
		end
		
	end
	-- 显示出来
	pItem:setVisible(true)
	-- 物品
	self.pArrItemToKey[ pItem ] = strKey;

	return pItem;
end

-- 将一个物品放到保存列表中
function BaseBackground:MoveToKeepItemSprite(pItem)
	local strKey = self.pArrItemToKey[ pItem ]
	-- 获取文件Keep列表
	local tItemList = self.pArrKeepItems[strKey];
	if tItemList == nil then
		tItemList = {}
		self.pArrKeepItems[strKey] = tItemList;
	end
	tItemList[#tItemList+1] = pItem
	pItem:setVisible(false)
	pItem:stopAllActions()
	return strKey;
end

-- 创建物品小部件
function BaseBackground:CreateRoadItems(nIdx)
	-- 创建数据数组
	local tItemList = BASE_BG_DATA[BgItemType.RoadItem];
	if tItemList == nil then
		tItemList = {}
		BASE_BG_DATA[BgItemType.RoadItem] = tItemList;
	end

	-- 物品类型
	local strTypeList = {
		"BG-025.png",
		"BG-026.png",
		"BG-027.png",
	}

	-- 获取物品的类型
	local nOneType = tItemList[nIdx]
	if nOneType == nil then
		nOneType = GetRandom(1, #strTypeList);
		tItemList[nIdx] = nOneType
	end
	-- 物品信息
	local pItem = self:GetKeepItemSprite( strTypeList[nOneType], BgItemType.RoadItem)
	pItem:setAnchorPoint( ccp(0.5, 0) )
	pItem:setTag(nIdx)

	if pItem:getZOrder() ~= 4 then
		pItem:getParent():reorderChild(pItem, 4)
	end

	return pItem
end

-- 创建树木
function BaseBackground:CreateRoadTrees(nIdx)
	-- 创建数据数组
	local tItemList = BASE_BG_DATA[BgItemType.RoadTree];
	if tItemList == nil then
		tItemList = {}
		BASE_BG_DATA[BgItemType.RoadTree] = tItemList;
	end

	-- 物品类型
	local strTypeList = {
		-- "BG-019.png",
		"BG-018.png",
		"BG-019.png",
	}

	-- 获取物品的类型
	local nOneType = tItemList[nIdx]
	if nOneType == nil then
		nOneType = GetRandom(1, #strTypeList);
		tItemList[nIdx] = nOneType
	end
	-- 物品信息
	local pItem = self:GetKeepItemSprite( strTypeList[nOneType], BgItemType.RoadTree )
	pItem:setAnchorPoint( ccp(0.5, 0) )
	pItem:setTag(nIdx)

	if pItem:getZOrder() ~= 3 then
		pItem:getParent():reorderChild(pItem, 3)
	end
	return pItem
end


-- 创建前面的建筑
function BaseBackground:CreatFrontBuilding(nIdx)
	
-- 创建数据数组
	local tItemList = BASE_BG_DATA[BgItemType.FrontBuild];
	if tItemList == nil then
		tItemList = {}
		BASE_BG_DATA[BgItemType.FrontBuild] = tItemList;
	end

	-- 物品类型
	local strTypeList = {
		"BG-002.png",
	}

	-- 获取物品的类型
	local nOneType = tItemList[nIdx]
	if nOneType == nil then
		nOneType = GetRandom(1, #strTypeList);
		tItemList[nIdx] = nOneType
	end
	-- 物品信息
	local pItem = self:GetKeepItemSprite( strTypeList[nOneType], BgItemType.FrontBuild )
	pItem:setAnchorPoint( ccp(0.5, 0) )
	pItem:setTag(nIdx)
	if pItem:getZOrder() ~= -1 then
		pItem:getParent():reorderChild(pItem, -1)
	end
	return pItem
end

-- 创建中间的建筑
function BaseBackground:CreatMidBuilding(nIdx)
	-- 创建数据数组
	local tItemList = BASE_BG_DATA[BgItemType.MidBuild];
	if tItemList == nil then
		tItemList = {}
		BASE_BG_DATA[BgItemType.MidBuild] = tItemList;
	end

	-- 物品类型
	local strTypeList = {
		"BG-002.png",
	}

	-- 获取物品的类型
	local nOneType = tItemList[nIdx]
	if nOneType == nil then
		nOneType = GetRandom(1, #strTypeList);
		tItemList[nIdx] = nOneType
	end
	-- 物品信息
	local pItem = self:GetKeepItemSprite( strTypeList[nOneType], BgItemType.MidBuild )
	pItem:setAnchorPoint( ccp(0.5, 0) )
	pItem:setTag(nIdx)
	if pItem:getZOrder() ~= -2 then
		pItem:getParent():reorderChild(pItem, -2)
	end
	return pItem
end

-- 创建云朵
function BaseBackground:CreatClound(nIdx)
	-- 创建数据数组
	local tItemList = BASE_BG_DATA[BgItemType.Cloud];
	if tItemList == nil then
		tItemList = {}
		BASE_BG_DATA[BgItemType.Cloud] = tItemList;
	end

	-- 物品类型
	local strTypeList = {
		"BG-021.png",
		"BG-022.png",
		"BG-023.png",
	}

	-- 获取物品的类型
	local nOneType = tItemList[nIdx]
	if nOneType == nil then
		nOneType = GetRandom(1, #strTypeList);
		tItemList[nIdx] = nOneType
	end
	-- 物品信息
	local pItem = self:GetKeepItemSprite( strTypeList[nOneType], BgItemType.Cloud )
	pItem:setAnchorPoint( ccp(0.5, 0) )
	pItem:setTag(nIdx)
	if pItem:getZOrder() ~= -3 then
		pItem:getParent():reorderChild(pItem, -3)
	end
	pItem:runAction( CCRepeatForever:create( CCMoveBy:create(30, ccp(200,0)) ) )
	return pItem
end

-- 根据类型和索引创建部件
function BaseBackground:CreateItemsByTypeAndIdx(nType, nIdx)
	-- 路面上的小部件
	if nType == BgItemType.RoadItem then
		return self:CreateRoadItems(nIdx)
	-- 树木
	elseif nType == BgItemType.RoadTree then
		return self:CreateRoadTrees(nIdx)
	-- 草丛
	elseif nType == BgItemType.RoadGrass then
		-- return self:CreateRoadGrass(nIdx)
	-- 前排建筑物
	elseif nType == BgItemType.FrontBuild then
		return self:CreatFrontBuilding(nIdx)
	-- 中排建筑物
	elseif nType == BgItemType.MidBuild then
		return self:CreatMidBuilding(nIdx)
	-- 云朵
	elseif nType == BgItemType.Cloud then
		return self:CreatClound(nIdx)
	end
	return nil
end

-- 改变物品到
function BaseBackground:SwitchToNextItem( tArrItems, bIsMoveLeft, nType )
	local bIsNotChange = true;
	-- 向左移动
	if bIsMoveLeft == true then
		-- 判断最后一个的变化
		local pOneBot = tArrItems[ #tArrItems ]
		local fTemp = pOneBot:getScaleX()*(1-pOneBot:getAnchorPoint().x)
		local fEndPosX = pOneBot:getPositionX()+pOneBot:getContentSize().width*fTemp;
		if fEndPosX <= (ctprwidth(1)+10) then
			-- 将第一个删除掉
			local pTmp = tArrItems[1];
			table.remove(tArrItems, 1)
			local tChangePos = ccpAdd(pTmp:getPositionInCCPoint(), ccp(self.fArrDis[nType], 0) );
			-- 转移到保留数组中
			local strKey = self:MoveToKeepItemSprite(pTmp)
			-- 再从保留数组取一个
			local tNewItm
			if nType == BgItemType.Road or nType == BgItemType.BackBuild then
				tNewItm = self:GetKeepItemSprite(strKey, nType)
			else
				tNewItm = self:CreateItemsByTypeAndIdx(nType, pOneBot:getTag()+1 )
			end
			tNewItm:setPosition( tChangePos )
			-- 放到显示数组中
			tArrItems[ #tArrItems+1 ] = tNewItm;

			bIsNotChange = false;
		end
	else
		-- 判断最后一个的变化
		local pOneBot = tArrItems[ 1 ]
		local fTemp = pOneBot:getScaleX()*pOneBot:getAnchorPoint().x
		local fEndPosX = pOneBot:getPositionX()-pOneBot:getContentSize().width*fTemp;
		if fEndPosX >= (-10) then
			-- 将第一个删除掉
			local pTmp = tArrItems[#tArrItems];
			table.remove(tArrItems, #tArrItems)
			local tChangePos = ccpAdd(pTmp:getPositionInCCPoint(), ccp(-self.fArrDis[nType], 0) );
			-- 转移到保留数组中
			local strKey = self:MoveToKeepItemSprite(pTmp)
			-- 再从保留数组取一个
			local tNewItm
			if nType == BgItemType.Road or nType == BgItemType.BackBuild then
				tNewItm = self:GetKeepItemSprite(strKey, nType)
			else
				tNewItm = self:CreateItemsByTypeAndIdx(nType, pOneBot:getTag()-1 )
			end
			tNewItm:setPosition( tChangePos )
			-- 放到显示数组中
			table.insert(tArrItems, 1, tNewItm)

			bIsNotChange = false;
		end
	end

	return bIsNotChange
end

-- 创建初始化的类型
function BaseBackground:CreateBackage()
	local nHeight = -100
	-- 长度路线
	local fLenTmp = -20;
	-- 最底层路面
	for i=1,2 do
		local pSpBg1 = self:GetKeepItemSprite("BG-001.png", BgItemType.Road)
		pSpBg1:setAnchorPoint(ccp(0.0, 0))
		pSpBg1:setPosition( fLenTmp, 0)
		-- pSpBg1:setScale(1.4)
		self.tArrRoad[i] = pSpBg1;
		-- 长度
		local fDisTmp =pSpBg1:getContentSize().width*pSpBg1:getScaleX()
		self.fArrDis[BgItemType.Road] = fDisTmp
		fLenTmp = fLenTmp + fDisTmp;
	end

	-- 路面小部件
	self.fArrDis[BgItemType.RoadItem] = ctprwidth(2)
	local fTmpHeight = ctipwidth(330 +nHeight)
	local tArrPos = { ccp(ctipwidth(100), fTmpHeight), ccp(ctipwidth(668), fTmpHeight), ccp(ctipwidth(868), fTmpHeight), ccp(ctipwidth(1436), fTmpHeight) }
	for i=1,3 do
		local pSpItem = self:CreateRoadItems(i)
		pSpItem:setPosition( tArrPos[i] )
		self.tArrRoadItems[i] = pSpItem;
	end

	-- 路面树木
	self.fArrDis[BgItemType.RoadTree] = ctipwidth(1450)
	fTmpHeight = ctipwidth(385 + nHeight)
	tArrPos = { ccp(ctipwidth(100), fTmpHeight), ccp(ctipwidth(550), fTmpHeight), ccp(ctipwidth(1000), fTmpHeight) }
	for i=1,3 do
		local pSpItem = self:CreateRoadTrees(i)
		pSpItem:setPosition( tArrPos[i] )
		self.tArrRoadTree[i] = pSpItem;
	end

	-- 中排建筑
	fTmpHeight = ctipwidth(400 + nHeight)
	fLenTmp = 350
	for i=1,3 do
		local pSpBg1 = self:GetKeepItemSprite("BG-002.png", BgItemType.MidBuild)
		pSpBg1:setAnchorPoint(ccp(0.5, 0))
		pSpBg1:setPosition( fLenTmp, fTmpHeight)
		self.tArrMidBuild[i] = pSpBg1;
		-- 长度
		local fDisTmp =pSpBg1:getContentSize().width*pSpBg1:getScaleX()

		self.fArrDis[BgItemType.MidBuild] = fDisTmp
		fLenTmp = fLenTmp + fDisTmp;

	end


	-- 后排建筑
	fTmpHeight = ctipwidth(430 + nHeight)
	fLenTmp = -20;
	for i=1,2 do
		local pSpBg1 = self:GetKeepItemSprite("BG-003.png", BgItemType.BackBuild)
		pSpBg1:setAnchorPoint(ccp(0.0, 0))
		pSpBg1:setPosition( fLenTmp, fTmpHeight)
		self.tArrBackBuild[i] = pSpBg1;
		if i == 2 then
			-- pSpBg1:setFlipX(true)
		end
		-- 长度
		local fDisTmp =pSpBg1:getContentSize().width*pSpBg1:getScaleX()
		self.fArrDis[BgItemType.BackBuild] = fDisTmp
		fLenTmp = fLenTmp + fDisTmp;
	end

	-- 云
	self.fArrDis[BgItemType.Cloud] = ctipwidth(1650)
	fTmpHeight = ctipwidth(800 + nHeight)
	local fTmpHeight2 = ctipwidth(950)
	tArrPos = { ccp(ctipwidth(20), GetFloatRandom(fTmpHeight, fTmpHeight2)),
				ccp(ctipwidth(570), GetFloatRandom(fTmpHeight, fTmpHeight2)),
				ccp(ctipwidth(1120), GetFloatRandom(fTmpHeight, fTmpHeight2)) }
	for i=1,3 do
		local pSpItem = self:CreatClound(i)
		pSpItem:setPosition( tArrPos[i] )
		self.tArrClond[i] = pSpItem;
	end
	-- 适配太阳的高度
	local nPosy = 1080
	local nShowType = SHARE_FUNC.nFrameType;
	-- ipad 
	if nShowType == CTDeviceTypeIpad or nShowType == CTDeviceTypeIpadHD then
		nPosy = 750
	-- iphone
	elseif nShowType == CTDeviceTypeIphoneHD or nShowType == CTDeviceTypeItouchHD then
		nPosy = 870
	end

	self.pSpSun = self:GetKeepItemSprite("BG-020.png", BgItemType.Sun)
	self.pSpSun:setPosition( ccp( ctipwidth(600), ctipwidth(nPosy) ) )
	self.pSpSun:getParent():reorderChild(self.pSpSun, -5)
	self.pSpSun:runAction( CCRepeatForever:create( CCRotateBy:create(20, 90) ) )

end

-- 显示滑动情况
function BaseBackground:ShowBgScrollDeltaLen( fDtLen )

	local fNowDt = fDtLen;

	-- 原始位置变化
	for i,pOneRoad in ipairs(self.tArrRoad) do
		pOneRoad:setPositionX( pOneRoad:getPositionX() + fNowDt )
	end

	for i,pOneRoad in ipairs(self.tArrRoadItems) do
		pOneRoad:setPositionX( pOneRoad:getPositionX() + fNowDt )
	end

	for i,pOneRoad in ipairs(self.tArrRoadTree) do
		pOneRoad:setPositionX( pOneRoad:getPositionX() + fNowDt )
	end

	-- for i,pOneRoad in ipairs(self.tArrRoadGrass) do
	-- 	pOneRoad:setPositionX( pOneRoad:getPositionX() + fNowDt )
	-- end

	-- 偏差的位置变化
	fNowDt = fNowDt/3;

	-- for i,pOneRoad in ipairs(self.tArrFrontBuild) do
	-- 	pOneRoad:setPositionX( pOneRoad:getPositionX() + fNowDt )
	-- end

	-- 中排建筑
	fNowDt = fNowDt/1.8;
	for i,pOneRoad in ipairs(self.tArrMidBuild) do
		pOneRoad:setPositionX( pOneRoad:getPositionX() + fNowDt )
	end
	-- 后排建筑
	fNowDt = fNowDt/2;
	for i,pOneRoad in ipairs(self.tArrBackBuild) do
		pOneRoad:setPositionX( pOneRoad:getPositionX() + fNowDt )
	end
	-- 云朵
	fNowDt = fNowDt/3;
	for i,pOneRoad in ipairs(self.tArrClond) do
		-- pOneRoad:setPositionX( pOneRoad:getPositionX() + fNowDt )
	end
	-- 太阳
	fNowDt = fNowDt/3;
	-- self.pSpSun:setPositionX( self.pSpSun:getPositionX() + fNowDt )
	

	-- 移动位置
	if fDtLen < 0 then
		local bIsNotChange = true;
		repeat
			bIsNotChange = true;

			-- 底部变化
			if self:SwitchToNextItem(self.tArrRoad, true, BgItemType.Road) == false then
				bIsNotChange = false;
			end
			-- 道路小物品
			if self:SwitchToNextItem(self.tArrRoadItems, true, BgItemType.RoadItem) == false then
				bIsNotChange = false;
			end

			-- 道路树
			if self:SwitchToNextItem(self.tArrRoadTree, true, BgItemType.RoadTree) == false then
				bIsNotChange = false;
			end
			-- -- 道路草丛
			-- if self:SwitchToNextItem(self.tArrRoadGrass, true, BgItemType.RoadGrass) == false then
			-- 	bIsNotChange = false;
			-- end

			-- -- 前排建筑物
			-- if self:SwitchToNextItem(self.tArrFrontBuild, true, BgItemType.FrontBuild) == false then
			-- 	bIsNotChange = false;
			-- end

			-- 中排建筑物
			if self:SwitchToNextItem(self.tArrMidBuild, true, BgItemType.MidBuild) == false then
				bIsNotChange = false;
			end

			-- 后排建筑物
			if self:SwitchToNextItem(self.tArrBackBuild, true, BgItemType.BackBuild) == false then
				bIsNotChange = false;
			end

			-- 云朵
			if self:SwitchToNextItem(self.tArrClond, true, BgItemType.Cloud) == false then
				bIsNotChange = false;
			end


		until bIsNotChange
		
	else
		local bIsNotChange = true;
		repeat
			bIsNotChange = true;

			-- 底部变化
			if self:SwitchToNextItem(self.tArrRoad, false, BgItemType.Road) == false then
				bIsNotChange = false;
			end
			-- 道路小物品
			if self:SwitchToNextItem(self.tArrRoadItems, false, BgItemType.RoadItem) == false then
				bIsNotChange = false;
			end

			-- 道路树
			if self:SwitchToNextItem(self.tArrRoadTree, false, BgItemType.RoadTree) == false then
				bIsNotChange = false;
			end
			-- -- 道路草丛
			-- if self:SwitchToNextItem(self.tArrRoadGrass, false, BgItemType.RoadGrass) == false then
			-- 	bIsNotChange = false;
			-- end

			-- 前排建筑物
			-- if self:SwitchToNextItem(self.tArrFrontBuild, false, BgItemType.FrontBuild) == false then
			-- 	bIsNotChange = false;
			-- end

			-- 中排建筑物
			if self:SwitchToNextItem(self.tArrMidBuild, false, BgItemType.MidBuild) == false then
				bIsNotChange = false;
			end

			-- 后排建筑物
			if self:SwitchToNextItem(self.tArrBackBuild, false, BgItemType.BackBuild) == false then
				bIsNotChange = false;
			end

			-- 云朵
			if self:SwitchToNextItem(self.tArrClond, false, BgItemType.Cloud) == false then
				bIsNotChange = false;
			end

		until bIsNotChange
	end
end

return BaseBackground