--
-- Author: Link
-- Date: 2014-06-17 14:49:24
--
local BuildingBase = class("BuildingBase", function()
	return display.newLayer();
end)

-- 点击的状态
local BuildingTouchType = {
	None 			= 0,	-- 没有触控
	NormalOnUp 		= 1,	-- 普通的点到上部
	NormalOnDown 	= 2,	-- 普通的点到下部
	NormalOut 		= 3,	-- 点到了外面
	NormalMove 		= 4,	-- 普通的移动
	BtnTouchOne		= 5,	-- 已经点击了按钮
}

-- 点击建筑的方位
local BuildingTouchPart = {
	OutSide 		= 0,	-- 点击到外面
	GetItemPart		= 1,	-- 获取物品的信息
	BuildingPart 	= 2,	-- 建筑物的位置
}

-- 按钮滑动的状态
local BuildBtnStatus = {
	None 			= 0,	-- 没有显示按钮状态
	Showing 		= 1,	-- 显示中
	Showed 			= 2,	-- 显示完成
	Removing 		= 4,	-- 删除按钮中
}

-- 初始化
function BuildingBase:ctor()
	-- 创建按钮列表
	self.pBtnMgr = CTMenuManager:new()
	self.pBtnMgr:init()
	self:addChild(self.pBtnMgr, 9999);
	self.pBtnMgr:release();

	self:setScale(1.2)

	-- 当前的建筑所在的建筑槽位,以0开始
	self.nBuildIdx = 0
	self.nBuildShowIdx = 0;

	-- 是否滑动过
	self.nTouchType = BuildingTouchType.None;

	-- 按钮的状态
	self.nBtnListStatus = BuildBtnStatus.None;

	-- 底部区域
	self.tBottomRect = CCRectMake(-200, -200, 400, 380)
	self.tTopRect = CCRectMake(self.tBottomRect:getMinX(), self.tBottomRect:getMaxY(), self.tBottomRect.size.width, 200)

	self:ignoreAnchorPointForPosition(false)
	self:setAnchorPoint( ccp(0, 0) )

end



-- 根据数据初始化
function BuildingBase:InitBuilding( tDataList )
end

-- 循环
function BuildingBase:BuildingLoop( dt , fCurTime)
end

-- 设置和获取建筑槽位
function BuildingBase:SetCurBuildIdx( nIdx )
	self.nBuildIdx = nIdx;
end
function BuildingBase:GetCurBuildIdx()
	return self.nBuildIdx;
end
-- 设置和获取建筑显示位置
function BuildingBase:SetCurBuildShowIdx( nIdx )
	self.nBuildShowIdx = nIdx
end
function BuildingBase:GetCurBuildShowIdx()
	return self.nBuildShowIdx
end

-- 删除建筑按钮结束
function BuildingBase:RemoveBuildingMenuItemsEnd()

end

-- 接收到升级，用于继承重写
function BuildingBase:ReceiveLevelUp()
	
end

-- 接收到建造完成，用于继承重写
function BuildingBase:ReceiveBuildingEnd()
	
end

-- 显示建筑物的名字
function BuildingBase:ShowBuildingName(strName, nZorder)
	if nZorder == nil  then
		nZorder = 20;
	end
	local pSpName = CCSprite:create(Public_GetResPathByKey(strName))
 	if pSpName == nil then 
 		return 
 	end
	pSpName:setPosition( ccp(0, 5) )
	pSpName:setScale(1.3)
	self:addChild(pSpName, nZorder)
end

-- 建造或者升级成功
function BuildingBase:ShowBuildOrLevelAct(pSpBuild, bIsBuild)
	CCSpriteFrameCache:sharedSpriteFrameCache():addSpriteFramesWithFile( Public_GetResPathByKey("Images/BuildAct.plist") )

	-- 底部动画
	local pBottom = CCSprite:createWithSpriteFrameName("BuildAct1_1.png")
	pBottom:setAnchorPoint(ccp(0.5, 0.0))
	pBottom:setScale(2.2)
	pBottom:setPosition( ccpAdd(ctspccp(pSpBuild, 0.5, 0), ccp(0, -8.5) ) )
	pSpBuild:addChild(pBottom, -1)

	local pArrBot = CCArray:create()
	for i=1,14 do
		pArrBot:addObject( CCSpriteFrameCache:sharedSpriteFrameCache():spriteFrameByName("BuildAct1_"..i..".png") )
	end
	local pAnim = CCAnimate:create( CCAnimation:createWithSpriteFrames(pArrBot, 0.1) )
	pBottom:runAction( CCSequence:createWithTwoActions(pAnim, CCRemoveSelf:create(true)) );

	-- 外面的动画
	local pSpMid = CCSprite:createWithSpriteFrameName("BuildAct2_1.png")
	pSpMid:setScale(3)
	pSpMid:setPosition( ctspccp(pSpBuild, 0.5, 0.7) )
	pSpBuild:addChild(pSpMid)

	local pArrMid = CCArray:create()
	for i=1,14 do
		pArrMid:addObject( CCSpriteFrameCache:sharedSpriteFrameCache():spriteFrameByName("BuildAct2_"..i..".png") )
	end
	local pAnim1 = CCAnimate:create( CCAnimation:createWithSpriteFrames(pArrMid, 0.1) )
	pSpMid:runAction( CCSequence:createWithTwoActions(pAnim1, CCRemoveSelf:create(true)) );

	-- 弹出来的字
	local pSpWord
	if bIsBuild == true then
		pSpWord = CCSprite:createWithSpriteFrameName("BuildActBuild.png")
	else
		pSpWord = CCSprite:createWithSpriteFrameName("BuildActLvUp.png")
	end
	pSpWord:setPosition( ctspccp(pSpBuild, 0.5, 0.75) )
	pSpWord:setScale(0)
	pSpBuild:addChild(pSpWord)

	local tArrAct = CCArray:create()
	tArrAct:addObject( CCScaleTo:create(0.45, 0.8) )
	tArrAct:addObject( CCCallFunc:create(function()
		local pSpOne  = CCSprite:createWithTexture( pSpWord:getTexture(), pSpWord:getTextureRect() )
		pSpOne:setPosition( ctspccp(pSpWord, 0.5, 0.5) )
		pSpOne:setOpacity(60)
		pSpWord:addChild(pSpOne)

		pSpOne:runAction( CCSequence:createWithTwoActions(CCScaleTo:create(0.3, 1.3), CCRemoveSelf:create(true)) )
	end) )
	tArrAct:addObject( CCDelayTime:create(0.4) )
	tArrAct:addObject( CCMoveBy:create(0.6, ccp(0, 100) ) )
	tArrAct:addObject( CCSpawn:createWithTwoActions(CCMoveBy:create(0.6, ccp(0, 100) ), CCFadeTo:create(0.6, 0)) )
	tArrAct:addObject( CCRemoveSelf:create(true) )

	pSpWord:runAction( CCSequence:create(tArrAct) )

	-- 播放音效
	SOUNDMGR:PlayEffBuildingBuildOrUpgrade()
end

-- 删除所有按钮显示
function BuildingBase:RemoveShowMenuItems()
	-- 如果不是已经全部显示，则不处理
	if self.nBtnListStatus ~= BuildBtnStatus.Showed and self.nBtnListStatus ~= BuildBtnStatus.Showing then
		return;
	end

	-- 切换成开始显示中
	self.nBtnListStatus = BuildBtnStatus.Removing;

	-- 删除所有
	local pArrChildren = self.pBtnMgr:getChildren();
	if pArrChildren then
		for i=1,pArrChildren:count() do
			local pOne = pArrChildren:objectAtIndex(i-1)
			tolua.cast(pOne, "CCNode")

			local tArrAct = CCArray:create();
			tArrAct:addObject( CCDelayTime:create( 0.06*(i-1) ) )
			tArrAct:addObject( CCMoveTo:create(0.18, ccp(0, 150)) )
			tArrAct:addObject( CCHide:create() )

			if i == pArrChildren:count() then
				tArrAct:addObject( CCCallFunc:create(function()
					self.nBtnListStatus = BuildBtnStatus.None;
					self.pBtnMgr:RemoveAllMenuItem();
					self:RemoveBuildingMenuItemsEnd();
				end) )
			end
			pOne:stopAllActions();
			pOne:runAction(CCSequence:create(tArrAct))
		end
	end
end

-- 添加一个按钮到层中
function BuildingBase:ShowMenuItemsToMgr(tItemList)
	if tItemList == nil or #tItemList <= 0 then
		return;
	end
	-- 只有不在显示状态是，才能显示
	if self.nBtnListStatus ~= BuildBtnStatus.None then
		CTAssert(0, "要显示建筑菜单时，当前菜单还没有删除!")
		return
	end

	-- 切换成开始显示中
	self.nBtnListStatus = BuildBtnStatus.Showing;

	-- 不同的数目，不同的位置
	local tPosList = {}
	local nItemCount = #tItemList;
	if nItemCount == 1 then
		tPosList = {ccp(0, 320)}
	elseif nItemCount == 2 then
		tPosList = {ccp(-90, 300), ccp(90, 300)}
	else
		tPosList = {ccp(-120, 280), ccp(0, 320), ccp(120, 280)}
	end


	-- 按钮,弹过去的动画
	for i,tOne in ipairs(tItemList) do
		self.pBtnMgr:AddMenuItem(tOne, tOne:getTag());

		tOne:setVisible(false)
		tOne:setPosition( ccp(0, 150) )

		local tArrAct = CCArray:create();
		tArrAct:addObject( CCDelayTime:create( 0.06*(i-1) ) )
		tArrAct:addObject( CCShow:create() )
		tArrAct:addObject( CCMoveTo:create(0.18, tPosList[i]) )
		if i == nItemCount then
			tArrAct:addObject( CCCallFunc:create(function ()
				-- 切换成开始显示中
				self.nBtnListStatus = BuildBtnStatus.Showed;
			end) )
		end

		tOne:runAction(CCSequence:create(tArrAct))
	end
end

-- 获取当前菜单状态
function BuildingBase:GetBuildBtnShowStatus()
	return self.nBtnListStatus;
end

-- 是否能显示按钮
function BuildingBase:GetIsCanShowButton()
	if self.nBtnListStatus == BuildBtnStatus.None then
		return true;
	end
	return false;
end
-- 是否能删除按钮显示
function BuildingBase:GetIsHadButton()
	-- 显示的过程都可以删除
	if self.nBtnListStatus == BuildBtnStatus.Showed or self.nBtnListStatus == BuildBtnStatus.Showing then
		return true;
	end
	return false;
end
-- -- -- 获取当前的按钮管理
-- -- function BuildingBase:GetCurMenuMgr()
-- -- 	return self.pBtnMgr
-- -- end

-- 创建底盘
function BuildingBase:CreateBuildingBottom()
	-- local pSpBottom1 = CCSprite:create("Images/JianZhuDi.png");
	-- pSpBottom1:setScale(1.5)
	-- self:addChild(pSpBottom1, -3)

	-- local pSpBottom2 = CCSprite:create("Images/CJ_Tu2.png");
	-- pSpBottom2:setPosition(ccp(0, 10))
	-- self:addChild(pSpBottom2, -2)
end


-- 点击建筑区域，nPart=0表示建筑本身，1表示物品
function BuildingBase:TouchBuildByPart( nPart, PartTypeEnum)
	
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function BuildingBase:BuildingTouchBegan(tLocation, nCount)
	self.nTouchType = BuildingTouchType.None;

	
	local tPos = self.pBtnMgr:convertToNodeSpace(tLocation)

	if self.pBtnMgr:touchMenuItemBeganByPoint(tPos) == true then
		self.nTouchType = BuildingTouchType.BtnTouchOne;
		return true;
	end


	-- 转成当前点
	local tCurPos = self:convertToNodeSpace(tLocation)
	-- 判断点击情况
	if self.tBottomRect:containsPoint(tCurPos) then
		self.nTouchType = BuildingTouchType.NormalOnDown;
	elseif self.tTopRect:containsPoint(tCurPos) then
		self.nTouchType = BuildingTouchType.NormalOnUp;
	else
		self.nTouchType = BuildingTouchType.NormalOut;
	end
	return false;
end

-- 触控移动
function BuildingBase:BuildingTouchMoved(tLocation, tPrevPos, tBeganPos)

	if self.nTouchType == BuildingTouchType.None then
		return false;
	end

	local tPos = self.pBtnMgr:convertToNodeSpace(tLocation)
	self.pBtnMgr:touchMenuItemMovedByPoint(tPos)

	-- 点击了按钮
	if self.nTouchType == BuildingTouchType.BtnTouchOne then
		return true;
	end

	if self.nTouchType == BuildingTouchType.NormalMove then
		return false;
	end

	if ccpDistance(tLocation, tBeganPos) > 6 then
		self.nTouchType = BuildingTouchType.NormalMove;
	end

	return false;
end

-- 触控结束
function BuildingBase:BuildingTouchEnded(tLocation, tPrevPos, tBeganPos)
	local tPos = self.pBtnMgr:convertToNodeSpace(tLocation)
	self.pBtnMgr:touchMenuItemEndedByPoint(tPos)

	if self.nTouchType == BuildingTouchType.None then
		return false;
	end

	-- 点击了按钮
	if self.nTouchType == BuildingTouchType.BtnTouchOne then
		self.nTouchType = BuildingTouchType.None;
		return true;
	end

	-- 点击了上部
	if self.nTouchType == BuildingTouchType.NormalOnUp then
		self:TouchBuildByPart(BuildingTouchPart.GetItemPart, BuildingTouchPart);
		self.nTouchType = BuildingTouchType.None;
		return false;
	-- 点击下部
	elseif self.nTouchType == BuildingTouchType.NormalOnDown then
		self:TouchBuildByPart(BuildingTouchPart.BuildingPart, BuildingTouchPart);
		self.nTouchType = BuildingTouchType.None;
		return false;
	-- 点击外部
	elseif self.nTouchType == BuildingTouchType.NormalOut then
		self:TouchBuildByPart(BuildingTouchPart.OutSide, BuildingTouchPart);
		self.nTouchType = BuildingTouchType.None;
		return false;
	end

	self.nTouchType = BuildingTouchType.None;
	return false;
end


return BuildingBase;