--
-- Author: Link
-- Date: 2014-03-19 10:20:19
--
local LogicBuilding = class("LogicBuilding", function()
	return display.newLayer()
end)

local BaseBuildingType = {
	CarPort 	= 1,	-- 汽车之家
	MergeCar 	= 2,	-- 合成汽车
	TmpValue1 	= 20,	-- 临时值
	TmpValue2 	= 21,	-- 临时值
}


function LogicBuilding:ctor()
	-- self.super.ctor(self)

	-- 是否在滑动中
	self.bIsScroll = false;
	-- 显示的建筑列表
	self.tShowBuildList = {}
	-- setmetatable(self.tShowBuildList, {__mode = "v"})
	-- 
	self.tBuildInfoList = {}

	self.fScrollLen		= 0;


	-- 合成车
	local tCarPortInfo = clone(OneBuildingInfo)
	tCarPortInfo.nBuildingID = BaseBuildingType.MergeCar;
	self.tBuildInfoList[1] = tCarPortInfo;

	-- 淘宝屋
	local tCarPortInfo = clone(OneBuildingInfo)
	tCarPortInfo.nBuildingID = BaseBuildingType.CarPort;
	self.tBuildInfoList[2] = tCarPortInfo;

	-- -- 其他临时值
	-- local tCarPortInfo = clone(OneBuildingInfo)
	-- tCarPortInfo.nBuildingID = BaseBuildingType.TmpValue1;
	-- self.tBuildInfoList[2] = tCarPortInfo;

	local tTmpList = DATAMGR:GetUserInfoMgr():GetAllUserBuildingList();

	-- 添加当前建筑
	for i,v in ipairs(tTmpList) do
		self.tBuildInfoList[#self.tBuildInfoList+1] = v;
	end

	-- 等级需求列表
	self.tLevelNeedList = DATAMGR:GetUserInfoMgr():GetAllUserBuildPosNeedLvList();
end

-- 释放前调用
function LogicBuilding:BeforeReleaseLayer()
	-- 有显示层，则释放对应信息
	-- self.pCtuiBuilding:removeFromParent()
	-- self.pCtuiBuilding = nil

	self.pScrollView:removeFromParent();
	self.pScrollView:release()
	self.pScrollView = nil;

	self.pLyCurShow:removeFromParent();
	self.pLyCurShow = nil;
end
-- 初始化
function LogicBuilding:InitInfo( pShowLayer, nStartShowIdx )
	if nStartShowIdx == nil then
		nStartShowIdx = 0;
	end

	-- 显示的层次 
	self.pLyCurShow = CCLayer:create()
	pShowLayer:addChild(self.pLyCurShow, GLobbyZOrder.Building)

	-- 开始显示
	self:StartToShowBuildingBg();

	local nCount = #self.tBuildInfoList

	
	local nShowType = SHARE_FUNC.nFrameType
	local fDelHeight = 230;
	-- ipad
    if nShowType == CTDeviceTypeIpad or nShowType == CTDeviceTypeIpadHD then

    elseif nShowType == CTDeviceTypeIphone5HD or nShowType == CTDeviceTypeItouch5HD then
    	fDelHeight = 230
    else
    	fDelHeight = 230
    end 

	-- 滑动层
	self.pScrollView = CTScrollCenter:new()
	self.pScrollView:init(CCSizeMake(ctipwidth(1000), ctipwidth(800)),
							ccp(ctipwidth(384), ctipwidth(fDelHeight) ),
							CCSizeMake(ctipwidth(768), ctipwidth(800)),
							nCount)

	self.pScrollView:SetIsClipRegion(false)
	self.pLyCurShow:addChild(self.pScrollView, -2)

	self.pScrollView:SetDelegateLua(function(event)
		-- 显示信息
		if event.type == "showItem" then
			self:ScrollItemContent(event.scroll, event.item, event.Idx)
		-- 开始滑动
		elseif event.type == "StartMove" then
			self:ScrollItemStartMove(event.scroll, event.item, event.Idx)
		-- 滑动结束
		elseif event.type == "DidStop" then
			self:ScrollItemDidStopInCenter(event.scroll, event.item, event.Idx)
		end
	end)
	self.pScrollView:SetDelSpeedValue(3300.0)
	self.pScrollView:SetMaxMoveSpeed(6000.0)
	self.pScrollView:SetSpeedRate(52)

	self.pScrollView:ShowScrollByCenterIndex(nStartShowIdx)
end

-- 强制显示到某个位置
function LogicBuilding:ForceShowBuildingByIdx( nIdx )

	local fOldLen = self.pScrollView:GetCurScrollLength();
	self.pScrollView:ReShowCenterIndex(nIdx)

	local fNewLen = self.pScrollView:GetCurScrollLength();
	-- 算上滑动
	if fOldLen < fNewLen then
		while fOldLen < fNewLen do
			local fTmp=fOldLen+200;
			if fTmp > fNewLen then
				fTmp = fNewLen
			end
			self:ShowBuildingBgScrollDeltaLen( fTmp-fOldLen )
			fOldLen = fTmp;
		end
	else
		while fOldLen > fNewLen do
			local fTmp=fOldLen-200;
			if fTmp < fNewLen then
				fTmp = fNewLen
			end
			self:ShowBuildingBgScrollDeltaLen( fTmp-fOldLen )
			fOldLen = fTmp;
		end
	end

	self.fScrollLen = fNewLen
end

-- 强制显示的建筑槽位
function LogicBuilding:ForceShowBuildingCaoByIdx( nIdx )
	local tTmpList = DATAMGR:GetUserInfoMgr():GetAllUserBuildingList();
	local nBuildIdx = nIdx + (#self.tBuildInfoList - #tTmpList);
	self:ForceShowBuildingByIdx(nBuildIdx)
end


-- 开始显示层次
function LogicBuilding:StartToShowBuildingBg()
	local BaseBackground = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/BaseBackground")
	local pBg = BaseBackground.new()
	self.pLyCurShow:addChild(pBg, -4)
	self.pBg = pBg;


	-- 树叶动画
	local BaseBackgroundLeafAct = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/BaseBackgroundLeafAct")
	self.pSpBg = BaseBackgroundLeafAct.new()
	self:addChild(self.pSpBg)
end

-- 建筑物光效果
function LogicBuilding:ShowBuildingLight( pSpCity, bIsFlip)
	if pSpCity == nil then 
		return
	end	
 	local fXCoordinate = 590
 	local fAnchYPoint = 0.5
 	if bIsFlip == true then 
 		fXCoordinate = 180
 		fAnchYPoint = 0.5
 	end
 	
 	local isFilp = false
 	-- 旋转角度
 	local fAngel = 180
 	local fScale = 1
 	-- 旋转时间
 	local fTime1 = 10
 	local fTime2 = 8
 	
 	if nShowType == CTDeviceTypeIpad or nShowType == CTDeviceTypeIpadHD then
    	fScale = 0.5
    elseif nShowType == CTDeviceTypeIphone5HD or nShowType == CTDeviceTypeItouch5HD then
    	fScale = 1
    else
    	fScale = 0.6
    end 

	for i = 1, 2 do 
		local pLight = CCSprite:create(Public_GetResPathByKey("Images/GuanShe.png"))
		pLight:setScale(fScale)
		pLight:setAnchorPoint( ccp(0.95, fAnchYPoint) )
		
		pLight:setFlipX(isFilp)
		-- 右边单个激光
		if bIsFlip == true then 
			if i == 2 then 
				pLight:setAnchorPoint( ccp(0.95, fAnchYPoint) )
				-- fXCoordinate = pLight:getContentSize().width + fXCoordinate - 340
				pLight:setPosition(ccp(fXCoordinate ,160))
				pSpCity:addChild(pLight, -1)
				pLight:setOpacity(120)
				pLight:setScale(0.6)
				fAngel = 150
				fTime1 = 7
				fTime2 = 5
			end
		-- 左边激光
		else
			isFilp = not isFilp
			-- fXCoordinate = pLight:getContentSize().width + fXCoordinate - 340
			if i == 2 then 
				fAngel = -180
				pLight:setAnchorPoint( ccp(0.045, 0.55) )
				fTime1 = 9
 				fTime2 = 7
 				pLight:setScale(0.4)
 				pLight:setOpacity(40)
 			else
 				pLight:setOpacity(255*0.9)
			end
			pLight:setPosition(ccp(fXCoordinate ,198))
			pSpCity:addChild(pLight, -1)

		end
		-- 动作
 		local actArry = CCArray:create()
		actArry:addObject(CCRotateTo:create(fTime1, fAngel))
		actArry:addObject(CCDelayTime:create(0.2))
		actArry:addObject(CCRotateTo:create(fTime2, 0))
		actArry:addObject(CCDelayTime:create(0.2))
		-- 执行光的动作
		pLight:runAction(CCRepeatForever:create(CCSequence:create(actArry)))
	end
end


-- 显示滑动情况
function LogicBuilding:ShowBuildingBgScrollDeltaLen( fDtLen )
	self.pBg:ShowBgScrollDeltaLen(fDtLen)
end

-- mark: ============== 循环处理 ================
function LogicBuilding:LogicBuildingLoop( dt )
	if self.pScrollView then
		self.pScrollView:UpdateScoll(dt)

		local fNewLen = self.pScrollView:GetCurScrollLength();
		if self.fScrollLen ~= fNewLen then
			self:ShowBuildingBgScrollDeltaLen( self.fScrollLen-fNewLen )
			self.fScrollLen = fNewLen
		end
	end
	-- 树叶动画
	-- 树叶动画
	if self.pSpBg ~= nil then 
		self.pSpBg:BackLeatActLoop(dt)
	end

	-- 获取当前时间
	local fCurTime = GetCurrentSystemTime1()
	-- 每个建筑的循环
	for i,pOneBuilding in pairs(self.tShowBuildList) do
		if pOneBuilding.BuildingLoop then
			pOneBuilding:BuildingLoop(dt, fCurTime)
		end
	end
end


--
-- mark: ============== 回调处理 ================
-- 对话框的按钮回调信息
function LogicBuilding:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
	-- 加速收获
	if nBtnTag == DLGTYPE.QuickGainRes then
		if nBtnIdx == 1 then
			self:SendQuickGainResource(tData[2], tData[3])
		end
	-- 弹出建筑
	elseif nBtnTag == DLGTYPE.BuildButNotGainRes then
		if nBtnIdx == 1 then
			-- 改建
			if tData[1] == 2 then
				GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.BuildChooseView, tData[2], tData[3])
			-- 升级
			elseif tData[1] == 3 then
				GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.BuildLevelUp, tData[2], tData[3], tData[4])
			end
		end
	-- 如果被掠夺
	elseif nBtnTag == DLGTYPE.BuildBeLooted then
		if nBtnIdx == 1 then
			-- 还没有获取过数据列表，则开始去获取
			if DATAMGR:GetUserInfoMgr():IsGetMessageListInfo() == false then
				-- 获取消息列表
				SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.GetMessageList, nil)
			else
				GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.MessageList)
			end	
		end
	end
end
-- 按钮回调
function LogicBuilding:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()

    print("点击了基地建筑按钮:".. nTag)

	-- 汽车配件抽卡界面
	if nTag == 24 then
		GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.GetCardPort)
	else
		-- DIALOGMGR:ShowToast("功能未开放，敬请期待!!")
	end
end


-- 显示item的内容
function LogicBuilding:ScrollItemContent( pScroll, pItem, nIdx )

	local tOneBuildInfo = self.tBuildInfoList[nIdx+1]

	local tTmpList = DATAMGR:GetUserInfoMgr():GetAllUserBuildingList();
	local nBuildIdx = nIdx - (#self.tBuildInfoList - #tTmpList);

	local pItemInfo = self:CreatBuildingByInfo(tOneBuildInfo, nIdx, nBuildIdx);
	pItem:addChild(pItemInfo);

	self.tShowBuildList[nIdx] = pItemInfo;
	-- collectgarbage();
	for k,v in pairs(self.tShowBuildList) do
		if v.BuildingTouchMoved == nil then
			self.tShowBuildList[k] = nil;
		end
	end
end

-- 开始点击滑动
function LogicBuilding:ScrollItemStartMove(pScroll, pItem, nIdx)
	-- 是否在滑动中
	self.bIsScroll = true;
end

-- 对应的item滑动到正中间
function LogicBuilding:ScrollItemDidStopInCenter( pScroll, pItem, nIdx )
	-- 是否在滑动中
	self.bIsScroll = false;
end


-- 根据对应的信息创建,nIdx 表示队列的位置，nBuildIdx 表示可以自由建造的槽位位置从0开始
function LogicBuilding:CreatBuildingByInfo(tOneBuildInfo, nIdx, nBuildIdx)
	if tOneBuildInfo == nil then
		return nil;
	end

	local pShowView 

	-- 如果是汽车之家
	if tOneBuildInfo.nBuildingID == BaseBuildingType.CarPort then
		pShowView = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/Building/BuildingCarPort").new()

	-- 如果是合成
	elseif tOneBuildInfo.nBuildingID == BaseBuildingType.MergeCar then
		pShowView = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/Building/BuildingMergeCard").new()

	-- 小型精力
	elseif tOneBuildInfo.nBuildingID == 4002 then
		pShowView = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/Building/BuildingJingLiS").new()
		pShowView:InitBuilding(tOneBuildInfo)
	-- 小型体力
	elseif tOneBuildInfo.nBuildingID == 4003 then
		pShowView = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/Building/BuildingTiLiS").new()
		pShowView:InitBuilding(tOneBuildInfo)
	-- 中型精力
	elseif tOneBuildInfo.nBuildingID == 4004 then
		pShowView = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/Building/BuildingJingLiM").new()
		pShowView:InitBuilding(tOneBuildInfo)
	-- 中型体力
	elseif tOneBuildInfo.nBuildingID == 4005 then
		pShowView = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/Building/BuildingTiLiM").new()
		pShowView:InitBuilding(tOneBuildInfo)
	-- 大型精力
	elseif tOneBuildInfo.nBuildingID == 4006 then
		pShowView = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/Building/BuildingJingLiL").new()
		pShowView:InitBuilding(tOneBuildInfo)
	-- 大型体力
	elseif tOneBuildInfo.nBuildingID == 4007 then
		pShowView = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/Building/BuildingTiLiL").new()
		pShowView:InitBuilding(tOneBuildInfo)

	-- 如果是未安装
	elseif tOneBuildInfo.nBuildingID == NOT_EQUIPED then
		pShowView = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/Building/BuildingUnBuild").new()
		pShowView:InitBuilding({self.tLevelNeedList[nBuildIdx+1]})
	end

	-- local pSpBuild = CCSprite:create("Images/4SJianZhu.png")
	-- pSpBuild:setAnchorPoint( ccp(0.5, 0.14) )

	if pShowView ~= nil then
		if SHARE_FUNC.nFrameType == CTDeviceTypeIphone5HD or SHARE_FUNC.nFrameType == CTDeviceTypeItouch5HD then
			pShowView:setScale(0.9)
		elseif SHARE_FUNC.nFrameType == CTDeviceTypeIpad or SHARE_FUNC.nFrameType == CTDeviceTypeIpadHD then
			pShowView:setScale(0.78)
		else
			pShowView:setScale(0.78)
		end
 
		pShowView:SetCurBuildIdx(nBuildIdx);
		pShowView:SetCurBuildShowIdx(nIdx)
	end

	return pShowView;
end

-- mark: ============== 发送的信息处理 ================
-- 发送加速获取,nType,1-金币，2-精力体力
function LogicBuilding:SendQuickGainResource( nPos, nType )
	-- 发送登陆信息
	local tLuaData = {}
	tLuaData.pos = nPos
	tLuaData.idx = nType

	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.SpeedUpGainBuilding, tLuaData)
end

-- mark: ============== 接收到的信息处理 ================
-- 建造建筑物成功
function LogicBuilding:ReceiveBuildOneBuildingScuess(nPos)
	self.pScrollView:ReShowCenterIndex( self.pScrollView:GetCurIndex() );

	local nCurPos = nPos-1
	-- 每个建筑的循环，找出接收的位置
	for i,pOneBuilding in pairs(self.tShowBuildList) do
		if pOneBuilding.GetCurBuildIdx and pOneBuilding:GetCurBuildIdx() == nCurPos then
			local nIdx = pOneBuilding:GetCurBuildShowIdx()
			local tOneBuildInfo = self.tBuildInfoList[nIdx+1]
			local pItemInfo = self:CreatBuildingByInfo(tOneBuildInfo, nIdx, pOneBuilding:GetCurBuildIdx() );
			pOneBuilding:getParent():addChild(pItemInfo);
			pOneBuilding:removeFromParent();
			
			self.tShowBuildList[nIdx] = pItemInfo;

			pItemInfo:ReceiveBuildingEnd()
			break;
		end
	end

	-- 建造成功，则指向下一条教学
	TEACHMGR:ShowSwichToNextStep();
end
-- 升级建筑物成功
function LogicBuilding:ReceiveLevelUpOneBuildingScuess(nPos)
	local nCurPos = nPos-1
	-- 每个建筑的循环，找出接收的位置
	for i,pOneBuilding in pairs(self.tShowBuildList) do
		if pOneBuilding.GetCurBuildIdx and pOneBuilding:GetCurBuildIdx() == nCurPos then
			if pOneBuilding.ReceiveLevelUp then
				pOneBuilding:ReceiveLevelUp()
			end
			break;
		end
	end

	-- CreatBuildingByInfo
end
-- 获取金币成功
function LogicBuilding:ReceiveGainOneBuildGold(nPos, nLootGold)
	local nCurPos = nPos-1
	-- 每个建筑的循环，找出接收的位置
	for i,pOneBuilding in pairs(self.tShowBuildList) do
		if pOneBuilding.GetCurBuildIdx and pOneBuilding:GetCurBuildIdx() == nCurPos then
			if pOneBuilding.ReceiveGainGoldScuess then
				pOneBuilding:ReceiveGainGoldScuess(nLootGold)
			end
			break;
		end
	end
end
-- 获取物品成功
function LogicBuilding:ReceiveGainOneBuildOtherValue(nPos)
	local nCurPos = nPos-1
	-- 每个建筑的循环，找出接收的位置
	for i,pOneBuilding in pairs(self.tShowBuildList) do
		if pOneBuilding.GetCurBuildIdx and pOneBuilding:GetCurBuildIdx() == nCurPos then
			if pOneBuilding.ReceiveGainOtherValueScuess then
				pOneBuilding:ReceiveGainOtherValueScuess()
			end
			break;
		end
	end
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function LogicBuilding:LgBuildingTouchBegan(tLocation, nCount)
	if tLocation.y < ctipwidth(SHARE_FUNC.bottomTagH) then
		return;
	end
	-- return self.pCtuiBuilding:CTUITouchBegan(tLocation)
	-- 是否在滑动中
	if self.bIsScroll == false then
		for i,pOneBuilding in pairs(self.tShowBuildList) do
			if pOneBuilding.BuildingTouchBegan then
				if pOneBuilding:BuildingTouchBegan(tLocation, nCount) then
					return true;
				end
			end
		end
	end
	self.pScrollView:ScrollTouchBegan(tLocation)
end

-- 触控移动
function LogicBuilding:LgBuildingTouchMoved(tLocation, tPrevPos, tBeganPos)
	-- return self.pCtuiBuilding:CTUITouchMoved(tLocation)
	for i,pOneBuilding in pairs(self.tShowBuildList) do
		if pOneBuilding.BuildingTouchMoved then
			if pOneBuilding:BuildingTouchMoved(tLocation, tPrevPos, tBeganPos) then
				return true;
			end
		end
	end

	-- 在教学中，则不给滑动
	if TEACHMGR:IsCurInTeaching() then
		return;
	end
	
	self.pScrollView:ScrollTouchMoved(tLocation, tPrevPos)
end

-- 触控结束
function LogicBuilding:LgBuildingTouchEnded(tLocation, tPrevPos, tBeganPos)
	-- return self.pCtuiBuilding:CTUITouchEnded(tLocation)
	for i,pOneBuilding in pairs(self.tShowBuildList) do
		if pOneBuilding.BuildingTouchEnded then
			if pOneBuilding:BuildingTouchEnded(tLocation, tPrevPos, tBeganPos) then
				return true;
			end
		end
	end

	self.pScrollView:ScrollTouchEnded(tLocation, tPrevPos)
end

return LogicBuilding