--
-- Author: Link
-- Date: 2013-12-10 14:57:32
--

local GameLgLobby = class("GameLgLobby", function()
	return display.newLayer()
end)

------------------ 界面tag值 -----------------
local GLobbyTabTag = {
	LyTopInfo		= 5,	-- 上一层
	LyBottomInfo	= 18,	-- 下一层
	-------- 人物信息 -------
	LbNickName		= 48,	-- 昵称
	LbLevelNum		= 9,	-- 等级数
	LbUserExp		= 1,	-- 人物经验数
	LbUserGold		= 28,	-- 人物金币数
	LbUserDiamond	= 30,	-- 人物钻石数
	LbUserEnergy	= 45,	-- 人物精力
	LbUserActivity	= 44,	-- 人物体力
	LbBattleValue	= 7,	-- 人物战斗力
	LbUserVipLv		= 8,	-- Vip等级

	-- SpExpCircle 	= 7,	-- 经验圈
	SpEnergyLine	= 16,	-- 精力长度
	SpActivityLine	= 21,	-- 体力长度
	-- SpCoin 			= 0,	-- 金币

	SpUserInfoBg 	= 0,	-- 玩家信息底图
	SpUserDiamod 	= 10,	-- 玩家金币 砖石层
	SpGoldBg 		= 13, 	-- 金币背景
	----------------------

	--- - 底下的按钮层 ----
	BtnBase = 15, -- 基地  
	BtnTeam = 6, -- 阵容
	BtnStory = 12, -- 副本
	BtnTopFight = 9, -- 竞技
	BtnPet = 0, -- 宠物
	BtnUnion = 15, -- 工会

	-- SpBotBase = 7,	-- 基地按钮底部标签
	-- SpBotTeam = 3,	-- 阵容按钮底部标签
	-- SpBotStory = 13,	-- 副本按钮底部标签
	-- SpBotTopFight = 19,	-- 竞技按钮底部标签
	-- SpBotPet = 29,	-- 宠物按钮底部标签
	-- SpBotUnion = 39,	-- 工会按钮底部标签

	-- SpSelBottom = 2, -- 底层选择的东西
	----------------------
}

-- 大厅所有信息的显示层次
GLobbyZOrder = {
	BackGround 	= 0,	-- 背景层
	Building 	= 1,	-- 建筑层
	TabLayer	= 3,	-- 每个tab主要内容层次
	TeamList 	= 4,	-- 阵容列表层次
	OtherInfo	= 5,	-- 其他信息层
	TopUserInfo	= 6,	-- 顶部信息层
	PopView		= 8,	-- 弹出视图层
	TabBtnLayer = 10,	-- 标签按钮层
	TopViewLy	= 11,	-- 视图管理里面顶部视图
}

-- 顶部通知栏
local Tag_Notice = {
	Lb_Notice 	= 40,
	Lb_Notice2 	= 11,

	SpBg = 2, 	-- 背景
	LyContent 	= 3, -- 内容层
}

-- 通知类型
Notice_Type = {
	System = 1,
	UserUpgrade = 2,
	UserGetHighStarCard = 3,
	UserPassStoryStage = 4,
}

---------------------------------------------
function GameLgLobby:ctor()
	-- self.super.ctor(self)

	-- 播放背景音乐
	SOUNDMGR:PlayLobbyBackgroundMusic()

	-- CD时间的显示
	self.fShowCdTime	= 0;
	-- 当前显示的类型
	self.nCurShowType 	= -1;

	-- 其他逻辑
	self.pLbLogicBase	= nil
	self.pLbLogicTeam	= nil
	self.pLbLogicFuBen	= nil
	self.pLbLogicRank	= nil
	self.pLbLogicPet	= nil
	self.pLbLogicUnion	= nil

	self.nTotalBatValue = 0

	self.tTmpUserInfo = clone(DATAMGR:GetUserInfoMgr():GetUserBaseInfo())

	-- 大厅管理的主要界面内容
	self.pCtuiLayer = CTUILayer:new();
	self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/CaiDanLan.ctui"))
	self.pCtuiLayer:setAnchorPoint(ccp(0, 0))
	self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	self.pCtuiLayer:ShowAllUiInfo()
	self:addChild(self.pCtuiLayer, GLobbyZOrder.TabBtnLayer)
	self.pCtuiLayer:release()

	-- 顶部通知。
	self:InitNoticeData()

	-- 顶部用户信息
	self.pCtuiUser = CTUILayer:new();
	self.pCtuiUser:InitWithFile(Public_GetResPathByKey("Images/CaiDanLanShang-ipad.ctui"))
	self.pCtuiUser:setAnchorPoint(ccp(0, 1))
	self.pCtuiUser:ignoreAnchorPointForPosition(false)
	self.pCtuiUser:setPosition(ctprccp(0, 1))
	self.pCtuiUser:SetUiScaleType(CTUIScaleType_Base_Width)
	self.pCtuiUser:ShowAllUiInfo()
	self:addChild(self.pCtuiUser, GLobbyZOrder.TopUserInfo)
	self.pCtuiUser:release()
	-- vip数目
	local nVipLevel = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nVipLevel
	if nVipLevel == nil then
		nVipLevel = 0;
	end
	-- Vip
	local pLbVipLvTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserVipLv)
	local pLbVipLv = CTImgLabel:new()
	pLbVipLv:InitLabel(""..nVipLevel, Public_GetResPathByKey("Images/Font/PlusShuZhi.json"), 40)
	pLbVipLv:setFontSize(35)
	-- local pLbVipLv = CCLabelAtlas:create(""..nVipLevel, "Images/GX-064.png", 40, 39, string.byte('0') )
	pLbVipLv:setAnchorPoint( ccp(0, 0.5) )
	pLbVipLv:setPosition( pLbVipLvTmp:getPositionInCCPoint() )
	pLbVipLvTmp:getParent():addChild(pLbVipLv, 0, pLbVipLvTmp:getTag())
	pLbVipLvTmp:removeFromParent()
	pLbVipLv:release()

	-- 将顶部的信息条重新定位
	local tTopLayer = self.pCtuiLayer:GetNodeByTag(GLobbyTabTag.LyTopInfo)
	tTopLayer:setAnchorPoint(ccp(0, 1))
	tTopLayer:ignoreAnchorPointForPosition(false)
	tTopLayer:setPosition( self.pCtuiLayer:convertToNodeSpace( ctprccp(0, 1) ) )

	self.pCtuiLayer:setDelegateLua(function(event)
	-- 如果是按钮点击事件
		if event.cbType == "btnCb" then
			self:CTUIButtonCallBack(event.Layer, event.Btn, event.Tag)
		end
	end)

	-- 显示一个视图
	self:ChangeLobbyView(GAME_STATUS.GetStatus(2), true)

	local tCurUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()

	-- 等级数
	local pLbLvTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbLevelNum)
	local CTWordImg = requires(ROOT_PATH, "app/GameConfig/CTWordImg")
	self.pLbUserLv = CTWordImg.new()
	self.pLbUserLv:InitWord("000", "UserLvNum", 60)
	self.pLbUserLv:setAnchorPoint( ccp(0.5, 0) )
	self.pLbUserLv:setPosition( ccpAdd(pLbLvTmp:getPositionInCCPoint(), ccp(-10,0)) )
	pLbLvTmp:getParent():addChild(self.pLbUserLv, 0, pLbLvTmp:getTag())
	pLbLvTmp:removeFromParent();
	if self.tTmpUserInfo.nLv < 10 then
		self.pLbUserLv:setScale(1.2)
	elseif self.tTmpUserInfo.nLv < 100 then
		self.pLbUserLv:setScale(1)
	else
		self.pLbUserLv:setScale(0.8)
	end


	-- 战斗力
	local pLbBattleVTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbBattleValue)
	-- local pLbBattleV = CCLabelAtlas:create(""..nVipLevel, "Images/GX-064.png", 40, 39, string.byte('0') )
	local pLbBattleV = CTImgLabel:new()
	pLbBattleV:InitLabel(""..nVipLevel, Public_GetResPathByKey("Images/Font/PlusShuZhi.json"), 40)
	pLbBattleV:setFontSize(28)
	pLbBattleV:setAnchorPoint( ccp(0, 0.5) )
	-- pLbBattleV:setScale(0.7)
	pLbBattleV:setPosition( pLbBattleVTmp:getPositionInCCPoint() )
	pLbBattleVTmp:getParent():addChild(pLbBattleV, 0, pLbBattleVTmp:getTag())
	pLbBattleVTmp:removeFromParent()
	pLbBattleV:release()

	-- 速度盘的修改
	-- local pSpPanTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.SpExpCircle)
	-- tolua.cast(pSpPanTmp, "CCSprite")
	-- local pSpSpeedPan = CCProgressTimer:create(CCSprite:createWithTexture(pSpPanTmp:getTexture()))
	-- pSpSpeedPan:setType(kCCProgressTimerTypeRadial)
	-- pSpSpeedPan:setMidpoint(CCPointMake(0.5074, 0.51))
	-- pSpSpeedPan:setRotation(pSpPanTmp:getRotation())
	-- local fPer = tCurUserInfo.nCurExp/tCurUserInfo.nLevelExp
	-- if fPer > 1.0 then
	-- 	fPer = 1.0
	-- end
	-- fPer = (fPer * (235.0/360) + (125.0/360) ) * 100.0
	-- pSpSpeedPan:setPercentage( toint(fPer) )
	-- pSpSpeedPan:setPosition(pSpPanTmp:getPositionInCCPoint())
	-- pSpSpeedPan:setScaleX(pSpPanTmp:getScaleX())
	-- pSpSpeedPan:setScaleY(pSpPanTmp:getScaleY())
	-- pSpPanTmp:getParent():addChild(pSpSpeedPan, 0, GLobbyTabTag.SpExpCircle)
	-- pSpPanTmp:removeFromParent()


	-- 获取触控区域
	self.tUserRect = self.pCtuiUser:GetNodeAbsShowRect(GLobbyTabTag.SpUserInfoBg)

	-- 点解金币 砖石弹出充值
	local pSpGoldBg = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.SpGoldBg)
	local tSpGoldPos = pSpGoldBg:getParent():convertToWorldSpace( pSpGoldBg:getPositionInCCPoint() )
	-- 用来处理掠夺按钮不可用时的点击
	self.tChongZhiRect = CCRectMake(tSpGoldPos.x - 120, tSpGoldPos.y - 50, 200, 150)
	-- 点击出现充值的标记
	self.IsChongZhi = false

	self.nTouchUserType = 0;-- 0是没有点击，1是点击中，2是滑动


	-- 显示当前用户信息
	self:ShowPlayerBaseInfo()

	-- 立刻更新
	self:LogicLobbyLoop(0)
	-- 根据状态判断显示
	self:JudgeShowUserInfoByStatus()

end

-- 释放前的调用
function GameLgLobby:BeforeReleaseLayer()
	-- 删除所有当前视图
	local nSecStatus = GAME_STATUS.GetStatus(2)
	self:RemoveOldLobbyView(nSecStatus, nSecStatus, true)
end
-- mark: ============== 获取各部分逻辑 ================
-- 基地
function GameLgLobby:GetBaseLogic()
	return self.pLbLogicBase
end
-- 队伍
function GameLgLobby:GetTeamLogic()
	return self.pLbLogicTeam
end
-- 副本
function GameLgLobby:GetFuBenLogic()
	return self.pLbLogicFuBen
end
-- 竞技
function GameLgLobby:GetRankLogic()
	return self.pLbLogicRank
end
-- 宠物
function GameLgLobby:GetPetLogic()
	return self.pLbLogicPet
end
-- 工会
function GameLgLobby:GetUnionLogic()
	return self.pLbLogicUnion
end

-- mark: ============== 循环处理 ================
function GameLgLobby:LogicLobbyLoop( dt )
	local fOldCdTime = self.fShowCdTime;
	self.fShowCdTime = self.fShowCdTime + dt;
	if self.fShowCdTime > 10.0 then
		self.fShowCdTime = 0.0;
		-- 更新时间
		self:ShowPlayerBaseInfo(5)
		self:ShowPlayerBaseInfo(6)
	end
	-- 显示倒计时的时间
	if self.fShowCdTime >= 7.0 then
		-- 倒计时
		if math.floor(fOldCdTime) ~= math.floor(self.fShowCdTime) then
			self:ShowPlayerBaseInfo(5, false)
			self:ShowPlayerBaseInfo(6, false)
		end
	end

	-------- 其他子逻辑的循环处理 --------
	local nSecStatus = GAME_STATUS.GetStatus(2)
	-- 基地
	if nSecStatus == STATUS_2.LB_BASE then
		if self.pLbLogicBase then
			self.pLbLogicBase:LogicBaseLoop(dt)
		end

		-- 队伍阵容
	elseif nSecStatus == STATUS_2.LB_TEAM then
		if self.pLbLogicTeam then
			self.pLbLogicTeam:LogicTeamLoop(dt)
		end

		-- 剧情
	elseif nSecStatus == STATUS_2.LB_STORY then
		if self.pLbLogicFuBen then
			self.pLbLogicFuBen:LogicFuBenLoop(dt)
		end

		-- 排位赛
	elseif nSecStatus == STATUS_2.LB_RANKWAR then
		if self.pLbLogicRank then
			self.pLbLogicRank:LogicRankLoop(dt)
		end

		-- 宠物等
	elseif nSecStatus == STATUS_2.LB_GAMEITEM then
		if self.pLbLogicPet then
			-- self.pLbLogicPet:LogicBaseLoop(dt)
		end

		-- 联盟
	elseif nSecStatus == STATUS_2.LB_UNION then
		if self.pLbLogicUnion then
			-- self.pLbLogicUnion:LogicBaseLoop(dt)
		end
	end
	-----------------------------------------

	--------- 判断值是否发生了变化，如果发生了变化，则自动修改显示 ----------
	local tCurUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()
	-- 等级
	if self.tTmpUserInfo.nLv ~= tCurUserInfo.nLv then
		self:ShowPlayerBaseInfo(1)
		self.tTmpUserInfo.nLv = tCurUserInfo.nLv
	end

	-- 经验
	if self.tTmpUserInfo.nCurExp ~= tCurUserInfo.nCurExp or self.tTmpUserInfo.nLevelExp ~= tCurUserInfo.nLevelExp then
		self:ShowPlayerBaseInfo(2)
		self.tTmpUserInfo.nCurExp = tCurUserInfo.nCurExp
		self.tTmpUserInfo.nLevelExp = tCurUserInfo.nLevelExp
	end

	-- 金币
	if self.tTmpUserInfo.nGoldNum ~= tCurUserInfo.nGoldNum then
		self:ShowPlayerBaseInfo(3)
		self.tTmpUserInfo.nGoldNum = tCurUserInfo.nGoldNum
	end

	-- 钻石
	if self.tTmpUserInfo.nDiamodNum ~= tCurUserInfo.nDiamodNum then
		self:ShowPlayerBaseInfo(4)
		self.tTmpUserInfo.nDiamodNum = tCurUserInfo.nDiamodNum
	end

	-- 精力
	if self.tTmpUserInfo.nCurEnergy ~= tCurUserInfo.nCurEnergy or self.tTmpUserInfo.nMaxEnergy ~= tCurUserInfo.nMaxEnergy then
		self:ShowPlayerBaseInfo(5)
		self.tTmpUserInfo.nCurEnergy = tCurUserInfo.nCurEnergy
		self.tTmpUserInfo.nMaxEnergy = tCurUserInfo.nMaxEnergy
	end

	-- 体力
	if self.tTmpUserInfo.nCurActivity ~= tCurUserInfo.nCurActivity or self.tTmpUserInfo.nMaxAcitivity ~= tCurUserInfo.nMaxAcitivity then
		self:ShowPlayerBaseInfo(6)
		self.tTmpUserInfo.nCurActivity = tCurUserInfo.nCurActivity
		self.tTmpUserInfo.nMaxAcitivity = tCurUserInfo.nMaxAcitivity
	end

	-- 战斗力
	local tCurTeam = DATAMGR:GetUserInfoMgr():GetCurTeamList()
	local nBatValue = math.floor( tCurTeam.fTotalSpeed + tCurTeam.fTotalASpeed + tCurTeam.fTotalTurn ) 
	if nBatValue ~= self.nTotalBatValue then
		self.nTotalBatValue = nBatValue
		self:ShowPlayerBaseInfo(7)		
	end

	-- vip
	if self.tTmpUserInfo.nVipLevel ~= tCurUserInfo.nVipLevel then
		self:ShowPlayerBaseInfo(8)
		self.tTmpUserInfo.nVipLevel = tCurUserInfo.nVipLevel
	end
	------------------------------------------------------------------
	
	-- 顶部通知处理循环.
	self:updateNotice(dt)

end

-- 根据状态判断顶部用户信息是否显示
function GameLgLobby:JudgeShowUserInfoByStatus()
	local nSecStatus = GAME_STATUS.GetStatus(2)
	local nThdStatus = GAME_STATUS.GetStatus(3)
	local bIsShow = false;
	-- 如果是竞技
	if nSecStatus == STATUS_2.LB_RANKWAR then
		if nThdStatus == STATUS_3.RK_CHOOSE then
			bIsShow = true;
		end
	-- 如果是基地
	elseif nSecStatus == STATUS_2.LB_BASE then
		bIsShow = true;

	-- 如果是剧情
	elseif nSecStatus == STATUS_2.LB_STORY then
		if nThdStatus ~= STATUS_3.FB_LIMITED and nThdStatus ~= STATUS_3.FB_CHOOSE then
			bIsShow = true;
		end
	end

	self.pCtuiUser:setVisible(bIsShow);
end

-- 游戏状态发生了改变,2级状态开始
function GameLgLobby:gameStatusHadChange(nStatusType, nOldStatus, nNewStatus)
	if nStatusType == 2 then
		if self.nCurShowType ~= nNewStatus then
			self:ChangeLobbyView(nNewStatus, true)			
		end
	end
	-- 判断是否要显示顶部信息
	self:JudgeShowUserInfoByStatus()
end

-- 进入赛车显示界面
function GameLgLobby:ShowFightView(tFightInfo)
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.StartShowLocalRace, tFightInfo)
end

-- 显示竞技里面的排位赛或者掠夺
function GameLgLobby:ForceToShowRankView( bIsLoot )
	if bIsLoot then
		-- 显示掠夺
		self:GetRankLogic():ShowPlunderView()
	else
		-- 显示排位
		self:GetRankLogic():ShowRankWarView()
	end
	self:JudgeShowUserInfoByStatus()
end

-- 根据对应的idx切换到阵容界面
function GameLgLobby:SwitchToTeamCarByIdx( nIdx )
	-- 创建逻辑
	if self.pLbLogicTeam == nil then
		-- 删除久层
		self:RemoveOldLobbyView(GAME_STATUS.GetStatus(2), STATUS_2.LB_TEAM)

		local LogicTeam = requires(ROOT_PATH, "app/GameLg/Lobby/GameTeam/LogicTeam")
		self.pLbLogicTeam = LogicTeam.new()
		self.pLbLogicTeam:InitInfo(self)
		self.pLbLogicTeam:ChangeTeamIdxAfterInit( nIdx )
		self:addChild(self.pLbLogicTeam, GLobbyZOrder.OtherInfo)

		-- 如果原本就不在阵容界面中，则开始处理
		if GAME_STATUS.GetStatus(2) ~= STATUS_2.LB_TEAM then
			-- 获取当前按钮
			local pBtnCur = self.pCtuiLayer:GetNodeByTag(GLobbyTabTag.BtnTeam)
			-- -- 位置的移动动画
			-- local pSpMoveBot = self.pCtuiLayer:GetNodeByTag(GLobbyTabTag.SpSelBottom)
			-- local tMovePos = ccp(pBtnCur:getPositionX(), pSpMoveBot:getPositionY())
			-- pSpMoveBot:stopAllActions()
			-- pSpMoveBot:runAction(CCMoveTo:create(0.1, tMovePos))
		end
		-- 大厅状态处理
		GAME_STATUS.ChangeOneStatus(2, STATUS_2.LB_TEAM)
		-- 标签效果
		self:ShowTabSelEffect(GLobbyTabTag.BtnTeam)
	else
		self.pLbLogicTeam:ChangeTeamIdxAfterInit( nIdx )
	end

	-- 新的类型
	self.nCurShowType = STATUS_2.LB_TEAM;
end

-- 更新显示所有信息,1是等级数，2是经验，3是金币，4是钻石，5是精力，6是体力，7是战斗力，8是vip， nil是所有都修改
function GameLgLobby:ShowPlayerBaseInfo(nType, bChangeLen)
	local tCurUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()

	if bChangeLen == nil then
		bChangeLen = true;
	end

	-- 修改等级数
	if nType == nil or nType == 1 then
		-- local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbLevelNum)
		-- tolua.cast(pLbTmp, "CTWordImg")
		-- pLbTmp:setString("" .. tCurUserInfo.nLv)

		self.pLbUserLv:setString("" .. tCurUserInfo.nLv)
		if tCurUserInfo.nLv < 10 then
			self.pLbUserLv:setScale(1.2)
		elseif tCurUserInfo.nLv < 100 then
			self.pLbUserLv:setScale(1)
		else
			self.pLbUserLv:setScale(0.8)
		end
	end

	-- SpExpCircle 	= 7,	-- 经验圈
	-- SpEnergyLine	= 16,	-- 精力长度
	-- SpActivityLine	= 5,	-- 体力长度

	-- 修改经验
	if nType == nil or nType == 2 then
		local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserExp)
		tolua.cast(pLbTmp, "CCLabelTTF")
		pLbTmp:setString(tCurUserInfo.nCurExp .. "/" .. tCurUserInfo.nLevelExp)

		-- local pSpPanTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.SpExpCircle)
		-- tolua.cast(pSpPanTmp, "CCProgressTimer")
		-- local fPer = tCurUserInfo.nCurExp*100.0/tCurUserInfo.nLevelExp
		-- local fPer = tCurUserInfo.nCurExp/tCurUserInfo.nLevelExp
		-- if fPer > 1.0 then
		-- 	fPer = 1.0
		-- end
		-- fPer = (fPer * (235.0/360) + (125.0/360) ) * 100.0
		-- pSpPanTmp:setPercentage( toint(fPer) )
		
	end

	-- 修改金币
	if nType == nil or nType == 3 then
		local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserGold)
		tolua.cast(pLbTmp, "CCLabelTTF")
		pLbTmp:setString( Public_GetNumShowType(tCurUserInfo.nGoldNum, NumberShowType.ShortWan) )
	end

	-- 修改钻石
	if nType == nil or nType == 4 then
		local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserDiamond)
		tolua.cast(pLbTmp, "CCLabelTTF")
		pLbTmp:setString( Public_GetNumShowType(tCurUserInfo.nDiamodNum, NumberShowType.ShortWan) )
	end

	-- 修改精力
	if nType == nil or nType == 5 then
		local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserEnergy)
		tolua.cast(pLbTmp, "CCLabelTTF")
		-- 最大值达到，则不倒计时
		if tCurUserInfo.nCurEnergy >= tCurUserInfo.nMaxEnergy then
			bChangeLen = true
		end
		-- 精力比例
		if bChangeLen == true then
			local fRate = tCurUserInfo.nCurEnergy * 1.0 / tCurUserInfo.nMaxEnergy
			if fRate > 1.0 then
				fRate = 1.0
			end
			local pSpTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.SpEnergyLine)
			tolua.cast(pSpTmp, "CCSprite")
			local pTexSize = pSpTmp:getTexture():getContentSize()
			pSpTmp:setTextureRect(CCRectMake((pTexSize.width-34)*(1-fRate), 0, 
												(pTexSize.width-34)*fRate+34, pTexSize.height))

			if self.fShowCdTime < 7.0 then
				pLbTmp:setString(tCurUserInfo.nCurEnergy .. "/" .. tCurUserInfo.nMaxEnergy)
			end
		else
			pLbTmp:setString( Public_ChangeToTimeFormat( DATAMGR:GetUserInfoMgr():GetResNextEnergyTime(), true) )
		end
	end

	-- 修改体力
	if nType == nil or nType == 6 then
		local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserActivity)
		tolua.cast(pLbTmp, "CCLabelTTF")
		-- 最大值达到，则不倒计时
		if tCurUserInfo.nCurActivity >= tCurUserInfo.nMaxAcitivity then
			bChangeLen = true
		end
		-- 体力比例
		if bChangeLen == true then
			local fRate = tCurUserInfo.nCurActivity * 1.0 / tCurUserInfo.nMaxAcitivity
			if fRate > 1.0 then
				fRate = 1.0
			end
			local pSpTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.SpActivityLine)
			tolua.cast(pSpTmp, "CCSprite")
			local pTexSize = pSpTmp:getTexture():getContentSize()
			pSpTmp:setTextureRect(CCRectMake((pTexSize.width-34)*(1-fRate), 0, 
												(pTexSize.width-34)*fRate+34, pTexSize.height))

			if self.fShowCdTime < 7.0 then
				pLbTmp:setString(tCurUserInfo.nCurActivity .. "/" .. tCurUserInfo.nMaxAcitivity)
			end
		else
			pLbTmp:setString( Public_ChangeToTimeFormat( DATAMGR:GetUserInfoMgr():GetResNextActivityTime(), true) )
		end
	end
	-- 修改战斗力
	if nType == nil or nType == 7 then
		local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbBattleValue)
		tolua.cast(pLbTmp, "CTImgLabel")
		pLbTmp:setString("" .. self.nTotalBatValue)
	end

	-- vip修改
	if nType == nil or nType == 8 then
		local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserVipLv)
		tolua.cast(pLbTmp, "CTImgLabel")

		local nVipLevel = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nVipLevel
		if nVipLevel == nil then
			nVipLevel = 0;
		end

		pLbTmp:setString("" .. nVipLevel)
	end

	-- 如果不是特定类型，则统一修改的
	if nType == nil then
		-- 昵称
		local pLbNick = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbNickName)
		tolua.cast(pLbNick, "CCLabelTTF")
		pLbNick:setString(tCurUserInfo.strNick)
	end
end

-- 显示建筑物获得资源的效果，bIsCoin表示是否是货币
function GameLgLobby:ShowGetResourceEffect(tStartPos, nType, bIsCoin)

	local tTargetPos

	local pSpTmp
	if bIsCoin == true then
		pSpTmp = CCSprite:create("Images/JinBi3.png")
		pSpTmp:setScale( ctipwidth(1) )
		tTargetPos = self.pCtuiUser:GetNodeAbsPosition(GLobbyTabTag.LbUserGold)
		-- 播放音效
		SOUNDMGR:PlayEffBuildingGetGold()
	else
		-- 分成精力和体力
		local pImgTex
		if nType == PlayValueType.Energy then
			pImgTex = DATAMGR:GetItemInfoMgr():GetItemIconNameByID(ITEM_ENERGY_2_ID)

			tTargetPos = self.pCtuiUser:GetNodeAbsPosition(GLobbyTabTag.LbUserEnergy)
		else
			pImgTex = DATAMGR:GetItemInfoMgr():GetItemIconNameByID(ITEM_ACTIVITY_2_ID)

			tTargetPos = self.pCtuiUser:GetNodeAbsPosition(GLobbyTabTag.LbUserActivity)
		end
		pSpTmp = CCSprite:createWithSpriteFrame(pImgTex)
		pSpTmp:setScale( ctipwidth(0.5) )

		-- 播放音效
		SOUNDMGR:PlayEffBuildingGetEngeryOrActivity()
	end

	local pLyResAdd = CCLayer:create()
	self:addChild(pLyResAdd, GLobbyZOrder.OtherInfo)

	local nShowCnt = 6;

	for i=1,nShowCnt do
		local pSpRes = CCSprite:createWithTexture( pSpTmp:getTexture(), pSpTmp:getTextureRect() )
		pSpRes:setScale( pSpTmp:getScale() )
		pSpRes:setPosition(tStartPos)
		pLyResAdd:addChild(pSpRes, -i)
		pSpRes:setVisible(false)


		local bezier2 = ccBezierConfig()
	    bezier2.controlPoint_1 = ccp(tStartPos.x + ctipwidth(20), tStartPos.y - ctipheight(50))
	    bezier2.controlPoint_2 = ccp(tStartPos.x - ctipwidth(20), tStartPos.y - ctipheight(50))
	    bezier2.endPosition = tTargetPos

	    local bezierTo1 = CCBezierTo:create(1.0, bezier2)

	    

	    bezierTo1 = CCEaseIn:create(bezierTo1, 1.8)

		local pArrAct = CCArray:create()
		pArrAct:addObject( CCDelayTime:create(0.16*i) )
		pArrAct:addObject( CCCallFunc:create(function()
			pSpRes:setVisible(true)
			if bIsCoin == true then
				-- 翻转的动画
				local pSpTmp1 = CCSprite:create("Images/JinBi3.png")
				local pArrAnim = CCArray:create();
				local nStartIdx = GetRandom(1, 4);
				for i=1,4 do
					pArrAnim:addObject( CCSpriteFrame:create("Images/JinBi"..((i+nStartIdx)%4+1)..".png", pSpTmp1:getTextureRect() ) )
				end
				local pAnim = CCAnimate:create( CCAnimation:createWithSpriteFrames(pArrAnim, 0.1) )
				pSpRes:runAction( CCRepeatForever:create(pAnim) )

				-- 
				local pSpYan = CCSprite:create("Images/JinBiGuang.png")
				pSpYan:setAnchorPoint( ccp(0.29, 0.5) )
				pSpYan:setPosition( ctspccp(pSpRes, 0.5, 0.5) )
				pSpYan:setRotation( GetP2PRotation(tTargetPos, pSpRes:getPositionInCCPoint() ) +90 )
 
				pSpYan:setVisible(false)
				pSpRes:addChild(pSpYan)
				pSpYan:runAction(CCSequence:createWithTwoActions(CCDelayTime:create(0.6), CCShow:create()))
			else
				pSpRes:setRotation( GetFloatRandom(0, 360) )
				pSpRes:runAction( CCRepeatForever:create( CCRotateBy:create(1, 90) ) )
				pSpRes:runAction( CCScaleTo:create(1, ctipwidth(0.8)) )
		    end
		end) )
		pArrAct:addObject( bezierTo1 )
		if i==nShowCnt then
			pArrAct:addObject( CCCallFunc:create(function()
				pLyResAdd:removeFromParent()

				if bIsCoin == true then
					local pCoinNode = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserGold)
					pCoinNode:runAction(CCSequence:createWithTwoActions(CCScaleTo:create(0.1, 1.2),
																		CCScaleTo:create(0.1, 1)))
				end

			end) )
		else
			pArrAct:addObject( CCRemoveSelf:create(true) )
		end
		pSpRes:runAction( CCSequence:create(pArrAct) )
	end
end

-- mark: ============== Tab切换 ================
-- 根据状态来删除视图，bIsForce为true表明，无论是否状态相同，都会强制删除
function GameLgLobby:RemoveOldLobbyView(nOldStatus, nNewStatus, bIsForce)
	-- bIsForce = bIsForce or false
	-- -- 如果状态相同，则不处理
	-- if bIsForce == false and nOldStatus == nNewStatus then
	-- 	return
	-- end

	-- 基地
	-- if nOldStatus == STATUS_2.LB_BASE then
		if self.pLbLogicBase then
			self.pLbLogicBase:BeforeReleaseLayer()
			self.pLbLogicBase:removeFromParent()
			self.pLbLogicBase = nil
		end

		-- 队伍阵容
	-- elseif nOldStatus == STATUS_2.LB_TEAM then
		if self.pLbLogicTeam then
			self.pLbLogicTeam:BeforeReleaseLayer()
			self.pLbLogicTeam:removeFromParent()
			self.pLbLogicTeam = nil
		end

		-- 剧情
	-- elseif nOldStatus == STATUS_2.LB_STORY then
		if self.pLbLogicFuBen then
			self.pLbLogicFuBen:BeforeReleaseLayer()
			self.pLbLogicFuBen:removeFromParent()
			self.pLbLogicFuBen = nil
		end

		-- 排位赛
	-- elseif nOldStatus == STATUS_2.LB_RANKWAR then
		if self.pLbLogicRank then
			self.pLbLogicRank:BeforeReleaseLayer()
			self.pLbLogicRank:removeFromParent()
			self.pLbLogicRank = nil
		end

		-- 宠物等
	-- elseif nOldStatus == STATUS_2.LB_GAMEITEM then
		if self.pLbLogicPet then
			self.pLbLogicPet:BeforeReleaseLayer()
			self.pLbLogicPet:removeFromParent()
			self.pLbLogicPet = nil
		end

		-- 联盟
	-- elseif nOldStatus == STATUS_2.LB_UNION then
		if self.pLbLogicUnion then
			self.pLbLogicUnion:BeforeReleaseLayer()
			self.pLbLogicUnion:removeFromParent()
			self.pLbLogicUnion = nil
		end
	-- end
end

-- 修改界面，nNewStatus 是要显示的界面，bIsForce表示是否要强制修改，默认是false
function GameLgLobby:ChangeLobbyView(nNewStatus, bIsForce)
	bIsForce = bIsForce or false

	local bIsChange = true
	-- 如果状态相同，且不想强制改变，则不处理
	if GAME_STATUS.GetStatus(2) == nNewStatus and bIsForce == false then
		bIsChange = false
	else
		self:RemoveOldLobbyView(GAME_STATUS.GetStatus(2), nNewStatus)
		-- 修改状态
		GAME_STATUS.ChangeOneStatus(2, nNewStatus, false)
	end

	local nBtnTag = 0
	-- 基地
	if nNewStatus == STATUS_2.LB_BASE then
		nBtnTag = GLobbyTabTag.BtnBase
		-- 创建逻辑
		if self.pLbLogicBase == nil then
			local LogicGameBase = requires(ROOT_PATH, "app/GameLg/Lobby/GameBase/LogicGameBase")
			self.pLbLogicBase = LogicGameBase.new()
			self.pLbLogicBase:InitInfo(self)
			self:addChild(self.pLbLogicBase, GLobbyZOrder.OtherInfo)


			-- 获取精力活动状态。
			self:SendGetHuoDongStatusList()
		end

		-- 队伍阵容
	elseif nNewStatus == STATUS_2.LB_TEAM then
		nBtnTag = GLobbyTabTag.BtnTeam
		-- 切换到显示第一个
		self:SwitchToTeamCarByIdx(0)

		-- 剧情
	elseif nNewStatus == STATUS_2.LB_STORY then
		nBtnTag = GLobbyTabTag.BtnStory
		-- 创建逻辑
		if self.pLbLogicFuBen == nil then
			local LogicFuBen = requires(ROOT_PATH, "app/GameLg/Lobby/GameStory/LogicFuBen")
			self.pLbLogicFuBen = LogicFuBen.new()
			self.pLbLogicFuBen:InitInfo(self)
			self:addChild(self.pLbLogicFuBen, GLobbyZOrder.OtherInfo)
		else
			self.pLbLogicFuBen:ShowChooseView(false)
		end

		-- 排位赛
	elseif nNewStatus == STATUS_2.LB_RANKWAR then
		nBtnTag = GLobbyTabTag.BtnTopFight
		-- 创建逻辑
		if self.pLbLogicRank == nil then
			local LogicRanking = requires(ROOT_PATH, "app/GameLg/Lobby/GameRank/LogicRanking")
			self.pLbLogicRank = LogicRanking.new()
			self.pLbLogicRank:InitInfo(self)
			self:addChild(self.pLbLogicRank, GLobbyZOrder.OtherInfo)
		end
		-- 显示选择界面
		self.pLbLogicRank:ShowChooseView()

		-- 宠物等
	elseif nNewStatus == STATUS_2.LB_GAMEITEM then
		nBtnTag = GLobbyTabTag.BtnPet
		bIsChange = false

		-- 联盟
	elseif nNewStatus == STATUS_2.LB_UNION then
		nBtnTag = GLobbyTabTag.BtnUnion
		bIsChange = false

		-- 其他错误
	else
		return
	end

	if bIsChange == false then
		return
	end
	-- 新的类型
	self.nCurShowType = nNewStatus;

	self:ShowTabSelEffect(nBtnTag)

	-- -- 获取当前按钮
	-- local pBtnCur = self.pCtuiLayer:GetNodeByTag(nBtnTag)
	-- -- 位置的移动动画
	-- local pSpMoveBot = self.pCtuiLayer:GetNodeByTag(GLobbyTabTag.SpSelBottom)
	-- local tOrigPos = pBtnCur:getParent():convertToWorldSpace(pBtnCur:getPositionInCCPoint())
	-- tOrigPos = self.pCtuiLayer:convertToNodeSpace(tOrigPos)
	-- local tMovePos = ccp(tOrigPos.x, pSpMoveBot:getPositionY())
	-- pSpMoveBot:stopAllActions()
	-- if bIsForce == true then
	-- 	pSpMoveBot:setPosition(tMovePos)
	-- else
	-- 	pSpMoveBot:runAction(CCMoveTo:create(0.1, tMovePos))
	-- end
end

-- 显示切换的标签
function GameLgLobby:ShowTabSelEffect(nTag)
	local nAllTag = {
		GLobbyTabTag.BtnBase, -- 基地
		GLobbyTabTag.BtnTeam, -- 阵容
		GLobbyTabTag.BtnStory, -- 副本
		GLobbyTabTag.BtnTopFight, -- 竞技
		GLobbyTabTag.BtnPet, -- 宠物
		-- GLobbyTabTag.BtnUnion, -- 工会
	}

	local strNorImages = {
		"Images/GX-003.png",
		"Images/GX-004.png",
		"Images/GX-005.png",
		"Images/GX-006.png",
		"Images/GX-007.png",
	};
	local strSelImages = {
		"Images/GX-003-2.png",
		"Images/GX-004-2.png",
		"Images/GX-005-2.png",
		"Images/GX-006-2.png",
		"Images/GX-007-2.png",
	};

	-- local nIdx = 1
	for i,nOneTag in ipairs(nAllTag) do
		local pOneBtn = self.pCtuiLayer:GetNodeByTag(nOneTag)
		tolua.cast(pOneBtn, "CTMenuItemSprite")
		pOneBtn:setCascadeColorEnabled(true)
		if nOneTag == nTag then
			local pTex = CCTextureCache:sharedTextureCache():addImage(strSelImages[i])
			if pOneBtn:GetNormalSprite() ~= pTex then
				pOneBtn:GetNormalSprite():setTexture( pTex );
				pOneBtn:GetSelectedSprite():setTexture( pTex );
			end
			
			-- pOneBtn:GetNormalSprite():setColor(ccc3(255, 255, 255))
			-- local pChild = pOneBtn:GetNormalSprite():getChildren()
			-- for i=1,pChild:count() do
			-- 	local pOne = pChild:objectAtIndex(i-1)
			-- 	tolua.cast(pOne, "CCNode")
			-- 	pOne:setColor(ccc3(0, 0, 0))
			-- end
			-- nIdx = i
		else
			local pTex = CCTextureCache:sharedTextureCache():addImage(strNorImages[i])
			if pOneBtn:GetNormalSprite() ~= pTex then
				pOneBtn:GetNormalSprite():setTexture( pTex );
				pOneBtn:GetSelectedSprite():setTexture( pTex );
			end
			-- local pChild = pOneBtn:GetNormalSprite():getChildren()
			-- for i=1,pChild:count() do
			-- 	local pOne = pChild:objectAtIndex(i-1)
			-- 	tolua.cast(pOne, "CCNode")
			-- 	pOne:setColor(ccc3(255, 255, 255))
			-- end
		end
	end

	-- local nBotTags = {
	-- 	GLobbyTabTag.SpBotBase,	-- 基地按钮底部标签
	-- 	GLobbyTabTag.SpBotTeam,	-- 阵容按钮底部标签
	-- 	GLobbyTabTag.SpBotStory,	-- 副本按钮底部标签
	-- 	GLobbyTabTag.SpBotTopFight,	-- 竞技按钮底部标签
	-- 	GLobbyTabTag.SpBotPet,	-- 宠物按钮底部标签
	-- 	-- SpBotUnion = 39,	-- 工会按钮底部标签
	-- }
	-- for i,nOntBot in ipairs(nBotTags) do
	-- 	local pOneSpBot = self.pCtuiLayer:GetNodeByTag(nOntBot)
	-- 	pOneSpBot:setCascadeColorEnabled(true)
	-- 	pOneSpBot:stopAllActions()
	-- 	if i == nIdx then
	-- 		pOneSpBot:runAction(CCMoveTo:create(0.1, ccp(pOneSpBot:getPositionX(), 61)))
	-- 		pOneSpBot:setColor(ccc3(255, 255, 255))
	-- 	else
	-- 		pOneSpBot:runAction(CCMoveTo:create(0.1, ccp(pOneSpBot:getPositionX(), 49)))
	-- 		pOneSpBot:setColor(ccc3(160, 160, 160))
	-- 	end
	-- end
end

-- mark: ============== 按钮的回调 ================
-- 按钮回调
function GameLgLobby:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)
	print("button call back, tag: ", nTag)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()

	-- 点击了基地
	if nTag == GLobbyTabTag.BtnBase then
		GameViewMgr:shareMgr():RemoveAllViewes()
		self:ChangeLobbyView(STATUS_2.LB_BASE)

		-- 进入下一步教学，如果没有教学，内部不处理
		TEACHMGR:ShowSwichToNextStep();
	-- 点击阵容
	elseif nTag == GLobbyTabTag.BtnTeam then
		GameViewMgr:shareMgr():RemoveAllViewes()
		self:ChangeLobbyView(STATUS_2.LB_TEAM)

		-- 进入下一步教学，如果没有教学，内部不处理
		TEACHMGR:ShowSwichToNextStep();

	-- 点击了副本
	elseif nTag == GLobbyTabTag.BtnStory then
		GameViewMgr:shareMgr():RemoveAllViewes()
		GAME_STATUS.ChangeOneStatus(3, STATUS_3.FB_CHOOSE);
		self:ChangeLobbyView(STATUS_2.LB_STORY)
		-- 进入下一步教学，如果没有教学，内部不处理
		TEACHMGR:ShowSwichToNextStep();

	-- 点击了竞技
	elseif nTag == GLobbyTabTag.BtnTopFight then
		GameViewMgr:shareMgr():RemoveAllViewes()
		self:ChangeLobbyView(STATUS_2.LB_RANKWAR)

		-- 进入下一步教学，如果没有教学，内部不处理
		TEACHMGR:ShowSwichToNextStep();

	-- 点击了宠物
	elseif nTag == GLobbyTabTag.BtnPet then
		DIALOGMGR:ShowToast("功能未开放，敬请期待!!")
		-- GameViewMgr:shareMgr():RemoveAllViewes()
		-- self:ChangeLobbyView(STATUS_2.LB_GAMEITEM)

	-- 点击了工会
	elseif nTag == GLobbyTabTag.BtnUnion then
		DIALOGMGR:ShowToast("功能未开放，敬请期待!!")
		-- GameViewMgr:shareMgr():RemoveAllViewes()
		-- self:ChangeLobbyView(STATUS_2.LB_UNION)
		-- GAME_STATUS.ChageAllStatus(STATUS_MAIN.LOGIN, STATUS_2.LG_STARTGAME)

	end
end

-- 对话框的按钮回调信息
function GameLgLobby:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
	-- 如果是金币不足
	if nBtnTag == DLGTYPE.LowGold then
		if nBtnIdx == 1 then
			GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.GameItemShop)
		end
		return;
	-- 如果是钻石不足
	elseif nBtnTag == DLGTYPE.LowDiamond then
		if nBtnIdx == 1 then
			GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.ChongZhi)
		end
		return;

	-- 如果是用货币购买东西
	elseif nBtnTag == DLGTYPE.UseMoneyBySomeThing then
		if nBtnIdx == 1 then
			self:SendBuyGoodInfo(tData[2], 1)
		end
		return;

	-- 如果是直接开启宝箱
	elseif nBtnTag == DLGTYPE.OpenBoxByDiamond then
		if nBtnIdx == 1 then
			-- 获取开箱子的视图逻辑，然后发送开箱子信息
			local pBoxView = GameViewMgr:shareMgr():GetViewByType(GameViewType.OpenBoxView)
			if pBoxView ~= nil then
				pBoxView:SendOpenBoxByDiamond(tData[2])
			end
		end
		return;

	-- 删除宿敌并增加为好友
	elseif nBtnTag == DLGTYPE.RemoveEnemyAndAddFriend then
		if nBtnIdx == 1 then
			self:SendRemoveEnemy(tData[2]);
			self:SendReqAddFriend(tData[2]);
		end
	-- 删除宿敌
	elseif nBtnTag == DLGTYPE.RemoveEnemy then
		if nBtnIdx == 1 then
			self:SendRemoveEnemy(tData[2]);
		end
		return;

	-- 从好友里面删除，并加为宿敌
	elseif nBtnTag == DLGTYPE.RemoveFriendAndAddEnemy then
		if nBtnIdx == 1 then
			self:SendAddToEnemy(tData[2]);
		end
		return;

	-- 删除好友
	elseif nBtnTag == DLGTYPE.RemoveDelFriend then
		if nBtnIdx == 1 then
			self:SendRemoveFriend(tData[2]);
		end
		return;
	-- 背包已满
	elseif nBtnTag == DLGTYPE.BagIsFull then
		-- 点击处理，则打开背包
		if nBtnIdx == 1 then
			GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.UserBag)
		end
		return;
	-- 判断等级比较高
	elseif nBtnTag == DLGTYPE.HeChengLevel then
		if nBtnIdx == 1 then
			-- 确定合成
			local pHeChengView = GameViewMgr:shareMgr():GetViewByType(GameViewType.HeChegnCar)
			if pHeChengView ~= nil then
			 	pHeChengView:HeChengProce()
			end
		end
		return;
	-- 背包不足
	elseif nBtnTag == DLGTYPE.BagHaveNoCard then
		if nBtnIdx == 1 then
			GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.GameItemShop)
		end
		return;
	-- 高级卡牌是否要被吃掉
	elseif nBtnTag == DLGTYPE.SeniorCardNorEat then
		if nBtnIdx == 1 then
			self:SendGetExpFromCards(tData[1], tData[2], tData[3])
		end
		return;
	-- 高级卡牌是否要被卖掉
	elseif nBtnTag == DLGTYPE.SeniorCardNorSell then
		if nBtnIdx == 1 then
			self:SendSellCards(tData[2], tData[1])
		end
		return;
	-- 充值成为vip
	elseif nBtnTag == DLGTYPE.NotEnoughUseTimes then
		if nBtnIdx == 1 then
			GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.ChongZhi)
		end
		return;
	end

	-------- 其他子逻辑的触控管理 --------
	local nSecStatus = GAME_STATUS.GetStatus(2)
	-- 基地
	if nSecStatus == STATUS_2.LB_BASE then
		if self.pLbLogicBase then
			self.pLbLogicBase:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
		end

		-- 队伍阵容
	elseif nSecStatus == STATUS_2.LB_TEAM then
		if self.pLbLogicTeam then
			self.pLbLogicTeam:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
		end

		-- 剧情
	elseif nSecStatus == STATUS_2.LB_STORY then
		if self.pLbLogicFuBen then
			self.pLbLogicFuBen:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
		end

		-- 排位赛
	elseif nSecStatus == STATUS_2.LB_RANKWAR then
		if self.pLbLogicRank then
			self.pLbLogicRank:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
		end

		-- 宠物等
	elseif nSecStatus == STATUS_2.LB_GAMEITEM then
		if self.pLbLogicPet then
			-- self.pLbLogicPet:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
		end

		-- 联盟
	elseif nSecStatus == STATUS_2.LB_UNION then
		if self.pLbLogicUnion then
			-- self.pLbLogicUnion:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
		end
	end
	-----------------------------------------
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function GameLgLobby:LgLobbyTouchBegan(tLocation, nCount)
	-- print("beganx:" .. tLocation.x .. " y: " .. tLocation.y .. " c: " .. nCount)

	self.nTouchUserType = 0;-- 0是没有点击，1是点击中，2是滑动
	-- 如果在显示弹框中，则不给其他操作了
	if GameViewMgr:shareMgr():IsCurShowPopView() == false then
		if self.pCtuiLayer:CTUITouchBegan(tLocation) == true then
			return;
		end
	end

	-- 如果拥有显示视图，则触控在显示视图里面
	if GameViewMgr:shareMgr():IsHadShowView() == true then
		GameViewMgr:shareMgr():ViewMgrTouchBegan(tLocation, nCount)
		return
	end

	
	-- 获取触控区域
	if self.tUserRect:containsPoint(tLocation) == true then

		if self.tChongZhiRect:containsPoint(tLocation) == true then 
			-- 这里是显示充值的标记
			self.IsChongZhi = true
		end
		self.nTouchUserType = 1; -- 0是没有点击，1是点击中，2是滑动
	end
 	

	-------- 其他子逻辑的触控管理 --------
	local nSecStatus = GAME_STATUS.GetStatus(2)
	-- 基地
	if nSecStatus == STATUS_2.LB_BASE then
		if self.pLbLogicBase then
			self.pLbLogicBase:LgBaseTouchBegan(tLocation, nCount)
		end

		-- 队伍阵容
	elseif nSecStatus == STATUS_2.LB_TEAM then
		if self.pLbLogicTeam then
			self.pLbLogicTeam:LgTeamTouchBegan(tLocation, nCount)
		end

		-- 剧情
	elseif nSecStatus == STATUS_2.LB_STORY then
		if self.pLbLogicFuBen then
			self.pLbLogicFuBen:LgFuBenTouchBegan(tLocation, nCount)
		end

		-- 排位赛
	elseif nSecStatus == STATUS_2.LB_RANKWAR then
		if self.pLbLogicRank then
			self.pLbLogicRank:LgRankTouchBegan(tLocation, nCount)
		end

		-- 宠物等
	elseif nSecStatus == STATUS_2.LB_GAMEITEM then
		if self.pLbLogicPet then
			-- self.pLbLogicPet:LgBaseTouchBegan(tLocation, nCount)
		end

		-- 联盟
	elseif nSecStatus == STATUS_2.LB_UNION then
		if self.pLbLogicUnion then
			-- self.pLbLogicUnion:LgBaseTouchBegan(tLocation, nCount)
		end
	end
	-----------------------------------------
end

-- 触控移动
function GameLgLobby:LgLobbyTouchMoved(tLocation, tPrevPos, tBeganPos)
	-- print("MoveLx:" .. tLocation.x .. " Ly: " .. tLocation.y .. " Px: " .. tPrevPos.x .. " Py: " .. tPrevPos.y)
	-- 如果在显示弹框中，则不给其他操作了
	if GameViewMgr:shareMgr():IsCurShowPopView() == false then
		self.pCtuiLayer:CTUITouchMoved(tLocation)
		-- return;
	end


	-- 如果拥有显示视图，则触控在显示视图里面
	if GameViewMgr:shareMgr():IsHadShowView() == true then
		GameViewMgr:shareMgr():ViewMgrTouchMoved(tLocation, tPrevPos, tBeganPos)
		return
	end

	-- 移动，则去掉
	if ccpDistance(tLocation, tBeganPos) > 5 then
		self.nTouchUserType = 2;-- 0是没有点击，1是点击中，2是滑动
	end
	

	-------- 其他子逻辑的触控管理 --------
	local nSecStatus = GAME_STATUS.GetStatus(2)
	-- 基地
	if nSecStatus == STATUS_2.LB_BASE then
		if self.pLbLogicBase then
			self.pLbLogicBase:LgBaseTouchMoved(tLocation, tPrevPos, tBeganPos)
		end

		-- 队伍阵容
	elseif nSecStatus == STATUS_2.LB_TEAM then
		if self.pLbLogicTeam then
			self.pLbLogicTeam:LgTeamTouchMoved(tLocation, tPrevPos, tBeganPos)
		end

		-- 剧情
	elseif nSecStatus == STATUS_2.LB_STORY then
		if self.pLbLogicFuBen then
			self.pLbLogicFuBen:LgFuBenTouchMoved(tLocation, tPrevPos, tBeganPos)
		end

		-- 排位赛
	elseif nSecStatus == STATUS_2.LB_RANKWAR then
		if self.pLbLogicRank then
			self.pLbLogicRank:LgRankTouchMoved(tLocation, tPrevPos, tBeganPos)
		end

		-- 宠物等
	elseif nSecStatus == STATUS_2.LB_GAMEITEM then
		if self.pLbLogicPet then
			-- self.pLbLogicPet:LgBaseTouchMoved(tLocation, tPrevPos, tBeganPos)
		end

		-- 联盟
	elseif nSecStatus == STATUS_2.LB_UNION then
		if self.pLbLogicUnion then
			-- self.pLbLogicUnion:LgBaseTouchMoved(tLocation, tPrevPos, tBeganPos)
		end
	end
	-----------------------------------------
end

-- 触控结束
function GameLgLobby:LgLobbyTouchEnded(tLocation, tPrevPos, tBeganPos)
	-- print("EndLx:" .. tLocation.x .. " Ly: " .. tLocation.y .. " Px: " .. tPrevPos.x .. " Py: " .. tPrevPos.y)
	-- 如果在显示弹框中，则不给其他操作了
	if GameViewMgr:shareMgr():IsCurShowPopView() == false then
		self.pCtuiLayer:CTUITouchEnded(tLocation)
		-- return
	end

	-- 如果拥有显示视图，则触控在显示视图里面
	if GameViewMgr:shareMgr():IsHadShowView() == true then
		GameViewMgr:shareMgr():ViewMgrTouchEnded(tLocation, tPrevPos, tBeganPos)
		return
	end

	local nSecStatus = GAME_STATUS.GetStatus(2)
	-- 0是没有点击，1是点击中，2是滑动
	if self.nTouchUserType == 1 then
		if self.pCtuiUser:isVisible() == true then
			if self.pLbLogicBase == nil or self.pLbLogicBase:IsCurShowPopMenu() == false then
				-- 点击菜单栏的金币和砖石背景 弹出充值界面
				if self.IsChongZhi == true then
					self.IsChongZhi = false
					GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.ChongZhi)
				else
					GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PlayerBaseInfo)
				end
			end
			
			return;
		end
	end
	self.nTouchUserType = 0;

	-------- 其他子逻辑的触控管理 --------
	-- 基地
	if nSecStatus == STATUS_2.LB_BASE then
		if self.pLbLogicBase then
			self.pLbLogicBase:LgBaseTouchEnded(tLocation, tPrevPos, tBeganPos)
		end

		-- 队伍阵容
	elseif nSecStatus == STATUS_2.LB_TEAM then
		if self.pLbLogicTeam then
			self.pLbLogicTeam:LgTeamTouchEnded(tLocation, tPrevPos, tBeganPos)
		end

		-- 剧情
	elseif nSecStatus == STATUS_2.LB_STORY then
		if self.pLbLogicFuBen then
			self.pLbLogicFuBen:LgFuBenTouchEnded(tLocation, tPrevPos, tBeganPos)
		end

		-- 排位赛
	elseif nSecStatus == STATUS_2.LB_RANKWAR then
		if self.pLbLogicRank then
			self.pLbLogicRank:LgRankTouchEnded(tLocation, tPrevPos, tBeganPos)
		end

		-- 宠物等
	elseif nSecStatus == STATUS_2.LB_GAMEITEM then
		if self.pLbLogicPet then
			-- self.pLbLogicPet:LgBaseTouchEnded(tLocation, tPrevPos, tBeganPos)
		end

		-- 联盟
	elseif nSecStatus == STATUS_2.LB_UNION then
		if self.pLbLogicUnion then
			-- self.pLbLogicUnion:LgBaseTouchEnded(tLocation, tPrevPos, tBeganPos)
		end
	end
	-----------------------------------------
end

-- 处理重新连接
function GameLgLobby:DealReconnect()
	local nSecStatus = GAME_STATUS.GetStatus(2)
	-- 基地
	if nSecStatus == STATUS_2.LB_BASE then
		-- 重新获取活动状态
		self:SendGetHuoDongStatusList()
	-- 排位赛|掠夺赛。
	elseif nSecStatus == STATUS_2.LB_RANKWAR then
		local nThdStatus = GAME_STATUS.GetStatus(3)
		-- 排位赛
		if nThdStatus == STATUS_3.LB_RANKWAR then
			self:SendGetRankList()
		-- 掠夺赛
		elseif nThdStatus == STATUS_3.RK_PLUNDER then
			self:SendGetLootList()
		end
	end

	self:SendGetOneBigChapterInfo()

end

-- mark: ===================== 数据发送 =============================
-- 获取章节信息
function GameLgLobby:SendGetOneBigChapterInfo()
	-- 发送登陆信息
	local tLuaData = {}
	-- tLuaData.stageId = nBigID

	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.GetTaskInfo, tLuaData)
end
-- 发送到购买BOSS关卡次数成功的处理
function GameLgLobby:SendGetTaskBossTime(nBigID, nChapterID, nTaskID)
	local tLuaData = {}
	tLuaData.stageId = nBigID
	tLuaData.pageId = nChapterID
	tLuaData.copyId = nTaskID

	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.BuyTaskBossTimes, tLuaData)
end
-- 发送购买商品
function GameLgLobby:SendBuyGoodInfo(nGoodID, nNum)
	local tLuaData = {}
	tLuaData.gid = nGoodID
	tLuaData.num = nNum

	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.BuyItemShopItem, tLuaData)
end
-- 发送添加到宿敌
function GameLgLobby:SendAddToEnemy(nUserID)
	local tJsonData = {}
	tJsonData.eid = nUserID

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.AddOneEnemy, tJsonData)
end
-- 发送添加到好友
function GameLgLobby:SendReqAddFriend(nUid)
	local tLuaData = {}
	tLuaData.fid = nUid

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.AddOneFriend, tLuaData)
end
-- 发送删除出宿敌
function GameLgLobby:SendRemoveEnemy(nUserID)
	local tJsonData = {}
	tJsonData.eid = nUserID

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.DelOneEnemy, tJsonData)
end

-- 发送删除出好友
function GameLgLobby:SendRemoveFriend(nUserID)
	local tJsonData = {}
	tJsonData.fid = nUserID

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.DelOneFriend, tJsonData)
end

-- 发送出售物品
function GameLgLobby:SendSellCards(tCardList, nType)
	-- 发送装备信息
	local tLuaData = {}
	tLuaData.arr = tCardList
	tLuaData.bagType = nType

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.SellBagItem, tLuaData)
end

-- 发送强化信息
function GameLgLobby:SendGetExpFromCards(nBagIdx, tGetExpList, nType)
	-- 发送装备信息
	local tLuaData = {}
	tLuaData.cardIdx = nBagIdx
	tLuaData.expArr = tGetExpList
	tLuaData.bagType = nType

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.StregthCard, tLuaData)
end

-- 发送获取任务状态列表信息
function GameLgLobby:SendGetHuoDongStatusList()
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.GetFreeEnergyStatus, {})

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.GetActListStatus, nil)
end

-- 发送获取临时任务的奖励
function GameLgLobby:SendGetTempActRewards(nActId, nIdx)
	local tLuaData = {}
	tLuaData.actId = nActId
	tLuaData.idx = nIdx

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.GetTempActReward, tLuaData)
end

-- 发送获取临时活动的信息。
function GameLgLobby:SendGetTempActInfo(nActId)
	local tLuaData = {}
	tLuaData.actId = nActId

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.GetTempActInfo, tLuaData)
end

-- 大转盘发送转动的信息。
function GameLgLobby:SendTurnExpendTable(nActId, nTurnNum, nFree)
	local tLuaData = {}
	tLuaData.actId = nActId
	tLuaData.turnNum = nTurnNum
	tLuaData.free = nFree

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.GetTableTurn, tLuaData)
end
-- 获取掠夺数据信息
function GameLgLobby:SendGetLootList()
	local tLuaData = {}
	-- 刷新
	tLuaData.flush = 0
	-- 不使用货币刷新。
	tLuaData.useCoin = 0
	
	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.StartGetLootList, tLuaData)
end
-- 获取排位数据信息
function GameLgLobby:SendGetRankList()
	
	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.StartGetRankList, nil)
end

-- mark: ===================== 数据接收 =============================
-- 接收任务的结果
function GameLgLobby:ReceiveDoTaskResult(tTastResult)
	-- 如果不是BOSS关卡
	if tTastResult.nFightType == 0 then
		-- 通知逻辑完成
		if self.pLbLogicFuBen then
			self.pLbLogicFuBen:ReceiveGetDoneTaskResult(tTastResult.tResult)
		end
	-- 如果是BOSS关卡
	else
		-- -- 通知逻辑完成
		-- if self.pLbLogicFuBen then
		-- 	self.pLbLogicFuBen:ReceiveGetDoneTaskResult(tTastResult.tResult)
		-- end
		-- 进入播放
		-- print2json(tTastResult)
		self:ShowFightView(tTastResult)
	end
end

-- 接收endless task基本信息.
function GameLgLobby:ReceiveGetEndlessBaseInfo(tTaskBaseInfo)
	DATA_USER:SetEndlessInfo(tTaskBaseInfo)

	local data = DATAMGR:GetUserInfoMgr():GetEndlessInfo()
	data.pTotalBuffArr = tTaskBaseInfo.totalBuffArr
	data.nNeedLv = tTaskBaseInfo.needLv

	if self:GetFuBenLogic() then
		self:GetFuBenLogic():ReceiveGetEndlessBaseInfo()
	end

	-- 通知每日任务
	local pDailyTaskView = GameViewMgr:shareMgr():GetViewByType(GameViewType.DailyTask)
	if pDailyTaskView ~= nil then
		pDailyTaskView:ReceiveGetEndlessBaseInfo()
	end
end

-- 切磋的数据处理
function GameLgLobby:ReceiveFriendQieCuoResult(nFriendID, tFightInfo)
	self:ShowFightView(tFightInfo)
end
-- 开始掠夺赛
function GameLgLobby:ReceiveGetLootFightInfo(tLootFightInfo)
	self:ShowFightView(tLootFightInfo)
end
-- 开始排位赛
function GameLgLobby:ReceiveGetRankFightInfo(tRankFightInfo)
	self:ShowFightView(tRankFightInfo)
end
-- mark: ====== 商城买卖 ======
-- 接收到购买商品的信息
function GameLgLobby:ReceiveBuyItemShopGoodsResult(nGoodID, nTotalGold, nTotalDiamond, tItemList)
	-- 重新赋值金币和钻石总数
	local tUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo();
	tUserInfo.nDiamodNum = nTotalDiamond
	tUserInfo.nGoldNum = nTotalGold
	
	-- 获取商城信息
	local pShopView = GameViewMgr:shareMgr():GetViewByType(GameViewType.GameItemShop)
	if pShopView ~= nil then
		pShopView:ReceiveBuyGoodsResult(nGoodID, 1)
	end

	-- 获取快速使用接口，更新显示数据
	local pQuickUseView = GameViewMgr:shareMgr():GetViewByType(GameViewType.QuickUserItem)
	if pQuickUseView ~= nil then
		pQuickUseView:RefreshItemShowInfo()
		pQuickUseView:ReceiveBuyItem(nGoodID, tItemList)
	end

	-- 扫荡卡数目
	local pSkipCardView = GameViewMgr:shareMgr():GetViewByType(GameViewType.SkipStoryCard)
	if pSkipCardView ~= nil then
		pSkipCardView:RefreshSkipCardNum()
	end
end

-- 接收到强化的信息
function GameLgLobby:ReceiveStregthCardInfo(nBagIdx, nNewLv, nNewExp, tRemoveCardList)

	local  tBagInfo, tItemInfo = DATAMGR:GetUserInfoMgr():GetBagItemByIdx(nBagIdx, true);
	-- 如果升级了
	if tBagInfo.nLv ~= nNewLv then
		if tItemInfo.tType.nBigType == ItemBigType.Car then
			SOUNDMGR:PlayEffCarLvUp();
		else
			SOUNDMGR:PlayEffEquipLvUp();
		end
	end
	-- 旧等级和经验值 速度  加速度  转弯
	local nOldLv = tBagInfo.nLv
	local nOldExp = tBagInfo.nExp
	local nOldTotalExp = tBagInfo.nTotalExp
	local nOldSpeed = tBagInfo.fTotalSpeed
	local nOldAsSpeed = tBagInfo.fTotalASpeed
	local nOldTurnSpeed = tBagInfo.fTotalTurn

	DATAMGR:GetUserInfoMgr():SetBagItemLevelInfo(nBagIdx, nNewLv, nNewExp)
	-- 删除一个信息列表
	self:ReceiveSellCardListInfo(tRemoveCardList)
	-- 再计算缘
	local tCurTeam = DATAMGR:GetUserInfoMgr():GetCurTeamList()
	DATAMGR:GetUserInfoMgr():CountOneTeamValue(tCurTeam, true)

	-- 更新队伍信息
	if self.pLbLogicTeam then
		self.pLbLogicTeam:RefreshTeamInfo()

		-- 隐藏教学
		TEACHMGR:ShowSwichToNextStep();
	end

	-- 队伍信息
	if self.pLbLogicBase then
		self.pLbLogicBase:RefreshTeamInfo()
	end

	-- 显示强化经验
	GameViewMgr:shareMgr():ShowChildViewWithInfo( GameViewType.ItemAddExp, nBagIdx, nOldLv, nOldExp, nOldTotalExp, nOldSpeed, nOldAsSpeed, nOldTurnSpeed)
end

-- 接收到出售信息
function GameLgLobby:ReceiveSellCardListInfo(tSellCardList)
	-- 删除一个信息列表
	for i,nBagIdx in ipairs(tSellCardList) do
		DATAMGR:GetUserInfoMgr():RemoveOneBagItem(nBagIdx)
	end

	-- 刷新背包信息
	local pBagView = GameViewMgr:shareMgr():GetViewByType(GameViewType.UserBag)
	if pBagView ~= nil then
		pBagView:RefreshShow()
	end

	-- 刷新多选界面
	local pStregthView = GameViewMgr:shareMgr():GetViewByType(GameViewType.MultiSelCard)
	if pStregthView ~= nil then
		pStregthView:RefreshShow()
	end
end

-- 接收到钥匙开启的消息, 这里的所有参数在其他地方添加修改物品了，这里的用处只是特殊显示
function GameLgLobby:ReceiveOpenBoxNormalResult(nKeyBagIdx, nLeftCount, tScaleList, tNewItemList)
	-- 物品的背包信息
	local nKeyItem = DATAMGR:GetUserInfoMgr():GetBagItemByIdx(nKeyBagIdx, false)
	local nKeyID = nKeyItem.nID
	
	------ 修改对应钥匙的数目 -----
	-- DATAMGR:GetUserInfoMgr():AddItemToBag(nKeyID, nLeftCount, nKeyBagIdx, 1, 0, true)
	local tKayBagItem = clone(BagSimpleInfo)
	tKayBagItem.nID = nKeyID
	tKayBagItem.nIdx = nKeyBagIdx
	tKayBagItem.nNum = nLeftCount
	tKayBagItem.nLv = 1
	DATAMGR:GetUserInfoMgr():AddItemToBag(tKayBagItem, true)


	-- 设置宝箱的倍数，要在最后设置
	local tUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()
	local nOldScale = 1;
	-- 铜钥匙
	if nKeyID == KEY_TONG_ID then
		nOldScale = tUserInfo.nKey1Scale
	-- 银钥匙
	elseif nKeyID == KEY_YIN_ID then
		nOldScale = tUserInfo.nKey2Scale
	-- 金钥匙
	elseif nKeyID == KEY_JIN_ID then
		nOldScale = tUserInfo.nKey3Scale
	end

	tUserInfo.nKey1Scale = tScaleList[1]
	tUserInfo.nKey2Scale = tScaleList[2]
	tUserInfo.nKey3Scale = tScaleList[3]

	-- 获取开箱子的视图逻辑，此处只用于特殊处理
	local pBoxView = GameViewMgr:shareMgr():GetViewByType(GameViewType.OpenBoxView)
	if pBoxView ~= nil then
		pBoxView:ReceiveOpenBoxNormalResult(nKeyID, nOldScale, nLeftCount, tNewItemList)
	end
end
-- 接收到用钻石开启钥匙的消息, 这里的所有参数在其他地方添加修改物品了，这里的用处只是特殊显示
function GameLgLobby:ReceiveOpenBoxByDiamondResult(nKeyID, tScaleList, tNewItemList, nTotalDiamond)
	local tUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()
	tUserInfo.nDiamodNum = nTotalDiamond

	-- 设置宝箱的倍数，要在最后设置
	local nOldScale = 1
	-- 铜钥匙
	if nKeyID == KEY_TONG_ID then
		nOldScale = tUserInfo.nKey1Scale
	-- 银钥匙
	elseif nKeyID == KEY_YIN_ID then
		nOldScale = tUserInfo.nKey2Scale
	-- 金钥匙
	elseif nKeyID == KEY_JIN_ID then
		nOldScale = tUserInfo.nKey3Scale
	end

	tUserInfo.nKey1Scale = tScaleList[1]
	tUserInfo.nKey2Scale = tScaleList[2]
	tUserInfo.nKey3Scale = tScaleList[3]

	-- 获取开箱子的视图逻辑，此处只用于特殊处理
	local pBoxView = GameViewMgr:shareMgr():GetViewByType(GameViewType.OpenBoxView)
	if pBoxView ~= nil then
		pBoxView:ReceiveOpenBoxByDiamondResult(nKeyID, nOldScale, tNewItemList)
	end
end

-- mark: ====== 收到建筑信息 ======
-- 接收到淘宝屋获取奖励列表信息。
function GameLgLobby:ReceiveGetBuildingPortCard(nGetCar, tBagList, fNewCDTime)
	-- 重置CD时间
	local tUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()
	if nGetCar == 1 then
		-- 重置车碎片cd时间。
		tUserInfo.tArrFreeCardTime[1] = fNewCDTime;
	else
		-- 重置配件碎片cd时间。
		tUserInfo.tArrFreeCardTime[2] = fNewCDTime;
	end

	-- 添加物品到背包。
	for i,tOneBagItem in ipairs(tBagList) do
		DATAMGR:GetUserInfoMgr():AddItemToBag(tOneBagItem, false)
	end
	-- 
	-- 获取淘宝屋信息
	local pShopView = GameViewMgr:shareMgr():GetViewByType(GameViewType.GetCardPort)
	if pShopView ~= nil then
		pShopView:ReceiveGetCard(tBagList)
	end
end

-- mark: ====== 收到top排名信息 ======
-- 收到世界排名信息
function GameLgLobby:ReceiveGetTopWorldRankList(pTopList, nSelfRank)
	-- 获取商城信息
	local pTopView = GameViewMgr:shareMgr():GetViewByType(GameViewType.TopList)
	if pTopView ~= nil then
		pTopView:ReceiveTopWorldRankList(pTopList, nSelfRank)
	end
end
-- 收到掠夺排名信息
function GameLgLobby:ReceiveGetTopLootRankList(pTopList, nSelfRank)
	-- 获取商城信息
	local pTopView = GameViewMgr:shareMgr():GetViewByType(GameViewType.TopList)
	if pTopView ~= nil then
		pTopView:ReceiveTopLootRankList(pTopList, nSelfRank)
	end
end

-- mark: ====== 收到活动相关 ======
-- 收到免费精力奖励
function GameLgLobby:ReceiveGetFreeEnergyReward(nEnergyNum, nStatus)
	-- 收到免费精力
	local tUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()
	tUserInfo.nCurEnergy = nEnergyNum
	DATAMGR:GetActivityMgr():GetHuoDongStatusList().nFreeEnergyStatus = nStatus;

	-- 状态
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.FreeEnergy)
	if pView ~= nil then
		pView:ReceiveGetFreeEnergy(nStatus)
	end
end
-- 收到免费精力状态
function GameLgLobby:ReceiveGetHuoDongStatusList(tStatusList)
	DATAMGR:GetActivityMgr():SetHuoDongStatusList( tStatusList )
end
-- 收到签到奖励
function GameLgLobby:ReceiveGetLogonReward(nItemId, nItemNum)

	-- 状态
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.LogonReward)
	if pView ~= nil then
		pView:ReceiveLogonReward(nItemId, nItemNum)
	end
end

-- 收到通知信息列表
function GameLgLobby:ReceiveNoticeList( pNoticeList )
	for i,oneNotice in ipairs(pNoticeList) do
		local strNotice = nil
		local noticeType = oneNotice[1]
		if noticeType == Notice_Type.System then
			strNotice = oneNotice[2]
		elseif noticeType == Notice_Type.UserGetHighStarCard then
			-- 3个参数
			local name = oneNotice[2]
			local itemId = oneNotice[3]
			local pItemInfo = DATAMGR:GetItemInfoMgr():GetItemInfoByID(itemId)
			local starLv = pItemInfo.strStar
			local itemName = pItemInfo.strName

			strNotice = clone(self.activeNoticeList[noticeType])
			strNotice = string.gsub(strNotice, "{0}", name)
			strNotice = string.gsub(strNotice, "{1}", starLv)
			strNotice = string.gsub(strNotice, "{2}", itemName)
			-- print("receive notice:", strNotice)
		elseif noticeType == Notice_Type.UserUpgrade or noticeType == Notice_Type.UserPassStoryStage then
			-- 2个参数
			strNotice = clone(self.activeNoticeList[noticeType])
			strNotice = string.gsub(strNotice, "{0}", oneNotice[2])
			strNotice = string.gsub(strNotice, "{1}", oneNotice[3])
			-- print("receive notice:", strNotice)
		end

		if strNotice ~= nil then
			-- 添加到第一个位置
			table.insert(self.tempNoticeList, 1, strNotice)
		end
	end
end

-- 接收到任务进度
function GameLgLobby:ReceiveGetTaskProgress(tProgress)
	DATAMGR:GetTaskInfoMgr():SetAllTaskProgressInfo(tProgress)
end

-- 接收获取任务信息
function GameLgLobby:ReceiveGetTaskInfo(tTaskInfo, bIsLastOne)
	-- 设置对应的章节任务信息
	DATAMGR:GetTaskInfoMgr():SetOneBigChapterFinishedInfo(tTaskInfo.nBigID, tTaskInfo)

	if self:GetFuBenLogic() then
		self:GetFuBenLogic():ReceiveGetTaskInfo(tTaskInfo, bIsLastOne)
	end
end

-- 获取得到图鉴信息
function GameLgLobby:ReceiveTuJianAllInfo(tHadGotCardList)
	DATAMGR:GetUserInfoMgr():SetCurPlayerTuJianGetCards(tHadGotCardList);
	GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.TuJianView)
end
-- 获取得到的信息
function GameLgLobby:ReceiveGetMessageReward(nMsgID, tItemList, nTotalGold, nTotalDiamond)
	local tUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo();

	tUserInfo.nGoldNum = nTotalGold;
	tUserInfo.nDiamodNum = nTotalDiamond;
	-- -- 显示获得到的金币
	-- if nAddGold > 0 then
	-- 	-- NEW_ITEM_MGR:ShowAddShowMoney(nAddGold, MoneyType.Gold)
	-- 	NEW_ITEM_MGR:ShowAddOneItem(ITEM_ID_GOLD, nAddGold)
	-- end
	-- -- 显示获得到的钻石
	-- if nAddDiamond > 0 then
	-- 	-- NEW_ITEM_MGR:ShowAddShowMoney(nAddDiamond, MoneyType.Diamond)
	-- 	NEW_ITEM_MGR:ShowAddOneItem(ITEM_ID_DIAMOND, nAddDiamond)
	-- end

	for i,tBagItem in ipairs(tItemList) do
		-- print("bagItem id: ", tBagItemId)
		DATAMGR:GetUserInfoMgr():AddItemToBag(tBagItem, false);
		-- 添加到显示中
		-- NEW_ITEM_MGR:ShowAddOneItem(v.nItemID, v.nAddCount)
		-- v.nID = v.nItemID
	end
	NEW_ITEM_MGR:ShowAddItemList(tItemList)

	-- 获取消息视图管理
	local tMsgView = GameViewMgr:shareMgr():GetViewByType(GameViewType.MessageList)
	if tMsgView ~= nil then
		tMsgView:SendDeleteOneMsg(nMsgID)
	end
end
-- 合成车收到数据
function GameLgLobby:ReceiveHeChengCarData(nSecondIdx, tListArry, nGoldNum)
	if tListArry == nil or nSecondIdx == nil then 
		return
	end 
	
	-- 更新金币值。
	DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nGoldNum = nGoldNum
	
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.HeChegnCar)
	if pView ~= nil then
		pView:ReceiveHeChengCarInfo(nSecondIdx, tListArry, nGoldNum)
	end


	-- 更新队伍信息
	if self.pLbLogicTeam then
		self.pLbLogicTeam:RefreshTeamInfo()
	end

	-- 队伍信息
	if self.pLbLogicBase then
		self.pLbLogicBase:RefreshTeamInfo()
	end
end

-- 临时活动奖励领取
function GameLgLobby:ReceiveTempActRewards(nActId, tInfo, tRewardList,pData)
	if nActId == ACT_ONLINE_BOX_ID then
		-- 在线宝箱活动。
		local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.OnlineBox)
		if pView ~= nil then
			pView:ReceiveActRewards(nActId, tInfo, tRewardList)
		end
		local nId = ITEM_ID_DIAMOND
		local nAddCount = 0
		-- 计算ID和要加的金币或者砖石
		for i, item in ipairs(pData) do 
			if i == 1 then 
				nId = item
			elseif i == 7 then 
				nAddCount = item
			end
		end
		-- 这里更新金币和砖石
		local tUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()
		-- ItemId  砖石
		if item == ITEM_ID_DIAMOND then 
			tUserInfo.nDiamodNum = tUserInfo.nDiamodNum + nAddCount
			local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserDiamond)
			tolua.cast(pLbTmp, "CCLabelTTF")
			pLbTmp:setString( Public_GetNumShowType(tUserInfo.nDiamodNum, NumberShowType.ShortWan) )
		-- 金币	
		elseif item == ITEM_ID_GOLD then 
			tUserInfo.nGoldNum = tUserInfo.nGoldNum + nAddCount
			local pLbTmp = self.pCtuiUser:GetNodeByTag(GLobbyTabTag.LbUserGold)
			tolua.cast(pLbTmp, "CCLabelTTF")
			pLbTmp:setString( Public_GetNumShowType(tUserInfo.nGoldNum, NumberShowType.ShortWan) )
		end
		
		-- 物品获得途径
		UM_SDK:GetItemListCollectInfo(tRewardList, UMGetItemType.OnlineBox)
	end
end

-- 临时活动的信息列表
function GameLgLobby:ReceiveTempActInfo(nActId, tInfo)
	-- 
	if nActId == ACT_ONLINE_BOX_ID then
		-- 在线宝箱活动。
		local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.OnlineBox)
		if pView ~= nil then
			pView:ReceiveActInfo(nActId, tInfo)
		end
	elseif nActId == ACT_EXPEND_TABLE_ID then
		-- 消费大转盘活动。
		local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.ExpendTable)
		if pView ~= nil then
			pView:ReceiveActInfo(nActId, tInfo)
		end
	end
end

-- 获取活动的状态列表
function GameLgLobby:ReceiveActStatusListInfo( tList, bGetNew )
	local tNewList = DATAMGR:GetActivityMgr():SetActStatusList( tList, bGetNew )
	if bGetNew == true then
		for i,nActId in ipairs(tNewList) do
			self:SendGetTempActInfo( nActId )
		end
	end
end

-- 获取转盘转动后的信息
function GameLgLobby:ReceiveTableTurn(nActId, nTurnNum, tIdxList, tRewardList, tActInfo, nDiamondCnt)
	-- 
	if nActId == ACT_EXPEND_TABLE_ID then
		-- 消费大转盘活动。
		local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.ExpendTable)
		if pView ~= nil then
			pView:ReceiveTableTurn(nActId, nTurnNum, tIdxList, tRewardList, tActInfo, nDiamondCnt)
		end
	end

end

-- 获取消费大转盘的其他人得高级物品列表。
function GameLgLobby:ReceiveExpendTableMsgList(tMsgList)
	-- 消费大转盘活动。
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.ExpendTable)
	if pView ~= nil then
		pView:ReceiveNewMsgList(tMsgList)
	end

end

-- 刷新一次大转盘物品。
function GameLgLobby:ReceiveRefreshTableOnce(tActInfo, nDiamondCnt)
	-- 消费大转盘活动。
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.ExpendTable)
	if pView ~= nil then
		pView:ReceiveRefreshTableOnce(tActInfo, nDiamondCnt)
	end
end

-- 招财猫接收信息 
function GameLgLobby:ReceiveLuckyCatInfor(idx, outDiamond, tActInfo, tData)
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.LuckyCat)
	if pView ~= nil then
		pView:ReceiveLuckyCatInfor(idx, outDiamond, tActInfo, tData)
	end
end

-- 招财猫接收具体信息 
function GameLgLobby:ReceiveLuckyCatDetailInfo(tData)
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.LuckyCat)
	if pView ~= nil then
		pView:ReceiveLuckyCatDetailInfo( tData )
	end
end


-- 接收购买礼包信息。
function GameLgLobby:ReceiveBuyVipGift( tItemList, tVipGiftStatus, nGoldNum, nDiamondNum )
	-- 
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.VipExchange)
	if pView ~= nil then
		pView:ReceiveBuyVipGift(tItemList, tVipGiftStatus, nGoldNum, nDiamondNum)
	end
end

-- 接收卡牌提升星级后的信息
function GameLgLobby:ReceiveCardUpgradeStarLv( tCardInfo, tUpCardInfo, nGoldNum, nDiamondNum )
	-- 
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.CardUpgradeStar)
	if pView ~= nil then
		pView:ReceiveUpgradeStarLvInfo(tCardInfo, tUpCardInfo, nGoldNum, nDiamondNum)
	end
end

-- 接收碎片合成卡牌后的信息
function GameLgLobby:ReceivePieceCombineCard( tCardInfo, tPieceInfo, nGoldNum, nDiamondNum )
	-- 更新物品列表。
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.UserBag)
	if pView ~= nil then
		pView:ReceivePieceCombineCard(tCardInfo, tPieceInfo, nGoldNum, nDiamondNum)
	end
end

-- 接收卡牌分解碎片的信息
function GameLgLobby:ReceiveCardDecomposePiece( nCardIdxArr, tPieceBagList, nGoldNum, nDiamondNum )
	-- 更新物品列表。
	-- 更新用户货币值。
	DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nGoldNum = nGoldNum
	DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamondNum = nDiamondNum
	-- 将碎片放到背包。
	for i,oneBagItem in ipairs(tPieceBagList) do
		-- print("bagItem1's id: , idx: , num: ", oneBagItem.nID, oneBagItem.nIdx, oneBagItem.nNum)
		local tBagItem = DATAMGR:GetUserInfoMgr():AddItemToBag(oneBagItem, true)
		-- print("bagItem2's id: , idx: , num: ", tBagItem.nID, tBagItem.nIdx, tBagItem.nNum)
	end
	-- 删除卡牌。
	for i,nCardIdx in ipairs(nCardIdxArr) do
		DATAMGR:GetUserInfoMgr():RemoveOneBagItem(nCardIdx)
	end

	-- 显示分解得到的碎片。
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.MultiSelCard)
	if pView ~= nil then
		pView:ReceiveCardDecomposePiece(tPieceBagList)
	end
end

-- 获取兑换码的奖励信息
function GameLgLobby:ReceiveGetExchangeReward( tItemList, nGoldNum, nDiamondNum )
	-- 更新物品列表。
	-- 更新用户货币值。
	DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nGoldNum = nGoldNum
	DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamondNum = nDiamondNum
	-- 将奖励放到背包。
	for i,oneBagItem in ipairs(tItemList) do
		local tBagItem = DATAMGR:GetUserInfoMgr():AddItemToBag(oneBagItem, true)
	end

	-- 显示获取的奖励。
	local pView = GameViewMgr:shareMgr():GetViewByType(GameViewType.GameSettingInfo)
	if pView ~= nil then
		pView:ReceiveGetExchangeItems(tItemList)
	end
end

-- 获取支付订单号。
function GameLgLobby:ReceiveGetPurchaseOrderId( nPayIdx, nOrderId, nGameSvrId )
	-- 。
	CT_SHOP_MGR:ShowPayMenu(nPayIdx, nOrderId, nGameSvrId)
end
-- 获取支付列表信息。
function GameLgLobby:ReceiveGetPurchaseList( tPayRewardsArr, nOrderId )
	-- 。
end

-- 装备反馈信息
function GameLgLobby:ReceiveEquipOneCar(nPos, nBagIdx, nKind)
	nPos = nPos + 1
	local tCurTeam = DATAMGR:GetUserInfoMgr():GetCurTeamList()
	-- 要装备的车子
	local tTeamCar = nil
	-- 辅助车
	if nKind ~= 1 then 
		tTeamCar = tCurTeam.tArrAidList[nPos]
		if tTeamCar == nil then 
			tTeamCar = NOT_EQUIPED
		end
	-- 阵容车
	else
		tTeamCar = tCurTeam.tArrCarList[nPos]
	end
	
	if tTeamCar == nil then
		CTAssert(0, "装备时，网络回复时，对应的阵容位置获取不到车子：".. " "..nPos .. " "..nBagIdx.. " "..nType)
		return
	end
	-- 战斗阵容
	if nKind == 1 then
		if tTeamCar.nCarIdx ~= NOT_EQUIPED then
			DATAMGR:GetUserInfoMgr():UnequipOneCardByBagIdx(tTeamCar.nCarIdx)
		end
		tTeamCar.nCarIdx = nBagIdx

	-- 辅助阵容
	else
		local nCarIdx = tTeamCar
		if nCarIdx ~= NOT_EQUIPED then
			DATAMGR:GetUserInfoMgr():UnequipOneCardByBagIdx(nCarIdx)
		end
		DATAMGR:GetUserInfoMgr():GetCurTeamList().tArrAidList[nPos] = nBagIdx
	end

	-- 装备了
	DATAMGR:GetUserInfoMgr():EquipOneCardByBagIdx(nBagIdx)

	-- 计算当前信息
	DATAMGR:GetUserInfoMgr():CountOneTeamValue(tCurTeam, true)

	if self:GetTeamLogic() then
		self:GetTeamLogic():ReceiveEquipOneCar(nPos, nBagIdx, nKind)
	end

	-- 改变阵容的
	local pView = GameViewMgr:shareMgr():GetViewByType( GameViewType.ChangeTeamView )
	if pView then
		pView:CreateFiveCarBtn()
	end
end


-- mark: ====== 顶部通知相关 ======
-- 初始化信息
function GameLgLobby:InitNoticeData()
	-- 静态通知列表。
	self.staticNoticeList = {}
	self.staticIdx = 1
	self.activeNoticeList = {}
	-- 解析活动公告.
	local strFile = Public_GetResPathByKey("Images/StaticDatas/notice.json")
	local strData = io.readfile(strFile)
	local jsonNotice = json.decode(strData)
	-- 静态文字
	for i,oneNotice in ipairs(jsonNotice.static) do
		-- print("static notice:", oneNotice)
		self.staticNoticeList[#self.staticNoticeList + 1] = oneNotice
	end
	-- 动态文字
	for key,oneNotice in pairs(jsonNotice.active) do
		-- print("active notice:", oneNotice, key)
		self.activeNoticeList[toint(key)] = oneNotice
	end

	-- 动态通知列表(一条通知显示一次，然后从列表中删除)。
	self.tempNoticeList = {}

	-- 初始化标签。
	local pLbNotice = self.pCtuiLayer:GetNodeByTag(Tag_Notice.Lb_Notice)
	tolua.cast(pLbNotice, "CTLabel")

	pLbNotice:setVisible(false)
	local pLbNotice2 = self.pCtuiLayer:GetNodeByTag(Tag_Notice.Lb_Notice2)
	tolua.cast(pLbNotice2, "CTLabel")
	pLbNotice2:setVisible(false)

	
	-- 对内容层进行属性设置
	local pTmpLy = self.pCtuiLayer:GetNodeByTag(Tag_Notice.LyContent)
	local fConWidth = pTmpLy:getContentSize().width ;
	tolua.cast(pTmpLy, "CTLayerClip")
	pTmpLy:SetClipRegion( CCRectMake(72, 0, display.width + 45,pTmpLy:getContentSize().height ) )
	pTmpLy:SetIsClipRegion(true)
	pTmpLy:SetIsMoveFollow(true)

	self:initNewNoticeLabel(pLbNotice)

	-- test
	-- local pNoticeList = {}
	-- local zerNotice = {}
	-- zerNotice[1] = 1
	-- zerNotice[2] = "紧急通知：服务器将关闭1小时维护。"
	-- zerNotice[3] = 2
	-- pNoticeList[#pNoticeList+1] = zerNotice
	-- local oneNotice = {}
	-- oneNotice[1] = 2
	-- oneNotice[2] = "收到建筑"
	-- oneNotice[3] = 2
	-- pNoticeList[#pNoticeList+1] = oneNotice
	-- local secNotice = {}
	-- secNotice[1] = 3
	-- secNotice[2] = "我是谁"
	-- secNotice[3] = 2210
	-- pNoticeList[#pNoticeList+1] = secNotice
	-- local thiNotice = {}
	-- thiNotice[1] = 4
	-- thiNotice[2] = "么么哒"
	-- thiNotice[3] = 33
	-- pNoticeList[#pNoticeList+1] = thiNotice
	-- self:ReceiveNoticeList(pNoticeList)

end
-- 初始化标签显示。
function GameLgLobby:initNewNoticeLabel( pLbNotice )
	tolua.cast(pLbNotice, "CTLabel")
	-- body
	local bShowNotice = false;
	local strNotice = ""
	if #self.tempNoticeList > 0 then
		-- 显示临时通知
		strNotice = self.tempNoticeList[#self.tempNoticeList]
		self.tempNoticeList[#self.tempNoticeList] = nil
		bShowNotice = true
	elseif #self.staticNoticeList > 0 then
		-- 显示静态通知
		strNotice = self.staticNoticeList[self.staticIdx]
		self.staticIdx = self.staticIdx + 1
		if self.staticIdx > #self.staticNoticeList then
			self.staticIdx = 1
		end
		bShowNotice = true
	end
	-- print("show string:", strNotice) 
	if bShowNotice == true then
		local pTmpLy = self.pCtuiLayer:GetNodeByTag(Tag_Notice.LyContent)
		local fConWidth = pTmpLy:getContentSize().width ;
		pLbNotice:setString(strNotice)
		local posX = pLbNotice:getContentSize().width * pLbNotice:getAnchorPoint().x
		pLbNotice:setPosition(fConWidth + posX, pLbNotice:getPositionY())
		pLbNotice:setVisible(true)

	end
end
-- 停止标签移动。
function GameLgLobby:stopNoticeLabelMove()
	-- body
end
-- 动态更新通知
function GameLgLobby:updateNotice( dt )
	
	local pLbNotice = self.pCtuiLayer:GetNodeByTag(Tag_Notice.Lb_Notice)
	local pLbNotice2 = self.pCtuiLayer:GetNodeByTag(Tag_Notice.Lb_Notice2)

	if pLbNotice:isVisible() then
		pLbNotice:setPosition(pLbNotice:getPositionX() - 2, pLbNotice:getPositionY())
		local pLbEndPosX = pLbNotice:getPositionX() + pLbNotice:getContentSize().width * (1.0 - pLbNotice:getAnchorPoint().x)
		if pLbEndPosX < 0 then
			-- stop.
			pLbNotice:setVisible(false)
		elseif pLbEndPosX < ctprwidth(1.0)*1/4 then
			-- new another one.
			if pLbNotice2:isVisible() == false then
				self:initNewNoticeLabel(pLbNotice2)
			end
		end
	end

	-- 
	if pLbNotice2:isVisible() then
		pLbNotice2:setPosition(pLbNotice2:getPositionX() - 2, pLbNotice2:getPositionY())
		local pLbEndPosX = pLbNotice2:getPositionX() + pLbNotice2:getContentSize().width * (1.0 - pLbNotice:getAnchorPoint().x)
		if pLbEndPosX < 0 then
			-- stop.
			pLbNotice2:setVisible(false)
		elseif pLbEndPosX < ctprwidth(1.0)*1/4 then
			-- new another one.
			if pLbNotice:isVisible() == false then
				--
				self:initNewNoticeLabel(pLbNotice)
			end
		end
	end
end

return GameLgLobby