--
-- Author: Link
-- Date: 2014-03-13 14:16:38
--

local LogicStoryStage = class("LogicStoryStage", function()
	return display.newLayer()
end)

-- UITag
local StoryStageUiTag = {
	------ 章节Cell层次 ----
	LyCell 			= 3,	-- cell层次
	BtnCell 		= 7,	-- cell按钮

	LbCellTitle		= 14,	-- 章节标题
	LbCellPercent	= 15,	-- 进度百分比
	LbCellInto1		= 11,	-- 进入和未开启的文字
	LbCellInto2		= 12,	-- 进入和未开启的文字
	LbCellInto3		= 13,	-- 进入和未开启的文字

	SpChapterBot1	= 16,	-- 章节底图1
	SpCellBg 		= 6,	-- Cell的底图

	SpCellPercent	= 26,	-- 进度条
	-----------------------

	SpFrame 		= 1,	-- 底层框
}


-- 初始化
function LogicStoryStage:ctor()
	
end

-- 释放前调用
function LogicStoryStage:BeforeReleaseLayer()
	-- 有显示层，则释放对应信息
	self.pCtuiLayer:removeFromParent()
	self.pCtuiLayer = nil
end

-- 获取信息
function LogicStoryStage:InitWithShowLayer( pShowLayer )
	-- 当前界面
	self.pCtuiLayer = CTUILayer:new();
	self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/FuBenBiaoTi.ctui"))
	self.pCtuiLayer:setAnchorPoint(ccp(0.5, 1))
	self.pCtuiLayer:setPosition(ctprccp(0.5, 1))
	self.pCtuiLayer:ignoreAnchorPointForPosition(false)
	self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	self.pCtuiLayer:ShowAllUiInfo()
	pShowLayer:addChild(self.pCtuiLayer, GLobbyZOrder.TabLayer)
	self.pCtuiLayer:release()

	self.pCtuiLayer:setDelegateLua(function(event)
	-- 如果是按钮点击事件
		if event.cbType == "btnCb" then
			self:CTUIButtonCallBack(event.Layer, event.Btn, event.Tag)
		end
	end)
	-- 去掉显示
	self.pCtuiLayer:SetCTUINodeVisiable(StoryStageUiTag.LyCell, false)

	-- -- 差异不一样
	-- local fTopDis = SHARE_FUNC:GetViewTopHeight(true)
	-- self.pCtuiLayer:setPositionY( self.pCtuiLayer:getPositionY() - fTopDis )

	local pSpBot = self.pCtuiLayer:GetNodeByTag(StoryStageUiTag.SpFrame)
	local fTmpY = pSpBot:getParent():convertToNodeSpace( ccp(0, ctipwidth(SHARE_FUNC.bottomTagH)) ).y;
	pSpBot:setScaleY( (pSpBot:getPositionInCCPoint().y-fTmpY)/pSpBot:getContentSize().height )
	

	-- 表格的回调的
	local function TabelViewCallBack( event )
		-- 表格总数
		if event.type == "count" then
			return self:CTTableViewShowCellCount(event.table)

		-- 表格的key
		elseif event.type == "key" then
			return self:CTTableViewCellByIdx(event.table, event.Idx)

		-- 创建
		elseif event.type == "Create" then
			return self:CTTableViewCreateNewCellByIdx(event.table, event.Idx, event.key)

		-- 显示具体信息
		elseif event.type == "showCell" then
			self:CTTableViewShowToExistCellByIdx(event.table, event.cell)

		-- 改变尺寸的回调中
		elseif event.type == "ReSize" then
			local tSize = CCSizeMake(event.width, event.height)
			self:CTTableViewChangeSize(event.table, event.cell, tSize)

		-- cell上面点中按钮的回调
		elseif event.type == "Btn" then
			self:CTTableViewClickCellButtonByIdx(event.table, event.Idx, event.BtnType)

		-- 在cell上面点击的回调
		elseif event.type == "Click" then
			self:CTTableViewClickCell(event.table, event.cell)
		end
	end

	local pSpRBot = self.pCtuiLayer:GetNodeByTag(StoryStageUiTag.SpCellBg)
	local tRBotRect = ctspbound(pSpRBot)
	tRBotRect.size.width = self.pCtuiLayer:getContentSize().width

	-- print("aaaaaa:", tRBotRect:getMinX(), tRBotRect:getMinY(), tRBotRect:getMaxX(), tRBotRect:getMaxY())
	local fTabHeight = fTmpY+ctipwidth(60*pSpBot:getScaleY())
	-- 章节列表
	self.pStoryTable = CTTableView:new()
	self.pStoryTable:initWithFrameLua(CCRectMake(tRBotRect:getMinX(), fTabHeight, tRBotRect.size.width, (tRBotRect:getMaxY()-fTabHeight)), TabelViewCallBack)
	self.pStoryTable:SetNormalCellSize(CCSizeMake(tRBotRect.size.width, tRBotRect.size.height+22))
	self.pStoryTable:setDelSpeedRate(SHARE_FUNC.fScorllDelSpeedRate)
	self.pCtuiLayer:addChild(self.pStoryTable, 1)

	self.pStoryTable:reloadTableData()
	self.pStoryTable:release()
	-- 统计点击故事剧情的次数(sdk)
	UM_SDK:StatisticsStoryTimes()
end

-- mark: ============== 循环处理 ================
function LogicStoryStage:LogicStoryStageLoop( dt )
	
end

-- mark: ============== 按钮的回调 ================
-- 按钮回调
function LogicStoryStage:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()
	DIALOGMGR:ShowToast("功能未开放，敬请期待!!")
end

-- mark: ============== 表格的管理 ================
-- 如果之前的没有创建cell，则提示创建新的cell
function LogicStoryStage:CTTableViewCreateNewCellByIdx(pTableView, nCreateIdx, strKey)
	local pCell = CTTableViewCell:new()


	local pCellInfo = self.pCtuiLayer:ReCreateUiNode(StoryStageUiTag.LyCell, false, StoryStageUiTag.LyCell, 0, UiNotShowEnd)
	local pSpBottom = self.pCtuiLayer:GetNodeByTag(StoryStageUiTag.SpCellBg)
	local tBotRect = ctspbound(pSpBottom)
	pCellInfo:setPosition(ccp(2 - tBotRect:getMinX(), 2 - tBotRect:getMinY()))
	pCell:addChild(pCellInfo, 2)

	-- 开始按钮管理
	local pMenuMgr = pCell:GetChildNodeByTag(StoryStageUiTag.BtnCell):getParent()
	tolua.cast(pMenuMgr, "CTMenuManager")
	pCell:StartToUseCellBtn(pMenuMgr)

	-- 可以移动
	pCell:setBottonMoveType(ctCellBtnTouchType_CanMoveAndNotUse)
	
	pCell:autorelease()
	return pCell
end
-- 如果之前已经提示创建了cell，则提示已经开始显示到对应的cell，提示开始修改数据
function LogicStoryStage:CTTableViewShowToExistCellByIdx(pTableView, pCell)
	-- 获取具体信息
		local nTaskIdx = pCell:getCellIdx()+1

		-- UITag
-- local StoryStageUiTag = {
-- 	------ 章节Cell层次 ----
-- 	LyCell 			= 3,	-- cell层次
-- 	BtnCell 		= 6,	-- cell按钮

	-- LbCellTitle		= 13,	-- 章节标题
	-- LbCellPercent	= 20,	-- 进度百分比
	
	-- SpChapterBot1	= 10,	-- 章节底图1
-- 	-----------------------
-- }
	-- 列表	
	local tOneChapter = DATAMGR:GetTaskInfoMgr():GetBigChapterList()[nTaskIdx]

	local tArrChapterName = {
		"第一章",
		"第二章",
		"第三章",
		"第四章",
		"第五章",
		"第六章",
		"第七章",
		"第八章",
		"第九章",
		"第十章",
		"第十一章",
		"第十二章",
		"第十三章",
		"第十四章",
	}

	local nCurShowIdx = (nTaskIdx-1)%(#tArrChapterName) + 1;

	-- 标题
	local pLbTitle = pCell:GetChildNodeByTag(StoryStageUiTag.LbCellTitle)
	tolua.cast(pLbTitle, "CCLabelTTF")
	pLbTitle:setString(tArrChapterName[ nCurShowIdx ].." "..tOneChapter.strName)

	-- 按钮的可以点击状态
	local pBtnCell = pCell:GetChildNodeByTag(StoryStageUiTag.BtnCell)
	tolua.cast(pBtnCell, "CTMenuItem")
	pBtnCell:SetEnable( tOneChapter.bIsUnlock )

	-- 任务的百分比
	local pLbPercent = pCell:GetChildNodeByTag(StoryStageUiTag.LbCellPercent)
	tolua.cast(pLbPercent, "CCLabelTTF")
	local pSpPercent = pCell:GetChildNodeByTag(StoryStageUiTag.SpCellPercent)
	tolua.cast(pSpPercent, "CCLabelTTF")

	-- 开启的文字 
	local pLbInto1 = pCell:GetChildNodeByTag(StoryStageUiTag.LbCellInto1)
	tolua.cast(pLbInto1, "CCLabelTTF")
	local pLbInto2 = pCell:GetChildNodeByTag(StoryStageUiTag.LbCellInto2)
	tolua.cast(pLbInto2, "CCLabelTTF")
	local pLbInto3 = pCell:GetChildNodeByTag(StoryStageUiTag.LbCellInto3)
	tolua.cast(pLbInto3, "CCLabelTTF")

	-- 底下的地图
	local pChaBot1 = pCell:GetChildNodeByTag(StoryStageUiTag.SpChapterBot1)
	tolua.cast(pChaBot1, "CCSprite")

	local tArrBot1 = {
			"Images/Objects/CarSmall1046.png",
			"Images/Objects/CarSmall1064.png",
			"Images/Objects/CarSmall1072.png",
			"Images/Objects/CarSmall1076.png",
			"Images/Objects/CarSmall1087.png",
			"Images/Objects/CarSmall1099.png",
			"Images/Objects/CarSmall1091.png",
	}
	local pBot1Tex = CCTextureCache:sharedTextureCache():addImage(tArrBot1[nCurShowIdx])
	pChaBot1:setTexture(pBot1Tex)

	-- 已经解锁的显示信息
	if tOneChapter.bIsUnlock == true then

		local tPerSize = pSpPercent:getContentSize();
		-- 获取进度信息
		local tProgress = DATAMGR:GetTaskInfoMgr():GetTaskCurProgress()
		if tProgress.nBigIdx == nTaskIdx then
			local fTotal = #(tOneChapter.tChapterList)*5.0
			local nPastCount = (tProgress.nCurMidIdx-1)*5.0 + tProgress.nCurTaskIdx-1
			local fPercent = nPastCount / fTotal

			pLbPercent:setString(""..toint(fPercent*100).."%")
			-- pSpProgress:setScaleX(fPercent)
			pSpPercent:setTextureRect( CCRectMake(0, 0, tPerSize.width*fPercent, tPerSize.height) )
		else
			pLbPercent:setString("100%")
			-- pSpProgress:setScaleX(1)
			pSpPercent:setTextureRect( CCRectMake(0, 0, tPerSize.width, tPerSize.height) )
		end

		pLbInto1:setString("进  入")
		pLbInto1:setColor(ccc3(0x93, 0x32, 0x3f))
		pLbInto2:setString("进  入")
		pLbInto2:setColor(ccc3(0x93, 0x32, 0x3f))

		pLbPercent:setColor(ccc3(0xff, 0xff, 0xff))
		pSpPercent:setVisible(true)
		-- pLbChaCount:setColor(ccc3(0xb3, 0x5d, 0x10))
	else

		pLbPercent:setString("0%")
		pLbInto3:setString("未开启")
		pLbInto3:setColor(ccc3(0x5a, 0x5a, 0x5a))

		pLbPercent:setColor(ccc3(0xc0, 0xc0, 0xc0))
		pSpPercent:setVisible(false)
		-- pLbChaCount:setColor(ccc3(0x4b, 0x4b, 0x4b))
	end

end
-- 获取对应idx的cell的key值
function LogicStoryStage:CTTableViewCellByIdx(pTableView, nCellIdx)
	return "a"
end
-- 获取显示的cell的总数
function LogicStoryStage:CTTableViewShowCellCount(pTableView)
	-- 剧情大章节信息
	local tChapterList = DATAMGR:GetTaskInfoMgr():GetBigChapterList()
	return #(tChapterList)
end
-- 点击了其中一个cell
function LogicStoryStage:CTTableViewClickCell(pTableView, pCell)
	
end
-- 点击cell上面的按钮
function LogicStoryStage:CTTableViewClickCellButtonByIdx(pTableView, nCellIdx, nButtonType)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()
	-- 获取具体信息
	local nTaskIdx = nCellIdx+1
	-- 通知上层，要进入章节列表信息
	self:getParent():StartToIntoChapterByChapterIdx(nTaskIdx)
end
-- 改变尺寸的回调
function LogicStoryStage:CTTableViewChangeSize(pTableView, pCell, suNewSize)

end

-- mark: ============== 触控的管理 ================
-- 开始触控
function LogicStoryStage:LgStageTouchBegan(tLocation, nCount)
	self.pCtuiLayer:CTUITouchBegan(tLocation)
	self.pStoryTable:CtScrollViewTouchBegan(tLocation)
end

-- 触控移动
function LogicStoryStage:LgStageTouchMoved(tLocation, tPrevPos, tBeganPos)
	self.pCtuiLayer:CTUITouchMoved(tLocation)
	-- 在教学中，则不给滑动
	if TEACHMGR:IsCurInTeaching() then
		return;
	end
	self.pStoryTable:CtScrollViewTouchMoved(tLocation, tPrevPos)
end

-- 触控结束
function LogicStoryStage:LgStageTouchEnded(tLocation, tPrevPos, tBeganPos)
	self.pCtuiLayer:CTUITouchEnded(tLocation)
	self.pStoryTable:CtScrollViewTouchEnded(tLocation, tPrevPos)
end

return LogicStoryStage