--
-- Author: Link
-- Date: 2013-12-27 20:11:56
--
local LogicTeamList = class("LogicTeamList", function()
	return display.newLayer()
end)

--------- 显示的Uitag值 ----------
local TeamListUiTag = {
	LyCell			= 5,	-- cell 层次
	LyCellInfo1 	= 9,	-- cell 的信息层
	LyCellInfo2 	= 4,	-- cell 的信息层

	BtnCellOne		= 1,	-- 一个按钮
	BtnChangeTeam	= 0,	-- 修改阵容按钮

	LbCellLevel1	= 39,	-- 等级数字
	LbCellLevel2	= 12,	-- 等级数字
	LbCellStar1		= 8,	-- 星级
	LbCellStar2		= 13,	-- 星级
	SpCellCar1 		= 10,	-- 车子的标签
	SpCellCar2 		= 15,	-- 车子的标签

	SpCellStarBot1 	= 7,	-- 星级的底图
	SpCellStarBot2 	= 11,	-- 星级的底图

	SpLeftArrow	= 3,	-- 左边的阵容箭头

	SpMergeStar 	= 14,	-- 合成等级的星星
	LbMergeLevel	= 2,	-- 合成等级的文字
}
---------------------------------

local TeamListType = {
	CarList 	= 1,	-- 当前阵容队伍类型
	CarListIcon = 2,	-- 当前阵容队伍类型有选中光标
	AidItemList	= 3,	-- 辅助类型，比如说宠物等
	ViewOtherCar = 4,	-- 查看其他车的列表处理
	PreReadTeam = 5,    -- 预备队
}

function LogicTeamList:ctor()
	-- self.super.ctor(self)

	-- 回调
	self.pCallBack = nil
	-- 当前显示的类型
	self.nShowType = TeamListType.CarList
	-- 查看其他队伍时的数据
	self.tViewTeamList = nil
	-- 预备队
	self.pCtuiPreTeamList = nil 
	-- 点击换整容的标记 用来更新当前的table
	self.updateTeam = false
	-- 当前选中的光环
	self.nSelIdx = 0
	self.pSpSelCir = CCSprite:create("Images/GX-017-9.png")
	self.pSpSelCir:setScale(1.2)
	self.pSpSelCir:retain()
end

-- 初始化数据
function LogicTeamList:InitWithInfo(pCallBack, nType, pShowLayer)
	nType = nType or TeamListType.CarList
	-- 用于把预备队层加在 战斗里层上
	self.pShowLayer = pShowLayer
	-- 回调
	self.pCallBack = pCallBack
	-- 当前显示的类型
	self.nShowType = nType


	-- 当前界面
	self.pCtuiLayer = CTUILayer:new();

	if self.nShowType == TeamListType.ViewOtherCar then
		self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/ZhenRongCao-ipad.ctui"))
		self.pCtuiLayer:setContentSize(CCSizeMake(768, 1024))
	else
		self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/ZhenRongCao-ipad.ctui"))
		self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	end
	self.pCtuiLayer:setAnchorPoint(ccp(0, 1))
	self.pCtuiLayer:setPosition(ctprccp(0, 1))
	self.pCtuiLayer:ignoreAnchorPointForPosition(false)
	self.pCtuiLayer:ShowAllUiInfo()
	self:addChild(self.pCtuiLayer)

	self.pCtuiLayer:setDelegateLua(function(event)
	-- 如果是按钮点击事件
		if event.cbType == "btnCb" then
			self:CTUIButtonCallBack(event.Layer, event.Btn, event.Tag)
		end
	end)

	self.pCtuiLayer:release()

	-- 不显示按钮层
	self.pCtuiLayer:SetCTUINodeVisiable(TeamListUiTag.LyCell, false)

	-- 按钮
	local tBtnCell = self.pCtuiLayer:GetNodeByTag(TeamListUiTag.BtnCellOne)
	local tCellRect = ctspbound(tBtnCell)
	-- 默认显示类型
	local fTbWidth = self.pCtuiLayer:getContentSize().width - tCellRect:getMinX()*2
	local fStartTbX = tCellRect:getMinX();
	if self.nShowType == TeamListType.ViewOtherCar then
		fTbWidth = self.pCtuiLayer:getContentSize().width - tCellRect:getMinX()*1-8
	-- 有光标类型
	elseif self.nShowType == TeamListType.CarListIcon then
		fTbWidth = self.pCtuiLayer:getContentSize().width - tCellRect:getMinX()*1 - 20
		fStartTbX = fStartTbX + 20;
	end
	self.pTableView = CTTableView:new()
	self.pTableView:initWithFrameLua(CCRectMake(fStartTbX, tCellRect:getMinY()-2, fTbWidth - 20, tCellRect.size.height+ 20), function(event)
		-- 表格总数
			if event.type == "count" then
				return self:CTTableViewShowCellCount(event.table)

			-- 表格的key
			elseif event.type == "key" then
				return self:CTTableViewCellByIdx(event.table, event.Idx)

			-- 创建
			elseif event.type == "Create" then
				return self:CTTableViewCreateNewCellByIdx(event.table, event.Idx, event.key)

			-- 显示具体信息
			elseif event.type == "showCell" then
				self:CTTableViewShowToExistCellByIdx(event.table, event.cell)

			-- 改变尺寸的回调中
			elseif event.type == "ReSize" then
				local tSize = CCSizeMake(event.width, event.height)
				self:CTTableViewChangeSize(event.table, event.cell, tSize)

			-- cell上面点中按钮的回调
			elseif event.type == "Btn" then
				self:CTTableViewClickCellButtonByIdx(event.table, event.Idx, event.BtnType)

			-- 在cell上面点击的回调
			elseif event.type == "Click" then
				self:CTTableViewClickCell(event.table, event.cell)
			end
	end, ctTableViewDirectionLeftToRight)

	self.pTableView:SetNormalCellSize( CCSizeMake(tCellRect.size.width+ 12, tCellRect.size.height+4) )
	self.pTableView:setDelSpeedRate(SHARE_FUNC.fScorllDelSpeedRate)
	self.pCtuiLayer:addChild(self.pTableView)

	self.pTableView:release()

	-- 没有选中光标的
	if self.nShowType == TeamListType.CarList then
		self.nSelIdx = -1
		self.pCtuiLayer:SetCTUINodeVisiable(TeamListUiTag.BtnChangeTeam, false)
	elseif self.nShowType == TeamListType.CarListIcon then
		self.pCtuiLayer:SetCTUINodeVisiable(TeamListUiTag.SpLeftArrow, false)
		self.pCtuiLayer:SetCTUINodeVisiable(TeamListUiTag.BtnChangeTeam, true)
	else

	end

	-- 如果不是队列里面的信息
	if self.nShowType ~= TeamListType.ViewOtherCar then
		self.pTableView:reloadTableData()
		self.pTableView:reloadTableData()
	else
		self:setContentSize(CCSizeMake(768, 1024))
		self.pCtuiLayer:setPosition( ccp(0, 1024) )
	end
end

-- 释放前调用
function LogicTeamList:BeforeReleaseLayer()
	self.pSpSelCir:release()
	self.pSpSelCir = nil
end

function LogicTeamList:RefreshShow()
	if self.pCtuiPreTeamList ~= nil then 
		if self:GeStateOfTeam() then 
			self.pTableView:reloadTableData()
		end
		self.pCtuiPreTeamList:RefreshShow()
	else
		self.pTableView:reloadTableData()
	end
	
end

-- 改变队伍的具体高度
function LogicTeamList:ChangeTeamPosY(fPosY)
	-- local tPos = self.pTableView:getParent():convertToWorldSpace( self.pTableView:getPositionInCCPoint() )
	local tRect = self.pCtuiLayer:GetNodeAbsShowRect(TeamListUiTag.BtnCellOne)
	local tPos = ccp(tRect:getMidX(), tRect:getMidY())
	self:setPositionY( self:getPositionY() + (fPosY-tPos.y) )
end

-- 设置查看的其他队伍的队列
function LogicTeamList:SetShowTeamView(tTeamList)
	self.tViewTeamList = tTeamList
	self.pTableView:reloadTableData()
end

-- mark: ============== 循环处理 ================
function LogicTeamList:LogicTeamListLoop( dt )
	
end

-- 显示选中的光圈
function LogicTeamList:ShowSeletedCir( nSelIdx )
	-- 显示预备对的光标
	if self.pCtuiPreTeamList ~= nil then 
		self.pCtuiPreTeamList:ShowSeletedCir( nSelIdx )
		return 
	end
	-- 如果不显示图标
	if self.nShowType ~= TeamListType.CarListIcon and self.nShowType ~= TeamListType.ViewOtherCar then
		return
	end
	-- 索引正确，则显示选中框
	if self.nSelIdx ~= nSelIdx then
		self.pSpSelCir:removeFromParent()
		-- 显示对应的索引
		local pOldCell = self.pTableView:GetShowCellByIndex(self.nSelIdx)
		if pOldCell then
			local pBtnIcon = pOldCell:GetChildNodeByTag(TeamListUiTag.BtnCellOne)
			tolua.cast(pBtnIcon, "CTMenuItemSprite")
			pBtnIcon:SetEnable(true)
			-- 解决出现预备队后  队列的高亮框消失
			if self.nSelIdx == -1 then 
				return 
			end
		end


		-- 显示对应的索引
		local pCell = self.pTableView:GetShowCellByIndex(nSelIdx)
		-- 获取按钮信息
		if pCell then
			local pBtnIcon = pCell:GetChildNodeByTag(TeamListUiTag.BtnCellOne)
			tolua.cast(pBtnIcon, "CTMenuItemSprite")

			self.pSpSelCir:setPosition( ctspccp(pBtnIcon:GetNormalSprite(), 0.5, 0.5) )
			pBtnIcon:GetNormalSprite():addChild(self.pSpSelCir)

			self.nSelIdx = nSelIdx

			pBtnIcon:SetEnable(false)
		end

		-- 滑动到对应的位置
		local fWidth = self.pTableView:getContentSize().width - self.pTableView:getShowFrame().size.width
		local fNextWidth = self.pTableView:GetNormalCellSize().width * (nSelIdx-3.5)
		if fNextWidth < 0 then
			fNextWidth = 0
		elseif fNextWidth > fWidth then
			fNextWidth = fWidth
		end
		self.pTableView:SetOffsetPos(ccp(-fNextWidth, 0), true)
	end
end

-- mark: ============== 按钮的回调 ================
-- 按钮回调
function LogicTeamList:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()
    -- 修改阵容按钮
	if nTag == TeamListUiTag.BtnChangeTeam then
	 	self.pTeamView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.ChangeTeamView)
	 	self.pTeamView:SetTeamDelegate(self)

	 	
	 	-- 教学的下一步
	 	TEACHMGR:ShowSwichToNextStep();
	else
		DIALOGMGR:ShowToast("功能未开放，敬请期待!!")
	end
end

-- mark: ============== 表格的管理 ================
-- 如果之前的没有创建cell，则提示创建新的cell
function LogicTeamList:CTTableViewCreateNewCellByIdx(pTableView, nCreateIdx, strKey)
	local pCell = CTTableViewCell:new()
	-- 创建一个新的信息
	local pCellInfo = self.pCtuiLayer:ReCreateUiNode(TeamListUiTag.LyCell, false, TeamListUiTag.LyCell, 0, UiNotShowEnd)
	local pSpBottom = self.pCtuiLayer:GetNodeByTag(TeamListUiTag.BtnCellOne)
	local tBotRect = ctspbound(pSpBottom)
	local tCellSize = pTableView:GetNormalCellSize() --  pSpBottom:getContentSize()   
	-- print("tBotRect:getMinX() ", tBotRect:getMinX(), tBotRect:getMinY())
	pCellInfo:setPosition(ccp(2 - tBotRect:getMinX() + nCreateIdx*10,  8 - tBotRect:getMinY() ))
	-- pCellInfo:setPosition(ccp( -90,  - 686 ))
	pCell:addChild(pCellInfo, 2)
	pCell:setScale(1.06)

	
	if (nCreateIdx + 1) >  MAX_TEAMCOUNT then
		newCellSize = CCSizeMake(tCellSize.width + 60, tCellSize.height)
		-- pCell:setCellSize(newCellSize)
		pCell:setCellSize(newCellSize)		
	end

	
 	local pMenuMgr = pCell:GetChildNodeByTag(TeamListUiTag.BtnCellOne):getParent()
	tolua.cast(pMenuMgr, "CTMenuManager")
	pCell:StartToUseCellBtn(pMenuMgr)


	-- 可以移动
	pCell:setBottonMoveType(ctCellBtnTouchType_CanMoveAndNotUse)
	pCell:autorelease()
	return pCell
end
-- 如果之前已经提示创建了cell，则提示已经开始显示到对应的cell，提示开始修改数据
function LogicTeamList:CTTableViewShowToExistCellByIdx(pTableView, pCell)
	-- 当前索引
	local nCellIdx = pCell:getCellIdx()

	-- 如果是车队伍类型
	if self.nShowType == TeamListType.CarList or self.nShowType == TeamListType.CarListIcon 
		or self.nShowType == TeamListType.ViewOtherCar then

		local pArrList
		-- 如果是查看其他队伍列表信息
		if self.nShowType == TeamListType.ViewOtherCar then
			pArrList = self.tViewTeamList.tCarList
		else
			pArrList = DATAMGR:GetUserInfoMgr():GetCurTeamList().tArrCarList
		end
		local nCount = #pArrList
		-- 车子等级信息层
		local pLyInfo1 = pCell:GetChildNodeByTag(TeamListUiTag.LyCellInfo1)
		local pLyInfo2 = pCell:GetChildNodeByTag(TeamListUiTag.LyCellInfo2)

		-- 获取按钮信息
		local pBtnIcon = pCell:GetChildNodeByTag(TeamListUiTag.BtnCellOne)
		tolua.cast(pBtnIcon, "CTMenuItemSprite")

		-- 获取车子图标
		local pSpBtnIcon1 = pCell:GetChildNodeByTag(TeamListUiTag.SpCellCar1)
		tolua.cast(pSpBtnIcon1, "CCSprite")
		local pSpBtnIcon2 = pCell:GetChildNodeByTag(TeamListUiTag.SpCellCar2)
		tolua.cast(pSpBtnIcon2, "CCSprite")
		-- self.nSelIdx  ＝ 0
		-- 索引正确，则显示选中框
		if self.nShowType == TeamListType.CarListIcon or self.nShowType == TeamListType.ViewOtherCar  then
			if self.nSelIdx == nCellIdx then
				self.pSpSelCir:removeFromParent()
				self.pSpSelCir:setPosition( ctspccp(pBtnIcon:GetNormalSprite(), 0.5, 0.5) )
				pBtnIcon:GetNormalSprite():addChild(self.pSpSelCir)

				pBtnIcon:SetEnable(false)
			-- 如果位置索引不正确，则删除出来
			elseif pBtnIcon == self.pSpSelCir:getParent() then
				self.pSpSelCir:removeFromParent()
				pBtnIcon:SetEnable(true)
			end
		else
			pBtnIcon:SetEnable(true)
		end

		-- 合成的星级
		local pSpMergeStar = pCell:GetChildNodeByTag(TeamListUiTag.SpMergeStar)

		-- 如果是显示车子信息
		if nCellIdx < nCount then
			-- 车子的信息
			local tTeamCar = pArrList[nCellIdx+1]
			local nCarValue = NOT_EQUIPED
			-- 如果是查看其他队伍列表信息
			if self.nShowType == TeamListType.ViewOtherCar then
				nCarValue = tTeamCar.nCarID
			--- 查看当前自己的列表
			else
				nCarValue = tTeamCar.nCarIdx
			end

			-- 如果车子没有装备
			if nCarValue == NOT_EQUIPED then

				local pTexTmp = CCTextureCache:sharedTextureCache():addImage("Images/GX-017-8.png")
				local tTexSize = pTexTmp:getContentSize()

				pBtnIcon:GetNormalSprite():setTexture(pTexTmp)
				pBtnIcon:GetNormalSprite():setTextureRect(CCRectMake(0, 0, tTexSize.width, tTexSize.height))
				pBtnIcon:GetSelectedSprite():setTexture(pTexTmp)
				pBtnIcon:GetSelectedSprite():setTextureRect(CCRectMake(0, 0, tTexSize.width, tTexSize.height))
				pSpBtnIcon1:setVisible(false)
				pSpBtnIcon2:setVisible(false)

				pLyInfo1:setVisible(false)
				pLyInfo2:setVisible(false)
				-- 隐藏起来
				pSpMergeStar:setVisible(false)

			-- 该阵容槽上面装备了
			else
				pSpBtnIcon1:setVisible(true)
				pSpBtnIcon2:setVisible(true)
				-- 获取车子信息
				local nShowLv = 1
				local tCarInfo = nil
				local tCarBagItem = nil
				-- 如果是查看其他队伍列表信息
				if self.nShowType == TeamListType.ViewOtherCar then
					tCarInfo = DATAMGR:GetItemInfoMgr():GetItemInfoByID(nCarValue)
					nShowLv = tTeamCar.nCarLv

					pSpMergeStar:setVisible(false)
				--- 查看当前自己的列表
				else
					tCarBagItem = DATAMGR:GetUserInfoMgr():GetBagItemByIdx(nCarValue, false)
					tCarInfo = DATAMGR:GetItemInfoMgr():GetItemInfoByID(tCarBagItem.nID)
					nShowLv = tCarBagItem.nLv

					-- 显示星星
					if tCarBagItem.nMergeLv > 0 then
						pSpMergeStar:setVisible(true)
						local pLbMergeLevel = pCell:GetChildNodeByTag(TeamListUiTag.LbMergeLevel)
						tolua.cast(pLbMergeLevel, "CCLabelTTF")
						pLbMergeLevel:setString(""..tCarBagItem.nMergeLv)
					else
						pSpMergeStar:setVisible(false)
					end
				end
				

				-- 图标信息
				local pIconFrame = DATAMGR:GetItemInfoMgr():GetItemIconNameByID(tCarInfo.nID)
				pSpBtnIcon1:setDisplayFrame(pIconFrame)
				pSpBtnIcon2:setDisplayFrame(pIconFrame)
				pLyInfo1:setVisible(true)
				pLyInfo2:setVisible(true)

				-- 显示等级
				local pLbLevel1 = pCell:GetChildNodeByTag(TeamListUiTag.LbCellLevel1)
				tolua.cast(pLbLevel1, "CTLabel")
				pLbLevel1:setString(""..nShowLv)
				local pLbLevel2 = pCell:GetChildNodeByTag(TeamListUiTag.LbCellLevel2)
				tolua.cast(pLbLevel2, "CTLabel")
				pLbLevel2:setString(""..nShowLv)

				-- 显示星级
				local pLbStar1 = pCell:GetChildNodeByTag(TeamListUiTag.LbCellStar1)
				tolua.cast(pLbStar1, "CCSprite")
				local pLbStar2 = pCell:GetChildNodeByTag(TeamListUiTag.LbCellStar2)
				tolua.cast(pLbStar2, "CCSprite")

				local pSpStarBot1 = pCell:GetChildNodeByTag(TeamListUiTag.SpCellStarBot1)
				tolua.cast(pSpStarBot1, "CCSprite")
				local pSpStarBot2 = pCell:GetChildNodeByTag(TeamListUiTag.SpCellStarBot2)
				tolua.cast(pSpStarBot2, "CCSprite")
				local pStarTex, pStarBot, pTexTmp = SHARE_FUNC:GetCardRelateTextureByStar(tCarBagItem.strStarLv)

				local tTexSize = pTexTmp:getContentSize()
				pBtnIcon:GetNormalSprite():setTexture(pTexTmp)
				pBtnIcon:GetNormalSprite():setTextureRect(CCRectMake(0, 0, tTexSize.width, tTexSize.height))
				pBtnIcon:GetSelectedSprite():setTexture(pTexTmp)
				pBtnIcon:GetSelectedSprite():setTextureRect(CCRectMake(0, 0, tTexSize.width, tTexSize.height))

				-- 
				pLbStar1:setTexture(pStarTex)
				pLbStar2:setTexture(pStarTex)
				pSpStarBot1:setTexture(pStarBot)
				pSpStarBot2:setTexture(pStarBot)
			end
			
		-- 如果没有
		else
			-- 隐藏起来
			pSpMergeStar:setVisible(false)
			-- 等级信息层不显示
			pLyInfo1:setVisible(false)
			pLyInfo2:setVisible(false)
			local pSpIcon = nil
			-- if  nCellIdx + 1 ~= MAX_TEAMCOUNT then 
			if  MAX_TEAMCOUNT >= (nCellIdx + 1) then
				pSpIcon = CCTextureCache:sharedTextureCache():addImage("Images/GX-017-2.png")
			else
				pSpIcon = CCTextureCache:sharedTextureCache():addImage("Images/YBD-001.png")
			end
			
			local tTexSize = pSpIcon:getContentSize()
			pBtnIcon:GetNormalSprite():setTexture(pSpIcon)
			pBtnIcon:GetNormalSprite():setTextureRect(CCRectMake(0, 0, tTexSize.width, tTexSize.height))
			pBtnIcon:GetSelectedSprite():setTexture(pSpIcon)
			pBtnIcon:GetSelectedSprite():setTextureRect(CCRectMake(0, 0, tTexSize.width, tTexSize.height))
			pSpBtnIcon1:setVisible(false)
			pSpBtnIcon2:setVisible(false)
			
		end
	end
	-- local pLbText = pCell:GetChildNodeByTag(1)
	-- tolua.cast(pLbText, "CCLabelTTF")
	-- pLbText:setString(""..pCell:getCellIdx())
end
-- 获取对应idx的cell的key值
function LogicTeamList:CTTableViewCellByIdx(pTableView, nCellIdx)
	return "a"
end
-- 获取显示的cell的总数
function LogicTeamList:CTTableViewShowCellCount(pTableView)
	local nCount = 0
	-- 如果是车队伍类型
	if self.nShowType == TeamListType.CarList or self.nShowType == TeamListType.CarListIcon then
		nCount = MAX_TEAMCOUNT
	elseif self.nShowType == TeamListType.ViewOtherCar then
		nCount = MAX_TEAMCOUNT
	end
	return nCount + 1
end
-- 点击了其中一个cell
function LogicTeamList:CTTableViewClickCell(pTableView, pCell)
	
end

-- 创建预备队阵容 nFlag: true 解锁创建  false: 点击进入创建
function LogicTeamList:CreatePreReadTeamList( bFlag)
	-- 首先显示当前选择预备队按钮的光标
	self:ShowSeletedCir( MAX_TEAMCOUNT)
	if self.pCtuiPreTeamList ~= nil then 
		-- self.pCtuiPreTeamList:removeFromParent()
		-- 重新加载预备队
		self.pCtuiPreTeamList:ReReloadTableData()
	else
		local LogicPreTeamList = requires(ROOT_PATH, "app/GameLg/Lobby/GameTeam/LogicPreTeamList")
		self.pCtuiPreTeamList = LogicPreTeamList.new()
		self.pCtuiPreTeamList:InitWithInfo(self, LogicPreTeamList:ListType().PreReadTeam, bFlag, self.pShowLayer)
		self.pCtuiPreTeamList:setPositionY( self.pCtuiPreTeamList:getPositionY()  )
		self.pShowLayer:addChild(self.pCtuiPreTeamList, -1)
	end
	-- 不显示技能
	if self.pCallBack ~= nil then
		self.pCallBack:ShowOrHideLayer(false)
	end
	self:SetIsClickPreReadyTeamList(true)
	-- 显示预备队控制队列光标
	self:ShowSeletedCir( self.nSelIdx )
end 

-- 移除预备队
function LogicTeamList:RemovePreReadTeamList(nIndex)
	if self.pCtuiPreTeamList ~= nil then
		self.pCtuiPreTeamList:removeFromParent()
		self.pCtuiPreTeamList = nil 
		self:SetIsClickPreReadyTeamList(false)
		self.pCallBack:ShowOrHideLayer(true)
		if self.pCallBack  then
			if (nIndex) == (MAX_TEAMCOUNT) then 
				nIndex = 0
			end
			self.pCallBack :ResteScrollCountAndIndex(1, nIndex, true)
		end
	end
end

-- 

-- 设置当前的点击的是预备队还是其他
function LogicTeamList:SetIsClickPreReadyTeamList(bIsClick)
	self.isClickPreTeam = bIsClick
end

-- 获取当前点击的类型
function LogicTeamList:GetClickState()
	return self.isClickPreTeam
end

-- 获取预备队对象  是否为nil
function LogicTeamList:GetInstance()
	if self.pCtuiPreTeamList == nil then 
		return false 
	else
		return true
	end
end

-- 统计解锁个数
function LogicTeamList:StaticLockCount()
	-- 已经解锁的
	local nCount = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nHelpCarUnlock
	local nGetTarget = 0;
	for i=1,10 do
		if nCount > 0 then
			nGetTarget = nGetTarget+1;
		end
		nCount = math.floor(nCount/10);
		if nCount <= 0 then
			break;
		end
	end
	-- 获取用户的等级
	local nUserLv = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nLv
	-- 等级到达之后的解锁
	local tnLv = DATA_USER:GetPreLevelUnlockLv()
	-- 查看是否有等级解锁个数
	for i = 1, #tnLv do 
		if tnLv[i] == nUserLv then 
			local tTemLv = {1, 11, 111, 1111, 11111}
			-- 当前级数的索引 如果大于解锁的个数 即当前级数可以解锁
			if i > nGetTarget then 
				nGetTarget = i
				DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nHelpCarUnlock = tTemLv[i]
			end
		end
	end

	return nGetTarget
end

-- 判断当前是否为预备队 并且解锁的个数是否为0 true:是预备对个数为0
function LogicTeamList:IsPreTeamAndUnlock()
	if self.pCtuiPreTeamList ~= nil then 
		local nCount = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nHelpCarUnlock
		if nCount == 0 then 
			return true 
		else
			return false 
		end
	else 
		return false
	end
end

-- 预备对回调
function LogicTeamList:TeamListClickCallBack(nClickIdx)
	if self.pCallBack then 
		self.pCallBack:TeamListClickCallBack(nClickIdx)
	end
end

-- 等级不够 是否用砖石开启 回调到team处理
function LogicTeamList:IsOpenPreTeam(nCellIdx)
	if self.pCallBack then 
		self.pCallBack:IsOpenPreTeam(nCellIdx)
	end
end

-- 点击cell上面的按钮
function LogicTeamList:CTTableViewClickCellButtonByIdx(pTableView, nCellIdx, nButtonType)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()
    -- 移除预备队 
    if self:GetClickState() == true then 
    	-- 解决点击队列为解锁时  关闭预备队列且显示队列车问题
    	self:RemovePreReadTeamList(nCellIdx)
    	return 
    end

    -- 点击按钮写成
    if nButtonType == TeamListUiTag.BtnCellOne then
    	local pArrList = DATAMGR:GetUserInfoMgr():GetCurTeamList().tArrCarList
    	local nCarCount = #pArrList;
    	local nIdx = nCellIdx+1
    	if nIdx > nCarCount then
    		-- 显示预备队layer
    		if nIdx == (MAX_TEAMCOUNT + 1) then 
				-- 回调到LogicTeam中 弹出预备队界面	
				if self.pCallBack then
					-- 这里是阵容的回调函数 idx还需要传递
					self.pCallBack:PreReadyTeamListClickCallBack(nIdx - 1)
				end
    			return;
    		else
    			-- local nCarIdx = {1, 1, 20, 40, 60};
    			local nCarIdxUnlockLvArr = DATAMGR:GetUserInfoMgr():GetRaceCarUnlockLvArr()
    			DIALOGMGR:ShowDialog(DLGTYPE.UnlockOneTeamCar, nCarIdxUnlockLvArr[ nIdx ])
    			return;
    		end
    		
    	end

    	-- 如果有回调，则处理 nIdx == MAX_TEAMCOUNT 
    	if self.pCallBack and self.nSelIdx ~= nCellIdx then
    		-- 如果允许选中效果 nIdx == MAX_TEAMCOUNT:表示预备队有回调
    		self:TeamListClickCallBack(nCellIdx) 
    	end
    end
end
-- 改变尺寸的回调
function LogicTeamList:CTTableViewChangeSize(pTableView, pCell, suNewSize)

end

-- 设置是否是切换阵容的状态
function LogicTeamList:SetStateOfTeam(bFlag)
	self.updateTeam = bFlag
end

-- 获取切换阵容的状态
function LogicTeamList:GeStateOfTeam()
	return self.updateTeam
end

-- 调整位置的回调
function LogicTeamList:ChangeTeamViewChangePos()
	self:SetStateOfTeam(true)
	-- 刷新队伍显示
	self:RefreshShow();
	-- 如果有回调，则处理
	if self.pCallBack then
		-- 如果允许选中效果
		self.pCallBack:TeamListClickCallBack(self.nSelIdx);
	end
end

-- 用来判断一键换阵实例是否存在 如果存在 更新当前的阵容
function LogicTeamList:GetChangeTeamInstace()
	-- 如果是预备队存在  不更新
	if self.pTeamView == nil  or self.pCtuiPreTeamList ~= nil then 
		return false;
	end

	local pTestView = GameViewMgr:shareMgr():GetViewByType(GameViewType.ChangeTeamView)
	if pTestView then
		return true;
	end

	return false
end

-- 用来更新一键换阵车
function LogicTeamList:UpdateChangeTeamCar()
	self.pTeamView:CreateFiveCarBtn()
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function LogicTeamList:LgTeamListTouchBegan(tLocation, nCount)
	self.pTableView:CtScrollViewTouchBegan(tLocation)
	if self.pCtuiPreTeamList ~= nil then 
		self.pCtuiPreTeamList:LgPreReadTeamListTouchBegan(tLocation, nCount)
	end
	return self.pCtuiLayer:CTUITouchBegan(tLocation)
end

-- 触控移动
function LogicTeamList:LgTeamListTouchMoved(tLocation, tPrevPos)
	-- 在教学中，则不给滑动
	if TEACHMGR:IsCurInTeaching() then
		return;
	end
	self.pTableView:CtScrollViewTouchMoved(tLocation, tPrevPos)
	if self.pCtuiPreTeamList ~= nil then 
		self.pCtuiPreTeamList:LgPreReadTeamListTouchMoved(tLocation, tPrevPos)
	end
	
	return self.pCtuiLayer:CTUITouchMoved(tLocation)
end

-- 触控结束
function LogicTeamList:LgTeamListTouchEnded(tLocation, tPrevPos)
	self.pTableView:CtScrollViewTouchEnded(tLocation, tPrevPos)
	if self.pCtuiPreTeamList ~= nil then 
		self.pCtuiPreTeamList:LgPreReadTeamListTouchEnded(tLocation, tPrevPos)
	end

	return self.pCtuiLayer:CTUITouchEnded(tLocation)
end


-- 获取list类型
function LogicTeamList:ListType()
	return TeamListType
end

return LogicTeamList,TeamListType