-- 游戏的基本视图
-- Author: Link
-- Date: 2014-02-08 15:17:02
--

local GameView = class("GameView", function()
	return display.newLayer()
end)

-- 初始化
function GameView:ctor()
	-- 是否
	self.bIsFullScreen	= true
	-- 回调到管理
	self.pViewConMgr 	= nil
	-- 当前类型
	self.nCurGameViewType		= 0
	-- 当前点击不会关闭的范围，之外的会关闭，不为nil表示开启
	self.tTouchOutCloseRect = nil
	-- 是否允许立刻能够点击关闭
	self.bIsOutSideTouchEnable = false;
end
-- mark: ============== 视图继承所用 ================
-- 弹出的动画
function GameView:GetPopUpViewAction(fBaseScale)

	if fBaseScale == nil then
		fBaseScale = 1.0
	end

	local pAct = CCArray:create()
	pAct:addObject( CCScaleTo:create(0.16, 1.2*fBaseScale) )
	-- pAct:addObject( CCScaleTo:create(0.12, 0.9*fBaseScale) )
	-- pAct:addObject( CCScaleTo:create(0.1, 1.1*fBaseScale) )
	pAct:addObject( CCScaleTo:create(0.08, 1.0*fBaseScale) )
	return CCSequence:create(pAct)
end

-- 添加背景
function GameView:ShowViewBg(nType)
	-- 地板的图片
	local pSpBg = CCSprite:create("Images/BG-001.jpg")
	pSpBg:setScaleX(display.width / pSpBg:getContentSize().width)
	pSpBg:setScaleY(display.height / pSpBg:getContentSize().height)
	pSpBg:setPosition(ctprccp(0.5, 0.5))
	self:addChild(pSpBg)
end

-- 弹出结束
function GameView:ShowPopUpViewActionEnd(pView)
	
end

-- 弹出
function GameView:ShowPopUpViewAction(pView, bForceAchor, fBaseScale)
	if bForceAchor == nil or bForceAchor == true then
		pView:ignoreAnchorPointForPosition(false)
		PlatformChangeAnchorPosAndStay(pView, ccp(0.5, 0.5))
	end
	pView:setScale(0)

	local pPopView = self:GetPopUpViewAction(fBaseScale)
	local pEnd = CCCallFuncN:create(function(pEndView)
		self:ShowPopUpViewActionEnd(pEndView)
	end)
	
	pView:runAction(CCSequence:createWithTwoActions(pPopView, pEnd));
end

-- 初始化数据显示
function GameView:InitViewWithData(tDataList)
	
end

-- 重新根据数据显示
function GameView:ReshowWithData(tDataList)
	
end

-- 重新登陆回来的处理
function GameView:ReconnectBack()
	
end

-- View里面的循环使用
function GameView:ViewLoop( dt )
	
end

-- 释放之前调用的
function GameView:BeforeReleaseView()
	-- 回调到管理
	self.pViewConMgr 	= nil
end

-- 对话框的按钮回调信息
function GameView:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
end

-- 设置当前非关闭区域
function GameView:StartCountUncloseRect(pCtuiLayer, nLeftTopTag, nRightBottomTag)
	if pCtuiLayer == nil then
		return
	end
	local tLeftRect = pCtuiLayer:GetNodeAbsShowRect(nLeftTopTag)
	local tRightRect = pCtuiLayer:GetNodeAbsShowRect(nRightBottomTag)
	-- 设置关闭区域
	self.tTouchOutCloseRect = CCRectMake(tLeftRect:getMinX(), tRightRect:getMinY(),
											tRightRect:getMaxX() - tLeftRect:getMinX(),
											tLeftRect:getMaxY() - tRightRect:getMinY())

	self.bIsOutSideTouchEnable = true;
end
-- 重新设置非关闭区域，如果为nil，则取消该功能，一定要是CCRect
function GameView:StartResetUncloseRect(tRect)
	self.tTouchOutCloseRect = tRect
	self.bIsOutSideTouchEnable = true;
end
-- 获取点击关闭的区域
function GameView:GetUncloseRect()
	return self.tTouchOutCloseRect;
end

-- 设置和获取能否点击关闭
function GameView:SetIsOutSideTouchEnable(bEnable)
	self.bIsOutSideTouchEnable = bEnable;
end
function GameView:GetIsOutSideTouchEnable()
	return self.bIsOutSideTouchEnable;
end

-- 关闭当前对话框
function GameView:CloseView()
	-- 删除
	if self.pViewConMgr then
		self.pViewConMgr:CloseOneViewCallBack(self)
	end
end

-- mark: ============== 视图管理所用 ================
-- 设置是否是全屏幕
function GameView:SetIsFullScreen(bIsFull)
	if bIsFull == nil then
		bIsFull = true;
	end
	self.bIsFullScreen = bIsFull
end
-- 获取是否是全屏幕
function GameView:GetIsFullScreen()
	return self.bIsFullScreen
end
-- 设置管理的回调
function GameView:SetViewConMgr(pViewConMgr)
	self.pViewConMgr = pViewConMgr
end
-- 设置当前类型
function GameView:SetCurViewType(nType)
	self.nCurGameViewType 	= nType
end
-- 获取当前类型
function GameView:GetCurViewType()
	return self.nCurGameViewType
end
-- 判断点击外部处理
function GameView:JudgeClickOutsideBegan(tLocation)
	if self.tTouchOutCloseRect == nil then
		return
	end

	-- 是否能点击处理
	if self.bIsOutSideTouchEnable == false then
		return;
	end

	if self.tTouchOutCloseRect:containsPoint(tLocation) == true then
		return
	end

	self:CloseView()
end

-- 弹出来

-- mark: ============== 触控的管理 ================
-- 开始触控
function GameView:ViewTouchBegan(tLocation, nCount)

end

-- 触控移动
function GameView:ViewTouchMoved(tLocation, tPrevPos, tBeganPos)

end

-- 触控结束
function GameView:ViewTouchEnded(tLocation, tPrevPos, tBeganPos)
	
end

return GameView