-- 作用是管理所有弹出的二级及其更加深层的界面
-- Author: Link
-- Date: 2014-02-08 14:56:32
--

local GameViewConMgr = class("GameViewConMgr", function()
	return display.newLayer()
end)

-- 所有子视图类型
GameViewType = {
	-- 车子的详细信息
	CarAllInfo 		= 1,
	-- 车子装备的详细信息
	CarEquipInfo	= 2,
	-- 游戏设置界面信息
	GameSettingInfo	= 3,
	-- 物品商城
	GameItemShop	= 4,
	-- 玩家背包
	UserBag 		= 5,
	-- 强化车子或装备
	MultiSelCard  	= 6,
	-- 选择车子或装备
	SelectCarOrEquip = 7,
	-- 汽车之家
	GetCardPort 	= 8,
	-- 修改队伍阵容
	ChangeTeamView	= 9,
	-- 显示其他玩家详细信息
	ViewOtherTeamInfo = 10,
	-- 升级提示界面
	LevelUpNotice	= 11,
	-- 显示Vs数据类型
	VsInfo 			= 12,
	-- 无尽试炼 弹出奖励界面
	EndlessReward 	= 13,
	-- 百宝箱
	OpenBoxView		= 14,
	-- 百宝箱详情
	OpenBoxHelp		= 15,
	-- 好友列表
	FriendList 		= 16,
	-- 弹出菜单
	PopMenu 		= 17,
	-- 留言板
	SendMessage 	= 18,
	-- 信息列表
	MessageList 	= 19,
	-- 排名列表
	TopList 		= 20,
	-- 物品吃经验之后的弹出框
	ItemAddExp 		= 21,
	-- 活动之免费精力界面
	FreeEnergy 		= 22,
	-- 活动之登录奖励界面
	LogonReward		= 23,
	-- 公告界面
	GongGaoMenu		= 24,
	-- 快捷使用窗口
	QuickUserItem	= 25,
	-- 图鉴
	TuJianView		= 26,
	-- 故事转盘
	StoryTurntable 	= 27,
	-- 玩家基本信息
	PlayerBaseInfo 	= 28,
	-- 弹出高级新卡牌
	PopGoodCardView	= 29,
	-- 玩家升级奖励
	UpgradeReward   = 30, 
	-- 建造选择界面
	BuildChooseView = 31,
	-- 建筑物升级提示
	BuildLevelUp 	= 32,
	-- 合成车
	HeChegnCar		= 33,
	-- 宝箱滚动
	BaoXiangScroll  = 34,
	-- 在线宝箱活动
	OnlineBox  		= 35,
	-- pvp 详情
	PvpDetailInfo	= 36,
	-- 消费大转盘活动
	ExpendTable  	= 37,
	-- 消费大转盘详情
	ExpendTableDetail = 38,
	-- 消费大转盘十连发
	TableTenGet 	= 39,
	-- 充值界面
	ChongZhi 		= 40,
	-- 充值月度活动
	ChongZhiYueDu	= 41,
	-- 每日任务
	DailyTask 		= 42,
	-- Vip详细信息
	VipHelpInfo 	= 43,
	-- Vip的具体使用情况
	VipExchange 	= 44,
	-- 车升星级(进阶)
	CardUpgradeStar	= 45,
	-- 卡牌分解。
	CardDecompose	= 46,
	-- 改造车间的界面
	ViewGaiZaoSel 	= 47,
	-- 剧情物品跳转信息
	ItemAddress		= 48,
	-- 扫荡卡的使用
	SkipStoryCard	= 49,
	-- 招财猫
	LuckyCat		= 50,
}

-- 对外使用的视图管理单例模式
GameViewMgr = {}
function GameViewMgr:shareMgr()
	if self.pMgr == nil then
		self.pMgr = GameViewConMgr.new()
		self.pMgr:retain()
	end
	return self.pMgr
end
function GameViewMgr:ReleaseMgr()
	if self.pMgr ~= nil then
		self.pMgr:release()
		self.pMgr = nil
	end
end

-- 初始化
function GameViewConMgr:ctor()
	-- 所有view的数组
	self.pArrViews = {}

	-- 弱引用回调
	self.pCView = {}
	setmetatable(self.pCView, {__mode = "v"})
end

-- mark: ============== 显示的对外接口 ================
-- 设置显示层次管理
function GameViewConMgr:SetShowLayerMgr(pShowLyMgr)
	self:RemoveAllViewes()
	self.pCView.pShow = pShowLyMgr
end

-- 根据类型和数据来显示视图
function GameViewConMgr:ShowChildViewWithInfo( nViewType, ... )
	-- 获取所有数据
	local tDataList = {}
	for i, v in ipairs{...} do  
 		tDataList[i] = v
 	end

	-- 如果创建成功
	local pShowView = self:JudgeAndSwitchExistViewToTop(nViewType)

	-- 如果切换成功
	if pShowView then
		pShowView:ReshowWithData(tDataList)
		return pShowView
	end

	---- 如果没有切换，则表明，之前没有创建，现在需要创建显示 ---
	local bIsFull = true
	-- 车子的详细信息
	if nViewType == GameViewType.CarAllInfo then
		local ViewCarInfo = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewCarInfo")
		pShowView = ViewCarInfo.new()

	-- 车子装备的信息
	elseif nViewType == GameViewType.CarEquipInfo then
		local ViewCarEquipInfo = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewCarEquipInfo")
		pShowView = ViewCarEquipInfo.new()
		-- 不是全屏显示
		bIsFull = false
	-- 游戏设置信息
	elseif nViewType == GameViewType.GameSettingInfo then
		local ViewGameSetting = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewGameSetting")
		pShowView = ViewGameSetting.new()

	-- 物品商城界面
	elseif nViewType == GameViewType.GameItemShop then
		 local ViewItemShop = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewItemShop")
		 pShowView = ViewItemShop.new()
	-- 玩家背包
	elseif nViewType == GameViewType.UserBag then
		local ViewUserBag = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewUserBag")
		pShowView = ViewUserBag.new()
		
	-- 强化车子或装备
	elseif nViewType == GameViewType.MultiSelCard then
		local ViewMultiSelCard = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewMultiSelCard")
		pShowView = ViewMultiSelCard.new()
		
	-- 选择车子或装备
	elseif nViewType == GameViewType.SelectCarOrEquip then
		local ViewItemSelect = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewItemSelect")
		pShowView = ViewItemSelect.new()

	-- 卡牌抽取
	elseif nViewType == GameViewType.GetCardPort then
		local ViewGetCarPort = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewGetCarPort")
		pShowView = ViewGetCarPort.new()

	-- 阵容切换界面
	elseif nViewType == GameViewType.ChangeTeamView then
		local ViewChangeTeam = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewChangeTeam")
		pShowView = ViewChangeTeam.new()
	-- 查看他人队伍信息
	elseif nViewType == GameViewType.ViewOtherTeamInfo then
		local ViewOtherTeamInfo = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewOtherTeamInfo")
		pShowView = ViewOtherTeamInfo.new()
		-- 不是全屏显示
		bIsFull = false
	-- 升级提示界面
	elseif nViewType == GameViewType.LevelUpNotice then
		local ViewLevelUp = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewLevelUp")
		pShowView = ViewLevelUp.new()
		-- 不是全屏显示
		bIsFull = false
	-- 挑战Vs信息框
	elseif nViewType == GameViewType.VsInfo then
		local ViewVsInfo = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewVsInfo")
		pShowView = ViewVsInfo.new()
		-- 不是全屏显示
		bIsFull = false
	-- 无尽挑战 奖励.
	elseif nViewType == GameViewType.EndlessReward then
		local ViewEndlessReward = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewEndlessReward")
		pShowView = ViewEndlessReward.new()
		-- 不是全屏显示
		bIsFull = false
	-- 开箱子
	elseif nViewType == GameViewType.OpenBoxView then
		local ViewOpenBox = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewOpenBox")
		pShowView = ViewOpenBox.new()
	-- 开箱子的帮助
	elseif nViewType == GameViewType.OpenBoxHelp then
		local ViewOpenBox = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewOpenBoxHelp")
		pShowView = ViewOpenBox.new()
	-- 好友列表
	elseif nViewType == GameViewType.FriendList then
		local ViewFriendList = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewFriendList")
		pShowView = ViewFriendList.new()
	-- 弹出菜单
	elseif nViewType == GameViewType.PopMenu then
		local ViewPopMenu = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewPopMenu")
		pShowView = ViewPopMenu.new()
		-- 不是全屏显示
		bIsFull = false
	-- 留言板
	elseif nViewType == GameViewType.SendMessage then
		local ViewSendMessage = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewSendMessage")
		pShowView = ViewSendMessage.new()
		-- 不是全屏显示
		bIsFull = false
	-- 信息列表
	elseif nViewType == GameViewType.MessageList then
		local ViewMessage = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewMessage")
		pShowView = ViewMessage.new()
	-- 排行榜列表
	elseif nViewType == GameViewType.TopList then
		local ViewMessage = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewTopList")
		pShowView = ViewMessage.new()
	-- 物品吃经验之后的弹出框
	elseif nViewType == GameViewType.ItemAddExp then
		local ViewItemAddExp = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewItemAddExp")
		pShowView = ViewItemAddExp.new()
		-- 不是全屏显示
		bIsFull = false
	-- 活动之免费精力UI显示
	elseif nViewType == GameViewType.FreeEnergy then
		local ViewFreeEnergy = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewFreeEnergy")
		pShowView = ViewFreeEnergy.new()
		-- 不是全屏显示
		bIsFull = false
	-- 活动之签到UI显示
	elseif nViewType == GameViewType.LogonReward then
		local ViewLogonReward = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewLogonReward")
		pShowView = ViewLogonReward.new()
		-- 不是全屏显示
		bIsFull = false
	-- 公告显示
	elseif nViewType == GameViewType.GongGaoMenu then
		local ViewGonggao = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewGonggao")
		pShowView = ViewGonggao.new()
		-- 不是全屏显示
		bIsFull = false
	-- 快捷使用窗口
	elseif nViewType == GameViewType.QuickUserItem then
		local ViewQuickUseItem = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewQuickUseItem")
		pShowView = ViewQuickUseItem.new()
		-- 不是全屏显示
		bIsFull = false
	-- 图鉴
	elseif nViewType == GameViewType.TuJianView then
		local ViewTuJian = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewTuJian")
		pShowView = ViewTuJian.new()
	-- 故事转盘
	elseif nViewType == GameViewType.StoryTurntable then
		local ViewStoryTurntable = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewStoryTurntable")
		pShowView = ViewStoryTurntable.new()
		-- 不是全屏显示
		bIsFull = false
	-- 玩家基本信息
	elseif nViewType == GameViewType.PlayerBaseInfo then
		local ViewPlayerInfo = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewPlayerInfo")
		pShowView = ViewPlayerInfo.new()
		-- 不是全屏显示
		bIsFull = false
	-- 弹出高级新卡牌
	elseif nViewType == GameViewType.PopGoodCardView then
		local ViewNewGoodCards = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewNewGoodCards")
		pShowView = ViewNewGoodCards.new()
		-- 不是全屏显示
		bIsFull = false

	elseif nViewType == GameViewType.UpgradeReward then
		local ViewUpgradeReward = requires(ROOT_PATH , "app/GameLg/Lobby/Others/ViewUpgradeReward")
		pShowView = ViewUpgradeReward.new()
	-- 建造选择界面
	elseif nViewType == GameViewType.BuildChooseView then
		local ViewChooseBuild = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewChooseBuild")
		pShowView = ViewChooseBuild.new()
	-- 建筑物升级提示
	elseif nViewType == GameViewType.BuildLevelUp then
		local ViewBuildingLevelUp = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewBuildingLevelUp")
		pShowView = ViewBuildingLevelUp.new()
		-- 不是全屏显示
		bIsFull = false
	-- 汽车合成 
	elseif nViewType == GameViewType.HeChegnCar then
		local ViewViewHeChengCar = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewHeChengCar")
		pShowView = ViewViewHeChengCar.new()
	-- 宝箱滚动  
	elseif nViewType == GameViewType.BaoXiangScroll then
		local ViewBaoXiangScroll = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewOpenBoxChouJiang")
		pShowView = ViewBaoXiangScroll.new()
		bIsFull = false
	-- 在线宝箱活动 
	elseif nViewType == GameViewType.OnlineBox then
		local ViewActOnlineBox = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewActOnlineBox")
		pShowView = ViewActOnlineBox.new()
		bIsFull = false
	-- pvp详情
	elseif nViewType == GameViewType.PvpDetailInfo then
		local ViewLogicPvpDetailInfo = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewLogicPvpDetailInfo")
		pShowView = ViewLogicPvpDetailInfo.new()
		bIsFull = false
	-- 消费大转盘
	elseif nViewType == GameViewType.ExpendTable then
		local ViewExpendTable = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewExpendTable")
		pShowView = ViewExpendTable.new()

	-- 消费大转盘详情
	elseif nViewType == GameViewType.ExpendTableDetail then
		local ViewExpendTableDetail = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewExpendTableDetail")
		pShowView = ViewExpendTableDetail.new()

		bIsFull = false

	-- 消费大转盘十连发
	elseif nViewType == GameViewType.TableTenGet then
		local ViewTableTenGet = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewTableTenGet")
		pShowView = ViewTableTenGet.new()

		bIsFull = false

	-- 充值界面
	elseif nViewType == GameViewType.ChongZhi then
		local ViewDiamondShop = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewDiamondShop")
		pShowView = ViewDiamondShop.new()
	-- 充值月度活动
	elseif nViewType == GameViewType.ChongZhiYueDu then
		local ViewDiamondMouth = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewDiamondMouth")
		pShowView = ViewDiamondMouth.new()

		bIsFull = false

	-- 每日任务
	elseif nViewType == GameViewType.DailyTask then
		local ViewDailyTask = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewDailyTask")
		pShowView = ViewDailyTask.new()

	-- VIP详细信息
	elseif nViewType == GameViewType.VipHelpInfo then
		local ViewVipHelpInfo = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewVipHelpInfo")
		pShowView = ViewVipHelpInfo.new()
		-- 不是全屏显示
		bIsFull = false

	-- Vip的具体使用情况
	elseif nViewType == GameViewType.VipExchange then
		local ViewVipExchange = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewVipExchange")
		pShowView = ViewVipExchange.new()

	-- 卡牌提升星级。
	elseif nViewType == GameViewType.CardUpgradeStar then
		local ViewCardUpgradeStar = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewCardUpgradeStar")
		pShowView = ViewCardUpgradeStar.new()
	
	-- 卡牌分解碎片。
	elseif nViewType == GameViewType.CardDecompose then
		local ViewCardDecompose = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewCardDecompose")
		pShowView = ViewCardDecompose.new()
	
	-- 改造车间的界面
	elseif nViewType == GameViewType.ViewGaiZaoSel then
		local ViewGaiZaoSel = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewGaiZaoSel")
		pShowView = ViewGaiZaoSel.new()

	-- 改造车间的界面
	elseif nViewType == GameViewType.ItemAddress then
		local ViewStoryItemAddress = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewStoryItemAddress")
		pShowView = ViewStoryItemAddress.new()
		bIsFull = false

	-- 扫荡卡的使用
	elseif nViewType == GameViewType.SkipStoryCard then
		local ViewSkipStory = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewSkipStory")
		pShowView = ViewSkipStory.new()
		bIsFull = false

	-- 招财猫 
	elseif nViewType == GameViewType.LuckyCat then
		local ViewLuckyCat = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewLuckyCat")
		pShowView = ViewLuckyCat.new()
		bIsFull = false
	end

	-- 添加进入
	pShowView:InitViewWithData( tDataList )
	if self.pCView.pShow ~= nil then
		self:PushOneView(pShowView, bIsFull, nViewType)
	else
		CTAssert(0, "添加视图是，显示管理还没有设置")
	end
	
	return pShowView
end

-- mark: ============== 外部管理方法 ================
-- 将所有视图全部删除掉
function GameViewConMgr:RemoveAllViewes()
	-- 循环当前所有视图
	for i,pOneView in ipairs(self.pArrViews) do
		pOneView:BeforeReleaseView()
		pOneView:removeFromParent()
	end
	-- 将所有视图删除出栈
	self.pArrViews = {}
end

-- 获取顶部的视图
function GameViewConMgr:GetTopView()
	-- 查看还有多少view
	local nViewCount = #(self.pArrViews)
	if nViewCount <= 0 then
		return nil
	end
	-- 取出最后一个视图进行操作
	return self.pArrViews[nViewCount]
end
-- 根据类型获取视图
function GameViewConMgr:GetViewByType(nType)
	-- 从最后一个开始算
	local nViewIdx = #self.pArrViews
	while nViewIdx > 0 do
		local pView = self.pArrViews[nViewIdx]
		if pView:GetCurViewType() == nType then
			return pView
		end
		nViewIdx = nViewIdx-1
	end
	return nil
end

-- 判断是否有视图要控制
function GameViewConMgr:IsHadShowView()
	local nViewCount = #(self.pArrViews)
	if nViewCount > 0 then
		return true
	end
	return false
end

-- 当前是否显示的是弹出的小窗口
function GameViewConMgr:IsCurShowPopView()
	local pShowView = self:GetTopView();
	if pShowView == nil then
		return false
	end
	if pShowView:GetIsFullScreen() == false then
		return true;
	end
	return false
end

-- 循环处理
function GameViewConMgr:ViewConLoop(dt)
	-- -- 取出最后一个视图进行操作
	-- local pLastView = self:GetTopView()
	-- if pLastView == nil or pLastView.ViewLoop == nil then
	-- 	return
	-- end
	-- pLastView:ViewLoop(dt)

	-- 循环当前所有视图
	for i,pOneView in ipairs(self.pArrViews) do
		pOneView:ViewLoop(dt)
	end
end

-- mark: ============== 内部方法 ================
-- 添加视图到显示上面去,bIsFull表示是否是全屏显示
function GameViewConMgr:PushOneView(pOneView, bIsFull, nType)
	if pOneView == nil then
		CTAssert(0, "添加一个View失败，View为空！")
		return
	end

	-- 查看还有多少view
	local nViewCount = #(self.pArrViews)

	-- 如果是全屏，则将底下的全部处理掉
	if bIsFull == true then
		-- 将底下的全部隐藏出来
		while nViewCount >= 1 do
			local pView = self.pArrViews[nViewCount]
			if pView:isVisible() == false then
				break
			end
			pView:setVisible(false)
			nViewCount = nViewCount-1
		end
	end
	
	nViewCount = #(self.pArrViews)

	-- 设置view的属性
	pOneView:SetIsFullScreen(bIsFull)
	pOneView:SetViewConMgr(self)
	pOneView:SetCurViewType(nType)
	-- 添加显示
	if bIsFull == true then
		self.pCView.pShow:addChild(pOneView, GLobbyZOrder.PopView)
	else
		self.pCView.pShow:addChild(pOneView, GLobbyZOrder.TopViewLy)
	end
	

	-- 添加到栈中
	self.pArrViews[nViewCount+1] = pOneView
end

-- 将最上面的视图出栈，删除掉
function GameViewConMgr:PopOneView()
	-- 取出最后一个视图进行操作
	local pLastView = self:GetTopView()
	if pLastView == nil then
		return
	end
	self:CloseOneView(pLastView)
	-- -- 删除出显示
	-- pLastView:removeFromParent()
	-- -- 删除出栈
	-- local nViewCount = #self.pArrViews
	-- self.pArrViews[nViewCount] = nil
	-- nViewCount = nViewCount-1

	-- -- 将底下的显示出来
	-- while nViewCount >= 1 do
	-- 	local pOneView = self.pArrViews[nViewCount]
	-- 	pOneView:setVisible(true)
	-- 	if pOneView:GetIsFullScreen() == true then
	-- 		break
	-- 	end
	-- 	nViewCount = nViewCount-1
	-- end
end

-- 删除一个对应的视图
function GameViewConMgr:CloseOneView( pView )
	local nViewCount = #self.pArrViews
	local nTmpCount = nViewCount

	local bIsLast = false

	while nTmpCount >= 1 do
		local pOneView = self.pArrViews[nTmpCount]
		-- 如果类型相同
		if pOneView:GetCurViewType() == pView:GetCurViewType() and pView == pOneView then
			if nTmpCount == nViewCount then
				bIsLast = true
			end
			pOneView:BeforeReleaseView()
			pOneView:removeFromParent()
			-- 删除掉
			table.remove(self.pArrViews, nTmpCount)
			break
		end
		nTmpCount = nTmpCount-1
	end

	-- 如果删除的是最后一个
	if bIsLast == true then
		-- 重新获取数目
		nViewCount = #self.pArrViews
		-- 将底下的显示出来
		while nViewCount >= 1 do
			local pOneView = self.pArrViews[nViewCount]
			pOneView:setVisible(true)
			if pOneView:GetIsFullScreen() == true then
				break
			end
			nViewCount = nViewCount-1
		end
	end
end

-- 关闭一个视图时的回调
function GameViewConMgr:CloseOneViewCallBack(pView)
	-- 删除当前视图
	self:CloseOneView(pView)
end

-- 判断是否有同样的视图在里面显示
function GameViewConMgr:JudgeAndSwitchExistViewToTop( nViewType )
	-- 找到相同视图
	local nIdx = -1
	for i,pOneView in ipairs(self.pArrViews) do
		if pOneView:GetCurViewType() == nViewType then
			nIdx = i
			break
		end
	end

	-- 如果找到索引
	if nIdx > 0 then
		-- 获取视图
		local pGetView = self.pArrViews[nIdx]

		-- 将所有视图向前移动
		local nViewCount = #(self.pArrViews)
		-- 如果是最顶上的那一个，则不需要如此重复处理
		if nIdx == nViewCount then
			return pGetView
		end

		for i=nIdx,(nViewCount-1) do
			self.pArrViews[i] = self.pArrViews[i+1]
		end
		self.pArrViews[nViewCount] = nil

		-- 将获取出来的置顶
		pGetView:setVisible(true)
		pGetView:retain()
		pGetView:removeFromParent()

		self:PushOneView(pGetView, pGetView:GetIsFullScreen(), nViewType)

		pGetView:release()

		return pGetView
	end

	return nil
end

-- 对话框的按钮回调信息
function GameViewConMgr:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
	-- 取出最后一个视图进行操作
	local pLastView = self:GetTopView()
	if pLastView == nil or pLastView.CTDialogClickButton == nil then
		return
	end
	pLastView:CTDialogClickButton(nBtnTag, nBtnIdx, tData)
end

-- 重新登陆回来的处理
function GameViewConMgr:ReconnectBack()
	-- 循环当前所有视图
	for i,pOneView in ipairs(self.pArrViews) do
		pOneView:ReconnectBack()
	end
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function GameViewConMgr:ViewMgrTouchBegan(tLocation, nCount)
	-- 取出最后一个视图进行操作
	local pLastView = self:GetTopView()
	if pLastView == nil then
		return
	end
	pLastView:ViewTouchBegan(tLocation, nCount)
	pLastView:JudgeClickOutsideBegan(tLocation)
end

-- 触控移动
function GameViewConMgr:ViewMgrTouchMoved(tLocation, tPrevPos, tBeganPos)
	-- 取出最后一个视图进行操作
	local pLastView = self:GetTopView()
	if pLastView == nil then
		return
	end
	pLastView:ViewTouchMoved(tLocation, tPrevPos, tBeganPos)
end

-- 触控结束
function GameViewConMgr:ViewMgrTouchEnded(tLocation, tPrevPos, tBeganPos)
	-- 取出最后一个视图进行操作
	local pLastView = self:GetTopView()
	if pLastView == nil then
		return
	end
	pLastView:ViewTouchEnded(tLocation, tPrevPos, tBeganPos)
end
