--
-- Author: NickWang
-- Date: 2014-08-06 14:22:00
--

-- 基础视图
local GameView = requires(ROOT_PATH, "app/GameLg/Lobby/Others/GameView")

-- 界面视图
local ViewExpendTable = class("ViewExpendTable", GameView)

-- ctui tag值
local Tag_ExpendTable = {
	Sp_Bg_Table = 58, -- 转盘
	-- btn
	Btn_Close 	= 10, -- 关闭按钮
	Btn_1Start 	= 59, -- 1次开始按钮
	Sp_1StartNormal = 11, -- 1次开始按钮的常规图片 --
	-- Sp_1StartSelect = 8, -- 1次开始按钮的选中图片
	Btn_10Start 	= 12, -- 10次开始按钮
	Sp_SuperScore 	= 52, -- 超级大奖的奖池数字
	Sp_SuperName 	= 48, -- 超级大奖的名称
	Lb_EndCDTime	= 8, -- 活动结束的倒计时。
	Lb_DiamondNum 	= 7, -- 用户钻石数。
	Btn_Detail 		= 74, -- 详情按钮。
	Lb_MyScore 		= 65, -- 我的积分。
	Btn_ReflashTable 	= 53, -- 刷新转盘物品按钮。
	Lb_ReflashTableCD	= 56, -- 刷新转盘物品时间。

	-- 奖励的图标
	Sp_RewardItem1 	= 16,	-- 第1个图标
	Sp_RewardItem2 	= 18,	-- 第2个图标
	Sp_RewardItem3 	= 20,	-- 第3个图标
	Sp_RewardItem4 	= 34,	-- 第4个图标
	Sp_RewardItem5 	= 36,	-- 第5个图标
	Sp_RewardItem6 	= 32,	-- 第6个图标
	Sp_RewardItem7 	= 30,	-- 第7个图标
	Sp_RewardItem8 	= 28,	-- 第8个图标
	Sp_RewardItem9 	= 26,	-- 第9个图标
	Sp_RewardItem10	= 24,	-- 第10个图标
	Sp_RewardItem11	= 22,	-- 第11个图标
	Sp_RewardItem12	= 14,	-- 第12个图标

	-- 奖励的底图
	Sp_RewardBot1 	= 15,	-- 第1个底图
	Sp_RewardBot2 	= 17,	-- 第2个底图
	Sp_RewardBot3 	= 19,	-- 第3个底图
	Sp_RewardBot4 	= 33,	-- 第4个底图
	Sp_RewardBot5 	= 35,	-- 第5个底图
	Sp_RewardBot6 	= 31,	-- 第6个底图
	Sp_RewardBot7 	= 29,	-- 第7个底图
	Sp_RewardBot8 	= 27,	-- 第8个底图
	Sp_RewardBot9 	= 25,	-- 第9个底图
	Sp_RewardBot10	= 23,	-- 第10个底图
	Sp_RewardBot11	= 21,	-- 第11个底图
	Sp_RewardBot12	= 13,	-- 第12个底图

	-- table Cell
	Cell_Ly = 66, -- 底板
	Cell_Lb_Nick = 68, -- 昵称
	Cell_Lb_TotalCnt = 69, -- 获得的物品
}

-- 转盘转动的参数（先转动2圈，第3圈确定奖励。）。
local DATA_TURN = {
	-- 最大转动次数
	MAX_TURN_CNT = 3,
	-- 最大停留时间（每个物品上）
	MAX_DELAY_TIME = 0.3,
	-- 最小停留时间（每个物品上）
	MIN_DELAY_TIME = 0.02,
	-- 时间间隔增减值（每个物品上）
	DELTA_ADD_TIME = 0.05,
	-- 转动一圈需要的物品总个数
	DELTA_ONE_CIRCLE_MAX = 12,
}

-- init
function ViewExpendTable:ctor()
	-- super init
 	self.super.ctor(self)

 	-- 获取活动配置数据.
	self.tableConfig = DATAMGR:GetActivityMgr():GetDataConfigWithExpendTable()
	-- print("table's size: ", #self.tableConfig)
	self.localIdx = 0

	self:initTurnTableVariable()

	local nOldDiamond = -1;

	-- 背景颜色
	local pLyColor = CCLayerColor:create( GRAY_BG_COLOR )
	self:addChild(pLyColor)

	-- 地板的图片
	self:ShowViewBg()

 	-- 当前界面
	self.pCtuiLayer = CTUILayer:new();
	self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/HuoDongZhuanPan.ctui"))
	self.pCtuiLayer:setAnchorPoint(ccp(0.5, 1.0))
	self.pCtuiLayer:setPosition(ctprccp(0.5, 1.0))
	self.pCtuiLayer:ignoreAnchorPointForPosition(false)
	self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	self.pCtuiLayer:ShowAllUiInfo()
	self:addChild(self.pCtuiLayer)
	self.pCtuiLayer:release()

	-- 点击事件设置.
	self.pCtuiLayer:setDelegateLua(function(event)
	-- 如果是按钮点击事件
		if event.cbType == "btnCb" then
			self:CTUIButtonCallBack(event.Layer, event.Btn, event.Tag)
		end
	end)

	-- 初始化界面UI
	self:initTableUI()

	-- self:StartCountUncloseRect(self.pCtuiLayer, Tag_ExpendTable.Sp_Bottom, Tag_ExpendTable.Sp_Bottom)

	-- 将ctui里的cell隐藏不显示。
	local pCellLayer = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Cell_Ly)
	pCellLayer:setVisible(false)

	-- table起始位置
	local fTabelPosy = 130
	-- cell的起始位置
	self.nCellPosy = -210
	self.nShowType = SHARE_FUNC.nFrameType
	-- ipad
  if self.nShowType == CTDeviceTypeIphone5HD or self.nShowType == CTDeviceTypeItouch5HD then
    	fTabelPosy = 150
    	self.nCellPosy = -310
    end


	-- 获取表格显示区域
	local pSpTopBottom = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Sp_Bg_Table)
	-- pSpTopBottom:setZOrder(0)
	local tTopRect = ctspbound(pSpTopBottom)
	local tBottomPos = self.pCtuiLayer:convertToNodeSpace( ccp(0, SHARE_FUNC.bottomTagH) )
	local tBottomRect = CCRectMake(0, fTabelPosy, ctprwidth(1.3), tTopRect:getMinY()-fTabelPosy)
	-- local tBottomRect = CCRectMake(0, 130, ctprwidth(1.3), 120)
	-- local tBottomRect = CCRectMake(0, 130, ctprwidth(1.3), 120)
	local cellNormalSize = CCSizeMake(1024, 50)
	-- print("rect: , ", tBottomRect.size.width, tBottomRect.size.height)
	-- print("cellSize: ", cellNormalSize.width, cellNormalSize.height)

	-- local pLyColor = CCLayerColor:create(ccc4(255, 0, 0, 60), tBottomRect.size.width, tBottomRect.size.height)
	-- -- pLyColor:ignoreAnchorPointForPosition(false)
	-- pLyColor:setPosition(ccp(tBottomRect.origin.x, tBottomRect.origin.y))
	-- self:addChild(pLyColor)

	-- 表格的回调
	local function TabelViewCallBack( event )
		-- 表格总数
		if event.type == "count" then
			return self:CTTableViewShowCellCount(event.table)

		-- 表格的key
		elseif event.type == "key" then
			return self:CTTableViewCellByIdx(event.table, event.Idx)

		-- 创建
		elseif event.type == "Create" then
			return self:CTTableViewCreateNewCellByIdx(event.table, event.Idx, event.key)

		-- 显示具体信息
		elseif event.type == "showCell" then
			self:CTTableViewShowToExistCellByIdx(event.table, event.cell)

		-- 改变尺寸的回调中
		elseif event.type == "ReSize" then
			local tSize = CCSizeMake(event.width, event.height)
			self:CTTableViewChangeSize(event.table, event.cell, tSize)

		-- cell上面点中按钮的回调
		elseif event.type == "Btn" then
			self:CTTableViewClickCellButtonByIdx(event.table, event.Idx, event.BtnType)

		-- 在cell上面点击的回调
		elseif event.type == "Click" then
			self:CTTableViewClickCell(event.table, event.cell)
		end
	end
	-- table列表
	self.pTableList = CTTableView:new()
	self.pTableList:initWithFrameLua(tBottomRect, TabelViewCallBack)
	self.pTableList:SetNormalCellSize(cellNormalSize)
	self.pTableList:setDelSpeedRate(SHARE_FUNC.fScorllDelSpeedRate)
	self.pCtuiLayer:addChild(self.pTableList, 1)
	self.pTableList:release()
	self.pTableList:reloadTableData()
	-- print("table anchorPos: ", self.pTableList:getAnchorPoint().x, self.pTableList:getAnchorPoint().y)
	-- print("table Pos: ", self.pTableList:getPositionX(), self.pTableList:getPositionY())
	-- print("table size: ", self.pTableList:getContentSize().width, self.pTableList:getContentSize().height)

	-- self:ShowPopUpViewAction(self.pCtuiLayer)

	-- 获取活动信息。
	-- SHARE_FUNC:GetMainLogic():GetLobbyLogic():SendGetTempActInfo(ACT_EXPEND_TABLE_ID)
	self:SendGetActInfo(true)
end

-- 初始化转盘数据
function ViewExpendTable:initTurnTableVariable()
	-- 更新活动信息倒计时
	self.getActCDTime = 2.0
	-- 转动开始
	self.turnStart = false
	-- 转动计时
	self.turnTime = 0
	-- 转动每个物品上的停留时间。
	self.turnOneTime = 0
	-- 转动的圈数
	self.turnCircleCnt = 0
	-- 转动一圈需要的个数。
	self.turnObjectCnt = 0
	-- 积分topX的信息。
	self.tTopInfoList = {}
	-- for i=1,10 do
	-- 	local oneInfo = {}
	-- 	oneInfo[1] = i
	-- 	oneInfo[2] = "我是"..i
	-- 	oneInfo[3] = 100 - i
	-- 	self.tTopInfoList[i] = oneInfo
	-- end
	-- print2json(self.tTopInfoList)

	-- 新消息列表。
	self.newMsgList = {}
	-- local msg = "添加用户，给数据库.."
	-- self.newMsgList[#self.newMsgList+1] = msg
	-- local msg = "广东省深圳市 电信."
	-- self.newMsgList[#self.newMsgList+1] = msg
	-- local msg = "ip地址常见问题解决【图文】百度经验"
	-- self.newMsgList[#self.newMsgList+1] = msg
	-- local msg = "经常上站长工具可以了解SEO数据变化."
	-- self.newMsgList[#self.newMsgList+1] = msg
	-- local msg = "在这里跟大家分享怎样设置IP地址."
	-- self.newMsgList[#self.newMsgList+1] = msg
	self.fMsgPauseTime = 0 -- 暂停时间。
end

-- mark: ============== 按钮的回调 ================
-- 对话框的按钮回调信息
function ViewExpendTable:CTDialogClickButton( nBtnTag, nBtnIdx, tData)
	-- 钻石购买
	if nBtnTag == DLGTYPE.RefreshExpendTableItems then
		if nBtnIdx == 1 then
			self:SendRefreshItemList()
		end
	end
end
function ViewExpendTable:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()
    
	-- 查看信息
	if pUiLayer == self.pCtuiLayer then
		if self.turnStart == true then

		elseif nTag == Tag_ExpendTable.Btn_Close then
			self:CloseView()
		elseif nTag == Tag_ExpendTable.Btn_1Start then
			-- 转动一次。
			-- 发送转动一次请求。
			local tExpendTableAct = DATAMGR:GetActivityMgr():GetActivityDataByID(ACT_EXPEND_TABLE_ID)
			local nFree = 0
			if tExpendTableAct.tData.nFreeCDTime <= 0 then
				nFree = 1
			end

			if nFree == 0 and self.tableConfig.needDiamondPerTurn > DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamodNum then
				-- 钻石不足。
				DIALOGMGR:ShowDialog(DLGTYPE.LowDiamond)
			else
				-- 发送信息。
				self:getParent():SendTurnExpendTable(ACT_EXPEND_TABLE_ID, 1, nFree)
				-- 显示减少后的钻石数
				-- local nDiamondCnt = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamodNum - 1 * self.tableConfig.needDiamondPerTurn
				-- self:updateUserDiamondNum(nDiamondCnt)
			end
		elseif nTag == Tag_ExpendTable.Btn_10Start then
			-- 十连发。
			if self.tableConfig.needDiamondPerTurn > DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamodNum * 10 then
				-- 钻石不足。
				DIALOGMGR:ShowDialog(DLGTYPE.LowDiamond)
			else
				-- 发送转动十连发请求。
				self:getParent():SendTurnExpendTable(ACT_EXPEND_TABLE_ID, 10, 0)
				-- 显示减少后的钻石数
				-- local nDiamondCnt = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamodNum - 10 * self.tableConfig.needDiamondPerTurn
				-- self:updateUserDiamondNum(nDiamondCnt)
			end
		elseif nTag == Tag_ExpendTable.Btn_Detail then
			-- 详情。
			self:initViewTableDetail()
		elseif nTag == Tag_ExpendTable.Btn_ReflashTable then
			-- 刷新大转盘物品列表。
			local tExpendTableAct = DATAMGR:GetActivityMgr():GetActivityDataByID(ACT_EXPEND_TABLE_ID)
			if tExpendTableAct.tData.nRefreshCDTime <= 0 then
				-- 免费刷新，发送。
				self:SendRefreshItemList()
			elseif self.tableConfig.needDiamondPerRefresh > DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamodNum then
				-- 钻石不足。
				DIALOGMGR:ShowDialog(DLGTYPE.LowDiamond)
			else
				-- 钻石刷新弹出框。
				DIALOGMGR:ShowDialog(DLGTYPE.RefreshExpendTableItems, self.tableConfig.needDiamondPerRefresh, MoneyType.Diamond)
			end
		else
			DIALOGMGR:ShowToast("该功能还没有开放！！")
		end
	end
end

-- mark: ============== 逻辑管理 ================
-- 初始化UI
function ViewExpendTable:initTableUI()
	local pUserBaseInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()
	
	-- 显示物品初始显示比例。
	local pSpItemIcon = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Sp_RewardBot1)
	self.itemScale = pSpItemIcon:getScale()

	-- 更新钻石数
	self:updateUserDiamondNum(pUserBaseInfo.nDiamodNum)

	-- 初始化超级大奖
	local pLbScore = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Sp_SuperScore)
	tolua.cast(pLbScore, "CCSprite")
	pLbScore:setVisible(false)
	-- self.pLbSuperScore = CCLabelAtlas:create("", "Images/GX-064.png", 40, 39, string.byte('0'))
	-- self.pLbSuperScore:setScale(0.76)

	self.pLbSuperScore = CTImgLabel:new()
	self.pLbSuperScore:InitLabel("", Public_GetResPathByKey("Images/Font/PlusShuZhi.json"), 40)
	self.pLbSuperScore:setFontSize(28)

	self.pLbSuperScore:setPosition(pLbScore:getPosition())
	self.pLbSuperScore:setAnchorPoint(pLbScore:getAnchorPoint())
	pLbScore:getParent():addChild(self.pLbSuperScore)
	self.pLbSuperScore:release()
	self:updateSuperScoreNum(0)

	-- 超级大奖动画。
	local pSpSuperReward = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Sp_SuperName)
	tolua.cast(pLbScore, "CCSprite")
	-- 
	local fLocalScale = 1.0 * pSpSuperReward:getScale()
	pSpSuperReward:setScale(0)
	local pArray = CCArray:create()
	pArray:addObject(CCScaleTo:create(0.2, 1.1*fLocalScale))
	pArray:addObject(CCScaleTo:create(0.1, 1.0*fLocalScale))
	pArray:addObject(CCScaleTo:create(0.1, 1.1*fLocalScale))
	pArray:addObject(CCScaleTo:create(0.1, 1.0*fLocalScale))
	pArray:addObject(CCDelayTime:create(2))

	local pAct = CCRepeatForever:create(CCSequence:create(pArray))
	pSpSuperReward:runAction(pAct)
end

-- 更新转盘物品列表。
function ViewExpendTable:updateTableShowItemList()

	local nItemIconTagList = {
		Tag_ExpendTable.Sp_RewardItem1,
		Tag_ExpendTable.Sp_RewardItem2,
		Tag_ExpendTable.Sp_RewardItem3,
		Tag_ExpendTable.Sp_RewardItem4,
		Tag_ExpendTable.Sp_RewardItem5,
		Tag_ExpendTable.Sp_RewardItem6,
		Tag_ExpendTable.Sp_RewardItem7,
		Tag_ExpendTable.Sp_RewardItem8,
		Tag_ExpendTable.Sp_RewardItem9,
		Tag_ExpendTable.Sp_RewardItem10,
		Tag_ExpendTable.Sp_RewardItem11,
		Tag_ExpendTable.Sp_RewardItem12,
	}
	local nItemCount = #nItemIconTagList


	-- 物品列表。
	for i=1,nItemCount do
		local pSpItemIcon = self.pCtuiLayer:GetNodeByTag( nItemIconTagList[i] )
		tolua.cast(pSpItemIcon, "CCSprite")

		if self.localIdx == i-1 then
			-- 选中物品
			pSpItemIcon:setScale(self.itemScale * 1.05)
		else
			-- 非选中物品
			pSpItemIcon:setScale(self.itemScale * 0.8)
		end

		

		local tTableActData = DATAMGR:GetActivityMgr():GetActivityDataByID(ACT_EXPEND_TABLE_ID)
		local item = tTableActData.tData.tItemList[i]
		local pSpFrame = DATAMGR:GetItemInfoMgr():GetItemIconNameByID(item.itemId)
		if pSpFrame ~= nil then
			pSpItemIcon:setDisplayFrame(pSpFrame)
		end

		-- 删除遗留的显示
		pSpItemIcon:removeAllChildrenWithCleanup(true)
		-- 如果是碎片。
		if DATAMGR:GetItemInfoMgr():JudgeIsPieceIdWithId(item.itemId) == true then
			local pSpPiece = CCSprite:create("Images/GX-068.png");
			pSpPiece:setPosition(ctspccp(pSpItemIcon, 0.22, 0.22))
			pSpItemIcon:addChild(pSpPiece)
		-- 如果是钻石。
		elseif item.itemId == ITEM_ID_DIAMOND then
			-- print2json(item)
			-- local item = self.tableConfig.itemList[i]
			-- 显示钻石个数
			local pLbItemNum = CCLabelTTF:create("", LABEL_BOLD_FONT_NAME, 40)
			pLbItemNum:setPosition(ccp(pSpItemIcon:getContentSize().width/2, pSpItemIcon:getContentSize().height/4))
			pLbItemNum:setString(""..item.itemCnt)
			pLbItemNum:setColor(ccc3(255, 255, 0))
			pSpItemIcon:addChild(pLbItemNum)
		end
	end
end

-- 更新用户钻石显示。
function ViewExpendTable:updateUserDiamondNum(nDiamondCnt)
	-- 
	local pLbDiamondNum = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Lb_DiamondNum)
	tolua.cast(pLbDiamondNum, "CCLabelTTF")
	pLbDiamondNum:setString( Public_GetNumShowType(nDiamondCnt, NumberShowType.ShortWan) )
end

-- 更新活动结束的倒计时。
function ViewExpendTable:updateActEndCDTime()
	-- 获取倒计时时间（秒）
	local endTime = DATAMGR:GetActivityMgr():GetActivityEndTimeSecond(ACT_EXPEND_TABLE_ID)
	local nowTime = DATAMGR:GetActivityMgr():GetCurServerTimeSecond()
	-- print("endTime: "..endTime..", nowTime: "..nowTime)
	local timeSecond = endTime - nowTime
	if timeSecond < 0 then
		timeSecond = 0
	end
	
	-- 时间的string.
	local timeString = ""
	-- 显示（小时：分钟：秒）
	local s = timeSecond%60
	local m = timeSecond/60%60
	local h = timeSecond/3600
	timeString = string.format("%d:%.2d:%.2d", math.floor(h), math.floor(m), math.floor(s))

	-- 显示。
	local pLbCDTime = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Lb_EndCDTime)
	tolua.cast(pLbCDTime, "CCLabelTTF")
	pLbCDTime:setString(timeString)
end

-- 更新超级大奖数字。
function ViewExpendTable:updateSuperScoreNum(nScore)
	-- 显示。
	-- local pLbSuperScore = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Sp_SuperScore)
	-- tolua.cast(pLbSuperScore, "CCLabelTTF")
	-- pLbSuperScore:setString(""..nScore)

	-- self.randomLabel = CCLabelAtlas:create("", "Images/ZP_ShuZi.png", 25, 33, string.byte('0'))
	-- self.randomLabel:setScale(0.7)
	-- self.BackGround:addChild(self.randomLabel, 2)
	self.pLbSuperScore:setString(""..nScore)
end

-- 更新用户免费抽取倒计时。
function ViewExpendTable:updateFreeGetTime(timeSecond)
	timeSecond = toint(timeSecond)
	-- 时间的string.
	local timeString = ""
	-- 显示（小时：分钟：秒）
	local s = timeSecond%60
	local m = timeSecond/60%60
	local h = timeSecond/3600
	timeString = string.format("%d:%.2d:%.2d", math.floor(h), math.floor(m), math.floor(s))
	
	-- 显示按钮上的文字。
	local pSpNormal = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Sp_1StartNormal)
	tolua.cast(pSpNormal, "CCSprite")
	-- local pSpSelect = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Sp_1StartSelect)
	-- tolua.cast(pSpSelect, "CCSprite")
	local pSpN = nil
	local pSpS = nil
	if timeSecond > 0 then
		-- 显示“开始”
		pSpN = CCSprite:create("Images/XYZX-017.png")
		-- pSpS = CCSprite:create("Images/ZP_KaiShi.png")
	else
		-- 显示“免费获取”
		pSpN = CCSprite:create("Images/XYZX-015.png")
		-- pSpS = CCSprite:create("Images/ZP_MianFeiChou.png")
	end

	pSpNormal:setTexture(pSpN:getTexture())
	pSpNormal:setTextureRect( pSpN:getTextureRect() )
	-- pSpSelect:setTexture(pSpS:getTexture())
end

-- 消息显示循环。
function ViewExpendTable:updateNewMsg(dt)

end

-- mark: ============== 转动动画的管理 ================
-- 更新转盘的转动动画。
function ViewExpendTable:updateTurnTableEffect(dt)
	-- 时间
	if self.turnStart then
		-- 计时
		self.turnTime = self.turnTime + dt

		if self.turnTime > self.turnOneTime then
			-- 播放音效。
			SOUNDMGR:PlayEffClickButton()

			-- 判断下一个物品的停留时间。
			if self.turnCircleCnt == 1 then
				-- 第1圈，加速。
				self.turnOneTime = self.turnOneTime - DATA_TURN.DELTA_ADD_TIME
				if self.turnOneTime < DATA_TURN.MIN_DELAY_TIME then
					self.turnOneTime = DATA_TURN.MIN_DELAY_TIME
				end
			elseif self.turnCircleCnt == 4 then
				-- 第3圈，减速。
				self.turnOneTime = self.turnOneTime + DATA_TURN.DELTA_ADD_TIME
				if self.turnOneTime > DATA_TURN.MAX_DELAY_TIME then
					self.turnOneTime = DATA_TURN.MAX_DELAY_TIME
				end
			end

			local nItemBotTagList = {
				Tag_ExpendTable.Sp_RewardBot1,
				Tag_ExpendTable.Sp_RewardBot2,
				Tag_ExpendTable.Sp_RewardBot3,
				Tag_ExpendTable.Sp_RewardBot4,
				Tag_ExpendTable.Sp_RewardBot5,
				Tag_ExpendTable.Sp_RewardBot6,
				Tag_ExpendTable.Sp_RewardBot7,
				Tag_ExpendTable.Sp_RewardBot8,
				Tag_ExpendTable.Sp_RewardBot9,
				Tag_ExpendTable.Sp_RewardBot10,
				Tag_ExpendTable.Sp_RewardBot11,
				Tag_ExpendTable.Sp_RewardBot12,
			}

			local nItemIconTagList = {
				Tag_ExpendTable.Sp_RewardItem1,
				Tag_ExpendTable.Sp_RewardItem2,
				Tag_ExpendTable.Sp_RewardItem3,
				Tag_ExpendTable.Sp_RewardItem4,
				Tag_ExpendTable.Sp_RewardItem5,
				Tag_ExpendTable.Sp_RewardItem6,
				Tag_ExpendTable.Sp_RewardItem7,
				Tag_ExpendTable.Sp_RewardItem8,
				Tag_ExpendTable.Sp_RewardItem9,
				Tag_ExpendTable.Sp_RewardItem10,
				Tag_ExpendTable.Sp_RewardItem11,
				Tag_ExpendTable.Sp_RewardItem12,
			}

			local pCache = CCTextureCache:sharedTextureCache();
			-- 旧的选择物品
			local pSpOldBot = self.pCtuiLayer:GetNodeByTag( nItemBotTagList[self.localIdx+1] )
			tolua.cast(pSpOldBot, "CCSprite")
			pSpOldBot:setTexture( pCache:addImage("Images/GX-058.png") )

			local pSpOldIcon = self.pCtuiLayer:GetNodeByTag( nItemIconTagList[self.localIdx+1] )
			tolua.cast(pSpOldIcon, "CCSprite")
			pSpOldIcon:setScale(self.itemScale * 0.8)

			-- next idx.
			self.turnTime = 0
			self.localIdx = self.localIdx + 1
			if self.localIdx > 11 then
				self.localIdx = 0
			end
			self.turnObjectCnt = self.turnObjectCnt + 1
			if self.turnObjectCnt >= DATA_TURN.DELTA_ONE_CIRCLE_MAX then
				self.turnObjectCnt = 0
				self.turnCircleCnt = self.turnCircleCnt + 1
			end

			-- 新的选择物品
			local pSpIcon = self.pCtuiLayer:GetNodeByTag( nItemIconTagList[self.localIdx+1] )
			tolua.cast(pSpIcon, "CCSprite")
			pSpIcon:setScale(self.itemScale * 1.05)
			-- 显示转动的当前idx
			local pSpSelect = self.pCtuiLayer:GetNodeByTag( nItemBotTagList[self.localIdx+1] )
			tolua.cast(pSpSelect, "CCSprite")
			pSpSelect:setTexture( pCache:addImage("Images/GX-059.png") )
		elseif self.turnTime > self.turnOneTime*2/3 then
			
			-- 判断是否转动到接收的物品并停止。
			if self.turnCircleCnt >= 4 and self.localIdx == self:getLocalReceivedItemIdx() then
				-- 转盘结束
				self:stopTurnTable()
				-- 弹出框。
				-- self:initViewTableGet()
				-- local idx = self.tIdxList[1]
				-- local tTableActData = DATAMGR:GetActivityMgr():GetActivityDataByID(ACT_EXPEND_TABLE_ID)
				-- local item = tTableActData.tData.tItemList[idx+1]
				-- print2json(item)
				NEW_ITEM_MGR:ShowAddOneItem(self.tGetOneItem.nID, self.tGetOneItem.nAddCount)

				-- 更新活动信息。
				DATAMGR:GetActivityMgr():AddOneActInfo(ACT_EXPEND_TABLE_ID, self.tTempActInfo)
				-- 刷新大转盘显示列表。
				self:updateTableShowItemList()
			end
		end
	end
end

-- 开始转动转盘
function ViewExpendTable:startTurnTable()
	-- 
	self.turnStart = true
	self.turnTime = 0
	self.turnOneTime = DATA_TURN.MAX_DELAY_TIME
	self.turnCircleCnt = 1
	self.turnObjectCnt = 0
end

-- 结束转动转盘
function ViewExpendTable:stopTurnTable()
	-- 
	self.turnStart = false
	self.turnTime = 0
end

-- 更新活动信息（超级大奖 + topX）
function ViewExpendTable:updateActInfoWitchCD( dt )
	-- 
	if self.getActCDTime <= 0 then
		--
		self:resetGetActCDTime()
		self:SendGetActInfo(false)
	else
		self.getActCDTime = self.getActCDTime - dt
	end
end

-- 重置用户获取时间倒计时
function ViewExpendTable:resetGetActCDTime()
	-- 
	self.getActCDTime = 5.0
end

-- 更新我的积分
function ViewExpendTable:updateMyScore( nMyScore )
	-- 
	local pLbMyScore = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Lb_MyScore)
	tolua.cast(pLbMyScore, "CCLabelTTF")
	pLbMyScore:setString(""..nMyScore)
end

-- 更新刷新倒计时
function ViewExpendTable:updateRefreshCDTime( timeSecond )
	timeSecond = toint(timeSecond)
	-- 时间的string.
	local timeString = ""
	-- 显示（小时：分钟：秒）
	local s = timeSecond%60
	local m = timeSecond/60%60
	local h = timeSecond/3600
	timeString = string.format("%d:%.2d:%.2d", math.floor(h), math.floor(m), math.floor(s))

	-- 显示时间。
	local pLbCDTime = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Lb_ReflashTableCD)
	tolua.cast(pLbCDTime, "CCLabelTTF")
	pLbCDTime:setString(timeString)
	
	-- -- 显示按钮上的文字。
	-- local pSpNormal = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Sp_1StartNormal)
	-- tolua.cast(pSpNormal, "CCSprite")
	-- local pSpSelect = self.pCtuiLayer:GetNodeByTag(Tag_ExpendTable.Sp_1StartSelect)
	-- tolua.cast(pSpSelect, "CCSprite")
	-- local pSpN = nil
	-- local pSpS = nil
	-- if timeSecond > 0 then
	-- 	-- 显示“开始”
	-- 	pSpN = CCSprite:create("Images/ZP_KaiShi.png")
	-- 	pSpS = CCSprite:create("Images/ZP_KaiShi.png")
	-- else
	-- 	-- 显示“免费获取”
	-- 	pSpN = CCSprite:create("Images/ZP_MianFeiChou.png")
	-- 	pSpS = CCSprite:create("Images/ZP_MianFeiChou.png")
	-- end

	-- pSpNormal:setTexture(pSpN:getTexture())
	-- pSpSelect:setTexture(pSpS:getTexture())
end

-- mark: ============== 主循环 ================
-- 循环。
function ViewExpendTable:ViewLoop(dt)
	-- 更新活动信息（超级大奖 + topX）
	self:updateActInfoWitchCD(dt)
	-- 转动转盘动画.
	self:updateTurnTableEffect(dt)
	-- 活动结束倒计时.
	self:updateActEndCDTime()
	-- 新消息显示
	self:updateNewMsg(dt)

	local tExpendTableAct = DATAMGR:GetActivityMgr():GetActivityDataByID(ACT_EXPEND_TABLE_ID)
	if tExpendTableAct ~= nil then
		-- 免费抽取倒计时.
		self:updateFreeGetTime(tExpendTableAct.tData.nFreeCDTime)
		-- super大奖的值
		self:updateSuperScoreNum(tExpendTableAct.tData.nActTotalScore)
		-- 刷新倒计时
		self:updateRefreshCDTime(tExpendTableAct.tData.nRefreshCDTime)
	end

	local nDiamond = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamodNum
	if nOldDiamond ~= nDiamond then
		self:updateUserDiamondNum(nDiamond)

		nOldDiamond = nDiamond;
	end
end

-- mark: ============== 详情界面的管理 ================
-- 显示详情界面
function ViewExpendTable:initViewTableDetail()
	
	if self.tTopInfoList ~= nil then 
		local nCount = #self.tTopInfoList
		GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.ExpendTableDetail, self.tTopInfoList)
	end
	
end

-- mark: ============== 收取物品界面的管理 ================
-- 显示抽取物品界面
function ViewExpendTable:initViewTableGet( tRewardList )
	local pView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.TableTenGet)
	-- local tTableActData = DATAMGR:GetActivityMgr():GetActivityDataByID(ACT_EXPEND_TABLE_ID)
	-- pView:initWithItemIdxList(self.tIdxList, tTableActData.tData.tItemList)
	local tItemList = {}
	local tIdxList = {}
	for i,tBagItem in ipairs(tRewardList) do
		local tOneItem = {}
		tOneItem.itemId = tBagItem.nID
		tOneItem.itemCnt = tBagItem.nAddCount
		tOneItem.nSmallType = tBagItem.nSmallType
		-- print("itemId: , itemCnt, type,", tOneItem.itemId, tOneItem.itemCnt, tOneItem.nSmallType)

		tItemList[#tItemList+1] = tOneItem
		tIdxList[#tIdxList+1] = #tItemList-1
	end
	pView:initWithItemIdxList(tIdxList, tItemList)
end

-- mark: ============== 表格的管理 ================
-- 更新创建的Cell内容。
function ViewExpendTable:updateCellDetail(nCreateIdx, pCell)
	
	local tMsgInfo = self.newMsgList[nCreateIdx+1]
	-- 名称.
	local pLbUserName = pCell:GetChildNodeByTag(Tag_ExpendTable.Cell_Lb_Nick)
	tolua.cast(pLbUserName, "CCLabelTTF")
	pLbUserName:setString(""..tMsgInfo.nick)
	-- 积分.
	local pLbTotalCnt = pCell:GetChildNodeByTag(Tag_ExpendTable.Cell_Lb_TotalCnt)
	tolua.cast(pLbTotalCnt, "CCLabelTTF")
	pLbTotalCnt:setString(""..tMsgInfo.info)
	-- 
end
-- 如果之前的没有创建cell，则提示创建新的cell
function ViewExpendTable:CTTableViewCreateNewCellByIdx(pTableView, nCreateIdx, strKey)
	-- print("create idx: "..nCreateIdx)
	local pCell = CTTableViewCell:new()

	-- 创建一个新的信息
	local pCellInfo = self.pCtuiLayer:ReCreateUiNode(Tag_ExpendTable.Cell_Ly, false, Tag_ExpendTable.Cell_Ly, 0, UiNotShowEnd)
	pCellInfo:setPosition(ccp(-0, self.nCellPosy))
	pCell:addChild(pCellInfo)
	
	-- color layer.
	-- local size = pTableView:GetNormalCellSize()
	-- local pLyColor = CCLayerColor:create(ccc4(255, 255, 255, 60), size.width, size.height)
	-- -- pLyColor:ignoreAnchorPointForPosition(false)
	-- -- pLyColor:setPosition(ccp(tBottomRect.origin.x, tBottomRect.origin.y))
	-- pCell:addChild(pLyColor)


	-- self:updateCellDetail(nCreateIdx, pCell)
	
	-- 开始按钮管理
	-- local pMenuMgr = pCell:GetChildNodeByTag(Tag_OnlineBox.Ly_L_Btn):getParent()
	-- tolua.cast(pMenuMgr, "CTMenuManager")
	-- pCell:StartToUseCellBtn(pMenuMgr)

	pCell:autorelease()
	return pCell
end
-- 如果之前已经提示创建了cell，则提示已经开始显示到对应的cell，提示开始修改数据
function ViewExpendTable:CTTableViewShowToExistCellByIdx(pTableView, pCell)
	local nCreateIdx = pCell:getCellIdx()
	self:updateCellDetail(nCreateIdx, pCell)
end
-- 获取对应idx的cell的key值
function ViewExpendTable:CTTableViewCellByIdx(pTableView, nCellIdx)
	return "a"
end
-- 获取显示的cell的总数
function ViewExpendTable:CTTableViewShowCellCount(pTableView)
	-- print("top list : "..#self.tTopInfoList)
	return toint(#self.newMsgList)
end
-- 点击了其中一个cell
function ViewExpendTable:CTTableViewClickCell(pTableView, pCell)
	-- 获取当前数据
	-- local tTopInfo = self:GetTopListData()[ pCell:getCellIdx()+1 ]
	
end
-- 点击cell上面的按钮
function ViewExpendTable:CTTableViewClickCellButtonByIdx(pTableView, nCellIdx, nButtonType)
	-- 播放点击按钮声音
    -- SOUNDMGR:PlayEffClickButton()
    -- print("cellIdx: , buttonTag: ", nCellIdx, nButtonType)
end
-- 改变尺寸的回调
function ViewExpendTable:CTTableViewChangeSize(pTableView, pCell, suNewSize)

end

-- mark: ============== 发送接收数据的管理 ================

-- 发送获取活动信息。
function ViewExpendTable:SendGetActInfo(bLoading)
	-- 发送
	local tLuaData = {}
	tLuaData.actId = ACT_EXPEND_TABLE_ID
	tLuaData.bLoading = bLoading

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.GetTempActInfo, tLuaData)
	-- print("发送获取免费精力请求。")
end

-- 发送刷新一次大转盘物品列表。
function ViewExpendTable:SendRefreshItemList()
	-- 发送
	local tLuaData = {}

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.RefreshTableItems, tLuaData)
	-- print("发送获取免费精力请求。")
end

-- 消费大转盘的信息
function ViewExpendTable:ReceiveActInfo(nActId, tActInfo)
	if self.turnStart == true then
		return;
	end
	-- 更新活动信息。
	DATAMGR:GetActivityMgr():AddOneActInfo(nActId, tActInfo)
	
	-- 更新积分topX
	local tTableActData = DATAMGR:GetActivityMgr():GetActivityDataByID(nActId)
	self.tTopInfoList = tTableActData.tData.tTopList
	-- 更新消息列表
	self:ReceiveNewMsgList(tActInfo[7])
	-- 重置活动CD时间。
	self:resetGetActCDTime()
	-- 更新我的积分
	self:updateMyScore(tTableActData.tData.nUserScore)
	-- 更新物品列表
	self:updateTableShowItemList()

end

-- 获取转盘转动后的信息
function ViewExpendTable:ReceiveTableTurn(nActId, nTurnNum, tIdxList, tRewardList, tActInfo, nDiamondCnt)
	-- 更新活动信息。
	self.tTempActInfo = tActInfo
	-- 添加到背包。
	if #tRewardList > 0 then
		for i,tBagCard in ipairs(tRewardList) do
			DATAMGR:GetUserInfoMgr():AddItemToBag(tBagCard, true)
		end
	end
	-- 更新用户钻石。
	DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamodNum = nDiamondCnt
	-- 单个转动。
	if nTurnNum == 1 then
		-- 
		self.tIdxList = tIdxList

		self.tGetOneItem = tRewardList[ 1 ]

		self:startTurnTable()
	-- 十连抽。
	elseif nTurnNum > 1 then
		self.tIdxList = tIdxList
		-- 弹出框。
		self:initViewTableGet(tRewardList)
		
		-- 更新活动信息。
		DATAMGR:GetActivityMgr():AddOneActInfo(nActId, tActInfo)
		-- 更新物品列表
		self:updateTableShowItemList()
	end
	-- 更新topX
	local tTableActData = DATAMGR:GetActivityMgr():GetActivityDataByID(nActId)
	self.tTopInfoList = tTableActData.tData.tTopList
	self.pTableList:reloadTableData()
	-- 重置活动CD时间。
	self:resetGetActCDTime()
	-- 更新我的积分
	self:updateMyScore(tTableActData.tData.nUserScore)
	-- 更新物品列表
	-- self:updateTableShowItemList()
end
	
-- 接收新消息。
function ViewExpendTable:ReceiveNewMsgList(tMsgList)
	-- 
	for i,oneMsg in ipairs(tMsgList) do
		-- 消息类型。
		-- 【1】：用户nick
		-- 【2】：物品id
		-- 【3】：物品个数
		local strMsg = {}
		local nick = oneMsg[1]
		local itemId = oneMsg[2]
		local itemCnt = oneMsg[3]
		local msgId = oneMsg[4]
		local strName = ""
		if itemId == ITEM_ID_GOLD then
			strName = "金币"
		elseif itemId == ITEM_ID_DIAMOND then
			strName = "钻石"
		elseif itemId == ITEM_ID_SUPER_REWARD then
			strName = "超级大奖"
		else
			local itemInfo = DATAMGR:GetItemInfoMgr():GetItemInfoByID(itemId)
			if itemInfo ~= nil then
				strName = itemInfo.strName
			else
				DIALOGMGR:ShowToast("未处理的物品id: "..itemId)
			end
		end
		-- strMsg = "用户["..nick.."]获得"..strName.." x"..itemCnt
		-- print("strMsg ....................", strMsg)
		strMsg.nick = nick
		strMsg.info = "获得"..strName.." x"..itemCnt
		strMsg.id = msgId

		if strMsg ~= nil then
			-- 判断消息是否重复
			local bSameMsg = false
			for i,sMsg in ipairs(self.newMsgList) do
				if sMsg.id == strMsg.id then
					bSameMsg = true
					break
				end
			end

			if bSameMsg == false then
				-- 添加到第一个位置
				table.insert(self.newMsgList, 1, strMsg)
				-- print("newMsgList ", self.newMsgList, strMsg)
				-- 删除多余的信息
				for i,v in ipairs(self.newMsgList) do
					if i > 5 then
						self.newMsgList[i] = nil
					end
				end
				-- 重新加载显示列表。
				self.pTableList:reloadTableData()
			end
		end
	end
	print("newMsg size: ", #self.newMsgList)
end

-- 刷新一次大转盘(奖励列表)。
function ViewExpendTable:ReceiveRefreshTableOnce(tActInfo, nDiamondCnt)
	-- 更新活动信息。
	DATAMGR:GetActivityMgr():AddOneActInfo(ACT_EXPEND_TABLE_ID, tActInfo)
	-- 更新用户钻石。
	DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nDiamodNum = nDiamondCnt
	
	-- 更新topX
	local tTableActData = DATAMGR:GetActivityMgr():GetActivityDataByID(ACT_EXPEND_TABLE_ID)
	self.tTopInfoList = tTableActData.tData.tTopList
	-- table.insert(self.tCurGetPrizeData, 1, "bbb")
	self.pTableList:reloadTableData()
	-- 重置活动CD时间。
	self:resetGetActCDTime()
	-- 更新我的积分
	self:updateMyScore(tTableActData.tData.nUserScore)
	-- 更新物品列表
	self:updateTableShowItemList()
end

-- 接收到的当前物品序号
function ViewExpendTable:getLocalReceivedItemIdx()
	-- 
	local localIdx = self.tIdxList[1]

	return localIdx
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function ViewExpendTable:ViewTouchBegan(tLocation, nCount)
	if self.pCtuiLayer:CTUITouchBegan(tLocation) == false then
		self.pTableList:CtScrollViewTouchBegan(tLocation)
	end
end

-- 触控移动
function ViewExpendTable:ViewTouchMoved(tLocation, tPrevPos, tBeganPos)
	if self.pCtuiLayer:CTUITouchMoved(tLocation) == false then
		self.pTableList:CtScrollViewTouchMoved(tLocation, tPrevPos)
	end
end
-- 触控结束
function ViewExpendTable:ViewTouchEnded(tLocation, tPrevPos, tBeganPos)
	if self.pCtuiLayer:CTUITouchEnded(tLocation) == false then
		self.pTableList:CtScrollViewTouchEnded(tLocation, tPrevPos)
	end
end

return ViewExpendTable