--
-- Author: Link
-- Date: 2014-04-09 10:22:13
--


local GameView = requires(ROOT_PATH, "app/GameLg/Lobby/Others/GameView")
local ViewFriendList = class("ViewFriendList", GameView)

local FriendListUiTag = {
	SpTopBtnBackBg	= 7,    --  顶部按钮背景图片
	BtnClose 		= 37,	-- 关闭按钮
	BtnSearch 		= 3,	-- 搜索按钮

	BtnFriend		= 13,	-- 好友按钮
	BtnEnemy		= 21,	-- 宿敌按钮
	BtnTuiJian		= 29,	-- 推荐按钮

	LbOneTagWord1	= 18,	-- 第1个标签的文字
	LbOneTagWord2	= 23,	-- 第2个标签的文字
	LbOneTagWord3	= 28,	-- 第3个标签的文字

	LbFriendCount	= 6,	-- 好友数据
	SpInputName		= 2,	-- 输入框的名字
	SpTabBottom		= 7,	-- 表格背景


	SpTabFlag 		= 25,	-- 标签的位置

	SpEnermyRocket 		= 28,   -- 宿敌箭头
	SpFriendRocket  	= 20,	-- 好友箭头
	SpRecommendRocket	= 36, -- 推荐箭头


}

local FriendListType = {
	Friend 		= 1,	-- 好友列表
	Enemy 		= 2,	-- 宿敌列表
	TuiJian 	= 3,	-- 推荐列表
}

-- 初始化
function ViewFriendList:ctor()
	self.super.ctor(self)
	-- 默认是好友列表
	self.nCurShowType = FriendListType.Friend;
	-- 推荐和搜索的用户列表
	self.tSearchList = {}
	-- 地板的图片
	self:ShowViewBg()

	-- 当前界面
	self.pCtuiLayer = CTUILayer:new();
	self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/haoyou.ctui"))
	self.pCtuiLayer:setAnchorPoint(ccp(0.5, 1))
	self.pCtuiLayer:setPosition(ctprccp(0.5, 1))
	self.pCtuiLayer:ignoreAnchorPointForPosition(false)
	self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	self.pCtuiLayer:ShowAllUiInfo()
	self:addChild(self.pCtuiLayer, 1)

	self.pCtuiLayer:release()
	
	self.pCtuiLayer:setDelegateLua(function(event)
	-- 如果是按钮点击事件
		if event.cbType == "btnCb" then
			self:CTUIButtonCallBack(event.Layer, event.Btn, event.Tag)
		end
	end)


	-- 默认搜索按钮不能用
	self.pCtuiLayer:SetButtonEnable(false, FriendListUiTag.BtnSearch)

	-- 输入框
	local pSpInput = self.pCtuiLayer:GetNodeByTag(FriendListUiTag.SpInputName)
	local tLgSize = CCSizeMake(pSpInput:getContentSize().width*0.95, pSpInput:getContentSize().height*0.95)
	self.pLbInput = CTTextInput:textFieldWithPlaceHolder("请输入名字", tLgSize, LABEL_BOLD_FONT_NAME, tLgSize.height*0.6)
	-- self.pLbInput:setString("请输入名字")
	self.pLbInput:setPosition(pSpInput:getContentSize().width/2.0, pSpInput:getContentSize().height/2.0 )
	pSpInput:addChild(self.pLbInput, 2)

	self.pLbInput:setInputDelegateLua(function ( event )
		if event.type == "change" then
			return self:CTTextInputChangeText(event.input, event.chText, event.text, event.back)
		elseif event.type == "detach" then
			self:CTTextInputStartClose(event.input, event.isReturn, event.didhide)
		end
	end)

	-- table起始位置
	local fTabelPosy = -310
	-- table高度
	local fTableHeight = 330
	local nShowType = SHARE_FUNC.nFrameType
    -- ipad
    if nShowType == CTDeviceTypeIpad or nShowType == CTDeviceTypeIpadHD then
    	fTabelPosy = 25
    	fTableHeight = -8
    	
    elseif nShowType == CTDeviceTypeIphoneHD or nShowType == CTDeviceTypeItouchHD then
    	fTabelPosy = -100
    	fTableHeight = 115
    end


	local pSpTopBottom = self.pCtuiLayer:GetNodeByTag(FriendListUiTag.SpTopBtnBackBg)
	local tTopRect = ctspbound(pSpTopBottom)
	local tBottomPos = self.pCtuiLayer:convertToNodeSpace( ccp(0, SHARE_FUNC.bottomTagH) )
	local tBottomRect = CCRectMake(0, ctipwidth(SHARE_FUNC.bottomTagH )+ fTabelPosy , ctprwidth(1.3), tTopRect:getMinY() - ctipwidth(SHARE_FUNC.bottomTagH )+  fTableHeight )

	-- self:RefreshType(0);
	-- self:RefreshType(1);

	-- 好友和对手列表
	local LogicEnemyList = requires(ROOT_PATH, "app/GameLg/Lobby/GameRank/LogicEnemyList")
	self.pFriendList = LogicEnemyList.new()
	self.pFriendList:InitInfo( tBottomRect , self)
	self.pCtuiLayer:addChild(self.pFriendList, 1)

	-- 显示
	self:ChangeCurShowListByBtnTag(FriendListUiTag.BtnFriend)

end

-- 单独处理按钮之间的切换
function ViewFriendList:DealTagBtn(nBtnTag)
	-- 3按钮和对应的箭头
	local tArryTag = {
		 FriendListUiTag.BtnFriend, 
		 FriendListUiTag.BtnEnemy, 
		 FriendListUiTag.BtnTuiJian, 
	}
	-- 3个按钮处理和箭头处理
	for i, nTempTag in ipairs(tArryTag) do 
		-- 按钮
		local pTemBtn = self.pCtuiLayer:GetNodeByTag(nTempTag)
		tolua.cast(pTemBtn, "CTMenuItem")
		
		if pTemBtn then 
			-- 按钮
			if nTempTag == nBtnTag then 
				pTemBtn:SetEnable(false)
			
			else
				-- 按钮
				pTemBtn:SetEnable(true)
				
			end
		end
	end
end


-- 根据商城的信息显示
function ViewFriendList:InitViewWithData(tDataList)
	self:SetCurShowInfo(tDataList[1])
end

-- 重新根据数据显示
function ViewFriendList:ReshowWithData(tDataList)
	
end

-- 获取当前的显示数据
function ViewFriendList:GetCurShowInfo()
	local tShowData = {}
	tShowData.nShowType = self.nCurShowType

	local tPos = self.pFriendList:GetOffsetPos();
	tShowData.fPosX = tPos.x;
	tShowData.fPosY = tPos.y;

	return tShowData;
end
-- 设置当前的显示数据
function ViewFriendList:SetCurShowInfo(tShowData)
	if tShowData == nil then
		-- self:SendGetTopWorldRankList()
	else
		-- 如果显示的界面不像，则修改
		if self.nCurShowType ~= tShowData.nShowType then
			-- 如果要显示宿敌
			if tShowData.nShowType == FriendListType.Enemy then
				self:CTUIButtonCallBack(nil, nil, FriendListUiTag.BtnEnemy)
			end
		end
		self.pFriendList:SetOffsetPos( ccp(tShowData.fPosX, tShowData.fPosY) );
	end
end

-- 重新
function ViewFriendList:RefreshShowInfo()
	local pLbCount = self.pCtuiLayer:GetNodeByTag(FriendListUiTag.LbFriendCount)
	tolua.cast(pLbCount, "CCLabelTTF")

	local nCount = 0;--DATAMGR:GetUserInfoMgr():GetUserSpecialInfo().nMaxFriend
	local nFiend = 0

	if self.nCurShowType == FriendListType.Enemy then
		nCount = DATAMGR:GetUserInfoMgr():GetCurUserVipData().nMaxEnemyCnt
		nFiend = #DATAMGR:GetUserInfoMgr():GetUserSpecialInfo().tEnemyList
	else
		nCount = DATAMGR:GetUserInfoMgr():GetCurUserVipData().nMaxFriendCnt
		nFiend = #DATAMGR:GetUserInfoMgr():GetUserSpecialInfo().tFriendList
	end
	pLbCount:setString(nFiend.."/"..nCount)
end

-- 刷新一种类型
function ViewFriendList:RefreshType( nType )
	local tCarIDList = DATAMGR:GetItemInfoMgr():GetCarIDList()
	local nCarCount = #tCarIDList

	local tList = {}
	-- 掠夺
	if nType == 1 then
		tList = DATAMGR:GetUserInfoMgr():GetUserSpecialInfo().tFriendList
	else
		tList = DATAMGR:GetUserInfoMgr():GetUserSpecialInfo().tEnemyList
	end

	-- 如果宿敌列表存在
	if #tList > 0 then
		return
	end

	-- 读取姓名信息
	if self.strArrNames == nil then
		self.strArrNames = string.split(io.readfile(Public_GetResPathByKey("Images/names.txt")), "\n")
		self.strArrNames[#self.strArrNames] = nil
		for i,v in ipairs(self.strArrNames) do
			self.strArrNames[i] = string.trim(v)
		end
	end
	
	if self.strArrFamily == nil then
		self.strArrFamily = string.split(io.readfile(Public_GetResPathByKey("Images/familyNames.txt")), "\n")
		self.strArrFamily[#self.strArrFamily] = nil
		for i,v in ipairs(self.strArrFamily) do
			self.strArrFamily[i] = string.trim(v)
		end
	end

	-- 删除旧的
	for k,v in pairs(tList) do
		tList[k] = nil
	end

	local nCount = 20+nType

	for i=1,nCount do
		local tOne = clone(SocialOneData)

		local nIdx = GetRandom(1, #self.strArrFamily)
		local strName = self.strArrFamily[nIdx]
		nIdx = GetRandom(1, #self.strArrNames)
		strName = strName .. self.strArrNames[nIdx]

		tOne.strName = strName
		tOne.nLv = GetRandom(10, 120)
		tOne.nRank = GetRandom(10, 10000)
		tOne.nGold = GetRandom(20, 1200000)

		for m=1,3 do
			tOne.nCarList[m] = tCarIDList[ GetRandom(1, nCarCount) ]
		end

		tList[i] = tOne
	end
end

-- 根据不同的按钮标签显示不同的界面
function ViewFriendList:ChangeCurShowListByBtnTag(nTag)
	-- 不同的按钮可以刷新不同的列表
	local LogicEnemyList = requires(ROOT_PATH, "app/GameLg/Lobby/GameRank/LogicEnemyList")
	if nTag == FriendListUiTag.BtnFriend then
		-- 好友列表
		self.nCurShowType = FriendListType.Friend;
		self.pFriendList:ChangeCurShowType( LogicEnemyList:EnemyListType().FriendList )
	elseif nTag == FriendListUiTag.BtnEnemy then
		-- 宿敌列表
		self.nCurShowType = FriendListType.Enemy;
		self.pFriendList:ChangeCurShowType( LogicEnemyList:EnemyListType().EnemyList )
	else
		self.nCurShowType = FriendListType.TuiJian;
		self.pFriendList:ChangeCurShowType( LogicEnemyList:EnemyListType().TuiJianList, self.tSearchList )
	end

	-- 刷新显示总数
	self:RefreshShowInfo();
end

-- mark: ============== 按钮的回调 ================
-- 按钮回调
function ViewFriendList:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)
	-- 关闭
	if nTag == FriendListUiTag.BtnClose then
		self:CloseView();

	-- 搜索按钮
	elseif nTag == FriendListUiTag.BtnSearch then
		-- 切换到对应的列表
		if self.nCurShowType ~= FriendListType.TuiJian then
			self:ChangeCurShowListByBtnTag(FriendListUiTag.BtnTuiJian)
		end
		-- 发送搜索数据
		self:SendSearchFriend( self.pLbInput:getString() )

	-- 标签切换
	elseif nTag == FriendListUiTag.BtnFriend or nTag == FriendListUiTag.BtnEnemy or nTag == FriendListUiTag.BtnTuiJian then
		self:DealTagBtn(nTag)
		-- 删除所有搜索的列表
		for k,v in pairs(self.tSearchList) do
			self.tSearchList[k] = nil
		end
		-- 切换到对应的列表
		self:ChangeCurShowListByBtnTag(nTag)

		if nTag == FriendListUiTag.BtnTuiJian then
			-- 发送获取推荐列表
			self:SendSearchFriend()
		end
	end
end

-- 好友和宿敌列表的点击回调
function ViewFriendList:EnemyListCallBack(nCellIdx, tCellInfo)
	local ViewPopMenu = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewPopMenu");
	-- 好友
	if self.nCurShowType == FriendListType.Friend then
		local tMenuList = {
			ViewPopMenu:PopMenuBtnType().OneFourBlank,
			ViewPopMenu:PopMenuBtnType().Message,
			ViewPopMenu:PopMenuBtnType().QieCuo,
			ViewPopMenu:PopMenuBtnType().DeleteFriend,
			ViewPopMenu:PopMenuBtnType().OneFourBlank,
		}
		local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tCellInfo)
		pShowView:SetMenuDelegate(self)
	-- 宿敌
	elseif self.nCurShowType == FriendListType.Enemy then
		local tMenuList = {
			ViewPopMenu:PopMenuBtnType().Message,
			ViewPopMenu:PopMenuBtnType().Revenge,
			ViewPopMenu:PopMenuBtnType().YaoQing,
			ViewPopMenu:PopMenuBtnType().DeleteEnemy,
		}
		local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tCellInfo)
		pShowView:SetMenuDelegate(self)
	end
end

-- 推荐列表的回调, nType为0，则是申请好友，其他值则是留言
function ViewFriendList:SocialTuiJianListCallBack(nCellIdx, tCellInfo, nType)
	if nType == 0 then
		self:ReqToAddFriend(tCellInfo.nUserID, tCellInfo.strName)
	else
		GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.SendMessage, tCellInfo.nUserID);
	end
end

-- 输入框文字发生改变，返回true表示不能发生对应的改变
function ViewFriendList:CTTextInputChangeText(pInput, strChangeText, strNewText, bIsBack)
	
	-- 
	if strNewText == nil or strNewText == "" then
		self.pCtuiLayer:SetButtonEnable(false, FriendListUiTag.BtnSearch)
	else
		-- 默认搜索按钮不能用
		self.pCtuiLayer:SetButtonEnable(true, FriendListUiTag.BtnSearch)
	end

	return false
end
-- 输入框开始关闭
function ViewFriendList:CTTextInputStartClose(pInput, bIsReturn, bIsDid)
	local strNewText = pInput:getString()
	if strNewText == nil then
		strNewText = ""
	end
	-- 去掉空格
	strNewText = string.trim(strNewText)
	-- 点击完成的所有
	if bIsReturn == true and bIsDid == false and strNewText ~= "" then
		self:CTUIButtonCallBack(nil, nil, FriendListUiTag.BtnSearch)
	end
end

-- 弹出菜单的回调处理,tBtnTypeList是拥有的按钮列表
function ViewFriendList:PopMenuViewCallBack(nBtnType, tKeepInfo, tBtnTypeList)

	-- 留言
	if nBtnType == tBtnTypeList.Message then
		GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.SendMessage, tKeepInfo.nUserID);
	end
	-- 好友
	if self.nCurShowType == FriendListType.Friend then
		-- local tMenuList = {
		-- 	ViewPopMenu:PopMenuBtnType().OneFourBlank,
		-- 	ViewPopMenu:PopMenuBtnType().Message,
		-- 	ViewPopMenu:PopMenuBtnType().QieCuo,
		-- 	ViewPopMenu:PopMenuBtnType().DeleteFriend,
		-- 	ViewPopMenu:PopMenuBtnType().OneFourBlank,
		-- }
		-- 切磋
		if nBtnType == tBtnTypeList.QieCuo then
			-- 发送切磋
			self:SendFriendQieCuoInfo(tKeepInfo.nUserID);
		-- 绝交
		elseif nBtnType == tBtnTypeList.DeleteFriend then
			-- 询问是否处理
			DIALOGMGR:ShowDialog(DLGTYPE.RemoveDelFriend, tKeepInfo.strName, tKeepInfo.nUserID, nil);
		end
	-- 宿敌
	elseif self.nCurShowType == FriendListType.Enemy then
		-- local tMenuList = {
		-- 	ViewPopMenu:PopMenuBtnType().Message,
		-- 	ViewPopMenu:PopMenuBtnType().Revenge,
		-- 	ViewPopMenu:PopMenuBtnType().YaoQing,
		-- 	ViewPopMenu:PopMenuBtnType().DeleteEnemy,
		-- }
		-- 邀请好友
		if nBtnType == tBtnTypeList.YaoQing then
			self:ReqToAddFriend(tKeepInfo.nUserID, tKeepInfo.strName);
		-- 删除宿敌
		elseif nBtnType == tBtnTypeList.DeleteEnemy then
			-- 询问是否处理
			DIALOGMGR:ShowDialog(DLGTYPE.RemoveEnemy, tKeepInfo.strName, tKeepInfo.nUserID, nil);
		-- 报仇
		elseif nBtnType == tBtnTypeList.Revenge then
			self:SendStartLootFight(tKeepInfo.nUserID, true);
		end
	end
end

-- 邀请好友的处理
function ViewFriendList:ReqToAddFriend(nUid, strNick)
	local nSocialType = DATAMGR:GetUserInfoMgr():GetSocialTypeByUserID(nUid)

	-- 如果是好友
	if nSocialType == SocialType.Friend then
		-- 通知已经是好友
		DIALOGMGR:ShowDialog(DLGTYPE.AlreadyFriend, strNick, nil);
	-- 如果是宿敌
	elseif nSocialType == SocialType.Enemy then
		DIALOGMGR:ShowDialog(DLGTYPE.RemoveEnemyAndAddFriend, strNick, nUid, nil);
	-- 如果还是陌生人
	else
		self:SendReqAddFriend(nUid);
	end
end

-- mark: ============== 发送消息 ================
-- 发送搜索好友
function ViewFriendList:SendSearchFriend(strNick)
	local tLuaData = {}
	tLuaData.nick = strNick

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.SearchFriend, tLuaData)
end
-- 发送邀请好友
function ViewFriendList:SendReqAddFriend(nUid)
	local tLuaData = {}
	tLuaData.fid = nUid

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.AddOneFriend, tLuaData)
end
-- 发送获取绝交
function ViewFriendList:SendDelFriend(nUid)
	local tLuaData = {}
	tLuaData.fid = nUid

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.DelOneFriend, tLuaData)
end
-- 发送切磋信息
function ViewFriendList:SendFriendQieCuoInfo(nUid)
	local tLuaData = {}
	tLuaData.fid = nUid
	tLuaData.bShiTan = false;

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.FightOneFriend, tLuaData)
end
-- 进行掠夺处理
function ViewFriendList:SendStartLootFight(nUid, nIsEnemy)
	local tLuaData = {}
	tLuaData.uid = nUid

	local nSocialType = DATAMGR:GetUserInfoMgr():GetSocialTypeByUserID(nUid)
	if nSocialType == SocialType.Enemy then
		tLuaData.enemy = 1
	else
		tLuaData.enemy = 0
	end
	tLuaData.recordId = 0;
	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.StartFightLoot, tLuaData)
end
-- mark: ============== 接收消息 ================
-- 接收搜索好友
function ViewFriendList:ReceiveSearchFriend(tSearchList)
	-- 删除所有搜索的列表
	for k,v in pairs(self.tSearchList) do
		self.tSearchList[k] = nil
	end
	-- 获取当前列表
	for k,v in pairs(tSearchList) do
		self.tSearchList[k] = v;
	end

	-- 刷新显示列表
	self.pFriendList:RefreshList()
end
-- 接收到一个社交关系发生改变，比如添加或删除成功
function ViewFriendList:ReceiveChangeSocialRelationship(bIsFriend)
	self.pFriendList:RefreshList()
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function ViewFriendList:ViewTouchBegan(tLocation, nCount)

	self.pCtuiLayer:CTUITouchBegan(tLocation)

	if self.pLbInput and self.pLbInput:isVisible() then
		self.pLbInput:TITouchBegan(tLocation)
	end

	if self.pFriendList:isVisible() then
		self.pFriendList:LgEnemyListTouchBegan(tLocation, nCount)
	end
end

-- 触控移动
function ViewFriendList:ViewTouchMoved(tLocation, tPrevPos, tBeganPos)
	self.pCtuiLayer:CTUITouchMoved(tLocation)

	if self.pLbInput and self.pLbInput:isVisible() then
		self.pLbInput:TITouchMoved(tLocation)
	end

	if self.pFriendList:isVisible() then
		self.pFriendList:LgEnemyListTouchMoved(tLocation, tPrevPos, tBeganPos)
	end
end

-- 触控结束
function ViewFriendList:ViewTouchEnded(tLocation, tPrevPos, tBeganPos)
	self.pCtuiLayer:CTUITouchEnded(tLocation)

	if self.pLbInput and self.pLbInput:isVisible() then
		self.pLbInput:TITouchEnded(tLocation)
	end

	if self.pFriendList:isVisible() then
		self.pFriendList:LgEnemyListTouchEnded(tLocation, tPrevPos, tBeganPos)
	end
end

return ViewFriendList
