--
-- Author: Link
-- Date: 2014-04-10 20:57:40
--
local GameView = requires(ROOT_PATH, "app/GameLg/Lobby/Others/GameView")
local ViewMessage = class("ViewMessage", GameView)

local MessageListUiTag = {
	SpBtnBackBg		= 8,   --  按钮背景图片
	BtnClose 		= 32,	-- 关闭按钮
	BtnFight 		= 9,	-- 战斗
	BtnSystem		= 17,	-- 系统信息

	SpSysNewMsgBot	= 30,	-- 系统新消息数目底图
	LbSysNewMsgNum	= 31,	-- 系统新消息数目

	----- 战斗表格标签 ----
	SpTb1CellBg 		= 36,	-- 信息背景
	SpTb1CellCar 		= 40,	-- 玩家的车子
	SpCarLv 			= 41, 	-- 玩家的车子等级

	LbTb1Time 			= 38,	-- 多少时间前
	LbTb1Name			= 48,	-- 玩家名字
	LbTb1Level 		 	= 49,	-- 玩家等级
	LbTb1FightValue		= 51,	-- 战斗力
	LbTb1Gold 			= 43,	-- 获得或失去的金币
	LbTb1MedalWord 		= 45,	-- 战功文字
	LbTb1Medal 			= 44,	-- 战功
	LbTb1RankInfo		= 52,	-- 排名
	LbTb1Content		= 46,	-- 描述信息

	----- 系统表格标签 ----
	SpTb2CellBg 		= 54,	-- 信息背景
	SpTb2CellIcon 		= 62,	-- 玩家的车子

	LbTb2Time 			= 59,	-- 多少时间前
	LbTb2Name			= 61,	-- 玩家名字
	LbTb2Content		= 63,	-- 描述信息

	LbOneTagWord1		= 8,	-- 第1个标签的文字
	LbOneTagWord2		= 14,	-- 第2个标签的文字

	SpTabFlag 			= 0,	-- 标签的位置

	LbGold              = 7,    -- 玩家金币
	LbDiamond 			= 6,    -- 玩家砖石

	SpFightRocket 		= 16 , 	-- 战斗箭头
	SpInforRocket 		= 24 , 	-- 信息箭头

}

local MessageListType = {
	Fight 		= 1,	-- 战斗列表
	System 		= 2,	-- 系统列表
}

-- 初始化
function ViewMessage:ctor()
	self.super.ctor(self)

	-- 数字的变化
	self.nCurDiamond = -100
	self.nCurGold 	 = -100

	-- 战斗列表
	self.nCurShowType = MessageListType.Fight;

	-- 文字配置信息
	local strTextInfo = DATAMGR:LoadOneDataFileAndJudgeIsChange("Images/StaticDatas/Text_config.json")
	self.tTextList = json.decode(strTextInfo)

	-- 将列表数据copy过来
	local tFight = DATAMGR:GetUserInfoMgr():GetBattleMessageList()
	self.tFightList = {}
	for k,v in pairs(tFight) do
		self.tFightList[k] = v;
	end
	-- 战斗新消息数，表明已经查看
	local nFightNew = DATAMGR:GetUserInfoMgr():GetNewMessageCount(1)
	if nFightNew > 0 then
		self:SendHadReadMsgType(1)
		DATAMGR:GetUserInfoMgr():ReadNewMessage(1)
	end

	local tSys = DATAMGR:GetUserInfoMgr():GetSystemMessageList()
	self.tSysList = {}
	for k,v in pairs(tSys) do
		self.tSysList[k] = v;
	end
	-- 系统新消息数
	self.nSysNew = DATAMGR:GetUserInfoMgr():GetNewMessageCount(2)

	-- 地板的图片
	self:ShowViewBg()

	-- 当前界面
	self.pCtuiLayer = CTUILayer:new();
	self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/xx-diban.ctui"))
	self.pCtuiLayer:setAnchorPoint(ccp(0.5, 1))
	self.pCtuiLayer:setPosition(ctprccp(0.5, 1))
	self.pCtuiLayer:ignoreAnchorPointForPosition(false)
	self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	self.pCtuiLayer:ShowAllUiInfo()
	self:addChild(self.pCtuiLayer, 1)
	self.pCtuiLayer:release()

	self.pCtuiLayer:setDelegateLua(function(event)
	-- 如果是按钮点击事件
		if event.cbType == "btnCb" then
			self:CTUIButtonCallBack(event.Layer, event.Btn, event.Tag)
		end
	end)
	-- table起始位置
	local fTabelPosy = -90
	-- table高度
	local fTableHeight = 105
	local tDeiveType = GetCurrentDeviceType();
	if tDeiveType == CTDeviceTypeIpad or tDeiveType == CTDeviceTypeIpadHD then
		fTableHeight = -15
		fTabelPosy = 30
	elseif tDeiveType == CTDeviceTypeIphone5HD or tDeiveType == CTDeviceTypeItouch5HD then
		fTableHeight = 315
		fTabelPosy = -300
	end
	-- 背景图片
 	local pSpBg = self.pCtuiLayer:GetNodeByTag(MessageListUiTag.SpBtnBackBg)
	-- pSpBg:setScaleY( display.height/pSpBg:getContentSize().height - 0.15)

	-- 获取显示区域
	local tBotRect = ctspbound(pSpBg)
	local tBottomRect = CCRectMake(0, ctipwidth(SHARE_FUNC.bottomTagH )+ fTabelPosy , ctprwidth(1.3), tBotRect:getMinY() - ctipwidth(SHARE_FUNC.bottomTagH )+  fTableHeight  )

	-- 表格的回调的
	local function TabelViewCallBack( event )
		-- 表格总数
		if event.type == "count" then
			return self:CTTableViewShowCellCount(event.table)

		-- 表格的key
		elseif event.type == "key" then
			return self:CTTableViewCellByIdx(event.table, event.Idx)

		-- 创建
		elseif event.type == "Create" then
			return self:CTTableViewCreateNewCellByIdx(event.table, event.Idx, event.key)

		-- 显示具体信息
		elseif event.type == "showCell" then
			self:CTTableViewShowToExistCellByIdx(event.table, event.cell)

		-- 改变尺寸的回调中
		elseif event.type == "ReSize" then
			local tSize = CCSizeMake(event.width, event.height)
			self:CTTableViewChangeSize(event.table, event.cell, tSize)

		-- cell上面点中按钮的回调
		elseif event.type == "Btn" then
			self:CTTableViewClickCellButtonByIdx(event.table, event.Idx, event.BtnType)

		-- 在cell上面点击的回调
		elseif event.type == "Click" then
			self:CTTableViewClickCell(event.table, event.cell)
		end
	end
	-- 章节列表
	self.pTableList = CTTableView:new()
	self.pTableList:initWithFrameLua(tBottomRect, TabelViewCallBack)
	self.pTableList:setDelSpeedRate(SHARE_FUNC.fScorllDelSpeedRate)
	self.pCtuiLayer:addChild(self.pTableList, 1)
	self.pTableList:release()

	-- 创建
	self:ChangeCurShowTabelType()

	-- 系统新消息数目
	local pSpSysNewMsgBg = self.pCtuiLayer:GetNodeByTag(MessageListUiTag.SpSysNewMsgBot)
	if self.nSysNew > 0 then
		pSpSysNewMsgBg:setVisible(true);
		local pLbCount = self.pCtuiLayer:GetNodeByTag(MessageListUiTag.LbSysNewMsgNum)
		tolua.cast(pLbCount, "CCLabelTTF")
		pLbCount:setString(""..self.nSysNew)
	else
		pSpSysNewMsgBg:setVisible(false);
	end

	self:ViewLoop(0)

end 

-- View里面的循环使用
function ViewMessage:ViewLoop( dt )
	-- 当前玩家信息
	local pCurUserInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo()

	-- 如果钻石数目发生了变化
	if self.nCurDiamond ~= pCurUserInfo.nDiamodNum then
		local pLbDiamond = self.pCtuiLayer:GetNodeByTag( MessageListUiTag.LbDiamond )
		tolua.cast(pLbDiamond, "CCLabelTTF")
		pLbDiamond:setString( Public_GetNumShowType(pCurUserInfo.nDiamodNum, NumberShowType.ShortWan) )

		self.nCurDiamond = pCurUserInfo.nDiamodNum
	end

	-- 如果黄金数目发生了变化
	if self.nCurGold ~= pCurUserInfo.nGoldNum then
		local pLbGold = self.pCtuiLayer:GetNodeByTag( MessageListUiTag.LbGold )
		tolua.cast(pLbGold, "CCLabelTTF")
		pLbGold:setString( Public_GetNumShowType(pCurUserInfo.nGoldNum, NumberShowType.ShortWan) )

		self.nCurGold = pCurUserInfo.nGoldNum
	end
end

-- 单独处理按钮之间的切换
function ViewMessage:DealTagBtn(nBtnTag)
	-- 3按钮和对应的箭头
	local tArryTag = {
		MessageListUiTag.BtnFight, 
		MessageListUiTag.BtnSystem,
	}
	-- 3个按钮处理和箭头处理
	for i, nTempTag in ipairs(tArryTag) do 
		-- 按钮
		local pTemBtn = self.pCtuiLayer:GetNodeByTag(nTempTag)
		tolua.cast(pTemBtn, "CTMenuItem")
		-- 箭头
		local pSpRocket = self:getChildByTag(nTempTag)
		tolua.cast(pSpRocket, "CCSprite")

		if pTemBtn then 
			-- 按钮
			if nTempTag == nBtnTag then 
				pTemBtn:SetEnable(false)
			else
				-- 按钮
				pTemBtn:SetEnable(true)
			end
		end
	end
end


function ViewMessage:InitViewWithData(tDataList)
	self:SetCurShowInfo(tDataList[1])
	local pTemBtn = self.pCtuiLayer:GetNodeByTag(MessageListUiTag.BtnSystem)
	tolua.cast(pTemBtn, "CTMenuItem")
	pTemBtn:SetEnable(true)
end
-- 获取当前的显示数据
function ViewMessage:GetCurShowInfo()
	local tShowData = {}
	tShowData.nShowType = self.nCurShowType

	local tPos = self.pTableList:GetOffsetPos();
	tShowData.fPosX = tPos.x;
	tShowData.fPosY = tPos.y;

	return tShowData;
end
-- 设置当前的显示数据
function ViewMessage:SetCurShowInfo(tShowData)
	if tShowData == nil then
	else
		self.pTableList:SetOffsetPos( ccp(tShowData.fPosX, tShowData.fPosY) );
	end
end

-- 修改的按钮的回调
-- 改变当前类型
function ViewMessage:ChangeCurShowTabelType()
	-- local pMenuItem

	-- 战斗信息
	if self.nCurShowType == MessageListType.Fight then
		self.pTableList:SetNormalCellSize(CCSizeMake(1024, 185))
	-- 系统信息
	else
		self.pTableList:SetNormalCellSize(CCSizeMake(1024, 180))
	end
	-- 显示位置回到初始位置
	self.pTableList:SetToOffsetPosRate(ccp(0, 0))
	self.pTableList:reloadTableData()
end

-- 获取当前要显示的列表
function ViewMessage:GetCurShowList()
	-- 战斗信息
	if self.nCurShowType == MessageListType.Fight then
		return self.tFightList;

	end
	return self.tSysList;
end

-- 根据时间返回时间描述文字
function ViewMessage:GetTimeWordWithTime(nPastTime)

	local strWord = "刚刚"

	if nPastTime < 60 then
		strWord = "刚刚";
	elseif nPastTime < 3600 then
		strWord = math.floor( nPastTime/60 ).."分钟前";
	elseif nPastTime < 86400 then
		strWord = math.floor( nPastTime/3600 ).."小时前"
	else
		strWord = math.floor( nPastTime/86400 ).."天前"
	end

	return strWord
end

-- mark: ============== 表格的管理 ================
-- 如果之前的没有创建cell，则提示创建新的cell
function ViewMessage:CTTableViewCreateNewCellByIdx(pTableView, nCreateIdx, strKey)
	local pCell = CTTableViewCell:new()

	local fConWidthTmp = 370;
	if GetCurrentSystemType() == ctSystemTypeAndroid then
		fConWidthTmp = 450;
	end

	-- 战斗信息列表
	if self.nCurShowType == MessageListType.Fight then
		-- 当前选择界面
		local pCtuiCell = CTUILayer:new();
		pCtuiCell:InitWithFile(Public_GetResPathByKey("Images/xx-xinxitiao.ctui"))
		pCell:addChild(pCtuiCell)
		pCtuiCell:release()

		pCtuiCell:ShowAllUiInfo()

		local pSpRBot = pCtuiCell:GetNodeByTag(MessageListUiTag.SpTb1CellBg)
		local tCellSize = pTableView:GetNormalCellSize()
		pCtuiCell:setPosition(ccp( 0, tCellSize.height*0.5 - pSpRBot:getPositionY()))

		-- 信息内容
		local pLbContent = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1Content)
		tolua.cast(pLbContent, "CCLabelTTF")

		-- 创建新的信息内容
		local pLbConTmp = CCLabelTTF:create(" ", pLbContent:getFontName(), pLbContent:getFontSize(),
											CCSizeMake(fConWidthTmp, 200), kCCTextAlignmentLeft, kCCVerticalTextAlignmentTop)
		pLbConTmp:setAnchorPoint(ccp(0.4, 0.9))
		pLbConTmp:setColor(pLbContent:getColor())
		pLbConTmp:setPosition( pLbContent:getPositionInCCPoint() )--ccp(tShowRect:getMinX(), tShowRect:getMaxY()))
		pLbContent:getParent():addChild(pLbConTmp, 0, MessageListUiTag.LbTb1Content)

		pLbContent:removeFromParent()
	-- 系统信息列表
	else
		-- 当前选择界面
		local pCtuiCell = CTUILayer:new();
		pCtuiCell:InitWithFile(Public_GetResPathByKey("Images/xx-xinxitiao2.ctui"))
		pCell:addChild(pCtuiCell)
		pCtuiCell:release()
		pCtuiCell:ShowAllUiInfo()

		local pSpRBot = pCtuiCell:GetNodeByTag(MessageListUiTag.SpTb2CellBg)
		local tCellSize = pTableView:GetNormalCellSize()
		pCtuiCell:setPosition(ccp( 0, tCellSize.height*0.5 - pSpRBot:getPositionY() ))

		-- 信息内容
		local pLbContent = pCell:GetChildNodeByTag(MessageListUiTag.LbTb2Content) 
		tolua.cast(pLbContent, "CCLabelTTF")

		-- 创建新的信息内容
		local pLbConTmp = CCLabelTTF:create(" ", pLbContent:getFontName(), pLbContent:getFontSize(),
											CCSizeMake(fConWidthTmp, 200), kCCTextAlignmentLeft, kCCVerticalTextAlignmentTop)
		pLbConTmp:setAnchorPoint(ccp(0.0, 1))
		pLbConTmp:setColor(pLbContent:getColor())
		pLbConTmp:setPosition( pLbContent:getPositionInCCPoint() )   --ccp(tShowRect:getMinX(), tShowRect:getMaxY()))
		pLbContent:getParent():addChild(pLbConTmp, 0, MessageListUiTag.LbTb2Content)

		pLbContent:removeFromParent()
	end
	
	pCell:autorelease()
	return pCell
end
-- 如果之前已经提示创建了cell，则提示已经开始显示到对应的cell，提示开始修改数据
function ViewMessage:CTTableViewShowToExistCellByIdx(pTableView, pCell)
	-- 获取当前数据
	local tMsgInfo = self:GetCurShowList()[ pCell:getCellIdx()+1 ]

	-- 战斗信息列表
	if self.nCurShowType == MessageListType.Fight then 
		-- 时间
		local pLbTime = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1Time)
		tolua.cast(pLbTime, "CCLabelTTF")
		pLbTime:setString( self:GetTimeWordWithTime( os.time()-tMsgInfo.nPastTime ) )

		-- 姓名
		local pLbName = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1Name)
		tolua.cast(pLbName, "CCLabelTTF")
		pLbName:setString( tMsgInfo.strNick )
		-- pLbName:setScale(5)

		-- 玩家等级
		local pLbLevel = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1Level)
		tolua.cast(pLbLevel, "CCLabelTTF")
		pLbLevel:setString( "Lv."..tMsgInfo.nUserLv )
		print2json(tMsgInfo)
		-- tMsgInfo.tCarInfo[1]:对方第一辆车 tMsgInfo.tCarInfo[2]：附加等级
		local pSpItem = DATAMGR:GetItemInfoMgr():GetCarSmallImageSpriteByID(tMsgInfo.tCarInfo[1])
		-- 修改显示
		local pSpCar = pCell:GetChildNodeByTag(MessageListUiTag.SpTb1CellCar)
		tolua.cast(pSpCar, "CCSprite")
		pSpCar:setTexture( pSpItem:getTexture() )
		pSpCar:setTextureRect( pSpItem:getTextureRect() )
		print2json(tMsgInfo)

		-- 车子等级 
		local tCarBagItem =  DATAMGR:GetItemInfoMgr():GetItemInfoByID(tMsgInfo.tCarInfo[1])
		local strLv = tCarBagItem.strStar
		local pCarLv = pCell:GetChildNodeByTag(MessageListUiTag.SpCarLv)
		tolua.cast(pCarLv, "CCSprite")

		local pStarTex, pStarBot, pTexTmp = SHARE_FUNC:GetCardRelateTextureByStar(strLv)
		pCarLv:setTexture(pStarTex)











		-- 玩家战斗力 self.tTextList
		local pLbFightValue = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1FightValue)
		tolua.cast(pLbFightValue, "CCLabelTTF")
		pLbFightValue:setString( ""..tMsgInfo.nTotalValue )

		-- 获取需要更换的信息
		local pLbMedalWord = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1MedalWord)
		local pLbMedal = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1Medal)
		local pLbRank = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1RankInfo)
		local pLbGold = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1Gold)
		tolua.cast(pLbGold, "CCLabelTTF")
		local strContent = ""
		-- 根据类型进行处理
		-- 掠夺
		if tMsgInfo.nMsgType == MessageInfoType.Loot then
			-- 不显示战功等信息
			pLbMedalWord:setVisible(false)
			pLbMedal:setVisible(false)
			pLbRank:setVisible(false)

			-- 输
			if tMsgInfo.tDataList[1] == 1 then
				-- 金币
				pLbGold:setString(""..tMsgInfo.tDataList[2])
				pLbGold:setColor(ccc3(255, 0, 0))

				-- 描述文字
				strContent = string.gsub(self.tTextList.fmsg[1], "{0}", tMsgInfo.strNick);
			-- 赢
			else
				-- 金币
				pLbGold:setString("+"..tMsgInfo.tDataList[2])
				pLbGold:setColor(ccc3(255, 255, 0))

				-- 描述文字
				strContent = string.gsub(self.tTextList.fmsg[2], "{0}", tMsgInfo.strNick);
			end


		-- 排位处理
		elseif tMsgInfo.nMsgType == MessageInfoType.Rank then
			-- 不显示战功等信息
			pLbMedalWord:setVisible(true)

			-- 战功
			tolua.cast(pLbMedal, "CCLabelTTF")
			pLbMedal:setVisible(true)

			-- 输
			if tMsgInfo.tDataList[1] == 1 then
				-- 排位发生了变化
				tolua.cast(pLbRank, "CCLabelTTF")
				pLbRank:setVisible(true)
				pLbRank:setString("排位降至"..tMsgInfo.tDataList[2])

				-- 金币
				pLbGold:setString(""..tMsgInfo.tDataList[4])
				pLbGold:setColor(ccc3(255, 0, 0))

				-- 战功
				pLbMedal:setString(""..tMsgInfo.tDataList[3])
				pLbMedal:setColor(ccc3(255, 0, 0))

				-- 描述文字
				strContent = string.gsub(self.tTextList.fmsg[3], "{0}", tMsgInfo.strNick);
			-- 赢
			else
				-- 赢了，排位没有改变
				pLbRank:setVisible(false)

				-- 金币
				pLbGold:setString("+"..tMsgInfo.tDataList[2])
				pLbGold:setColor(ccc3(255, 255, 0))

				-- 战功
				pLbMedal:setString("0")
				pLbMedal:setColor(ccc3(255, 255, 0))

				-- 描述文字
				strContent = string.gsub(self.tTextList.fmsg[4], "{0}", tMsgInfo.strNick);
			end

		-- 切磋
		elseif tMsgInfo.nMsgType == MessageInfoType.QieCuo then
			-- 不显示战功等信息
			pLbMedalWord:setVisible(false)
			pLbMedal:setVisible(false)
			pLbRank:setVisible(false)

			-- 金币
			pLbGold:setString(""..tMsgInfo.tDataList[2])
			pLbGold:setColor(ccc3(255, 0, 0))

			-- 描述文字
			strContent = string.gsub(self.tTextList.fmsg[5], "{0}", tMsgInfo.strNick);
		end

		-- 修改描述文字
		local pLbContent = pCell:GetChildNodeByTag(MessageListUiTag.LbTb1Content)
		tolua.cast(pLbContent, "CCLabelTTF")
		pLbContent:setString(strContent)

	-- 系统信息列表
	else
		local pLbTime = pCell:GetChildNodeByTag(MessageListUiTag.LbTb2Time)
		tolua.cast(pLbTime, "CCLabelTTF")
		pLbTime:setString( self:GetTimeWordWithTime( os.time()-tMsgInfo.nPastTime ) )
		-- 姓名
		local pLbName = pCell:GetChildNodeByTag(MessageListUiTag.LbTb2Name)
		tolua.cast(pLbName, "CCLabelTTF")
		pLbName:setString( tMsgInfo.strNick )

		local strContent = ""
		--	int	加好友请求
		if tMsgInfo.nMsgType == MessageInfoType.ReqFriend then
			-- 描述文字
			strContent = string.gsub(self.tTextList.smsg[1], "{0}", tMsgInfo.strNick);

		-- 同意加为好友
		elseif tMsgInfo.nMsgType == MessageInfoType.AgreeFriend then
			-- 描述文字
			strContent = string.gsub(self.tTextList.smsg[3], "{0}", tMsgInfo.strNick);

		-- 拒绝加为好友
		elseif tMsgInfo.nMsgType == MessageInfoType.RefuseFriend then
			-- 描述文字
			strContent = string.gsub(self.tTextList.smsg[2], "{0}", tMsgInfo.strNick);

		-- 删除好友
		elseif tMsgInfo.nMsgType == MessageInfoType.RemoveFriend then
			-- 描述文字
			strContent = string.gsub(self.tTextList.smsg[4], "{0}", tMsgInfo.strNick);

		else
			if tMsgInfo.strContent then
				strContent = tMsgInfo.strContent
			end
		end

		-- 修改描述文字
		local pLbContent = pCell:GetChildNodeByTag(MessageListUiTag.LbTb2Content)
		tolua.cast(pLbContent, "CCLabelTTF")
		pLbContent:setString(strContent)

	end
end
-- 获取对应idx的cell的key值
function ViewMessage:CTTableViewCellByIdx(pTableView, nCellIdx)
	return "a"
end
-- 获取显示的cell的总数
function ViewMessage:CTTableViewShowCellCount(pTableView)
	-- 剧情大章节信息
	local tList = self:GetCurShowList()
	return #tList
	
end
-- 点击了其中一个cell
function ViewMessage:CTTableViewClickCell(pTableView, pCell)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()
	-- 获取当前数据
	local tMsgInfo = self:GetCurShowList()[ pCell:getCellIdx()+1 ]

	local ViewPopMenu = requires(ROOT_PATH, "app/GameLg/Lobby/Others/ViewPopMenu");
	-- 根据类型进行处理
	-- 掠夺
	if tMsgInfo.nMsgType == MessageInfoType.Loot then
		-- 输
		if tMsgInfo.tDataList[1] == 1 then
			local tMenuList = {
				ViewPopMenu:PopMenuBtnType().Message,
				ViewPopMenu:PopMenuBtnType().Revenge,
				ViewPopMenu:PopMenuBtnType().AddEnemy,
				ViewPopMenu:PopMenuBtnType().YaoQing,
				ViewPopMenu:PopMenuBtnType().Replay,
			}
			local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tMsgInfo)
			pShowView:SetMenuDelegate(self)
		-- 赢
		else
			local tMenuList = {
				ViewPopMenu:PopMenuBtnType().OneFourBlank,
				ViewPopMenu:PopMenuBtnType().Message,
				ViewPopMenu:PopMenuBtnType().YaoQing,
				ViewPopMenu:PopMenuBtnType().Replay,
				ViewPopMenu:PopMenuBtnType().OneFourBlank,
			}
			local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tMsgInfo)
			pShowView:SetMenuDelegate(self)
		end
	-- 排位处理
	elseif tMsgInfo.nMsgType == MessageInfoType.Rank then
		-- 输
		if tMsgInfo.tDataList[1] == 1 then
			local tMenuList = {
				ViewPopMenu:PopMenuBtnType().Message,
				ViewPopMenu:PopMenuBtnType().Revenge,
				ViewPopMenu:PopMenuBtnType().AddEnemy,
				ViewPopMenu:PopMenuBtnType().YaoQing,
				ViewPopMenu:PopMenuBtnType().Replay,
			}
			local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tMsgInfo)
			pShowView:SetMenuDelegate(self)
		-- 赢
		else
			local tMenuList = {
				ViewPopMenu:PopMenuBtnType().OneFourBlank,
				ViewPopMenu:PopMenuBtnType().Message,
				ViewPopMenu:PopMenuBtnType().YaoQing,
				ViewPopMenu:PopMenuBtnType().Replay,
				ViewPopMenu:PopMenuBtnType().OneFourBlank,
			}
			local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tMsgInfo)
			pShowView:SetMenuDelegate(self)
		end
	-- 切磋
	elseif tMsgInfo.nMsgType == MessageInfoType.QieCuo then
		local tMenuList = {
			ViewPopMenu:PopMenuBtnType().OneFourBlank,
			ViewPopMenu:PopMenuBtnType().Message,
			ViewPopMenu:PopMenuBtnType().QieCuo,
			ViewPopMenu:PopMenuBtnType().Replay,
			ViewPopMenu:PopMenuBtnType().OneFourBlank,
		}
		local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tMsgInfo)
		pShowView:SetMenuDelegate(self)

	-- 留言
	elseif tMsgInfo.nMsgType == MessageInfoType.Msg then
		local tMenuList = {
			ViewPopMenu:PopMenuBtnType().OneHalfBlank,
			ViewPopMenu:PopMenuBtnType().Message,
			ViewPopMenu:PopMenuBtnType().DeleteMsg,
			ViewPopMenu:PopMenuBtnType().OneHalfBlank,
		}
		local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tMsgInfo)
		pShowView:SetMenuDelegate(self)
	-- 加好友请求
	elseif tMsgInfo.nMsgType == MessageInfoType.ReqFriend then
		local tMenuList = {
			ViewPopMenu:PopMenuBtnType().OneHalfBlank,
			ViewPopMenu:PopMenuBtnType().AgreeAddFriend,
			ViewPopMenu:PopMenuBtnType().RefuseAddFriedn,
			ViewPopMenu:PopMenuBtnType().OneHalfBlank,
		}
		local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tMsgInfo)
		pShowView:SetMenuDelegate(self)

	-- 如果是各种奖励信息
	elseif tMsgInfo.nMsgType >= MessageInfoType.CostGift then
		local tMenuList = {
			ViewPopMenu:PopMenuBtnType().OneHalfBlank,
			ViewPopMenu:PopMenuBtnType().GetGift,
			ViewPopMenu:PopMenuBtnType().DeleteMsg,
			ViewPopMenu:PopMenuBtnType().OneHalfBlank,
		}
		local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tMsgInfo)
		pShowView:SetMenuDelegate(self)
	-- 其他信息
	else
		local tMenuList = {
			ViewPopMenu:PopMenuBtnType().DeleteMsg,
		}
		local pShowView = GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.PopMenu, tMenuList, tMsgInfo)
		pShowView:SetMenuDelegate(self)
	end
end
-- 点击cell上面的按钮
function ViewMessage:CTTableViewClickCellButtonByIdx(pTableView, nCellIdx, nButtonType)
	
end
-- 改变尺寸的回调
function ViewMessage:CTTableViewChangeSize(pTableView, pCell, suNewSize)

end

-- mark: ============== 按钮的回调 ================

-- 显示切换到系统消息
function ViewMessage:StartToShowSystemMsg()
	self:DealTagBtn(MessageListUiTag.BtnSystem)
	-- 系统列表
	self.nCurShowType = MessageListType.System;

	-- 系统新消息数目
	if self.nSysNew > 0 then
		self.nSysNew = 0;
		local pSpSysNewMsgBg = self.pCtuiLayer:GetNodeByTag(MessageListUiTag.SpSysNewMsgBot)
		pSpSysNewMsgBg:setVisible(false);

		-- 标志已读取
		self:SendHadReadMsgType(2)
		for k,tOneMsg in pairs(self.tSysList) do
			if tOneMsg.nStatus == MessageStatus.NewMsg then
				tOneMsg.nStatus = MessageStatus.HadRead;
			end
		end
	end

	self:ChangeCurShowTabelType()
end

-- 按钮回调
function ViewMessage:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()
	-- 关闭按钮
	if nTag == MessageListUiTag.BtnClose then
		self:CloseView()
		-- 标签切换
	elseif nTag == MessageListUiTag.BtnFight or nTag == MessageListUiTag.BtnSystem then
		-- 不同的按钮可以刷新不同的列表
		local LogicEnemyList = requires(ROOT_PATH, "app/GameLg/Lobby/GameRank/LogicEnemyList")
		if nTag == MessageListUiTag.BtnFight then
			-- 战斗列表
			self.nCurShowType = MessageListType.Fight;
			self:DealTagBtn(nTag)
			
			self:ChangeCurShowTabelType()
		else
			self:StartToShowSystemMsg();
		end
	end
end


-- 弹出菜单的回调处理,tBtnTypeList是拥有的按钮列表
function ViewMessage:PopMenuViewCallBack(nBtnType, tKeepInfo, tBtnTypeList)
	-- 获取当前数据
	local tMsgInfo = tKeepInfo


	-- 通用的处理
	-- 消息
	if nBtnType == tBtnTypeList.Message then
		GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.SendMessage, tMsgInfo.nUserID);
	-- 加为宿敌
	elseif nBtnType == tBtnTypeList.AddEnemy then
		local nSocialType = DATAMGR:GetUserInfoMgr():GetSocialTypeByUserID(tMsgInfo.nUserID)

		-- 如果是好友
		if nSocialType == SocialType.Friend then
			-- 通知是否删除好友并加为宿敌
			DIALOGMGR:ShowDialog(DLGTYPE.RemoveFriendAndAddEnemy, tMsgInfo.strNick, tMsgInfo.nUserID, nil);
		-- 如果是宿敌
		elseif nSocialType == SocialType.Enemy then
			DIALOGMGR:ShowDialog(DLGTYPE.AlreadyEnemy, tMsgInfo.strNick, nil);
		-- 如果还是陌生人
		else
			self:SendAddToEnemy(tMsgInfo.nUserID);
		end
	-- 加为好友
	elseif nBtnType == tBtnTypeList.YaoQing then
		local nSocialType = DATAMGR:GetUserInfoMgr():GetSocialTypeByUserID(tMsgInfo.nUserID)

		-- 如果是好友
		if nSocialType == SocialType.Friend then
			-- 通知是否删除好友并加为宿敌
			DIALOGMGR:ShowDialog(DLGTYPE.AlreadyFriend, tMsgInfo.strNick, nil);
		-- 如果是宿敌
		elseif nSocialType == SocialType.Enemy then
			DIALOGMGR:ShowDialog(DLGTYPE.RemoveEnemyAndAddFriend, tMsgInfo.strNick, tMsgInfo.nUserID, nil);
		-- 如果还是陌生人
		else
			self:SendReqAddFriend(tMsgInfo.nUserID);
		end
	-- 重播
	elseif nBtnType == tBtnTypeList.Replay then
		self:SendShowReply(tMsgInfo.nMsgID)
	-- 删除消息
	elseif nBtnType == tBtnTypeList.DeleteMsg then
		self:SendDeleteOneMsg(tMsgInfo.nMsgID)

	-- 收取礼物
	elseif nBtnType == tBtnTypeList.GetGift then

		self:SendGetMessageReward(tMsgInfo.nMsgID)
	end

	-- 根据类型进行处理
	-- 掠夺
	if tMsgInfo.nMsgType == MessageInfoType.Loot then
		-- 如果是报仇
		if nBtnType == tBtnTypeList.Revenge then
			self:SendStartLootFight(tMsgInfo.nUserID, true, tMsgInfo.nMsgID)
		end
	-- 排位处理
	elseif tMsgInfo.nMsgType == MessageInfoType.Rank then
		-- 如果是报仇
		if nBtnType == tBtnTypeList.Revenge then
			self:SendStartRankFight(tMsgInfo.nUserID, 1, 1, tMsgInfo.nMsgID)
		end
	-- 切磋
	elseif tMsgInfo.nMsgType == MessageInfoType.QieCuo then
		-- 如果是切磋
		if nBtnType == tBtnTypeList.QieCuo then
			self:SendFriendQieCuoInfo(tMsgInfo.nUserID)
		end

	-- 留言
	elseif tMsgInfo.nMsgType == MessageInfoType.Msg then
		
	-- 加好友请求
	elseif tMsgInfo.nMsgType == MessageInfoType.ReqFriend then
		-- 同意添加
		if nBtnType == tBtnTypeList.AgreeAddFriend then
			self:SendResAddFriend(tMsgInfo.nUserID, tMsgInfo.nMsgID, true);
		-- 拒绝添加
		elseif nBtnType == tBtnTypeList.RefuseAddFriedn then
			self:SendResAddFriend(tMsgInfo.nUserID, tMsgInfo.nMsgID, false);
		end
	end
end

-- mark: ============== 发送消息 ================
-- 发送已经读取，nType表明，已经读取,已查看的消息类型（1：战斗信息，2：非战斗信息）
function ViewMessage:SendHadReadMsgType(nType)
	local tJsonData = {}
	tJsonData.aNum = nType

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.HadReadNewMsg, tJsonData)
end
-- 发送回复加好友消息
function ViewMessage:SendResAddFriend(nUid, nMsgID, bIsAgree)
	local tJsonData = {}
	tJsonData.uid = nUid;
	tJsonData.aid = nMsgID;
	if bIsAgree == true then
		tJsonData.status = 1;
	else
		tJsonData.status = 0;
	end

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.ResAddOneFriend, tJsonData)
end
-- 发送删除单条邮件信息
function ViewMessage:SendDeleteOneMsg(nMsgID)
	local tLuaData = {}
	tLuaData.idArr = {}
	tLuaData.idArr[1] = nMsgID

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.DeleteOneMsg, tLuaData)
end

-- 发送列为宿敌
function ViewMessage:SendAddToEnemy(nUserID)
	local tJsonData = {}
	tJsonData.eid = nUserID

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.AddOneEnemy, tJsonData)
end
-- 发送邀请好友
function ViewMessage:SendReqAddFriend(nUid)
	local tLuaData = {}
	tLuaData.fid = nUid

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.AddOneFriend, tLuaData)
end
-- 发送重播功能
function ViewMessage:SendShowReply(nMsgID)
	local tLuaData = {}
	tLuaData.aid = nMsgID

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.ReplyBattle, tLuaData)
end
-- 发送切磋信息
function ViewMessage:SendFriendQieCuoInfo(nUid)
	local tLuaData = {}
	tLuaData.fid = nUid

	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.FightOneFriend, tLuaData)
end
-- 进行掠夺处理
function ViewMessage:SendStartLootFight(nUid, nIsEnemy, nMsgID)
	local tLuaData = {}
	tLuaData.uid = nUid

	local nSocialType = DATAMGR:GetUserInfoMgr():GetSocialTypeByUserID(nUid)
	if nSocialType == SocialType.Enemy then
		tLuaData.enemy = 1
	else
		tLuaData.enemy = 0
	end
	tLuaData.recordId = nMsgID;
	
	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.StartFightLoot, tLuaData)
end
-- 进行排位战
function ViewMessage:SendStartRankFight(nUid, nIsEnemy, nFree, nMsgID)
	local tLuaData = {}
	tLuaData.uid = nUid
	tLuaData.enemy = nIsEnemy
	tLuaData.free = nFree
	tLuaData.recordId = nMsgID;
	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.StartFightRank, tLuaData)
end
-- 获取邮件奖励信息
function ViewMessage:SendGetMessageReward(nMsgID)
	local tLuaData = {}
	tLuaData.mid = nMsgID;
	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.GetOneMsgRewards, tLuaData)
end
-- mark: ============== 接收 ================
-- 收到删除消息对应消息的id，里面存的是ID
function ViewMessage:ReceiveDeleteOneMsg(tDelList)
	-- 删除消息列表
	DATAMGR:GetUserInfoMgr():DeleteMessageList(tDelList)

	local bIsDelFight = false;
	local bIsDelSys = false;
	-- 遍历删除的ID
	for i,nDelID in ipairs(tDelList) do
		repeat
			local bIsDone = false;
			-- 遍历消息，获取状态
			for m,tOneMsg in ipairs(self.tFightList) do
				if tOneMsg.nMsgID == nDelID then
					table.remove(self.tFightList, m);
					bIsDone = true;
					break
				end
			end
			-- 已经删除，则跳过
			if bIsDone then
				bIsDelFight = true;
				break;
			end

			-- 遍历消息，获取状态
			for m,tOneMsg in ipairs(self.tSysList) do
				if tOneMsg.nMsgID == nDelID then
					table.remove(self.tSysList, m);
					bIsDelSys = true;
					break
				end
			end
		until true
	end

	-- 根据情况来判断是否重新刷新列表
	-- 战斗信息列表
	if self.nCurShowType == MessageListType.Fight then
		if bIsDelFight then
			self.pTableList:reloadTableData();
		end
	-- 系统信息列表
	else
		if bIsDelSys then
			self.pTableList:reloadTableData();
		end
	end
end

-- 进入重播
function ViewMessage:ReceiveReplyInfo(tFightInfo, nMsgId)
	-- 调转到战斗界面.
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.StartShowLocalRace, tFightInfo)
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function ViewMessage:ViewTouchBegan(tLocation, nCount)
	self.pCtuiLayer:CTUITouchBegan(tLocation)

	self.pTableList:CtScrollViewTouchBegan(tLocation)
end

-- 触控移动
function ViewMessage:ViewTouchMoved(tLocation, tPrevPos, tBeganPos)
	self.pCtuiLayer:CTUITouchMoved(tLocation)

	self.pTableList:CtScrollViewTouchMoved(tLocation, tPrevPos)

end

-- 触控结束
function ViewMessage:ViewTouchEnded(tLocation, tPrevPos, tBeganPos)
	self.pCtuiLayer:CTUITouchEnded(tLocation)

	self.pTableList:CtScrollViewTouchEnded(tLocation, tPrevPos)

end

return ViewMessage
