--
-- Author: Link
-- Date: 2014-04-04 15:55:32
--

local GameView = requires(ROOT_PATH, "app/GameLg/Lobby/Others/GameView")
local ViewOpenBoxHelp = class("ViewOpenBoxHelp", GameView)

local OpenBoxHelpUiTag = {
	BtnClose 		= 4,

	SpBackBg		= 1,    --  背景图片
	SpTableBg1		= 2,	-- 表格底图

	-- 要是按钮
	BtnKeyJin		= 7,	-- 金钥匙
	BtnKeyYin		= 11,	-- 银钥匙
	BtnKeyTong		= 15,	-- 铜钥匙

	LyCellLayer		= 3,	-- 表格滑动层
	SpCellBottom1 	= 4,	-- cell的底图
	SpCellBottom2 	= 6,	-- cell的底图
	SpCellIcon		= 19,	-- 物品图标
	LbItemName		= 29,	-- 物品名字
	LbItemContent	= 30,	-- 物品描述

}

-- 初始化
function ViewOpenBoxHelp:ctor()
	-- 调用父初始化
	self.super.ctor(self)
	-- -- 底部灰色
	-- local pLyColor = CCLayerColor:create( GRAY_BG_COLOR )
	-- self:addChild(pLyColor, -1)

	-- 地板的图片
	self:ShowViewBg()

	-- 当前界面
	self.pCtuiLayer = CTUILayer:new();
	self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/BBX-xinxi.ctui"))
	self.pCtuiLayer:setAnchorPoint(ccp(0.5, 1))
	self.pCtuiLayer:setPosition(ctprccp(0.5, 1))
	self.pCtuiLayer:ignoreAnchorPointForPosition(false)
	self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	self.pCtuiLayer:ShowAllUiInfo()
	self:addChild(self.pCtuiLayer)
	self.pCtuiLayer:release()

	self.pCtuiLayer:setDelegateLua(function(event)
	-- 如果是按钮点击事件
		if event.cbType == "btnCb" then
			self:CTUIButtonCallBack(event.Layer, event.Btn, event.Tag)
		end
	end)

	-- table起始位置
	local fTabelPosy = 20
	-- table高度
	local fTableHeight = 40
	local nShowType = SHARE_FUNC.nFrameType
	-- cell 的位置
	self.nCellx = 450
	self.nCelly = 150
	
    -- ipad
    if nShowType == CTDeviceTypeIpad or nShowType == CTDeviceTypeIpadHD then
    	
    	fTableHeight = 35

    	self.nCellx = 535
    	self.nCelly = 180

    	fTabelPosy = 16
    	
    elseif nShowType == CTDeviceTypeIphoneHD or nShowType == CTDeviceTypeItouchHD then
    	fTabelPosy = 16
    	fTableHeight = 33

    	self.nCellx = 440
		self.nCelly = 150
    end

	-- 表格数据
	local strJson = DATAMGR:LoadOneDataFileAndJudgeIsChange("Images/StaticDatas/OpenBox.json")
	self.tItemAllInfo = json.decode(strJson)
	self.tItemList = {}
	-- self.tItemList = self.tItemAllInfo[""..KEY_JIN_ID]


	-- 表格的回调的
	local function TabelViewCallBack( event )
		-- 表格总数
		if event.type == "count" then
			return self:CTTableViewShowCellCount(event.table)

		-- 表格的key
		elseif event.type == "key" then
			return self:CTTableViewCellByIdx(event.table, event.Idx)

		-- 创建
		elseif event.type == "Create" then
			return self:CTTableViewCreateNewCellByIdx(event.table, event.Idx, event.key)

		-- 显示具体信息
		elseif event.type == "showCell" then
			self:CTTableViewShowToExistCellByIdx(event.table, event.cell)

		-- 改变尺寸的回调中
		elseif event.type == "ReSize" then
			local tSize = CCSizeMake(event.width, event.height)
			self:CTTableViewChangeSize(event.table, event.cell, tSize)

		-- cell上面点中按钮的回调
		elseif event.type == "Btn" then
			self:CTTableViewClickCellButtonByIdx(event.table, event.Idx, event.BtnType)

		-- 在cell上面点击的回调
		elseif event.type == "Click" then
			self:CTTableViewClickCell(event.table, event.cell)
		end
	end

	-- 隐藏
	self.pCtuiLayer:SetCTUINodeVisiable(OpenBoxHelpUiTag.LyCellLayer, false)

	local pSpTableBg = self.pCtuiLayer:GetNodeByTag(OpenBoxHelpUiTag.SpBackBg);
	local tBgRect = ctspbound(pSpTableBg);

	local tHeightPos = ccp(0, ctipwidth(SHARE_FUNC.bottomTagH))
	tHeightPos = self.pCtuiLayer:convertToNodeSpace(tHeightPos)

	-- 创建列表表格
	self.pItemTable = CTTableView:new()
	self.pItemTable:initWithFrameLua(CCRectMake(tBgRect:getMinX(), tBgRect:getMinY() + fTabelPosy, ctprwidth(1), (tBgRect:getMaxY()-tBgRect:getMinY()) - fTableHeight ), TabelViewCallBack)
	self.pItemTable:SetNormalCellSize(CCSizeMake(1024, 140))
	self.pItemTable:setDelSpeedRate(SHARE_FUNC.fScorllDelSpeedRate)
	self.pCtuiLayer:addChild(self.pItemTable)

	self.pItemTable:release()

	-- 第一个条显示
	local tTags = {
		OpenBoxHelpUiTag.BtnKeyJin,
		OpenBoxHelpUiTag.BtnKeyYin,
		OpenBoxHelpUiTag.BtnKeyTong,
	}
	for i,nOneTag in ipairs(tTags) do
		local pOtherBtn = self.pCtuiLayer:GetNodeByTag(nOneTag)
		tolua.cast(pOtherBtn, "CTMenuItem")
		if i == 1 then
			pOtherBtn:SetEnable(false)
		else
			pOtherBtn:SetEnable(true)
		end -- nOneTag ~= nTag
	end -- for循环

	-- 重新显示宝箱信息
	self:ShowKeyInfoListByBtnTag(OpenBoxHelpUiTag.BtnKeyJin)
end

-- 改变table列表信息
function ViewOpenBoxHelp:ShowKeyInfoListByBtnTag( nTag )
	local nKeyID = KEY_JIN_ID;
	if nTag == OpenBoxHelpUiTag.BtnKeyYin then
		nKeyID = KEY_YIN_ID;
	elseif nTag == OpenBoxHelpUiTag.BtnKeyTong then
		nKeyID = KEY_TONG_ID;
	end

	self.tItemList = self.tItemAllInfo[""..nKeyID]

	self.pItemTable:SetToOffsetPosRate(ccp(0, 0));
	self.pItemTable:reloadTableData()
end

-- mark: ============== 按钮的回调 ================
-- 按钮回调
function ViewOpenBoxHelp:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()
    
	if nTag == OpenBoxHelpUiTag.BtnClose then
		self:CloseView()
	elseif nTag == OpenBoxHelpUiTag.BtnKeyJin or nTag == OpenBoxHelpUiTag.BtnKeyYin or nTag == OpenBoxHelpUiTag.BtnKeyTong then
		local pChildren = pMenuItem:getChildren()
		pMenuItem:SetEnable(false)
		if pChildren then
			-- 遍历过去
			for i=0,(pChildren:count()-1) do
				local pChild = pChildren:objectAtIndex(i)
				tolua.cast(pChild, "CCLabelTTF")
				if pChild:getTag() >= 0 then
					pChild:setFontSize(40)
					pChild:setColor(ccc3(255, 255, 0))
				end
			end
		end
		-- 重新显示宝箱信息
		self:ShowKeyInfoListByBtnTag(nTag)

		local tTags = {
			OpenBoxHelpUiTag.BtnKeyJin,
			OpenBoxHelpUiTag.BtnKeyYin,
			OpenBoxHelpUiTag.BtnKeyTong,
		}

		for i,nOneTag in ipairs(tTags) do
			if nOneTag ~= nTag then
				local pOtherBtn = self.pCtuiLayer:GetNodeByTag(nOneTag)
				tolua.cast(pOtherBtn, "CTMenuItem")
				if pOtherBtn:GetEnable() == false then
					pOtherBtn:SetEnable(true)
					local pChildren = pOtherBtn:getChildren()
					if pChildren then
						-- 遍历过去
						for i=0,(pChildren:count()-1) do
							local pChild = pChildren:objectAtIndex(i)
							tolua.cast(pChild, "CCLabelTTF")
							if pChild:getTag() >= 0 then
								pChild:setFontSize(36)
								pChild:setColor(ccc3(166, 202, 240))
							end
						end
					end-- 遍历childrend
				end -- pOtherBtn:GetEnable() ~= false
			end -- nOneTag ~= nTag
		end -- for循环
	end
end

-- mark: ============== 表格的管理 ================
-- 如果之前的没有创建cell，则提示创建新的cell
function ViewOpenBoxHelp:CTTableViewCreateNewCellByIdx(pTableView, nCreateIdx, strKey)
	local pCell = CTTableViewCell:new()

	-- 创建一个新的信息
	local pCellInfo = self.pCtuiLayer:ReCreateUiNode(OpenBoxHelpUiTag.LyCellLayer, false, OpenBoxHelpUiTag.LyCellLayer, 0, UiNotShowEnd)
	local tBotRect = ctspbound( self.pCtuiLayer:GetNodeByTag(OpenBoxHelpUiTag.SpCellBottom1) )
	pCellInfo:setPosition(ccp(-tBotRect:getMinX() + self.nCellx, -tBotRect:getMinY() + self.nCelly))

	pCell:addChild(pCellInfo)

	pCell:autorelease()
	return pCell
end
-- 如果之前已经提示创建了cell，则提示已经开始显示到对应的cell，提示开始修改数据
function ViewOpenBoxHelp:CTTableViewShowToExistCellByIdx(pTableView, pCell)
	-- 获取对应的物品信息
	local nItemID = self.tItemList[pCell:getCellIdx()+1][1];
	local nItemCount = self.tItemList[pCell:getCellIdx()+1][3];
	local tItemInfo = DATAMGR:GetItemInfoMgr():GetItemInfoByID(nItemID)
	-- 获取物品名字
	local pLbName = pCell:GetChildNodeByTag(OpenBoxHelpUiTag.LbItemName)
	tolua.cast(pLbName, "CCLabelTTF")
	pLbName:setString(tItemInfo.strName .."x"..nItemCount)

	-- 物品描述
	local pLbContent = pCell:GetChildNodeByTag(OpenBoxHelpUiTag.LbItemContent)
	tolua.cast(pLbContent, "CCLabelTTF")
	pLbContent:setString(tItemInfo.strContent)

	-- 物品图标
	local pSpIcon = pCell:GetChildNodeByTag(OpenBoxHelpUiTag.SpCellIcon)
	tolua.cast(pSpIcon, "CCSprite")
	pSpIcon:setDisplayFrame(DATAMGR:GetItemInfoMgr():GetItemIconNameByID(nItemID))
end
-- 获取对应idx的cell的key值
function ViewOpenBoxHelp:CTTableViewCellByIdx(pTableView, nCellIdx)
	return "a"
end
-- 获取显示的cell的总数
function ViewOpenBoxHelp:CTTableViewShowCellCount(pTableView)
	return #self.tItemList
	-- return 1
end
-- 点击了其中一个cell
function ViewOpenBoxHelp:CTTableViewClickCell(pTableView, pCell)
	
end
-- 点击cell上面的按钮
function ViewOpenBoxHelp:CTTableViewClickCellButtonByIdx(pTableView, nCellIdx, nButtonType)
	
end
-- 改变尺寸的回调
function ViewOpenBoxHelp:CTTableViewChangeSize(pTableView, pCell, suNewSize)

end

-- mark: ============== 触控的管理 ================
-- 开始触控
function ViewOpenBoxHelp:ViewTouchBegan(tLocation, nCount)
	
	if self.pCtuiLayer:CTUITouchBegan(tLocation) == false then
		self.pItemTable:CtScrollViewTouchBegan(tLocation)
	end
end

-- 触控移动
function ViewOpenBoxHelp:ViewTouchMoved(tLocation, tPrevPos, tBeganPos)
	if self.pCtuiLayer:CTUITouchMoved(tLocation) == false then
		self.pItemTable:CtScrollViewTouchMoved(tLocation, tPrevPos)
	end
end

-- 触控结束
function ViewOpenBoxHelp:ViewTouchEnded(tLocation, tPrevPos, tBeganPos)
	if self.pCtuiLayer:CTUITouchEnded(tLocation) == false then
		self.pItemTable:CtScrollViewTouchEnded(tLocation, tPrevPos)
	end
end

return ViewOpenBoxHelp
