--
-- Author: Link
-- Date: 2014-04-09 21:37:53
--
-- 基础视图
local GameView = requires(ROOT_PATH, "app/GameLg/Lobby/Others/GameView")
local ViewPopMenu = class("ViewPopMenu", GameView)


-- 弹出的按钮列表类型
local PopMenuBtnType = {
	CloseBtn 		= 100,	-- 关闭，在外部不要使用的

	OneBlankLine	= -9,	-- 一个空行
	OneHalfBlank	= -8,	-- 半个空行
	OneFourBlank	= -7,	-- 四分之一空行
	Message 		= 1,	-- 留言
	Revenge 		= 2,	-- 报仇
	YaoQing 		= 3,	-- 邀请好友
	Replay 			= 4,	-- 战斗回放
	AddEnemy 		= 5,	-- 列为宿敌
	DeleteMsg 		= 6,	-- 删除信息
	GetGift 		= 7,	-- 收礼物
	QieCuo 			= 8,	-- 好友切磋
	DeleteFriend	= 9,	-- 删除好友
	DeleteEnemy 	= 10,	-- 删除宿敌
	AgreeAddFriend	= 11,	-- 同意加为好友
	RefuseAddFriedn	= 12,	-- 拒绝加为好友
}

-- 当前弹出框出现的情形
local PopMenuStatus = {
	None 		= 1,	-- 没有显示
	PopShowing 	= 2,	-- 显示处理
	PopHadShow	= 3,	-- 显示完毕
	PopHiding 	= 4,	-- 隐藏起来
}

-- 构造函数
function ViewPopMenu:ctor()
	-- 调用父初始化
	self.super.ctor(self)

	-- 保留在弹出框的信息
	self.tKeepInfo = nil;

	self.nCurStatus = PopMenuStatus.None;

	-- 底部灰色
	local pLyColor = CCLayerColor:create( GRAY_BG_COLOR )
	self:addChild(pLyColor, -1)

	self.pSpBottom 	= nil
	self.pBtnMgr = nil

	-- 弱引用回调
	self.pCBack = {}
	setmetatable(self.pCBack, {__mode = "v"})

	-- 按钮的回调
	self.pBtnCallBack = function (pBtn)
		self:ButtonCallBack(pBtn)
	end
end

-- 根据商城的信息显示
function ViewPopMenu:PopMenuBtnType()
	return PopMenuBtnType;
end

-- 设置回调
function ViewPopMenu:SetMenuDelegate( pCallBack )
	self.pCBack.delegate = pCallBack
end

-- 根据数据初始化
function ViewPopMenu:InitViewWithData(tDataList)
	self:ShowMenuList(tDataList[1], tDataList[2])
end

-- 重新根据数据显示
function ViewPopMenu:ReshowWithData(tDataList)
	
end

-- 重新刷新显示
function ViewPopMenu:ShowMenuList(tMenuList, tKeepInfo)
	-- 保存的数据
	self.tKeepInfo = tKeepInfo;

	if self.pSpBottom ~= nil or #tMenuList < 0 then
		CTAssert(0, "创建社交弹出按钮列表框的时候发生了错误！")
		return
	end

	-- 状态处理
	self.nCurStatus = PopMenuStatus.PopShowing;

	local fScale1 = ctipwidth(1)

	-- 创建主界面背景
	-- self.pSpBottom = CCScale9Sprite:createWithInsets(CCRectMake(100, 100, 300, 400), "Images/xzfwq-diban.png")
	-- self.pSpBottom:setContentSize(CCSizeMake(tTotalSize.width-40, tConSize.height+20))
	self.pSpBottom = CCSprite:create("Images/xzfwq-diban.png");
	self.pSpBottom:setPosition(ctprccp(0.5, 0.5))
	self.pSpBottom:setScale(fScale1)
	self:addChild(self.pSpBottom)


	-- 获取显示区域
	local tBottomRect = ctspbound(self.pSpBottom)
	self:StartResetUncloseRect(tBottomRect)

	-- 更改倍数大小，用来拉升
	self.pSpBottom:setScale(0.0)

	-- 创建按钮列表
	self.pBtnMgr = CTMenuManager:new()
	self.pBtnMgr:init();
	self.pSpBottom:addChild(self.pBtnMgr, 1)
	self.pBtnMgr:setPosition(ccp(0,0))
	self.pBtnMgr:setAnchorPoint(ccp(0.065,0))
	self.pBtnMgr:ignoreAnchorPointForPosition(false)
	self.pBtnMgr:setScale(0.85/fScale1)
	self.pBtnMgr:release()

	-- 添加关闭按钮
	local pItemBtn = CTMenuItemSprite:new()
	pItemBtn:initWithOneFile("Images/QuXiaoAnNiu.png", 1, 0, 0, -1);
	-- 按钮灰色
	pItemBtn:GetSelectedSprite():setColor(GRAY_COLOR)
	pItemBtn:SetTouchEndedLuaCallBack(self.pBtnCallBack)
	pItemBtn:setScale( ctipwidth(1) )
	pItemBtn:setPosition( ctspccp(self.pSpBottom, 0.99*fScale1/0.86 , 0.96*fScale1/0.86) )
	-- 菜单按钮
	self.pBtnMgr:AddMenuItem(pItemBtn, PopMenuBtnType.CloseBtn)
	pItemBtn:release()

	-- 根据数目排位
	local fMaxCount	= -1.0;
	for i,nBtnType in ipairs(tMenuList) do
		if PopMenuBtnType.OneHalfBlank == nBtnType then
			fMaxCount = fMaxCount+0.5;
		elseif PopMenuBtnType.OneFourBlank == nBtnType then
			fMaxCount = fMaxCount+0.25;
		else
			fMaxCount = fMaxCount+1.0;
		end
	end

	local fStartRate = 0.0
	local fDelRate = 1.0/fMaxCount;
	if fMaxCount <= 0.1 then
		fStartRate = 0.5;
	end
	local tStartPos = ccp(280, 500)
	local tEndPos = ccp(280, 86)
	local tDelPos = ccpSub(tEndPos, tStartPos)

	-- 物品信息
	for i,nBtnType in ipairs(tMenuList) do
		-- 半行
		if nBtnType == PopMenuBtnType.OneHalfBlank then
			fStartRate = fStartRate+fDelRate*0.5;
		elseif nBtnType == PopMenuBtnType.OneFourBlank then
			fStartRate = fStartRate+fDelRate*0.25;
		elseif nBtnType == PopMenuBtnType.OneBlankLine then
			fStartRate = fStartRate+fDelRate*1.0;
		else
			local tOneBtn = self:CreateOneMenu(nBtnType)
			tOneBtn:setPosition( ccpAdd(tStartPos, ccp(tDelPos.x*fStartRate, tDelPos.y*fStartRate)) )
			-- tOneBtn:setScale(0.9)
			fStartRate = fStartRate+fDelRate*1.0;
		end
	end

	-- 显示的动作
	self:ShowPopUpViewAction(self.pSpBottom, false, fScale1)
	
end

-- 弹出结束
function ViewPopMenu:ShowPopUpViewActionEnd(pView)
	-- 状态处理，已经显示完毕
	self.nCurStatus = PopMenuStatus.PopHadShow;
end

-- 创建不同的按钮
function ViewPopMenu:CreateOneMenu(nBtnType)
	local strBtnIcon = "Images/hy-qiechou.png"
	local strName 	= "切 磋"

	
	-- 留言
	if nBtnType == PopMenuBtnType.Message then
		strBtnIcon = "Images/hy-liuyanpng.png"
		strName 	= "留       言"
	-- 报仇
	elseif nBtnType == PopMenuBtnType.Revenge then
		strBtnIcon = "Images/hy-baochou.png"
		strName 	= "报       仇"
	-- 邀请好友
	elseif nBtnType == PopMenuBtnType.YaoQing then
		strBtnIcon = "Images/hy-haoyoutubiao.png"
		strName 	= "邀 请 好 友"
	-- 战斗回放
	elseif nBtnType == PopMenuBtnType.Replay then
		strBtnIcon = "Images/hy-chongbo.png"
		strName 	= "比 赛 回 放"
	-- 列为宿敌
	elseif nBtnType == PopMenuBtnType.AddEnemy then
		strBtnIcon = "Images/hy-shudi.png"
		strName 	= "列 为 宿 敌"
	-- 删除信息
	elseif nBtnType == PopMenuBtnType.DeleteMsg then
		strBtnIcon = "Images/hy-shanchu.png"
		strName 	= "删 除 信 息"
	-- 收礼物
	elseif nBtnType == PopMenuBtnType.GetGift then
		strBtnIcon = "Images/hy-liwu.png"
		strName 	= "收  礼  物"
	-- 好友切磋
	elseif nBtnType == PopMenuBtnType.QieCuo then
		strBtnIcon = "Images/hy-qiechou.png"
		strName 	= "切       磋"
	-- 绝交
	elseif nBtnType == PopMenuBtnType.DeleteFriend then
		strBtnIcon = "Images/hy-juejiao.png"
		strName 	= "绝       交"
	-- 删除宿敌
	elseif nBtnType == PopMenuBtnType.DeleteEnemy then
		strBtnIcon = "Images/hy-shanchu.png"
		strName 	= "删       除"
	-- 同意加为好友
	elseif nBtnType == PopMenuBtnType.AgreeAddFriend then
		strBtnIcon = "Images/xx-tongyi.png"
		strName 	= "同       意"
	-- 拒绝加为好友
	elseif nBtnType == PopMenuBtnType.RefuseAddFriedn then
		strBtnIcon = "Images/hy-juejiao.png"
		strName 	= "拒       绝"
	end
	--
	local pItemBtn = CTMenuItemSprite:new()
	pItemBtn:initWithOneFile("Images/hy-xuanxiang.png", 1, 0, 0, -1);

	-- 按钮灰色
	pItemBtn:GetSelectedSprite():setColor(GRAY_COLOR)
	pItemBtn:SetTouchEndedLuaCallBack(self.pBtnCallBack)
	-- 菜单按钮
	self.pBtnMgr:AddMenuItem(pItemBtn, nBtnType)
	pItemBtn:release()

	-- 图标
	local pSpIcon = CCSprite:create(strBtnIcon)
	pSpIcon:setPosition( ctspccp(pItemBtn, 0.16, 0.55) )
	pItemBtn:addChild(pSpIcon)
	-- 按钮文字
	local pLbWord = CCLabelTTF:create(strName, LABEL_BOLD_FONT_NAME, 40)
	pLbWord:setPosition( ctspccp(pItemBtn, 0.6, 0.5) )
	-- pLbWord:enableStroke(ccc3(197, 0, 0), 2)
	pItemBtn:addChild(pLbWord)



	return pItemBtn
end


-- 按钮的回调处理
function ViewPopMenu:ButtonCallBack(tButton)
	-- 如果不是已经显示完毕的状态，则不处理
	if self.nCurStatus ~= PopMenuStatus.PopHadShow then
		return;
	end
	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()

	local nType = tButton:getTag();

	-- 如果不是关闭按钮，则回调处理
	if nType ~= PopMenuBtnType.CloseBtn then
		-- 回调通知
		if self.pCBack.delegate then
			self.pCBack.delegate:PopMenuViewCallBack(nType, self.tKeepInfo, PopMenuBtnType);
		end
	end
	-- 点击按钮之后，删除当前弹出框
	self:CloseView();
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function ViewPopMenu:ViewTouchBegan(tLocation, nCount)
	-- 没有显示处理，不给按钮处理
	if self.nCurStatus ~= PopMenuStatus.PopHadShow then
		return;
	end

	if self.pBtnMgr == nil then
		return;
	end

	local tPos = self.pBtnMgr:convertToNodeSpace(tLocation)
	self.pBtnMgr:touchMenuItemBeganByPoint(tPos)
end

-- 触控移动
function ViewPopMenu:ViewTouchMoved(tLocation, tPrevPos, tBeganPos)
	-- 没有显示处理，不给按钮处理
	if self.nCurStatus ~= PopMenuStatus.PopHadShow then
		return;
	end

	if self.pBtnMgr == nil then
		return;
	end
	local tPos = self.pBtnMgr:convertToNodeSpace(tLocation)
	self.pBtnMgr:touchMenuItemMovedByPoint(tPos)
end

-- 触控结束
function ViewPopMenu:ViewTouchEnded(tLocation, tPrevPos, tBeganPos)
	-- 没有显示处理，不给按钮处理
	if self.nCurStatus ~= PopMenuStatus.PopHadShow then
		return;
	end

	if self.pBtnMgr == nil then
		return;
	end
	local tPos = self.pBtnMgr:convertToNodeSpace(tLocation)
	self.pBtnMgr:touchMenuItemEndedByPoint(tPos)
end

return ViewPopMenu
