--
-- Author: Link
-- Date: 2014-06-13 11:19:32
--

local GameView = requires(ROOT_PATH, "app/GameLg/Lobby/Others/GameView")
local ViewStoryTurntable = class("ViewStoryTurntable", GameView)

local StoryTurnTag = {
	SpAwardList 		= 1,	-- 奖励类型

	LbAwardExpNum		= 7,	-- 奖励经验数
	LbAwardGoldNum		= 8,	-- 奖励金币数
}

function ViewStoryTurntable:ctor()
	self.super.ctor(self)

	-- 是否已经结束
	self.bIsEnd	= false
	-- 自己的内容
	self.tResult = nil
	self.tOtherInfo = {}

	-- 当前的回调
	self.pCBack = {}
	setmetatable(self.pCBack, {__mode = "v"})

	-- 底部灰色
	local pLyColor = CCLayerColor:create( GRAY_BG_COLOR )
	self:addChild(pLyColor, -1)

	-- 当前界面
	self.pCtuiLayer = CTUILayer:new();
	self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/ZhuanPan.ctui"))
	self.pCtuiLayer:setAnchorPoint(ccp(0.5, 0.5))
	self.pCtuiLayer:setPosition(ctprccp(0.5, 0.5))
	self.pCtuiLayer:ignoreAnchorPointForPosition(false)
	self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	self.pCtuiLayer:ShowAllUiInfo()
	self:addChild(self.pCtuiLayer)
	self.pCtuiLayer:release()

	-- self.pCtuiLayer:setDelegateLua(function(event)
	-- -- 如果是按钮点击事件
	-- 	if event.cbType == "btnCb" then
	-- 		self:CTUIButtonCallBack(event.Layer, event.Btn, event.Tag)
	-- 	end
	-- end)

	-- 经验值
	local pLbExp = self.pCtuiLayer:GetNodeByTag(StoryTurnTag.LbAwardExpNum)
	tolua.cast(pLbExp, "CCLabelTTF")
	pLbExp:setString("0")

	-- 金钱
	local pLbGoldNum = self.pCtuiLayer:GetNodeByTag(StoryTurnTag.LbAwardGoldNum)
	tolua.cast(pLbGoldNum, "CCLabelTTF")
	pLbGoldNum:setString("0")

end

-- 点击操作的回调
function ViewStoryTurntable:SetTableDelegate(pDelegate)
	self.pCBack.delegate = pDelegate;
end

-- 根据商城的信息显示
function ViewStoryTurntable:InitViewWithData(tDataList)
	self.tOtherInfo = tDataList[2];
	-- 初始化显示扭蛋
	self:StartToShowResult(tDataList[1])
end

-- 重新根据数据显示
function ViewStoryTurntable:ReshowWithData(tDataList)

end

-- 获取是否已经转动结束
function ViewStoryTurntable:StartToShowResult(tResult)
	local pSpAward = self.pCtuiLayer:GetNodeByTag(StoryTurnTag.SpAwardList)

	-- 转盘的声音
	SOUNDMGR:PlayEffTurnTable()

	-- 默认是至少10圈
	local fRotValue = 720

	self.tResult = clone(tResult)

	-- 经验翻倍
	if tResult.nRewardType == 2 then
		fRotValue = fRotValue + 0

	-- 金币翻倍
	elseif tResult.nRewardType == 3 then
		fRotValue = fRotValue + 270

	-- 卡牌
	elseif tResult.nRewardType == 4 then
		fRotValue = fRotValue + 90

	-- 没有奖励
	else
		fRotValue = fRotValue + 180
	end

	local pArrAct = CCArray:create()
	pArrAct:addObject(CCRotateBy:create(1.1, fRotValue))
	pArrAct:addObject(CCDelayTime:create(0.15))
	pArrAct:addObject(CCCallFunc:create(function ()
		-- 经验值
		local pLbExp = self.pCtuiLayer:GetNodeByTag(StoryTurnTag.LbAwardExpNum)
		tolua.cast(pLbExp, "CCLabelTTF")
		pLbExp:setString("+" .. self.tResult.nAddExp)

		-- 金钱
		local pLbGoldNum = self.pCtuiLayer:GetNodeByTag(StoryTurnTag.LbAwardGoldNum)
		tolua.cast(pLbGoldNum, "CCLabelTTF")
		pLbGoldNum:setString("+" .. self.tResult.nAddGold)

		self.bIsEnd = true
		-- 卡牌
		if tResult.nRewardType == 4 then
			local tBagCard = tResult.tCardInfo
			-- 添加物品进入背包
			DATAMGR:GetUserInfoMgr():AddItemToBag(tBagCard)
			-- 显示获取得到新物品
			NEW_ITEM_MGR:ShowAddOneItem(tBagCard.nID, tBagCard.nNum)
		end

		-- 升级消息
		local tLevelUpInfo = DATAMGR:GetUserInfoMgr():GetLevelUpInfo()
		if tLevelUpInfo.bIsNeedShow == true then
			GameViewMgr:shareMgr():ShowChildViewWithInfo(GameViewType.LevelUpNotice)
		end

		-- 进入下一步教学，如果没有教学，内部不处理
		TEACHMGR:ShowSwichToNextStep();
	end))

	-- 开始转动
	pSpAward:runAction(CCSequence:create(pArrAct))
end


-- 获取是否已经转动结束
function ViewStoryTurntable:GetIsTurnEnd()
	return self.bIsEnd
end

-- 获取结束信息
function ViewStoryTurntable:GetResultInfo()
	return self.tResult
end
-- mark: ============== 触控的管理 ================
-- 开始触控
function ViewStoryTurntable:ViewTouchBegan(tLocation, nCount)
end

-- 触控移动
function ViewStoryTurntable:ViewTouchMoved(tLocation, tPrevPos, tBeganPos)
end

-- 触控结束
function ViewStoryTurntable:ViewTouchEnded(tLocation, tPrevPos, tBeganPos)
	if self:GetIsTurnEnd() == true then
		if self.pCBack.delegate then
			self.pCBack.delegate:StoryTurntableCallBack( self:GetResultInfo(), self.tOtherInfo )
		end
		-- 进入下一步教学，如果没有教学，内部不处理
		TEACHMGR:ShowSwichToNextStep();
		self:CloseView();
	end
end

return ViewStoryTurntable;