--
-- Author: NickWang
-- Date: 2014-04-23 14:31:39
--

local LogicAccount = class("LogicAccount", function()
	return display.newLayer()
end)

-- 账户相关。
local Tag_Account = {

	--背景层的tag
	Ly_BgTag	= 1,

	-- 修改密码(btn)
	Btn_Login_ChangePSW = 23,
	-- 创建新账户(btn)
	Btn_Login_NewAccount = 28,
	
	Sp_Normal_ChangePSW = 8,
	Sp_Select_ChangePSW = 10,

	-- **登录界面** --
	Ly_Login = 0,
	Sp_LoginAccount = 3,
	Sp_LoginPSW = 6,
	Btn_LoginEnter = 9, -- 进入按钮
	Btn_LoginReturn = 16, -- 返回按钮

	-- **创建新账户** --
	Ly_Register = 33, -- 层
	Sp_RegisterEmail = 37, -- 点击输入邮箱
	Sp_RegisterPSW = 40, -- 点击输入密码
	Sp_RegisterConfirmPSW = 44, -- 点击确认密码
	Btn_RegisterEnter = 48, -- 注册按钮
	Btn_RegisterReturn = 55, -- 返回按钮

	-- **修改密码** --
	Ly_ChangePSW = 62, -- 层
	Sp_ChangeEmail = 79, -- 账户邮箱

	Sp_ChangeOldPSW = 83, -- 旧密码

	Sp_ChangeNewPSW = 87, -- 新密码
	-- Sp_ChangeConfirmPSW = 5, -- 确认密码

	Btn_ChangeEnter = 65, -- 修改按钮
	Btn_ChangeReturn = 72, -- 返回按钮
}
-- 账户状态。
local Account_Status = {
	Login = 1, -- 登录（默认）
	Register = 2, -- 注册新用户
	ChangePSW = 3, -- 修改密码
	QuickGame = 4, -- 快速游戏。
}

-- init
function LogicAccount:ctor()
	-- self.super.ctor(self);

	-- 创建昵称界面
	self.pCtuiLayer = CTUILayer:new();
	self.pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/dljiemian.ctui"))
	self.pCtuiLayer:setAnchorPoint(ccp(0.5, 0.5))
	self.pCtuiLayer:setPosition(ctprccp(0.5, 0.5))
	self.pCtuiLayer:ignoreAnchorPointForPosition(false)
	self.pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
	self.pCtuiLayer:ShowAllUiInfo()
	self:addChild(self.pCtuiLayer)

	self.pCtuiLayer:setDelegateLua(function(event)
		-- 如果是按钮点击事件
		if event.cbType == "btnCb" then
			self:CTUIButtonCallBack(self.pCtuiLayer, event.Btn, event.Tag)
		end
	end)

	self.pCtuiLayer:release()

	-- 设定当前状态
	self:setStatusAndUpdateUI(Account_Status.Login); -- 登录状态.

end

-- 切换状态更新UI显示
function LogicAccount:setStatusAndUpdateUI( status )
	-- body
	if self.nStatus ~= status then
		self.nStatus = status; -- 账户状态.

		-- 输入框相关。
		local pSpUserName = nil; -- 用户名（邮箱）.
		local pSpPsw = nil; -- 密码。
		local pSpCertainPsw = nil; -- 确认密码。
		local pSpCertain2Psw = nil; -- 确认密码2。

		-- 是否显示默认用户名+密码
		local bShowDefaultAccount = false;

		-- 设置UI层显示 
		if self.nStatus == Account_Status.Login or self.nStatus == Account_Status.QuickGame then
			-- 显示登录层
			self.pCtuiLayer:SetCTUINodeVisiable(Tag_Account.Ly_Login, true);
			self.pCtuiLayer:SetCTUINodeVisiable(Tag_Account.Ly_Register, false);
			self.pCtuiLayer:SetCTUINodeVisiable(Tag_Account.Ly_ChangePSW, false);

			-- 显示账户名 + 密码.
			pSpUserName = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_LoginAccount);
			pSpPsw = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_LoginPSW);

			bShowDefaultAccount = true;

			-- 替换按钮图片。
			local pSpQuickGame1 = nil
			local pSpQuickGame2 = nil
			if self.nStatus == Account_Status.QuickGame then
				-- 快速游戏 修改密码--》快速游戏.
				pSpQuickGame1 = CCSprite:create("Images/DLJM-024.png")
				pSpQuickGame2 = CCSprite:create("Images/DLJM-024.png")
			else
				-- 
				pSpQuickGame1 = CCSprite:create("Images/DLJM-011.png")
				pSpQuickGame2 = CCSprite:create("Images/DLJM-011.png")
			end
			-- normal.
			local pSpChangePswNormal = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_Normal_ChangePSW)
			tolua.cast(pSpChangePswNormal, "CCSprite")
			pSpChangePswNormal:setTexture(pSpQuickGame1:getTexture())
			-- normal.
			local pSpChangePswSelect = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_Select_ChangePSW)
			tolua.cast(pSpChangePswSelect, "CCSprite")
			pSpChangePswSelect:setTexture(pSpQuickGame2:getTexture())
		elseif self.nStatus == Account_Status.Register then
			-- 显示注册层
			self.pCtuiLayer:SetCTUINodeVisiable(Tag_Account.Ly_Login, false);
			self.pCtuiLayer:SetCTUINodeVisiable(Tag_Account.Ly_Register, true);
			self.pCtuiLayer:SetCTUINodeVisiable(Tag_Account.Ly_ChangePSW, false);

			-- 显示账户名 + 密码 + 确认密码.
			pSpUserName = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_RegisterEmail);
			pSpPsw = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_RegisterPSW);
			pSpCertainPsw = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_RegisterConfirmPSW);

			bShowDefaultAccount = false;
		else
			-- 显示修改密码层
			self.pCtuiLayer:SetCTUINodeVisiable(Tag_Account.Ly_Login, false);
			self.pCtuiLayer:SetCTUINodeVisiable(Tag_Account.Ly_Register, false);
			self.pCtuiLayer:SetCTUINodeVisiable(Tag_Account.Ly_ChangePSW, true);

			-- 显示账户名 + 密码 + 确认密码.
			pSpUserName = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_ChangeEmail);
			pSpPsw = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_ChangeOldPSW);
			pSpCertainPsw = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_ChangeNewPSW);
			-- pSpCertain2Psw = self.pCtuiLayer:GetNodeByTag(Tag_Account.Sp_ChangeConfirmPSW);

			bShowDefaultAccount = true;
		end

		-- clear 输入框
		if self.pInputAccount ~= nil then
			self.pInputAccount:removeFromParent();
			self.pInputAccount = nil
		end
		if self.pInputPsw ~= nil then
			self.pInputPsw:removeFromParent();
			self.pInputPsw = nil
		end
		if self.pInputConfirmPsw ~= nil then
			self.pInputConfirmPsw:removeFromParent();
			self.pInputConfirmPsw = nil
		end
		if self.pInputConfirm2Psw ~= nil then
			self.pInputConfirm2Psw:removeFromParent();
			self.pInputConfirm2Psw = nil
		end

		-- 输入法的回调
		local tFucInputCallBack = function ( event )
			if event.type == "change" then
				return self:CTTextInputChangeText(event.input, event.chText, event.text, event.back)
			elseif event.type == "detach" then
				self:CTTextInputStartClose(event.input, event.isReturn, event.didhide)
			end
		end

		-- 登陆名字的输入框
		if pSpUserName ~= nil then
			local tLgSize = CCSizeMake(pSpUserName:getContentSize().width*0.95, pSpUserName:getContentSize().height*0.95)
			self.pInputAccount = CTTextInput:textFieldWithPlaceHolder("请输入你的邮箱", tLgSize, "", tLgSize.height*0.56)
			self.pInputAccount:setPosition(pSpUserName:getContentSize().width/2.0, pSpUserName:getContentSize().height/2.0 )
			pSpUserName:addChild(self.pInputAccount, 2)

			self.pInputAccount:setInputDelegateLua(tFucInputCallBack)
		end

		-- 密码输入框
		if pSpPsw ~= nil then
			local tLgSize = CCSizeMake(pSpPsw:getContentSize().width*0.95, pSpPsw:getContentSize().height*0.95)
			self.pInputPsw = CTTextInput:textFieldWithPlaceHolder("请输入密码", tLgSize, "", tLgSize.height*0.56)
			self.pInputPsw:setPosition(pSpPsw:getContentSize().width/2.0, pSpPsw:getContentSize().height/2.0 )
			pSpPsw:addChild(self.pInputPsw, 2)
			self.pInputPsw:setSecureTextEntry(true)

			self.pInputPsw:setInputDelegateLua(tFucInputCallBack)
		end

		-- 密码确认框
		if pSpCertainPsw ~= nil then
			local tLgSize = CCSizeMake(pSpCertainPsw:getContentSize().width*0.95, pSpCertainPsw:getContentSize().height*0.95)
			self.pInputConfirmPsw = CTTextInput:textFieldWithPlaceHolder("请再输入一次密码", tLgSize, "", tLgSize.height*0.56)
			self.pInputConfirmPsw:setPosition(pSpCertainPsw:getContentSize().width/2.0, pSpCertainPsw:getContentSize().height/2.0 )
			pSpCertainPsw:addChild(self.pInputConfirmPsw, 2)
			self.pInputConfirmPsw:setSecureTextEntry(true)

			self.pInputConfirmPsw:setInputDelegateLua(tFucInputCallBack)
		end
		-- if pSpCertain2Psw ~= nil then
		-- 	local tLgSize = CCSizeMake(pSpCertain2Psw:getContentSize().width*0.95, pSpCertain2Psw:getContentSize().height*0.95)
		-- 	self.pInputConfirm2Psw = CTTextInput:textFieldWithPlaceHolder("请再输入一次密码", tLgSize, "", tLgSize.height*0.56)
		-- 	self.pInputConfirm2Psw:setPosition(pSpCertain2Psw:getContentSize().width/2.0, pSpCertain2Psw:getContentSize().height/2.0 )
		-- 	pSpCertain2Psw:addChild(self.pInputConfirm2Psw, 2)
		-- 	self.pInputConfirm2Psw:setSecureTextEntry(true)

		-- 	self.pInputConfirm2Psw:setInputDelegateLua(tFucInputCallBack)
		-- end

		-- 点击登陆用户的处理
		-- local pLbFName = self.pCtuiLayer:GetNodeByTag(GameLoginTag.SiLbUserName)
		-- tolua.cast(pLbFName,"CTLabel")
		if bShowDefaultAccount and DATAMGR.pLoginData.strUser and DATAMGR.pLoginData.strPsw then
			-- 给登陆显示登陆信息
		    self.pInputAccount:setString(DATAMGR.pLoginData.strUser)
		    if self.nStatus == Account_Status.Login or self.nStatus == Account_Status.QuickGame then
		    	self.pInputPsw:setString("******")
		    else
		    	self.pInputPsw:setString("")
		    end

		    -- 登陆名字
			-- pLbFName:setString(DATAMGR.pLoginData.strUser)
	    else
		    -- pLbFName:setString("点击登陆")
	    end

	end
end

-- 进入等级状态
function LogicAccount:setStatusLogin()
	self:setStatusAndUpdateUI(Account_Status.Login);
end

-- 进入等级状态
function LogicAccount:setStatusQuickGame()
	self:setStatusAndUpdateUI(Account_Status.QuickGame);
end

function LogicAccount:GetAccount()
	return self.pInputAccount:getString();
end

function LogicAccount:GetPassword()
	return self.pInputPsw:getString();
end
function LogicAccount:GetChangedPassword()
	return self.pInputConfirmPsw:getString();
end


-- 通知开始用账号登陆
function LogicAccount:StartToLoginPlayer()
	local sAccount = self.pInputAccount:getString();
	local sPsw = self.pInputPsw:getString();

	if sAccount == nil or sAccount == "" or sPsw == nil or sPsw == "" then
		DIALOGMGR:ShowNormalInfo("请填好所有登陆信息", "提示")
	else 
		self:getParent():SendUserLogin(sAccount, sPsw);
	end
end


-- mark: ============== 按钮的回调 ================
-- 按钮回调
function LogicAccount:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)

	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()

    print("nTag: "..nTag)
    -- **** 登录界面按钮 ****
	-- 退出账户界面，返回主登录界面
	if nTag == Tag_Account.Btn_LoginReturn then
		self:getParent():quitLogicAccount();
	-- 用户登录。
	elseif nTag == Tag_Account.Btn_LoginEnter then
		local sAccount = self.pInputAccount:getString();
		local sPsw = self.pInputPsw:getString();

		if sAccount == nil or sAccount == "" or sPsw == nil or sPsw == "" then
			DIALOGMGR:ShowNormalInfo("请填好所有登陆信息", "提示")
		else 

			self:getParent():SendGetClientVersion(sAccount, sPsw);
		end
	-- 进入注册界面 按钮
	elseif nTag == Tag_Account.Btn_Login_NewAccount then
		self:setStatusAndUpdateUI(Account_Status.Register)
	-- 进入修改密码界面 按钮
	elseif nTag == Tag_Account.Btn_Login_ChangePSW then
		if self.nStatus == Account_Status.QuickGame then
			-- 快速游戏 修改密码--》快速游戏.
			self:getParent():SendQuickGame()
		else
			-- 修改密码。
			self:setStatusAndUpdateUI(Account_Status.ChangePSW)
		end
	
	-- **** 注册界面按钮 ****
	-- 退出注册界面，返回账户界面
	elseif nTag == Tag_Account.Btn_RegisterReturn then
		if DATAMGR.pLoginData.strUser == nil or DATAMGR.pLoginData.strPsw == nil then
	    	self:setStatusAndUpdateUI(Account_Status.QuickGame)
	    -- 常规开始界面。
	    else
			self:setStatusAndUpdateUI(Account_Status.Login)
	    end
	-- 点击注册
	elseif nTag == Tag_Account.Btn_RegisterEnter then
		local sAccount = self.pInputAccount:getString();
		local sPsw = self.pInputPsw:getString();
		local sConfirmPsw = self.pInputConfirmPsw:getString();

		if sAccount == nil or sAccount == "" or 
			sPsw == nil or sPsw == "" or 
			sConfirmPsw == nil or sConfirmPsw == "" then
			DIALOGMGR:ShowNormalInfo("请填好所有注册信息", "提示")
		elseif sPsw ~= sConfirmPsw then
			DIALOGMGR:ShowNormalInfo("密码输入不一致！", "提示")
		else 
			self:getParent():SendAccountRegister(sAccount, sPsw);
		end

	-- **** 修改密码界面按钮 ****
	-- 退出修改密码界面，返回账户界面
	elseif nTag == Tag_Account.Btn_ChangeReturn then
		self:setStatusAndUpdateUI(Account_Status.Login)
	-- 修改密码
	elseif nTag == Tag_Account.Btn_ChangeEnter then
		local sAccount = self.pInputAccount:getString();
		local sOldPsw = self.pInputPsw:getString();
		local sNewPsw = self.pInputConfirmPsw:getString();
		-- local sNew2Psw = self.pInputConfirm2Psw:getString();

		if sAccount == nil or sAccount == "" or 
			sOldPsw == nil or sOldPsw == "" or 
			sNewPsw == nil or sNewPsw == "" then
			DIALOGMGR:ShowNormalInfo("请填好所有修改信息", "提示")
		-- elseif sNew2Psw ~= sNewPsw then
		-- 	DIALOGMGR:ShowNormalInfo("新密码输入不一致！", "提示")
		else 
			self:getParent():SendChangeAccountPassword(sAccount, sOldPsw, sNewPsw);
		end

	-- 其它。
	else 
		DIALOGMGR:ShowToast("该按钮未处理.")
	end
end
-- mark: ============== 输入框的回调 ================
-- 输入框文字发生改变，返回true表示不能发生对应的改变
function LogicAccount:CTTextInputChangeText(pInput, strChangeText, strNewText, bIsBack)
	return false
end
-- 输入框开始关闭
function LogicAccount:CTTextInputStartClose(pInput, bIsReturn, bIsDid)
	if pInput == self.pInputAccount then
		-- 如果是点击
		if bIsReturn == true and bIsDid == true then
			self.pInputPsw:GetTextField():attachWithIME()
		end

	elseif pInput == self.pInputPsw then
		
		if self.nStatus == Account_Status.Register then
			self.pInputConfirmPsw:GetTextField():attachWithIME()
		elseif self.nStatus == Account_Status.ChangePSW then
			self.pInputConfirmPsw:GetTextField():attachWithIME()
		end

	end
end

-- mark: ============== 触控的管理 ================
-- 开始触控
function LogicAccount:LgAccTouchBegan(tLocation, nCount)
	self.pCtuiLayer:CTUITouchBegan(tLocation)

	-- 输入框的触控
	if self.pInputAccount ~= nil then
		self.pInputAccount:TITouchBegan(tLocation);
	end
	if self.pInputPsw ~= nil then
		self.pInputPsw:TITouchBegan(tLocation);
	end
	if self.pInputConfirmPsw ~= nil then
		self.pInputConfirmPsw:TITouchBegan(tLocation);
	end
	if self.pInputConfirm2Psw ~= nil then
		self.pInputConfirm2Psw:TITouchBegan(tLocation);
	end
end

-- 触控移动
function LogicAccount:LgAccTouchMoved(tLocation, tPrevPos)
	self.pCtuiLayer:CTUITouchMoved(tLocation)

	-- 输入框的触控
	if self.pInputAccount ~= nil then
		self.pInputAccount:TITouchMoved(tLocation);
	end
	if self.pInputPsw ~= nil then
		self.pInputPsw:TITouchMoved(tLocation);
	end
	if self.pInputConfirmPsw ~= nil then
		self.pInputConfirmPsw:TITouchMoved(tLocation);
	end
	if self.pInputConfirm2Psw ~= nil then
		self.pInputConfirm2Psw:TITouchMoved(tLocation);
	end
end

-- 触控结束
function LogicAccount:LgAccTouchEnded(tLocation, tPrevPos)
	self.pCtuiLayer:CTUITouchEnded(tLocation)

	-- 输入框的触控
	if self.pInputAccount ~= nil then
		self.pInputAccount:TITouchEnded(tLocation);
	end
	if self.pInputPsw ~= nil then
		self.pInputPsw:TITouchEnded(tLocation);
	end
	if self.pInputConfirmPsw ~= nil then
		self.pInputConfirmPsw:TITouchEnded(tLocation);
	end
	if self.pInputConfirm2Psw ~= nil then
		self.pInputConfirm2Psw:TITouchEnded(tLocation);
	end
end

return LogicAccount