--
-- Author: NickWang
-- Date: 2014-05-01 13:49:47
-- 选择初始车。

local LogicSelInitCar = class("LogicSelInitCar", function()
	return display.newLayer()
end)

-- 车卡牌的显示
local Tag_SelCar = {
	-- 底板
	Sp_Bg = 0,
	-- 车名称
	Sp_Car_Name = 2,
	-- 车星级
	Sp_Car_Star = 1,
	-- 车icon
	Sp_Car_Icon = 18,
	
	-- 车速度
	-- Sp_Speed_Name = 5,
	Lb_Speed_Num = 15,
	Sp_Speed_Line = 7,

	-- 车加速度
	-- Sp_Accel_Name = 4,
	Lb_Accel_Num = 16,
	Sp_Accel_Line = 10,
	
	-- 车转弯
	-- Sp_Turn_Name = 6,
	Lb_Turn_Num = 17,
	Sp_Turn_Line = 13,
	
	Sp_Bot_Add 	= 19,	-- 属性加成图标
	
	-- 路面名字
	Lb_Name_Grass = 28, -- 草地
	Lb_Name_Shamo = 20, -- 沙漠
	Lb_Name_HighWay = 32, -- 公路
	Lb_Name_Ice = 22, -- 冰雪
	Lb_Name_Moto = 26, -- 越野
	Lb_Name_Fire = 30, -- 火山
	Lb_Name_Stone = 24, -- 石路

	-- 路面加成的数字
	Lb_Add_Grass = 29, -- 草地
	Lb_Add_Shamo = 21, -- 沙漠
	Lb_Add_HighWay = 33, -- 公路
	Lb_Add_Ice = 23, -- 冰雪
	Lb_Add_Moto = 27, -- 越野
	Lb_Add_Fire = 31, -- 火山
	Lb_Add_Stone = 25, -- 石路
}

-- 初始化.
function LogicSelInitCar:ctor()
	-- self.super.ctor(self)
	-- 触控区域处理
	self.bIsTouchInside = false;

	-- 
	self.s_fBaseScale = 0.5; -- 卡牌默认缩放大小.
	self.s_centerPos = ccp(ctprwidth(0.5), ctprheight(1)-ctipwidth(542)); -- 椭圆中心坐标.
	self.s_fXRadias = 300; -- x轴半径
	self.s_fYRadias = 25; -- y轴半径
	self.s_fMaxSkewY = 4; -- y方向倾斜度.

	-- ctui
	self.pCarIdList = {1046, 1047, 1048, 1049, 1050, 1021}
	local maxCardNum = #self.pCarIdList; -- 显示的卡牌总个数.
	
	self.pAngleList = {} -- 保持角度
	self.pCtuiList = {} -- 保持卡牌CtuiLayer
	self.nSelectCarIdx = 1; -- 默认选择第一个序号的车。
	local fAnrPosy = 0.06
	local nShowType = SHARE_FUNC.nFrameType;--GetCurrentDeviceType()
    -- ipad
    if nShowType == CTDeviceTypeIpad or nShowType == CTDeviceTypeIpadHD then
    	fAnrPosy = 0
    -- iphone5高清
    elseif nShowType == CTDeviceTypeIphone5HD or nShowType == CTDeviceTypeItouch5HD then
    	fAnrPosy = 0.1
    end

	for i=1,maxCardNum do
		-- print("idx: "..i)

		-- 椭圆属性
		local fAngle = 360 / maxCardNum * (i-1) + 90; -- 默认选择第一个序号的车。
		self.pAngleList[i] = fAngle;

		fAngle = CC_DEGREES_TO_RADIANS(fAngle);
		local fx = self.s_centerPos.x + self.s_fXRadias * math.cos(fAngle)
		local fy = self.s_centerPos.y + self.s_fYRadias * math.sin(fAngle)
		-- print("fx: "..fx..", fy: "..fy)

		-- 单个ctui
		local pCtuiLayer = CTUILayer:new();
		pCtuiLayer:InitWithFile(Public_GetResPathByKey("Images/hdkp.ctui"))
		pCtuiLayer:setAnchorPoint(ccp(0.5, fAnrPosy))
		-- pCtuiLayer:setPosition(ccp(fx, fy))
		pCtuiLayer:ignoreAnchorPointForPosition(false)
		pCtuiLayer:SetUiScaleType(CTUIScaleType_Base_Width)
		pCtuiLayer:ShowAllUiInfo()

		-- 设置选择车的属性.
		self:setSelectCarInfo(pCtuiLayer, self.pCarIdList[i]);

		self.pCtuiList[i] = pCtuiLayer;

		-- 设置透视效果
		self:setUIEffect(self.pCtuiList[i], self.pAngleList[i]);

		-- y值越高，显示层次越高.
		self:addChild(pCtuiLayer, fy, i)

		pCtuiLayer:setDelegateLua(function(event)
			-- 如果是按钮点击事件
			if event.cbType == "btnCb" then
				self:CTUIButtonCallBack(pCtuiLayer, event.Btn, event.Tag)
			end
		end)

		pCtuiLayer:release()
	end

	self.bgBaseScale = self:getBgBaseScale()
	-- 显示选中的卡牌变大
	self:showSelectCarToBig(true)
	
end

-- 显示车属性.
function LogicSelInitCar:setSelectCarInfo( pCtuiLayer, nCarId )
	local carInfo = DATAMGR:GetItemInfoMgr():GetItemInfoByID(nCarId)
	
	--背景底
	local pSpBg = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Bg)
	tolua.cast(pSpBg, "CCSprite")
	-- 设置抗锯齿.
	pSpBg:getTexture():setAntiAliasTexParameters()

	-- 车图标icon
	local pSpCarIcon = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Car_Icon)
	tolua.cast(pSpCarIcon, "CCSprite")
	if nCarId ~= nil then
		local spTex = DATAMGR:GetItemInfoMgr():GetCarBigImageByID(nCarId)
		pSpCarIcon:setTexture(spTex)
		-- 设置抗锯齿.
		spTex:setAntiAliasTexParameters()
	end

	-- 车名称
	local pSpCarName = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Car_Name)

	local CTWordImg = requires(ROOT_PATH, "app/GameConfig/CTWordImg")
	local pLbWord = CTWordImg.new()
	pLbWord:InitWord(carInfo.strName, "CarName", 60*pSpCarName:getScale())
	pLbWord:setAnchorPoint( ccp(0, 0.5) )
	pLbWord:setPosition( ccpAdd(pSpCarName:getPositionInCCPoint(), ccp(-30, 0)) )
	pSpCarName:getParent():addChild(pLbWord, 0, pSpCarName:getTag())

	pSpCarName:removeFromParent()

	-- 获取不同的纹理
	local pBgTex, pStarTex
	local nStarLv = DATAMGR:GetItemInfoMgr():GetLevelStarTypeByName(carInfo.strStar)
	if nStarLv == StarTypeDef.StarS then
		pBgTex = CCTextureCache:sharedTextureCache():addImage("Images/HDKP-001.png")
		pStarTex = CCTextureCache:sharedTextureCache():addImage("Images/GX-046.png")
	elseif nStarLv == StarTypeDef.StarA then
		pBgTex = CCTextureCache:sharedTextureCache():addImage("Images/HDKP-005.png")
		pStarTex = CCTextureCache:sharedTextureCache():addImage("Images/GX-047.png")
	elseif nStarLv == StarTypeDef.StarB then
		pBgTex = CCTextureCache:sharedTextureCache():addImage("Images/HDKP-002.png")
		pStarTex = CCTextureCache:sharedTextureCache():addImage("Images/GX-048.png")
	else
		pBgTex = CCTextureCache:sharedTextureCache():addImage("Images/HDKP-003.png")
		pStarTex = CCTextureCache:sharedTextureCache():addImage("Images/GX-049.png")
	end

	-- -- 车星级
	local pSpCarStar = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Car_Star)
	tolua.cast(pSpCarStar, "CCSprite")
	pSpCarStar:setTexture(pStarTex)

	pSpBg:setTexture(pBgTex)

	local NumMax = 300
	-- ****车速度****
	-- 速度数值
	local pLbCarSpeedNum = pCtuiLayer:GetNodeByTag(Tag_SelCar.Lb_Speed_Num)
	tolua.cast(pLbCarSpeedNum, "CTLabel")
	pLbCarSpeedNum:setString(""..math.floor(carInfo.fSpeed))
	-- 速度线卡通
	local pSpSpeedLine = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Speed_Line)
	tolua.cast(pSpSpeedLine, "CCSprite")
	local scaleX = carInfo.fSpeed/NumMax
	if scaleX > 1 then
		scaleX = 1
	elseif scaleX < 0 then
		scaleX = 0
	end
	pSpSpeedLine:setScaleX(scaleX)

	-- ****车加速度****
	-- 加速度数值
	local pLbCarAccelNum = pCtuiLayer:GetNodeByTag(Tag_SelCar.Lb_Accel_Num)
	tolua.cast(pLbCarAccelNum, "CTLabel")
	pLbCarAccelNum:setString(""..math.floor(carInfo.fASpeed))
	-- 加速度线卡通
	local pSpASpeedLine = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Accel_Line)
	tolua.cast(pSpASpeedLine, "CCSprite")
	local scaleX = carInfo.fASpeed/NumMax
	if scaleX > 1 then
		scaleX = 1
	elseif scaleX < 0 then
		scaleX = 0
	end
	pSpASpeedLine:setScaleX(scaleX)

	-- ****车转弯****
	-- 转弯数值
	local pLbCarTurnNum = pCtuiLayer:GetNodeByTag(Tag_SelCar.Lb_Turn_Num)
	tolua.cast(pLbCarTurnNum, "CTLabel")
	pLbCarTurnNum:setString(""..math.floor(carInfo.fTurn))
	-- 转弯线卡通
	local pSpTurnLine = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Turn_Line)
	tolua.cast(pSpTurnLine, "CCSprite")
	local scaleX = carInfo.fTurn/NumMax
	if scaleX > 1 then
		scaleX = 1
	elseif scaleX < 0 then
		scaleX = 0
	end
	pSpTurnLine:setScaleX(scaleX)

	-- 图表
	local pSpAddBot = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Bot_Add)
	SHARE_FUNC:ShowMapAdditionToCar(clone(carInfo.tRoadEffect), pSpAddBot)

	-- 类型的预定义
	local tRoadType = {
		RaceRoadType.CaoDi,
		RaceRoadType.ShangDi,
		RaceRoadType.GongLu,
		RaceRoadType.BingLu,
		RaceRoadType.YueYeLu,
		RaceRoadType.HuoShan,
		RaceRoadType.ShiLu,
	}

	-- 路面名字
	local tRoadNames = {
		Tag_SelCar.Lb_Name_Grass,
		Tag_SelCar.Lb_Name_Shamo,
		Tag_SelCar.Lb_Name_HighWay,
		Tag_SelCar.Lb_Name_Ice,
		Tag_SelCar.Lb_Name_Moto,
		Tag_SelCar.Lb_Name_Fire,
		Tag_SelCar.Lb_Name_Stone,
	}
	

	-- 路面加成的数字
	local tRoadAdds = {
		Tag_SelCar.Lb_Add_Grass,
		Tag_SelCar.Lb_Add_Shamo,
		Tag_SelCar.Lb_Add_HighWay,
		Tag_SelCar.Lb_Add_Ice,
		Tag_SelCar.Lb_Add_Moto,
		Tag_SelCar.Lb_Add_Fire,
		Tag_SelCar.Lb_Add_Stone,
	}

	for i,nRoadType in ipairs(tRoadType) do
		-- 名字
		local pLbMapName = pCtuiLayer:GetNodeByTag(tRoadNames[i])
		tolua.cast(pLbMapName, "CTLabel")
		pLbMapName:setString( DATAMGR:GetItemInfoMgr():GetRoadTypeNameByType(nRoadType) )

		-- 加成的数字
		local pLbMapAdd = pCtuiLayer:GetNodeByTag(tRoadAdds[i])
		tolua.cast(pLbMapAdd, "CTLabel")
		local nRoadEff = carInfo.tRoadEffect[nRoadType]
		local nShowRoadEff = math.floor( nRoadEff )
		if nShowRoadEff>0 then
			pLbMapAdd:setString( "+"..nShowRoadEff.."%" )
		else
			pLbMapAdd:setString( nShowRoadEff.."%" )
		end
		-- 信息
		if nShowRoadEff < 0 then
			pLbMapAdd:setColor( display.COLOR_RED )
		elseif nShowRoadEff > 0 then
			pLbMapAdd:setColor( display.COLOR_GREEN )
		else
			pLbMapAdd:setColor( ccc3(255, 255, 0) )
		end
	end

end

-- 设置透视效果
function LogicSelInitCar:setUIEffect( pCtuiLayer, fAngle )
	-- body
	-- tolua.cast(pCtuiLayer,"CTUILayer")

	fAngle = CC_DEGREES_TO_RADIANS(fAngle);
	local fx = self.s_centerPos.x + self.s_fXRadias * math.cos(fAngle)
	local fy = self.s_centerPos.y + self.s_fYRadias * math.sin(fAngle)

	local pos = ccp(fx, fy);
	pCtuiLayer:setPosition(pos)
	pCtuiLayer:setZOrder(fy)

	-- posY值低于中心点，卡牌才不透明.
	local pBgSp = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Bg);
	tolua.cast(pBgSp,"CCSprite")
	pBgSp:setCascadeOpacityEnabled(true)
	if pos.y < self.s_centerPos.y then
		pBgSp:setOpacity(0xff/3)
	else
		pBgSp:setOpacity(0xff/1)
	end

	-- 缩放，背面的图片需要缩小(并且x方向翻转)。
	if pos.y < self.s_centerPos.y then
		pCtuiLayer:setScale(self.s_fBaseScale * 0.8)
		pCtuiLayer:setScaleX( pCtuiLayer:getScaleX() * (-1))
	else
		pCtuiLayer:setScale(self.s_fBaseScale * 1.0)
	end

	-- x方向缩放，距离中心点越远，缩放越大
	pCtuiLayer:setScaleX( pCtuiLayer:getScaleX() * math.abs( math.sin(fAngle) ))

	-- y方向倾斜.
	local fSkewY = -math.cos(fAngle) * self.s_fMaxSkewY;
	-- if pos.x < self.s_centerPos.x then
	-- 	-- 中心点左侧
	-- 	fSkewY = self.s_fMaxSkewY;
	-- elseif pos.x > self.s_centerPos.x then
	-- 	-- 中心点右侧
	-- 	fSkewY = self.s_fMaxSkewY * (-1);
	-- else
	-- 	fSkewY = 0;
	-- end
	pCtuiLayer:setSkewY(fSkewY)

end

-- 获取卡牌背景默认的scale.
function LogicSelInitCar:getBgBaseScale()
	local pCtuiLayer = self.pCtuiList[self.nSelectCarIdx]
	local pSpBg = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Bg)
	tolua.cast(pBgSp,"CCSprite")
	return pSpBg:getScale()
end

-- 放大当前选中的车卡牌
function LogicSelInitCar:showSelectCarToBig( bBig )
	-- 
	local pCtuiLayer = self.pCtuiList[self.nSelectCarIdx]
	local pSpBg = pCtuiLayer:GetNodeByTag(Tag_SelCar.Sp_Bg)
	tolua.cast(pBgSp,"CCSprite")
	if bBig == true then
		pSpBg:setScale( self.bgBaseScale * 1.4 )
	else
		pSpBg:setScale( self.bgBaseScale * 1.0 )
	end
end

-- 当前选中的车id
function LogicSelInitCar:GetSelectCarId()
	-- 
	return self.pCarIdList[self.nSelectCarIdx]
end

-- mark: ============== 按钮的回调 ================
-- 按钮回调
function LogicSelInitCar:CTUIButtonCallBack(pUiLayer, pMenuItem, nTag)

	-- 播放点击按钮声音
    SOUNDMGR:PlayEffClickButton()

    print("nTag: "..nTag)
	-- 退出账户界面，返回主登录界面
	-- if nTag == Tag_Servers.Btn_LateLogon_Selected then
	-- 	self:getParent():quitLogicServers();
	
	-- -- 其它。
	-- else 
		DIALOGMGR:ShowToast("该按钮未处理.")
	-- end
end

-- mark: ============== 循环处理 ================
function LogicSelInitCar:LogicLoop( dt )
	-- 自动旋转动画。
	if self.startFlag == true then
		local moveAngle = 0;
		if self.autoMoveAngle > 0 then
			moveAngle = 100 * dt;
		elseif self.autoMoveAngle < 0 then
			moveAngle = -100 * dt;
		end
		-- print("moveAngle: "..moveAngle)

		if math.abs(self.autoMoveAngle) <= math.abs(moveAngle) then
			-- 自动动画结束
			moveAngle = self.autoMoveAngle
			self.autoMoveAngle = 0
			self:endAutoEffect()
		else
			self.autoMoveAngle = self.autoMoveAngle - moveAngle;
		end

		-- 卡牌自动旋转。
		for i,v in ipairs(self.pAngleList) do
			local fAngle = self.pAngleList[i] + moveAngle
			self.pAngleList[i] = fAngle
			self:setUIEffect(self.pCtuiList[i], fAngle)
		end

		if self.startFlag == false then
			return true;
		end
	end

	return false;
end

-- mark: ============== 动画还原 ================
-- 动画效果自然结束
function LogicSelInitCar:endAutoEffect()
	-- 
	self.startFlag = false;

	-- 显示选中的卡牌变大
	self:showSelectCarToBig(true)
	
end

-- 强迫停止动画效果
function LogicSelInitCar:forceStopAutoEffect()
	-- 
	self.startFlag = false;

	-- 显示选中的卡牌不变大
	self:showSelectCarToBig(false)
	
end

-- 开始动画效果
function LogicSelInitCar:startAutoEffect()
	
	-- 显示选中的卡牌不变大
	self:showSelectCarToBig(false)
	
	-- 
	self.startFlag = true;
	-- self.time = 0.5; -- 恢复的最长时间

	-- 查找最近的车在数组中的序号。
	local minAngle = 100;
	for i,v in ipairs(self.pAngleList) do
		-- print("i: "..i..", angle: "..v)
		local fAngle = math.abs(v - 90)
		if fAngle < minAngle then
			self.nSelectCarIdx = i;
			minAngle = fAngle;
			-- print("minAngle: "..minAngle)
		end
	end

	-- 
	self.autoMoveAngle = 90 - self.pAngleList[self.nSelectCarIdx];

end

-- mark: ============== 触控的管理 ================
-- 开始触控
function LogicSelInitCar:LgTouchBegan(tLocation, nCount)
	-- 触控区域处理
	self.bIsTouchInside = true;
	self:forceStopAutoEffect();
end

-- 触控移动
function LogicSelInitCar:LgTouchMoved(tLocation, tPrevPos)

	if self.bIsTouchInside == false then
		return;
	end
	local dx = tLocation.x - tPrevPos.x
	if dx ~= 0 then
		for i,pCtuiLayer in ipairs(self.pCtuiList) do
			-- print(i)
			self.pAngleList[i] = self.pAngleList[i] - dx/3.;

			-- 保持角度在[0, 360]之间.
			if self.pAngleList[i] < 0 then
				self.pAngleList[i] = self.pAngleList[i] + 360;
			elseif self.pAngleList[i] >= 360 then
				self.pAngleList[i] = self.pAngleList[i] - 360;
			end
			self:setUIEffect(pCtuiLayer, self.pAngleList[i])
		end
	end
end

-- 触控结束
function LogicSelInitCar:LgTouchEnded(tLocation, tPrevPos)
	if self.bIsTouchInside == false then
		self.bIsTouchInside = false;
		return;
	end
	self:startAutoEffect();
	self.bIsTouchInside = false;
end

return LogicSelInitCar
