--
-- Author: Link
-- Date: 2013-12-28 09:51:43
--
local LogicRaceCar = class("LogicRaceCar", function ()
	return display.newNode()
end)

local MAX_DEL_SPEED_SCALE 	= 3.6
local MAX_DEL_TMP	= (MAX_DEL_SPEED_SCALE*2)/(MAX_DEL_SPEED_SCALE+1)

-- 技能效果类型
local SkEffectType = SkillEffResMgr:SkEffectType();

-- 车子移动的路段类型
CarMoveRoadType = {
	Road1 = 0,
	Mid = 1,
	Road2 = 2,
	Total = 3,
}
-- 技能效果类型
local EffectOnCarType = {
	None 		= 0,	-- 当前没有技能效果
	AddAtt 		= 1,	-- 加属性
	SubAtt 		= 2,	-- 减属性
	YouXianLine	= 3,	-- 优先级分界线，下面是按显示优先级分，数字越高，显示优先级越高
	Reflex 		= 4,	-- 反伤
	OnFire 		= 5,	-- 集火效果
	MarkAsEnemy	= 6,	-- 伪装成敌人
	Frozen 		= 7,	-- 被冰封
	MianYi 		= 8,	-- 免疫
}

-- 车子的移动
local CarMoveStatus = {
	NotStart 	= 0,	-- 没有开始移动
	Moving 		= 1,	-- 正在移动中
	Pause 		= 2,	-- 暂停
}

function LogicRaceCar:ctor()

	----- 车子的属性值 -----
	-- 车子从网络获得的信息
	self.tCarInfo = {
		nCarID = 1001,
		fMaxSpeed  = 80.0,
		fAddSpeed  = 60.0,
		fRotValue  = 90.0,
		nLv = 0,
		nMergeLv = 0,
		strStar    = "E",
		bIsSelf = false,
	}
	-- 适合当前车子当前赛道的比例
	self.tShowRate = {
		fSpeedRate = 1.0,
		fASpeedRate = 1.0,
		fTurnRate = 1.0,

		fMaxSpeed  = 80.0,
		fAddSpeed  = 60.0,
		fRotValue  = 90.0,
	}
	-------------------------

	-- 当前移动到的点的索引
	----- 当前要计算的值 -----
	self.fCurMaxSpeed = self.tShowRate.fMaxSpeed
	self.fCurAddSpeed = self.tShowRate.fAddSpeed
	-- 当前车的属性倍数
	self.fAttScale 	= 1.0;
	self.fDelAttScale = 0.0;
	-- 在自己身上的技能效果
	self.tArrSkillEffect = { --[[  nType:3,nEffType:2,fTime:2.0,fPercent:0.1,  ]] };
	-- 当前速度
	self.fSpeed 	= 0
	-- 移动到的点，也就是路段Idx
	self.nMoveIdx	= 0
	-- 当前所在的圈数
	self.nCurCircle	= 1
	-- 当前车子所在的排位
	self.nCurQualifying = 0
	-- 移动的权重
	self.nCarMoveValue	= 0;
	-- 当前车子初始所在的位置
	self.nStartCarPos 	= 1
	-- 当前路段的角度
	self.fLineRot	= 0
	-- 当前选中的移动路线
	self.nSelRoadType = CarMoveRoadType.Total
	-- 当前车子是否在移动的状态
	self.nMoveStatus	= CarMoveStatus.NotStart;
	-- 下次要移动的两点
	self.tNextPos1	= nil
	self.tNextPos2	= ccp(0, 0)
	-- 车身的转向速度
	-- 车子速度的转向速度
	self.fRotSpeedValue	= 120.0
	-- 当前方向开始转向的距离
	self.fStartTurnDis	= 100000.0
	-- 当前方向上开始转向时的理想速度
	self.fStartTurnSpeed = 60.0
	-- 地图的回调
	self.pMapDelegate	= nil
	-------------------------

	------ 当前技能效果 -------
	self.pSpEffects = {}
	-- 效果的大类型
	self.nShowEffectType = EffectOnCarType.None;
	-- 对应技能的效果
	self.nShowSkillEffType = SkEffectType.None;
	-------------------------
	self.pSpCar = nil;
	-- 四个轮胎的数组
	self.pSpLunTai = {}
	-- 烟尘
	self.pSpYan	= nil
	-- 总共的移动时间
	self.fTotalTime = 0.0
	-- 是否移动结束
	self.bIsMoveEnd = false;
	self.bIsCanEndCallBack = false;

	self.nSharkIdx	= 8

	-- 车子的初始位置
	self.tStartPos = ccp(0, 0)
	-- 设置图片
	self:setIgnoreAnchorPointForPosition(false)
	self:setAnchorPoint(ccp(0.5, 0.6))

	-- 顶部标记，排名
	self.pSpNumBottom = nil;
	self.pLbCarNum = nil;


	------ 用于判断车子是否进入死胡同的判断 -------
	-- 开始判断车子的时间间隔
	self.fJudgeTime 	= 0;
	-- 开始之前车子的判断时间
	self.fJudgeRoadLen 	= -1;
	------------------------------------------
end

-- fFirstRot是初始化角度，tCarInfo是车的属性信息，tArrRoad当前要走的路的信息
function LogicRaceCar:InitCar(fFirstRot, tCarInfo, tArrRoad1, tArrRoadMid, tArrRoad2)
	-- 最大速度
	self.tCarInfo.fMaxSpeed  = tCarInfo.nSpeed
	self.tShowRate.fMaxSpeed = tCarInfo.nSpeed
	self:SetCurMaxSpeed(self.tShowRate.fMaxSpeed)
	-- 加速度
	self.tCarInfo.fAddSpeed  = tCarInfo.nASpeed
	self.tShowRate.fAddSpeed = tCarInfo.nASpeed
	self:SetCurAddSpeed(self.tShowRate.fAddSpeed)
	-- 转弯属性值
	self.tCarInfo.fRotValue  = tCarInfo.nTurn
	self.tShowRate.fRotValue = tCarInfo.nTurn
	-- 车子的类型ID
	self.tCarInfo.nCarID = tCarInfo.nCarID
	-- 车子的等级
	self.tCarInfo.nLv = tCarInfo.nLv
	-- 车子的合成等级
	self.tCarInfo.nMergeLv = tCarInfo.nMergeLv
	self.tCarInfo.strStar = tCarInfo.strStar
	-- 是否是当前玩家自己的车
	self.tCarInfo.bIsSelf = tCarInfo.bIsSelf;
	
	self.fRotSpeedValue	= self.tShowRate.fRotValue

	-- 车子的所有行动点数
	self.tArrRoads1 = tArrRoad1
	self.tArrRoadsMid = tArrRoadMid
	self.tArrRoads2 = tArrRoad2

	-- 默认的是中间的路
	self.tArrRoads = self.tArrRoadsMid

	-- 车子的ID
	local strImgName  = "Images/Objects/MapCar"..tCarInfo.nCarID..".png"

	self.pSpCar = CCSprite:create(strImgName)

	-- 设置当前大小一致
	local tConSize = self.pSpCar:getContentSize()
	self:setContentSize(tConSize)

	self:setScale(0.53)
	-- 添加显示车子
	self.pSpCar:setPosition(tConSize.width * 0.5, tConSize.height * 0.5)
	self:addChild(self.pSpCar)

	-- 四个轮胎
	local tTmpPos = {
		ccp(0.1875, 0.7119),
		ccp(0.1875, 0.2119),
		ccp(0.8125, 0.2119),
		ccp(0.8125, 0.7119),
	}
	for i=1,4 do
		local pSpLunTai = CCSprite:create("Images/CheLuen1_1.png")
		if i <= 2 then
			pSpLunTai:setAnchorPoint(ccp(1, 0.5))
		else
			pSpLunTai:setAnchorPoint(ccp(0, 0.5))
		end
		pSpLunTai:setPosition(ccp(tConSize.width * tTmpPos[i].x, tConSize.height * tTmpPos[i].y))
		self.pSpCar:addChild(pSpLunTai, -1)
		self.pSpLunTai[i] = pSpLunTai
	end
	-- 车的影子
	local pSpShadow = CCSprite:create("Images/CheTouYing.png")
	pSpShadow:setPosition(ccp(tConSize.width * 0.41, tConSize.height * 0.45))
	self.pSpCar:addChild(pSpShadow, -2)

	-- 速度
	-- local fLightPath = Public_GetResPathByKey("Images/weiqi.plist")
	-- nickXml.NickTexCacheMgr:shareCache():AddOnePlistFile(fLightPath, 1)

	-- local strPath = Public_GetResPathByKey("Images/weiqi.xml")
	-- self.pSpYan = nickXml.NickXmlSprite:createWithStart(strPath, 0);
	-- self.pSpYan:setPosition( ccp(tConSize.width*0.5, -50) );
	-- self.pSpYan:setRotation( 180 )
	-- self.pSpCar:addChild(self.pSpYan)
	-- self.pSpYan:startPlay(true);

	SkillEffResMgr:RetainOneEff( SkEffectType.CarYanWuEff )
	-- 创建动画
	self.pSpYan = SkillEffResMgr:CreateSkillSwfEff( SkEffectType.CarYanWuEff, true )
	-- 根据情况添加到不同的显示中
	-- 调整位置
	self.pSpYan:setPosition( ccp(tConSize.width*0.5, 0) )
	self.pSpYan:setRotation(-90)
	self.pSpYan:setScale(0.5)
	self.pSpCar:addChild(self.pSpYan,1)

	-- 开始播放动画
	SkillEffResMgr:StartAnimation( SkEffectType.CarYanWuEff, self.pSpYan )
	self.pSpYan:setVisible(false)


	-- self.pSpYan = CCSprite:create("Images/PaiYan1.png")
	-- self.pSpYan:setAnchorPoint(ccp(0.5, 1))
	-- self.pSpYan:setPosition(ccp(tConSize.width*0.5, 0))
	-- self.pSpCar:addChild(self.pSpYan)

	-- local animation = CCAnimation:create()
	-- for i = 1, 4 do
	-- 	local name = "Images/PaiYan"..i..".png"
	-- 	animation:addSpriteFrameWithFileName(name)
	-- end
	-- animation:setDelayPerUnit(0.1)
	-- animation:setRestoreOriginalFrame(true)
	-- local action = CCAnimate:create(animation)
	-- self.pSpYan:runAction(CCRepeatForever:create(action))
	-- self.pSpYan:setVisible(false)

	self:setAnchorPoint(ccp(0.5, 0.6))
	-- 车轴的长度
	-- self.fWheelBase = tConSize.height * 0.8
	-- CAR_LEN_RATE = tConSize.height * 0.85 / self.fWheelBase
	-- 重新处理角度
	self:SetCurCarRot(fFirstRot)
	self.fMoveTargetRot = fFirstRot

	-- 当前路段的角度
	self.fLineRot = fFirstRot
	-- 自己的车的处理
	if self.tCarInfo.bIsSelf then
		self.pSpNumBottom = CCSprite:create("Images/CheBiaoQuan.png");
		-- pSpRotP:setRotation()
		self.pSpNumBottom:setScale(1.1);
		self.pSpNumBottom:setRotation( GetRandom(0, 360) )
		self.pSpNumBottom:setPosition( ctspccp(self, 0.5, 0.5) )
		self:addChild(self.pSpNumBottom, -1)
		self.pSpNumBottom:runAction( CCRepeatForever:create( CCRotateBy:create(0.4, 90) ) )
	end

end

-- 显示的东西
function LogicRaceCar:ShowCurCarSelInfo(bIsSel)
	if self.tCarInfo.bIsSelf == false then
		return;
	end

	if self.pSpNumBottom == nil then
		return;
	end

	local pTex
	if bIsSel then
		pTex = CCTextureCache:sharedTextureCache():addImage("Images/CheBiaoQuan1.png")
	else
		pTex = CCTextureCache:sharedTextureCache():addImage("Images/CheBiaoQuan.png")
	end
	self.pSpNumBottom:setTexture(pTex)

end

-- 显示名次变化
function LogicRaceCar:ShowCurQualifyEffect(nNum)

	if self.pLbCarNum then
		self.pLbCarNum:removeFromParent();
		self.pLbCarNum = nil;
	end

	-- 还没有开始，则不显示
	if self.nMoveStatus	== CarMoveStatus.NotStart then
		return;
	end

	-- local pLbNum = CCLabelTTF:create(""..nNum, LABEL_BOLD_FONT_NAME, 36)


	-- 自己的车的处理
	-- if self.tCarInfo.bIsSelf then
	-- 	self.pLbCarNum = CCSprite:create("Images/CheBiaoDi.png");

	-- 	pLbNum:setColor( ccc3(0x9c, 0x1d, 0x00) )
	-- else
	-- 	-- return
	-- 	self.pLbCarNum = CCSprite:create("Images/CheBiaoDi1.png");

	-- 	pLbNum:setColor( ccc3(0x00, 0x41, 0x84) )
	-- end
	
	-- self.pLbCarNum:setPosition( ctspccp(self, 0.5, 0.5) )
	-- self:addChild(self.pLbCarNum, 20)

	-- pLbNum:setPosition( ctspccp(self.pLbCarNum, 0.5, 0.5) )
	-- self.pLbCarNum:addChild(pLbNum)


	-- self.pLbCarNum:setScale(0)

	-- local pArrAct = CCArray:create()
	-- pArrAct:addObject( CCScaleTo:create(0.25, 2.85) )
	-- pArrAct:addObject( CCScaleTo:create(0.10, 1.0) )
	-- pArrAct:addObject( CCScaleTo:create(0.10, 1.2) )
	-- pArrAct:addObject( CCScaleTo:create(0.10, 1.0) )
	-- pArrAct:addObject( CCScaleTo:create(0.10, 1.2) )
	-- pArrAct:addObject( CCScaleTo:create(0.10, 1.0) )
	-- pArrAct:addObject( CCDelayTime:create(0.3) )
	-- pArrAct:addObject( CCCallFunc:create(function()
	-- 	self.pLbCarNum:removeFromParent()
	-- 	self.pLbCarNum = nil;
	-- end) )

	-- self.pLbCarNum:runAction( CCSequence:create(pArrAct) )
end

-- 显示车子的烟尘效果
function LogicRaceCar:ShowCarYanChen()
	-- 如果已经在显示，则不处理
	if self.pSpYan:isVisible() then
		return
	end

	self.pSpYan:setVisible(true)

	local tDelay = CCDelayTime:create(0.3)
	local tHide = CCHide:create()
	self.pSpYan:runAction(CCSequence:createWithTwoActions(tDelay, tHide))
end

-- 计算转弯时候的偏差量
function LogicRaceCar:GetTurnDeltaLen(fAbsRot, fAvgSpeedRate, fMaxSpeed, fPrevTime)
	if fAvgSpeedRate > 1 or fAvgSpeedRate < 0 then
		CTAssert(0, "计算转弯时候的偏差量时，速度比例错误")
		return 0
	end
	local fTmpLen = 0
	if fAbsRot ~= 90 then
		-- 新的平均速度
		local fNewAvgSpeed = 1 - fAvgSpeedRate*fAvgSpeedRate
		fNewAvgSpeed = math.sqrt( math.abs(fNewAvgSpeed) ) * fMaxSpeed

		-- 算出提前量
		local fTanValue = math.tan( CC_DEGREES_TO_RADIANS(fAbsRot) )
		if fTanValue ~= 0 then
			local fNewMove = fNewAvgSpeed * fPrevTime
			fTmpLen = (fNewMove / fTanValue)
		end
	end

	return fTmpLen
end

-- 根据最多允许的提前量来计算允许的最大速度
function LogicRaceCar:GetTurnMaxSpeedByPrevDis(fPreDis, fAbsRot, fAvgSpeedRate, fPrevTime)
	-- 根据计算得到的公式
	if fAvgSpeedRate > 1 or fAvgSpeedRate < 0 then
		CTAssert(0, "计算转弯时候的允许的最大速度时，速度比例错误:"..fAvgSpeedRate)
		return 10
	end
	if fPrevTime <= 0 then
		CTAssert(0, "计算转弯时候的允许的最大速度时，时间错误:"..fPrevTime)
		return 10
	end
	local fMaxSpeed = 0
	if fAbsRot ~= 90 then
		-- 另一边的速度比例
		local fOtherRate = math.sqrt(1 - fAvgSpeedRate*fAvgSpeedRate)
		-- 算出提前量
		local fTanValue = math.tan( CC_DEGREES_TO_RADIANS(fAbsRot) )
		if fTanValue ~= 0 then
			local fTmpValue = fAvgSpeedRate*fTanValue - fOtherRate
			if fTmpValue <= 0 then
				-- 0度角的计算比较简单
				fMaxSpeed = (fPreDis / fPrevTime) / fAvgSpeedRate
			else
				-- 0度角的计算比较简单
				fMaxSpeed = (fPreDis * fTanValue) / (fTmpValue*fPrevTime)
			end
		else
			-- 0度角的计算比较简单
			fMaxSpeed = (fPreDis / fPrevTime) / fAvgSpeedRate
		end
	else
		-- 直角的计算比较简单
		fMaxSpeed = (fPreDis / fPrevTime) / fAvgSpeedRate
	end

	-- 不能超过最大速度
	if fMaxSpeed > self.tShowRate.fMaxSpeed then
		fMaxSpeed = self.tShowRate.fMaxSpeed
	end

	return fMaxSpeed
end

-- 或者两条线段之间的夹角和转弯所用数据
function LogicRaceCar:GetTwoLineTurnInfo(tPoint1, tPoint2, tPoint3, fMaxSpeed, bIsCurRoad)
	local fCurRot = ChangeRotTo180Range( GetP2PRotation(tPoint2, tPoint1) )
	-- local fNextRot = ChangeRotTo180Range( GetP2PRotation(tPoint2, tPoint3) )
	-- -- 找出和当前的偏差值
	-- local fRot = math.abs(fNextRot) - math.abs(fCurRot)

	local fTotalDis = ccpDistance(tPoint1, tPoint2)
	-- 获取两个向量
	local tVec1 = ccpSub(tPoint2, tPoint1)
	local tVec2 = ccpSub(tPoint3, tPoint2)
	local fVecValue = (tVec1.x*tVec2.x + tVec1.y*tVec2.y) / ( fTotalDis*ccpDistance(tPoint2, tPoint3) )
	local fRot = math.acos( fVecValue )
	-- fRot = CC_RADIANS_TO_DEGREES(fRot)
	-- 转弯所用的时间
	local fPrevTime = 0

	-- 以 当前车 的正面向作为x的正向轴，正向x的速度变化
	local fPreDis = 0;
	local fAbsRot = math.abs(fRot)
	local fOtherRot = CC_RADIANS_TO_DEGREES(fAbsRot)
	local fAvgSpeedRate = 1.0 -- 最高速度与平均速度的比例值
	local fTmpLen = 0

	if fOtherRot ~= 90 and fOtherRot > 0 then
		fPrevTime = fOtherRot/math.abs(self.fRotSpeedValue *MAX_DEL_TMP)
		fAvgSpeedRate = math.abs(math.sin( fAbsRot ) / fAbsRot)
		local fOldAvgSpeed = fMaxSpeed * fAvgSpeedRate

		-- 算出提前量
		fTmpLen = self:GetTurnDeltaLen(fOtherRot, fAvgSpeedRate, fMaxSpeed, fPrevTime)
		if fTmpLen ~= 0 then
			fPreDis = fOldAvgSpeed * fPrevTime - fTmpLen
		else
			fPreDis = 1
		end
		-- print("fTmpLen:", fTmpLen, fPreDis, fMaxSpeed, fAvgSpeedRate, fTotalDis)
	elseif fOtherRot == 0 then
		fPrevTime = 0.01
		fAvgSpeedRate = 1
		fPreDis = 0
	else
		fPrevTime = 90/math.abs(self.fRotSpeedValue *MAX_DEL_TMP)
		fAvgSpeedRate = (1 / (math.pi/2.0))
		local fOldAvgSpeed = fMaxSpeed * fAvgSpeedRate

		fPreDis = fOldAvgSpeed * fPrevTime
	end

	local bHadChangePrev = false
	-- 获取已经行进的距离
	if bIsCurRoad == true then
		local fCos = math.cos( CC_DEGREES_TO_RADIANS( -(fCurRot-90)) )
	    local fSin = math.sin( CC_DEGREES_TO_RADIANS( -(fCurRot-90)) )
	    local fPosRelX = math.abs( (self:getPositionX() - tPoint1.x)*fCos + (self:getPositionY() - tPoint1.y)*fSin )
	    if (fPosRelX+fPreDis) > fTotalDis then
	    	-- print("1111111:", fPreDis, fTmpLen, fPosRelX, fTotalDis)
	    	fPreDis = fTotalDis - fPosRelX
	    	bHadChangePrev = true
	    end
	end
	
	-- 开始转弯的距离
	local fOldPrev = fPreDis
	if fTotalDis < fPreDis then
		-- print("222222:", fPreDis, fTmpLen)
		fPreDis = fTotalDis
		bHadChangePrev = true
	end
	if fPreDis < 5 then
		fPreDis = 5
		bHadChangePrev = true
	end

	if bHadChangePrev == true then
		-- if fPrevTime == 0 then
		-- else
		-- end
		fMaxSpeed = self:GetTurnMaxSpeedByPrevDis(fPreDis, fOtherRot, fAvgSpeedRate, fPrevTime)
		fTmpLen = self:GetTurnDeltaLen(fOtherRot, fAvgSpeedRate, fMaxSpeed, fPrevTime)
	end

	-- print("fTmpLen111111:", fTmpLen, fPreDis, fMaxSpeed, fAvgSpeedRate, fTotalDis, CC_RADIANS_TO_DEGREES(fRot), fVecValue)

	local tTurnInfo = {}
	tTurnInfo.fPreDis = fPreDis
	tTurnInfo.fMaxSpeed = fMaxSpeed
	tTurnInfo.fNextDis = fTmpLen/math.cos(fAbsRot)
	tTurnInfo.fTotalDis = fTotalDis
	tTurnInfo.fAvgSRate = fAvgSpeedRate
	tTurnInfo.fPrevTime = fPrevTime
	tTurnInfo.fAbsRot = fOtherRot
	return tTurnInfo
end

-- 设置开始移动的下两个关键点
function LogicRaceCar:CarMoveNextTwoKeyPos( tNextPos1, tNextPos2, tNextPos3 )
	-- local tPrevPos = self.tNextPos1
	-- if self.tNextPos1 == nil then
	local tPrevPos = self:getPositionInCCPoint()
	-- end
	-- 下一个转弯的信息
	local tTurnInfo1 = self:GetTwoLineTurnInfo(tPrevPos, tNextPos1, tNextPos2, self.tShowRate.fMaxSpeed, true)
	local fShowMaxSpeed = tTurnInfo1.fMaxSpeed
	-- 下下个转弯的信息处理
	local tTurnInfo2 = self:GetTwoLineTurnInfo(tNextPos1, tNextPos2, tNextPos3, fShowMaxSpeed, false)
	-- 转弯的距离不能太长，导致下一个转弯无法转
	local fNextDistance = tTurnInfo1.fNextDis + tTurnInfo2.fPreDis + 30
	local fPreDis = tTurnInfo1.fPreDis
	if fNextDistance > tTurnInfo2.fTotalDis then
		local fNextDis = tTurnInfo1.fNextDis * (tTurnInfo2.fTotalDis / fNextDistance)
		local fNextHDis = fNextDis * math.abs( math.sin( CC_DEGREES_TO_RADIANS(tTurnInfo1.fAbsRot) ) )
		local fTmpLen = fNextDis * math.abs( math.cos( CC_DEGREES_TO_RADIANS(tTurnInfo1.fAbsRot) ) )
		if tTurnInfo1.fAvgSRate >= 1 then
			fPreDis = fNextDis
		else
			fPreDis = (tTurnInfo1.fAvgSRate * fNextHDis)/(math.sqrt(1 - tTurnInfo1.fAvgSRate*tTurnInfo1.fAvgSRate)) - fTmpLen
		end

		fShowMaxSpeed = self:GetTurnMaxSpeedByPrevDis(fPreDis, tTurnInfo1.fAbsRot, tTurnInfo1.fAvgSRate, tTurnInfo1.fPrevTime)
	end
	local fTmpDis = fShowMaxSpeed * tTurnInfo1.fAvgSRate * tTurnInfo1.fPrevTime
	-- 开始转向时的理想速度
	self.fStartTurnSpeed = fShowMaxSpeed
	-- 在理想速度下的理想转弯点
	self.fStartTurnDis = tTurnInfo1.fTotalDis - fPreDis

	-- printf("移动的距离：%f     ddd:%f   ddd:%f", fTotalDis, self.fStartTurnDis, fRot)
	-- print("计算的提前信息：", self:GetStartCarPos(), tTurnInfo1.fTotalDis, fPreDis, fTmpDis, tTurnInfo1.fAbsRot, tTurnInfo1.fPrevTime, self.fStartTurnSpeed)
	self.tNextPos1 = tNextPos1
	self.tNextPos2 = tPrevPos
end

-- 是否开始运行的速度
function LogicRaceCar:StartMoveCar( bIsMove )
	if bIsMove == true then
		self.nMoveStatus = CarMoveStatus.Moving;
	else
		self.nMoveStatus = CarMoveStatus.Pause;
	end
	-- 不给移动的时候，速度改成0
	if bIsMove == false then
		local tPhySpeed = b2Vec2(0, 0)
		self.pPhysicObject:SetLinearVelocity(tPhySpeed)
		self.pPhysicObject:SetAngularVelocity(0)
	end
end

-- 是否开始转弯,fMoveDis是在当前直线上移动的距离
function LogicRaceCar:IsStartTurnDirect( fMoveDis )
	if fMoveDis > self.fStartTurnDis then
		return true
	end

	return false
end


function LogicRaceCar:SetCurPhysicObject( tObject )
	self.pPhysicObject = tObject
end
-- mark: =========================== 设置和获取属性 =============================

-- 设置地图回调
function LogicRaceCar:SetMapDelegate(pDelegate)
	self.pMapDelegate = pDelegate
end

-- 设置适合当前属性的显示比例值
function LogicRaceCar:SetCarShowRateInfo(fSpeedRate, fASpeedRate, fTurnRate)
	self.tShowRate.fSpeedRate = fSpeedRate
	self.tShowRate.fASpeedRate = fASpeedRate
	self.tShowRate.fTurnRate = fTurnRate

	-- 算出确切的速度和加速度
	self.tShowRate.fMaxSpeed = self.tCarInfo.fMaxSpeed * fSpeedRate
	self.tShowRate.fAddSpeed = self.tCarInfo.fAddSpeed * fASpeedRate
	self.tShowRate.fRotValue = self.tCarInfo.fRotValue * fTurnRate

	self:SetCurMaxSpeed(self.tShowRate.fMaxSpeed)
	self:SetCurAddSpeed(self.tShowRate.fAddSpeed)
	-- 
	self.fRotSpeedValue	= self.tShowRate.fRotValue
end

-- 设置车子的初始位置
function LogicRaceCar:SetCarStartPosition( tStartPos )
	self.tStartPos = tStartPos

	self:setPosition(tStartPos)
end

-- 设置当前要运行的路段，0是一边的路，2是另一边的路，其他则为中间的路
function LogicRaceCar:SetCurRunRoad( nRoadIdx, bIsForce )
	if type(nRoadIdx) ~= "number" and nRoadIdx >= 0 and nRoadIdx < CarMoveRoadType.Total then
		CTLOG("重新设置移动的路线错误")
		return false
	end

	if bIsForce == nil then
		bIsForce = true;
	end

	if bIsForce == false and nRoadIdx == self:GetCurRunRoad() then
		return false
	end

	local tRoadIdx = self.tArrRoads;

	if nRoadIdx == CarMoveRoadType.Road1 then
		tRoadIdx = self.tArrRoads1
	elseif nRoadIdx == CarMoveRoadType.Road2 then
		tRoadIdx = self.tArrRoads2
	else
		tRoadIdx = self.tArrRoadsMid
	end

	-- local nNext = (self:GetCurRoadIdx() % #tRoadIdx) + 1
	-- local nNext1 = (nNext % #tRoadIdx) + 1
	-- local nNext2 = (nNext1 % #tRoadIdx) + 1
	-- -- 当前移动角度
	-- local fRot1 = GetP2PRotation(tRoadIdx[nNext], self:getPositionInCCPoint())
	-- local fRotDel = math.abs( ChangeRotTo180Range( fRot1 ) - ChangeRotTo180Range( self.pSpCar:getRotation() ) ); 

	-- if bIsForce == false and fRotDel > 45 and self.tNextPos2 ~= nil then

	local tStart = tRoadIdx[self:GetCurRoadIdx()]
	if tStart == nil then
		tStart = self:getPositionInCCPoint();
	end

	local nNext = (self:GetCurRoadIdx() % #tRoadIdx) + 1
	local nNext1 = (nNext % #tRoadIdx) + 1
	local nNext2 = (nNext1 % #tRoadIdx) + 1
	-- 当前移动角度
	local fRot1 = GetP2PRotation(tRoadIdx[nNext], self:getPositionInCCPoint())
	local fRot2 = GetP2PRotation(tRoadIdx[nNext], tStart)
	local fRotDel = math.abs( ChangeRotTo180Range( fRot1 ) - ChangeRotTo180Range( self.pSpCar:getRotation() ) ); 
	local fRotDel2 = math.abs( ChangeRotTo180Range( fRot2 ) - ChangeRotTo180Range( self.pSpCar:getRotation() ) ); 

	if bIsForce == false and (fRotDel > 45 or fRotDel2 > 55) and self.tNextPos2 ~= nil then
		return false
	else
		self.tNextPos1	= nil
		self.tNextPos2	= nil
		self.tArrRoads = tRoadIdx;
		self.nSelRoadType = nRoadIdx
		self:CarMoveNextTwoKeyPos(tRoadIdx[nNext], tRoadIdx[nNext1], tRoadIdx[nNext2])
	end

	return true
end
-- 获取当前走的路线类型
function LogicRaceCar:GetCurRunRoad()
	return self.nSelRoadType
end
-- 判断情况来设置跑的路线
function LogicRaceCar:JudgeToChangeRoadType()
	if self.pMapDelegate then
		return self.pMapDelegate:CallBackJudgeChangeCarRoadType( self, self:GetStartCarPos(), self:GetCurRoadIdx())
	end

	return false
end

-- 设置当前车子的所在
function LogicRaceCar:SetCurRoadIdx( nCur, bIsJustChange)
	if type(nCur) ~= "number" then
		CTLOG("设置单辆车的路段索引错误")
		return
	end

	self.nMoveIdx = nCur

	-- 重新获取当前路段的角度
	local nNext1 = (nCur % (#self.tArrRoads)) + 1
	local tLocalPos = self.tArrRoads[nCur]
	local tNextPos = self.tArrRoads[nNext1]
	self.fLineRot = ChangeRotTo360Range( GetP2PRotation(tNextPos, tLocalPos) )

	-- if self:GetCurCirCle() == 1 then
	-- 	local pLyParent = self:getParent():getParent()
	-- 	local pLbWord = CCLabelTTF:create("第 " .. (self.nMoveIdx-1) .. "点的时间:" .. self.fTotalTime, "", 20)
	-- 	pLbWord:setPosition(ccp(600 - 500 * (self:GetCurQualify()%2), 1000 - 26 * self.nMoveIdx))
	-- 	pLyParent:addChild(pLbWord, 20)
	-- end
	-- 如果没有移动结束，判断换道
	if self:GetIsMoveEnd() == false then
		self:JudgeToChangeRoadType()
	end

	-- 当前已经过了一圈
	if self.nMoveIdx <= 1 and bIsJustChange ~= true then
		self:SetCurCirCle( self:GetCurCirCle()+1 )
	end
end
-- 获取当前车子所在的路段
function LogicRaceCar:GetCurRoadIdx()
	return self.nMoveIdx
end

-- 设置当前圈数
function LogicRaceCar:SetCurCirCle( nCurCir, bIsJustChange )
	if type(nCurCir) ~= "number" then
		CTLOG("重新设置当前车子所在圈数的时候发生错误")
		return
	end
	-- 判断是否移动结束
	if self.pMapDelegate and bIsJustChange ~= true then
		if self.pMapDelegate:CallBackJudgeMoveCircleEnd( self, nCurCir ) == true then
			self.bIsMoveEnd = true;
			self.bIsCanEndCallBack = true;
		end
	end

	self.nCurCircle = nCurCir
end
-- 获取当前圈数
function LogicRaceCar:GetCurCirCle()
	return self.nCurCircle
end
-- 设置当前排位
function LogicRaceCar:SetCurQualify( nCurQualify, bShowEff )
	if type(nCurQualify) ~= "number" then
		CTLOG("重新设置当前车子所在圈数的时候发生错误")
		return
	end

	-- 显示效果
	-- print("fdfffffffffff:::", nCurQualify, bShowEff, self.nCurQualifying)
	-- if bShowEff==true and self.nCurQualifying ~= nCurQualify then
	-- 	self:ShowCurQualifyEffect( nCurQualify )
	-- end
	self.nCurQualifying = nCurQualify	
end
-- 获取当前排位名次
function LogicRaceCar:GetCurQualify()
	return self.nCurQualifying
end

-- 设置起始位置
function LogicRaceCar:SetStartCarPos(nPos)
	self.nStartCarPos = nPos
end
-- 设置位置的权重
function LogicRaceCar:SetCurRoadMoveValue(nValue)
	self.nCarMoveValue = nValue;
end
-- 获取位置的权重
function LogicRaceCar:GetCurRoadMoveValue()
	return self.nCarMoveValue;
end

-- 获取起始位置
function LogicRaceCar:GetStartCarPos()
	return self.nStartCarPos
end

-- 获取当前路线的角度
function LogicRaceCar:GetCurRoadRotation()
	return self.fLineRot
end
-- 获取当前路线的起始点
function LogicRaceCar:GetCurRoadStartPt()
	if self:GetCurRoadIdx() == 0 then
		return self.tStartPos
	end
	return self.tArrRoads[self:GetCurRoadIdx()]
end

-- 获取车子的显示区域
function LogicRaceCar:GetCarBound()
	local tRect = CCRectMake(0, 0, self:getContentSize().width, self:getContentSize().height)
	tRect.origin.x = self:getPositionX() - tRect.size.width * self:getAnchorPoint().x
	tRect.origin.y = self:getPositionY() - tRect.size.height * self:getAnchorPoint().y
	return tRect
end

-- 获取车子的时间列表
function LogicRaceCar:GetCarTimeInfo()
	return self.tTimeInfo
end

-- 在警戒范围类型，0是不在，1是在后面，2是在前面，3是差不多在并排中
function LogicRaceCar:GetAlertRangeType( pOtherCar )
	if pOtherCar == nil then
		CTLOG("判断其他车在警戒范围类型错误，比较的车子为nil！")
		return 0
	end

	-- 如果不是在同一段路上，则不处理
	if pOtherCar:GetCurRoadIdx() ~= self:GetCurRoadIdx() then
		return 0
	end

	-- 获取它们之间的距离
	local fDistance = ccpDistance(self:getPosition(), pOtherCar:getPosition())

	-- 获取相对当前车子坐标系的位置
	local tRelPosCur = self:convertToNodeSpace(self:getPosition())
	local tRelPosOther = self:convertToNodeSpace(pOtherCar:getPosition())

	if tRelPosCur.y > tRelPos.y then
		
	end
end

-- 获取车子的数据数据
function LogicRaceCar:GetCurCarInfo()
	return self.tCarInfo
end

-- 获取当前车子在本地数据中的详细信息
function LogicRaceCar:GetCurLocateInfo()
	return DATAMGR:GetItemInfoMgr():GetItemInfoByID(self.tCarInfo.nCarID)
end

-- 获取当前移动的总时间
function LogicRaceCar:CetCurMoveTotalTime()
	return self.fTotalTime;
end

-- 重新改变当前移动的总时间
function LogicRaceCar:ResetCurMoveTotalTime(fTotalTime)
	if fTotalTime ~= nil then
		self.fTotalTime = fTotalTime;
	end
end

-- 获取当前车子是否已经移动结束
function LogicRaceCar:GetIsMoveEnd()
	return self.bIsMoveEnd;
end
-- 获取是否能
function LogicRaceCar:GetIsMoveEndAndCanDealInfo()
	return self.bIsCanEndCallBack
end
-- 获取是否能
function LogicRaceCar:SetCannotMoveEndAndCanDealInfo()
	self.bIsCanEndCallBack = false;
end

-- 获取当前路段的长度
function LogicRaceCar:GetCurRoadLen()
	local nPointCount = #self.tArrRoads
	-- 获取下一个点和上一次经过的点
    local nNext = (self:GetCurRoadIdx() % nPointCount) + 1
    local tNextPos = self.tArrRoads[nNext]
    local tLocalPos = self.tNextPos2--self:GetCurRoadStartPt()
    local fDistance = ccpDistance(tLocalPos, tNextPos)
    return fDistance;
end

-- 获取相对与中心路段的位置信息还有权重值
function LogicRaceCar:GetRelMidRoadValues() 
	-- 获取路面信息
	local nCur = self:GetCurRoadIdx()
	if nCur == 0 then
		nCur = 1;
    end
	-- 重新获取当前路段的角度
	local nNext1 = (nCur % (#self.tArrRoads)) + 1
	local tLocalPos = self.tArrRoadsMid[nCur]
	local tNextPos = self.tArrRoadsMid[nNext1]
	local fCurRot = ChangeRotTo360Range( GetP2PRotation(tNextPos, tLocalPos) )


	-- 算出当前车子在当前路段位移的横向距离
    local fCos = math.cos( CC_DEGREES_TO_RADIANS( -fCurRot+90) )
	local fSin = math.sin( CC_DEGREES_TO_RADIANS( -fCurRot+90) )
	local fPosRelX = (self:getPositionX() - tLocalPos.x)*fCos + (self:getPositionY() - tLocalPos.y)*fSin

	-- 获取当前车子所在位置的权重值
    local fCurLen = ccpDistance(tLocalPos, tNextPos)

    return fPosRelX, fCurLen
end

-- mark: ========================== 内部属性处理 ============================
-- 当前速度最大值
function LogicRaceCar:SetCurMaxSpeed( fCurMax )
	self.fCurMaxSpeed = fCurMax
end
function LogicRaceCar:GetCurMaxSpeed()
	return self.fCurMaxSpeed
end
-- 获取实际三维
function LogicRaceCar:GetOrigMaxSpeed()
	return self.tShowRate.fMaxSpeed
end
function LogicRaceCar:GetOrigMaxAddSpeed()
	return self.tShowRate.fAddSpeed
end
function LogicRaceCar:GetOrigMaxTurn()
	return self.tShowRate.fRotValue
end
-- 获取当前的实际速度的价值
function LogicRaceCar:GetOrigCarFightValue()
	return (self:GetOrigMaxSpeed() + self:GetOrigMaxAddSpeed()*2 + self:GetOrigMaxTurn()*2)
end
-- 当前的加速度
function LogicRaceCar:SetCurAddSpeed( fCurAdd )
	self.fCurAddSpeed = fCurAdd
end
function LogicRaceCar:GetCurAddSpeed()
	return self.fCurAddSpeed
end
-- 当前的速度
function LogicRaceCar:SetCurSpeed( fCurS)
	self.fSpeed = fCurS
end
function LogicRaceCar:GetCurSpeed()
	return self.fSpeed
end

-- 设置当前车子的角度
function LogicRaceCar:SetCurCarRot(fRot)
	self.pSpCar:setRotation(fRot)
end

-- mark: ============================ 循环处理 ==================================

-- 循环处理
function LogicRaceCar:CarLoop( dt )
-- 技能效果类型
-- local EffectOnCarType = {
-- None 		= 0,	-- 当前没有技能效果
-- AddAtt 		= 1,	-- 加属性
-- SubAtt 		= 2,	-- 减属性
-- MianYi 		= 3,	-- 免疫
-- Frozen 		= 4,	-- 被冰封
-- } --[[  nType:3,nEffType:2,fTime:2.0,fPercent:0.1,  ]]
	-- 属性倍数重新处理
	self.fAttScale = 1.0;
	-- 遍历所有技能效果，
	local nSkIdx = 1;
	repeat
		local tOneEffect = self.tArrSkillEffect[nSkIdx]
		-- 找不到技能，则跳出
		if tOneEffect == nil then
			break;
		end
		tOneEffect.fTime = tOneEffect.fTime-dt;

		-- 技能效果结束
		if tOneEffect.fTime < 0 then
			table.remove(self.tArrSkillEffect, nSkIdx); 
			-- 删除过去功能效果
			self:ShowOneSkillEffectEnd(tOneEffect.nEffType, tOneEffect.nType);
		else
			-- 如果是改变属性的，则加回去
			if tOneEffect.nEffType == EffectOnCarType.AddAtt or tOneEffect.nEffType == EffectOnCarType.SubAtt then
				self.fAttScale = self.fAttScale+tOneEffect.fPercent;
			end
			nSkIdx = nSkIdx+1;
		end
	until false
	-- -- 如果移动结束，则只有一半的速度
	if self:GetIsMoveEnd() == true then
		self.fDelAttScale = self.fDelAttScale + 0.25*dt;
		if self.fDelAttScale > 0.7 then
			self.fDelAttScale = 0.7;
		end
		self.fAttScale = self.fAttScale*(1-self.fDelAttScale);
	end
	-- 消减的属性不能低于0倍
	if self.fAttScale < 0.01 then
		self.fAttScale = 0.01;
	end
	if self.nMoveStatus == CarMoveStatus.Moving then
		-- -- 晃动
		-- self.nSharkIdx = self.nSharkIdx % 16
		-- local nDtLen = (self.nSharkIdx-8) / 8
		-- self.pSpCar:setAnchorPoint( ccp( (self.pSpCar:getContentSize().width/2+nDtLen)/self.pSpCar:getContentSize().width, 0.5) )
		-- self.nSharkIdx = self.nSharkIdx+1

		-- 速度的处理
		local fCurSpeed = self:GetCurSpeed()
		local fCurMaxS = self:GetCurMaxSpeed()
		if fCurSpeed > (fCurMaxS+0.001) or fCurSpeed < (fCurMaxS-0.001) then
			
			local fCurAddS = self:GetCurAddSpeed() * self.fAttScale;
			-- 速度偏差大,才会冒烟尘
			if math.abs(fCurSpeed - fCurMaxS) > fCurMaxS*0.2 then
				self:ShowCarYanChen()
			end
			

			-- 如果速度大于当前最大速度，则开始减速
			if fCurSpeed > fCurMaxS then
				fCurSpeed = fCurSpeed - fCurAddS*dt *MAX_DEL_SPEED_SCALE
				if fCurSpeed < fCurMaxS then
					fCurSpeed = fCurMaxS
				end
			-- 如果速度小于当前最大速度，则开始加速
			else
				fCurSpeed = fCurSpeed + fCurAddS*dt
				if fCurSpeed > fCurMaxS then
					fCurSpeed = fCurMaxS
				end
			end
		end

		-- 如果速度小于0，则转到下一个点
		if fCurSpeed < 0 then
			local nPointCount = #self.tArrRoads
			local nCurIdx = self:GetCurRoadIdx()
			if nCurIdx == nil  or nCurIdx < 1 then
				nCurIdx = 0
			end
			-- 获取下一个点和上一次经过的点
			local nNext = (nCurIdx % nPointCount) + 1
			self:SetCurRoadIdx(nNext)
			-- 再获取下一个点
			nNext = (nNext % nPointCount) + 1
			local nNext1 = (nNext % nPointCount) + 1
			local nNext2 = (nNext1 % nPointCount) + 1
			local tNextPos = self.tArrRoads[nNext]
			local tNextPos1 = self.tArrRoads[nNext1]
			local tNextPos2 = self.tArrRoads[nNext2]

			self:CarMoveNextTwoKeyPos(tNextPos, tNextPos1, tNextPos2)

			fCurSpeed = 0;
		end

		self:SetCurSpeed(fCurSpeed)
	end

	-- 处理位置移动
	if self.nMoveStatus > CarMoveStatus.NotStart then
		-- 如果没有移动结束
		if self.bIsMoveEnd == false then
			self.fTotalTime = self.fTotalTime + dt
		end
		self:DealCarPhysicMoving(dt)
	end
	
end

-- 重置车子的位置，如果当前车子被卡住了。就调用这里
function LogicRaceCar:ResetCurCarPosition()
	self.fJudgeRoadLen = -1;

	local nPointCount = #self.tArrRoads
	-- 获取当前路段的两个点
	local nCurIdx = self:GetCurRoadIdx()
	if nCurIdx == nil  or nCurIdx < 1 then
		nCurIdx = 1
	end
    local nNext = (nCurIdx % nPointCount) + 1

    local tLocalPos = self.tArrRoads[nCurIdx]
    local tNextPos = self.tArrRoads[nNext]

    local tRot1 = ChangeRotTo360Range( GetP2PRotation(tNextPos, tLocalPos) )
    local tPhyPos = b2Vec2(tLocalPos.x/PTM_RATIO, tLocalPos.y/PTM_RATIO)

    local fPhyRot = CC_DEGREES_TO_RADIANS(ChangeRotTo180Range(tRot1))
	self.pPhysicObject:SetTransform( tPhyPos, -fPhyRot);
end

function LogicRaceCar:DealCarPhysicMoving( dt )

	-- -- 开始判断车子的时间间隔
	-- self.fJudgeTime 	= 0;
	-- -- 开始之前车子的判断时间
	-- self.fJudgeRoadLen 	= -1;
	-- 只有移动的时候才开始判断车子是否卡住了
	if self.nMoveStatus == CarMoveStatus.Moving then
		self.fJudgeTime = self.fJudgeTime+dt;
		if self.fJudgeTime > 2 then
			self.fJudgeTime = 0;
			-- 判断当前移动的距离和上次移动的距离偏差
			local fCurRoadingValue = self:GetCurRoadMoveValue();
			if self.fJudgeRoadLen >= 0 then
				-- 如果是被卡住了，则将车子强制移回到路段节点的起始点
				if math.abs( fCurRoadingValue - self.fJudgeRoadLen ) < 20 and math.abs(self.fAttScale) > 0.08 then
					-- 重置当前车子的位置
					self:ResetCurCarPosition()
					return;

				else
					self.fJudgeRoadLen = fCurRoadingValue;
				end
			else
				self.fJudgeRoadLen = fCurRoadingValue;
			end
		end
	end

	local nPointCount = #self.tArrRoads
	local nCurIdx = self:GetCurRoadIdx()
	if nCurIdx == nil  or nCurIdx < 1 then
		nCurIdx = 1
	end
	-- 获取下一个点和上一次经过的点
    local nNext = (nCurIdx % nPointCount) + 1
    local tNextPos = self.tArrRoads[nNext]
    local tLocalPos = self.tNextPos2--self:GetCurRoadStartPt()
    -- 角度
    local tRot1 = ChangeRotTo360Range( GetP2PRotation(tNextPos, tLocalPos) )

    local tCarPos = self:getPositionInCCPoint()
    local fCos = math.cos( CC_DEGREES_TO_RADIANS( -tRot1+90) )
	local fSin = math.sin( CC_DEGREES_TO_RADIANS( -tRot1+90) )
	local fPosRelX = (self:getPositionX() - tLocalPos.x)*fCos + (self:getPositionY() - tLocalPos.y)*fSin
	local fDistanceNow = fPosRelX

	local fCurSpeed = self:GetCurSpeed() * self.fAttScale


	-- 如果进入减速时间段的距离
	local fStartTurnSpeed = self.fStartTurnSpeed * self.fAttScale
	local fStartAddDis = math.abs(fCurSpeed*fCurSpeed - fStartTurnSpeed*fStartTurnSpeed)
	fStartAddDis = fStartAddDis / ( 2*self:GetCurAddSpeed() * self.fAttScale *MAX_DEL_SPEED_SCALE )
	fStartAddDis = self.fStartTurnDis - fStartAddDis

	-- print("ffdsafdsa:::", self:GetCurAddSpeed())

	local fDistance = ccpDistance(tLocalPos, tNextPos)
	local fCarDistance = ccpDistance(tLocalPos, self:getPositionInCCPoint() )

	local fSpeedX = 0
	local fSpeedY = 0

	local fCurCarRot = self.pSpCar:getRotation()
	local fTargetRot = ChangeRotTo180Range( tRot1 - fCurCarRot );
	-- -- 如果进入变速期间
	-- if fDistanceNow > self.fStartTurnDis then
	-- 	-- 下下个目标点
	-- 	local nNext1 = (nNext % nPointCount) + 1
	-- 	local tNextPos1 = self.tArrRoads[nNext1];
	-- 	-- 下一个转角
	-- 	local nNextRot1 = ChangeRotTo360Range( GetP2PRotation(tNextPos1, tNextPos) )
	-- 	local nToRot = ChangeRotTo360Range( ChangeRotTo180Range( nNextRot1-tRot1 ) * 0.8 + tRot1 )
	-- 	local nDelRot = ChangeRotTo180Range( nToRot-fCurCarRot )

	-- 	local fCurDelRot = dt*self.fRotSpeedValue
	-- 	if nDelRot < 0 then
	-- 		fCurDelRot = -fCurDelRot;
	-- 	end
	-- 	fCurCarRot = fCurCarRot+fCurDelRot;
	-- 	fCurCarRot = ChangeRotTo360Range(fCurCarRot)
	-- 	if math.abs(ChangeRotTo180Range( nToRot - fCurCarRot )) < math.abs(fCurDelRot*0.7)  then
	-- 		fCurCarRot = nToRot
	-- 	end

	-- 	fSpeedX = fCurSpeed * math.cos(CC_DEGREES_TO_RADIANS(fCurCarRot-90))
	-- 	fSpeedY = -fCurSpeed * math.sin(CC_DEGREES_TO_RADIANS(fCurCarRot-90))
	-- elseif fTargetRot > 8 or fTargetRot < -8  then
	-- 	local fCurDelRot = dt*self.fRotSpeedValue
	-- 	if fTargetRot < 0 then
	-- 		fCurDelRot = -fCurDelRot;
	-- 	end
	-- 	fCurCarRot = fCurCarRot+fCurDelRot;
	-- 	fCurCarRot = ChangeRotTo360Range(fCurCarRot)
	-- 	if math.abs(ChangeRotTo180Range( tRot1 - fCurCarRot )) < math.abs(fCurDelRot*0.7)  then
	-- 		fCurCarRot = tRot1
	-- 	end

	-- 	fSpeedX = fCurSpeed * math.cos(CC_DEGREES_TO_RADIANS(fCurCarRot-90))
	-- 	fSpeedY = -fCurSpeed * math.sin(CC_DEGREES_TO_RADIANS(fCurCarRot-90))

	-- 	-- if self:GetStartCarPos() == 1 then
	-- 	-- 	print("222222::::", fTargetRot, self.fRotSpeedValue, fCurDelRot, fCurCarRot, tRot1)
	-- 	-- end
	-- else
		fCurCarRot = tRot1;
		fSpeedX = fCurSpeed * math.cos(CC_DEGREES_TO_RADIANS(tRot1-90))
		fSpeedY = -fCurSpeed * math.sin(CC_DEGREES_TO_RADIANS(tRot1-90))

		-- if self:GetStartCarPos() == 1 then
		-- 	print("33333::::", fTargetRot, fCurCarRot)
		-- end
	-- end
	-- 特殊处理还没有移动的
	if self.nMoveStatus ~= CarMoveStatus.Moving then
		fSpeedX = 0;
		fSpeedY = 0;
	end

	-- self.pPhysicObject.m_force:SetZero()
    local tPhySpeed = b2Vec2(fSpeedX/PTM_RATIO, fSpeedY/PTM_RATIO)
    local vel = self.pPhysicObject:GetLinearVelocity();
    local m = self.pPhysicObject:GetMass();-- the mass of the body
    local force = b2Vec2((tPhySpeed.x-vel.x) * m / dt, (tPhySpeed.y-vel.y) * m / dt)--m * velChange / t; --f = mv/t
    self.pPhysicObject:ApplyForce(force, self.pPhysicObject:GetWorldCenter(), true);
    -- self.pPhysicObject:SetLinearVelocity(tPhySpeed)
    -- print("force:", force.x, force.y)

    -- local tRot2 = ChangeRotTo360Range( GetP2PRotation(tNextPos, tCarPos) )

	-- if self:GetStartCarPos() == 1 then
	-- 	print("111111::::", dt, fSpeedX, fSpeedY)
	-- end
    local fPhyRot = CC_DEGREES_TO_RADIANS(ChangeRotTo180Range(tRot1))
	self.pPhysicObject:SetTransform( self.pPhysicObject:GetPosition(), -fPhyRot);

	-- if self:GetStartCarPos() == 6 then
	-- 	print("33333::::", fTargetRot, fCurCarRot, fPosRelX, fDistance)
	-- end

    if fPosRelX > fDistance or (math.abs(fTargetRot) > 90 and math.abs(fPosRelX - fDistance)<80) then
    	self:SetCurRoadIdx(nNext)
    	 -- 再获取下一个点
        nNext = (nNext % nPointCount) + 1
        local nNext1 = (nNext % nPointCount) + 1
        local nNext2 = (nNext1 % nPointCount) + 1
        tNextPos = self.tArrRoads[nNext]
        local tNextPos1 = self.tArrRoads[nNext1]
        local tNextPos2 = self.tArrRoads[nNext2]

        self:CarMoveNextTwoKeyPos(tNextPos, tNextPos1, tNextPos2)
    end
	
	if fDistanceNow > fStartAddDis then
		self:SetCurMaxSpeed(self.fStartTurnSpeed)
	else
		self:SetCurMaxSpeed( self:GetOrigMaxSpeed() )
	end

	-- if math.abs(fTargetRot) >= 50 then
	-- 	self:SetCurMaxSpeed(0)
	-- end
end

-- 获取当前车子的的显示位置
function LogicRaceCar:GetCurCarShowPosByRate(fRateX, fRateY)
	local tTmpPos = self.pSpCar:convertToWorldSpace( ctspccp(self.pSpCar, fRateX, fRateY) )
	return self:convertToNodeSpace( tTmpPos )
end

-- 获取
function LogicRaceCar:GetCurCarShowRot()
	local fCarRot = self.pSpCar:getRotation()-90;
	 return fCarRot--ChangeRotTo360Range( fCarRot );
end

-- mark: ================================= 技能的处理 ======================================
-- 游戏开始点击的处理，返回的是播放动画的时间
function LogicRaceCar:StartClickToSpeedUpEffect(fAddScale, fKeepTime)
	local pSpEffects = CCSprite:create("Images/")
	-- 特殊效果
	CCSpriteFrameCache:sharedSpriteFrameCache():addSpriteFramesWithFile( Public_GetResPathByKey("Images/Skill/SkStrongAll.plist") )

	local pSpSelfEffect = CCSprite:createWithSpriteFrameName("SkStrongAll6.png");
	pSpSelfEffect:setAnchorPoint(ccp(0.5, 0.257));
	pSpSelfEffect:setPosition(ctspccp(self, 0.5, 0.5))
	pSpSelfEffect:setScaleX(1.6);
	pSpSelfEffect:setScaleY(2.0);
	self:addChild(pSpSelfEffect, -1);

	-- 车子播放使用技能的动作
	-- 给自己的特效
	local animFrames = CCArray:createWithCapacity(15)
	for i=6,1,-1 do
		animFrames:addObject( CCSpriteFrameCache:sharedSpriteFrameCache():spriteFrameByName("SkStrongAll"..i..".png") );
	end
	local tArrAct = CCArray:create()
	tArrAct:addObject( CCAnimate:create( CCAnimation:createWithSpriteFrames(animFrames, 0.1) ) )
	tArrAct:addObject( CCCallFunc:create(function()
		self:AddAttribute(fAddScale, fKeepTime, SkEffectType.Follow_HuanXiang)
	end) )
	tArrAct:addObject( CCRemoveSelf:create(true) )
	pSpSelfEffect:runAction(CCSequence:create(tArrAct))

	CCSpriteFrameCache:sharedSpriteFrameCache():removeSpriteFramesFromFile( Public_GetResPathByKey("Images/Skill/SkStrongAll.plist") )

	return 0.6;
end

-- 给当前车子加层，添加加成百分比10%的值就是0.1
function LogicRaceCar:AddAttribute(fAddScale, fKeepTime, nSkillType)
	-- 技能效果类型
-- 技能效果类型
-- local EffectOnCarType = {
-- None 		= 0,	-- 当前没有技能效果
-- AddAtt 		= 1,	-- 加属性
-- SubAtt 		= 2,	-- 减属性
-- MianYi 		= 3,	-- 免疫
-- Frozen 		= 4,	-- 被冰封
-- } 
	-- 如果是削弱的
	if fAddScale < 0 then
		-- 判断当前是否有免疫
		if self:IsCurCarHaveMianYi() == true then
			-- self:ShowMissEnemySkillEffect();
			return;
		end
	end


	local tEffect = {};
	tEffect.nType = nSkillType;
	if fAddScale < 0 then
		tEffect.nEffType = EffectOnCarType.SubAtt;
	else
		tEffect.nEffType = EffectOnCarType.AddAtt;
	end
	tEffect.fTime = fKeepTime;
	tEffect.fPercent = fAddScale;

	self.tArrSkillEffect[#self.tArrSkillEffect+1] = tEffect;

	-- 播放效果
	self:StartToShowEffect(tEffect.nEffType, nSkillType)
end

-- 锁住当前车子
function LogicRaceCar:ShowSkillLockCurCar(fKeepTime, nSkillType)
	-- 判断当前是否有免疫
	if self:IsCurCarHaveMianYi() == true then
		return;
	end
	-- 判断当前是否有免疫
	for i,tOneEff in ipairs(self.tArrSkillEffect) do
		if tOneEff.nEffType == EffectOnCarType.Frozen then
			if tOneEff.fTime < fKeepTime then
				tOneEff.fTime = fKeepTime;
			end
			return;
		end
	end
	local tEffect = {};
	tEffect.nType = nSkillType;
	tEffect.nEffType = EffectOnCarType.Frozen;
	tEffect.fTime = fKeepTime;

	self.tArrSkillEffect[#self.tArrSkillEffect+1] = tEffect;

	-- 将当前车子停住
	self:StartMoveCar(false);

	-- 播放效果
	self:StartToShowEffect(EffectOnCarType.Frozen, nSkillType)
end

-- 当前开始免疫效果
function LogicRaceCar:ShowSkillCurCarStartMianYi(fKeepTime, nSkillType)
	-- 判断当前是否有免疫，有的话，使用当前最长的时间
	for i,tOneEff in ipairs(self.tArrSkillEffect) do
		if tOneEff.nEffType == EffectOnCarType.MianYi then
			if tOneEff.fTime < fKeepTime then
				tOneEff.fTime = fKeepTime;
			end
			return;
		end
	end

	-- 判断当前是否有负面效果，有则全部去除掉，设置时间结束，在循环中去除，走原本的去除流程
	for i,tOneEff in ipairs(self.tArrSkillEffect) do
		if tOneEff.nEffType == EffectOnCarType.SubAtt or tOneEff.nEffType == EffectOnCarType.Frozen then
			tOneEff.fTime = -1.0;
		-- 对方的集火的时候，才会处理
		elseif tOneEff.nEffType == EffectOnCarType.OnFire then
			if tEffect.fPercent < 0 then
				tOneEff.fTime = -1.0;
			end
		end
	end

	local tEffect = {};
	tEffect.nType = nSkillType;
	tEffect.nEffType = EffectOnCarType.MianYi;
	tEffect.fTime = fKeepTime;
	self.tArrSkillEffect[#self.tArrSkillEffect+1] = tEffect;
	-- 播放效果
	self:StartToShowEffect(EffectOnCarType.MianYi, nSkillType)
end

-- 显示开启反伤
function LogicRaceCar:ShowSkillStartReflex(fKeepTime, fPercent, nSkillType)
	local tEffect = {};
	tEffect.nType = nSkillType;
	tEffect.nEffType = EffectOnCarType.Reflex;
	tEffect.fTime = fKeepTime;
	tEffect.fPercent = fPercent;

	self.tArrSkillEffect[#self.tArrSkillEffect+1] = tEffect;

	-- 播放效果
	self:StartToShowEffect(tEffect.nEffType, nSkillType)
end

-- 获取当前的反伤情况情况
function LogicRaceCar:GetCarReflexSkillInfo()
	local fReflex = -1;
	-- 判断当前是否有反伤，有的话，使用当前反伤比例最大的
	for i,tOneEff in ipairs(self.tArrSkillEffect) do
		if tOneEff.nEffType == EffectOnCarType.Reflex then
			if tOneEff.fPercent > fReflex then
				fReflex = tOneEff.fPercent;
			end
		end
	end
	if fReflex > 1 then
		fReflex = 1;
	end
	return fReflex
end

-- 显示伪装成对手，持续时间和可以承受的对方的加成效果
function LogicRaceCar:ShowSkillMarkAsEnemy(fKeepTime, fPercent, nSkillType)
	local tEffect = {};
	tEffect.nType = nSkillType;
	tEffect.nEffType = EffectOnCarType.MarkAsEnemy;
	tEffect.fTime = fKeepTime;
	tEffect.fPercent = fPercent;

	self.tArrSkillEffect[#self.tArrSkillEffect+1] = tEffect;

	-- 播放效果
	self:StartToShowEffect(tEffect.nEffType, nSkillType)
end

-- 获取当前的伪装情况
function LogicRaceCar:GetMarkAsEnemySkillInfo()
	-- 判断当前是否有免疫，有的话，使用当前最长的时间
	for i,tOneEff in ipairs(self.tArrSkillEffect) do
		if tOneEff.nEffType == EffectOnCarType.MarkAsEnemy then
			return tOneEff.fPercent
		end
	end
	return -1
end

-- 显示伪装成对手，持续时间和可以承受的对方的加成效果
function LogicRaceCar:ShowSkillFireOnEffect(fRate, fKeepTime, nSkillType)
	-- 判断当前是否有集火，有的话，使用当前最长的时间
	for i,tOneEff in ipairs(self.tArrSkillEffect) do
		if tOneEff.nEffType == EffectOnCarType.OnFire then
			tOneEff.fTime = fKeepTime;
			tEffect.fPercent = fRate;
			return;
		end
	end

	local tEffect = {};
	tEffect.nType = nSkillType;
	tEffect.nEffType = EffectOnCarType.OnFire;
	tEffect.fTime = fKeepTime;
	tEffect.fPercent = fRate;

	self.tArrSkillEffect[#self.tArrSkillEffect+1] = tEffect;

	-- 播放效果
	self:StartToShowEffect(tEffect.nEffType, nSkillType)
end

-- 删除集火效果
function LogicRaceCar:RemoveOnFireEffect()
	-- 判断当前是否有集火，有的话，使用当前最长的时间
	for i,tOneEff in ipairs(self.tArrSkillEffect) do
		if tOneEff.nEffType == EffectOnCarType.OnFire then
			tOneEff.fTime = 0;
		end
	end
end

-- 获取当前的伪装情况
function LogicRaceCar:GetOnFireSkillInfo()
	-- 判断当前是否有集火
	for i,tOneEff in ipairs(self.tArrSkillEffect) do
		if tOneEff.nEffType == EffectOnCarType.OnFire and tOneEff.fTime > 0 then
			return math.abs(tOneEff.fPercent);
		end
	end
	return -1
end

-- 判断当前是否有免疫
function LogicRaceCar:IsCurCarHaveMianYi()
	-- 判断当前是否有免疫，有的话，使用当前最长的时间
	for i,tOneEff in ipairs(self.tArrSkillEffect) do
		if tOneEff.nEffType == EffectOnCarType.MianYi then
			return true;
		end
	end
	return false;
end

-- 获取当前车子的位置相关信息
function LogicRaceCar:GetCurCarPositonAllInfo()
	local tInfo ={}
	-- 信息
	tInfo.tPos = self:getPositionInCCPoint();
	tInfo.nLinePt = self:GetCurRoadIdx()
	tInfo.fRoadValue = self:GetCurRoadMoveValue()
	tInfo.nRoadType = self:GetCurRunRoad();
	tInfo.nCircle = self:GetCurCirCle();
	tInfo.nNum = self:GetCurQualify();
	tInfo.fCarRot = self.pSpCar:getRotation();

	return tInfo
end

-- 获取当前车子的位置相关信息
function LogicRaceCar:SetCurCarPositonAllInfo(tInfo)
	self:setPosition( tInfo.tPos )
	self.pSpCar:setRotation( tInfo.fCarRot )

	local fPhyRot = CC_DEGREES_TO_RADIANS(ChangeRotTo180Range(tInfo.fCarRot))
	local tPhyPos = b2Vec2(tInfo.tPos.x/PTM_RATIO, tInfo.tPos.y/PTM_RATIO)
    self.pPhysicObject:SetTransform( tPhyPos, -fPhyRot);

	self:SetCurRoadMoveValue( tInfo.fRoadValue )
	self:SetCurQualify( tInfo.nNum, true )
	self:SetCurCirCle( tInfo.nCircle, true)
	self:SetCurRunRoad(tInfo.nRoadType, false)
	self:SetCurRoadIdx( tInfo.nLinePt, true)
end

-- 跟其他车子互换位置
function LogicRaceCar:ShowSkillSwitchPos(tOtherCar)
	if tOtherCar == nil then
		return;
	end

	-- 判断当前是否有免疫
	if self:IsCurCarHaveMianYi() == true then
		return;
	end

	-- 互换位置
	local tInfoTmp1 = tOtherCar:GetCurCarPositonAllInfo();
	local tInfoTmp2 = self:GetCurCarPositonAllInfo();
	self:SetCurCarPositonAllInfo(tInfoTmp1)
	tOtherCar:SetCurCarPositonAllInfo(tInfoTmp2)
end

-- 判断显示小效果
function LogicRaceCar:StartToShowEffect( nEffectType, nSkillEffType )
	-- 保存效果
	SkillEffResMgr:RetainOneEff( nSkillEffType )
	-- 效果一样，则不处理
	if self.nShowEffectType == nEffectType and self.nShowSkillEffType == nSkillEffType then
		return;
	end
	-- 如果当前是免疫或者没有效果，则不处理了
	if self.nShowEffectType == EffectOnCarType.MianYi or 
		nEffectType == EffectOnCarType.None or 
		nSkillEffType == SkEffectType.None then
		return;
	end
	-- 如果是有显示优先级的效果
	if self.nShowEffectType > EffectOnCarType.YouXianLine then
		-- 如果显示的优先级比当前显示的低，则不显示
		if nEffectType < self.nShowEffectType then
			return;
		end
	end

	-- 如果之前有效果，则删除掉
	if self.nShowEffectType ~= EffectOnCarType.None then
		self:RemoveCurSkillEffect();
	end
	-- print("显示的技能效果:", nEffectType, nSkillEffType)
	-- 创建动画
	local pSpEffect = SkillEffResMgr:CreateSkillSwfEff( nSkillEffType, true )
	-- 根据情况添加到不同的显示中
	if SkillEffResMgr:GetIsNeedRotByCar( nSkillEffType ) == true then
		-- 调整位置
		pSpEffect:setPosition( ccpAdd(ctspccp(self.pSpCar, 0.5, 0.5), 
								SkillEffResMgr:GetSkillShowOffsetPos(nSkillEffType)) )
		pSpEffect:setRotation(-90)
		self.pSpCar:addChild(pSpEffect,1)
	else
		pSpEffect:setPosition( ccpAdd( self:GetCurCarShowPosByRate(0.5, 0.5), 
								SkillEffResMgr:GetSkillShowOffsetPos(nSkillEffType)) )
		self:addChild(pSpEffect,1)
	end
	-- 开始播放动画
	SkillEffResMgr:StartAnimation( nSkillEffType, pSpEffect )

	-- 添加到精灵数组
	self.pSpEffects[1] = pSpEffect;

	self.nShowEffectType = nEffectType;
	self.nShowSkillEffType = nSkillEffType;
end

-- 某个技能结束
function LogicRaceCar:ShowOneSkillEffectEnd(nEffectType, nSkillType)
	-- print("技能效果结束：", nEffectType, nSkillType)

	-- 释放当前效果
	SkillEffResMgr:ReleaseOneEff( nSkillType )

	-- 如果是当前效果，则考虑是否删除掉
	if nEffectType == self.nShowEffectType and self.nShowSkillEffType == nSkillType then

		local nRestCout = #self.tArrSkillEffect
		-- 找出下一个要重新显示的效果
		local tNextEffInfo
		-- 当前最大的效果
		local nMaxEffCnt = EffectOnCarType.None;
		for i=nRestCout,1,-1 do
			local tOneInfo = self.tArrSkillEffect[i];
			-- 如果是有显示优先级的效果
			if tOneInfo.nEffType > EffectOnCarType.YouXianLine then
				-- 如果优先级比较高，则选择
				if tOneInfo.nEffType > nMaxEffCnt then
					tNextEffInfo = tOneInfo;
				end

			else
				if tNextEffInfo == nil then
					tNextEffInfo = tOneInfo;
				end
			end
		end

		local nNextType = EffectOnCarType.None;
		if tNextEffInfo then
			nNextType = tNextEffInfo.nEffType;
		end
		-- 删除掉旧的
		self:RemoveCurSkillEffect();
		-- 如果下一个效果播放的效果是有，则继续播放下一个效果
		if nNextType ~= EffectOnCarType.None then
			self:StartToShowEffect(nNextType, tNextEffInfo.nType);
		end
	end
	-- 如果停止的冰冻效果，则速度恢复
	if nEffectType == EffectOnCarType.Frozen then
		self:StartMoveCar(true);
	end
end

-- 删除当前的技能效果
function LogicRaceCar:RemoveCurSkillEffect()
	for i,pSpEffect in ipairs(self.pSpEffects) do
		pSpEffect:removeFromParent();
	end

	self.pSpEffects = {};
	-- 效果的大类型
	self.nShowEffectType = EffectOnCarType.None;
	-- 对应技能的效果
	self.nShowSkillEffType = SkEffectType.None;
end

return LogicRaceCar