--
-- Author: Link
-- Date: 2013-12-28 09:51:17
--

local LogicRaceMap = class("LogicRaceMap", function()
	return display.newLayer()
end)

---------------- 调整镜头所用的所有倍数，数字越小，离得越远 -----------------
-- 开始入场的倍数
local MAP_START_INTO_SCALE	= 1.0
-- 当前地图所能承受的最远倍数
local MAP_FARTHERT_SCALES	= 1.0;
-- 入场结束的倍数
local MAP_INTO_END_SCALE 	= 1.0
-- 跑动中的倍数
local MAP_RACING_SCALE 		= 1.6;
-- 技能开始播放的角度
local MAP_START_SKILL_SCALE	= 1.6;
-- 技能向下拉动的倍数
local MAP_SKILL_INTO_SCALE	= 1.6 
-- 第一辆车过完跑道时的显示倍数
local MAP_FIRST_END_SCALE 	= 1.0


local function ChangeMapBiggestScale( fScale )
	MAP_FARTHERT_SCALES = fScale;

	MAP_INTO_END_SCALE = fScale*1.35
	MAP_RACING_SCALE = fScale*1.35
end

------------------------------------------------------------------------



local CAR_ENTER_TIME    = 1.4

-- 能达到的最大速度（不加成）
MAX_CAR_RUN_SPEED       = 1100
-- 当前的最大速度
MAX_CUR_RUN_SPEED 		= MAX_CAR_RUN_SPEED
-- 最短的泡完地图的时间（不加成的情况下）
MIN_CAR_RUN_TIME 		= 17*0.9

local UNSET_SKILL_VALUE = -1000000

function LogicRaceMap:ctor()
	-- self.super.ctor(self)

    -- 当前是否暂停
    self.bIsPause = true;
    self.bIsAllEnd = false;

    self.bIsFirstEnd    = true;
    self.pEndMapPos = ccp(0,0)

    -- 当前的回调
    self.pCBack = {}
    setmetatable(self.pCBack, {__mode = "v"})

    -- 一圈的权重，等下会重新算
    self.fOneCircleValue = 1000000;
    -- 一圈中，移动到每个节点所需的总价值量
    self.tCirclePtValue = {}
    -- 每一段的价值
    self.tCircleOnePtValue = {}

    -- 播放技能的时间，显示技能的信息，
    self.tArrShowSkill = {
-- nCarPos = {
--     nCarPos = 1,
--     fRoadValue = 0,
--     tSkill = {},
--     bIsUse = false;
--     nEndUsePos = -1; -- 在对方有害技能结束之后立刻播放技能主动释放位置
-- },
    };

	-- 所有车
	self.pAllCars = {}
	self.nShowCarIdx = 1

    self.pSkillMgr = nil;
    -- 地图属性
    self.tMapInfo = nil;

    self:ignoreAnchorPointForPosition(true)
end

function LogicRaceMap:InitMap( tRaceInfo , pCallBack)
	-- body
	local strPathName = "Images/Map" .. tRaceInfo.nMapID

	MAX_CUR_RUN_SPEED = MAX_CAR_RUN_SPEED;

    -- 设置回调
    self.pCBack.delegate = pCallBack;

    -- 当前地图信息
    self.tFightInfo = clone(tRaceInfo)
    -- 读取信息
    if self:LoadMapInfo(tRaceInfo.nMapID) == false then
        return false
    end

    -- 层次要很高的显示
    self.pSkillMgr = requires(ROOT_PATH, "app/GameLg/Race/RaceSkillMgr").new();
    self.pSkillMgr:InitSkillMgr(self);
    self:getParent():addChild(self.pSkillMgr, 9999)

    -- 添加所有车子的显示
    -- self:ReAddAllCarToShow()

    return true
end

-- 删除赛道层次时的处理
function LogicRaceMap:BeforeReleaseLayer()
    self:unscheduleUpdate();

     -- 删除所有旧车子
    local nCarCount = #self.pAllCars
    for i=1,nCarCount do
        local pOneCar = self.pAllCars[i]
        pOneCar:removeFromParent()
        self.pAllCars[i] = nil
    end

    
    for i,v in ipairs(self.body_list) do
        self.pWorld:DestroyBody(v)
    end
    self.body_list = {}
end

-- 读取地图的
function LogicRaceMap:LoadMapInfo(nMapID)

    -- body
    local strPathName = "Images/map/Map" .. nMapID

    -- 地图文件
    local strMapName = strPathName .. ".tmx"
    strMapName = Public_GetResPathByKey(strMapName)
    -- 获取失败，则返回失败
    if CCFileUtils:sharedFileUtils():isFileExist(strMapName) == false then
        CTLOG("地图文件不存在，读取失败,ID为:%d", nMapID)
        return false
    end

    -- 创建一个世界
    local gravity = b2Vec2(0.0, 0.0)
    self.pWorld = b2World:new_local(gravity)

    -- 
    self.pMap = CCTMXTiledMap:create(strMapName)
    self:addChild(self.pMap)
    self.pMap:setScale(MAP_START_INTO_SCALE)
    self.pMap:setVisible(true)

    -- 最大倍数
    local tMapCon = self.pMap:getContentSize();
    -- 最大的倍数
    local pBigestScale = display.height/tMapCon.height
    ChangeMapBiggestScale(pBigestScale)

	-- 创建debug绘制物理区域
	if g_tSelSuperInfo ~= nil and g_tSelSuperInfo.nMapPhy == true then
		self.pRender = GLESDebugDraw:new_local()
		self.pRender:setRatio(PTM_RATIO)
		self.pWorld:SetDebugDraw(self.pRender);
		local flags = 0;
		flags = flags + b2Draw.e_shapeBit;
		-- //        flags += b2Draw::e_jointBit;
		-- //        flags += b2Draw::e_aabbBit;
		-- //        flags += b2Draw::e_pairBit;
		-- //        flags += b2Draw::e_centerOfMassBit;
		self.pRender:SetFlags(flags);

		-- 地图绘制时回调这里
		self.pMap:setNodeLuaDeleate(function( event )
		    if event.type == "draw" then
		        kmGLPushMatrix();
		        self.pWorld:DrawDebugData();
		        kmGLPopMatrix();
		    end
		end)
	end


    local pChildrenArray = self.pMap:getChildren();
    for i=1,pChildrenArray:count() do
        local pObject = pChildrenArray:objectAtIndex(i-1)
        tolua.cast(pObject, "CCSpriteBatchNode")
        pObject:getTexture():setAntiAliasTexParameters()
    end

    -- 获取地图信息
    local strInfoFile = strPathName .. ".road"
    local strMapInfo = ""
    strMapInfo = GetRoadMapInfo(Public_GetResPathByKey(strInfoFile), strMapInfo)

    -- 解析出来
    local jsonType = require("framework.json")
    local tMapAllInfo = jsonType.decode(strMapInfo);
    -- print_r(tMapAllInfo)
    -- 获取最佳路线
    self.pArrPointMid = {}
    self.pArrPoint1 = {}
    self.pArrPoint2 = {}
    for i=0,tMapAllInfo.road.count-2 do
        -- mid
        local tOneInfo = tMapAllInfo.road[i .. ""]
        local tPos = ccp(tOneInfo.x, tOneInfo.y)
        self.pArrPointMid[#self.pArrPointMid + 1] = tPos
        -- road1
        tOneInfo = tMapAllInfo.road1[i .. ""]
        tPos = ccp(tOneInfo.x, tOneInfo.y)
        self.pArrPoint1[#self.pArrPoint1 + 1] = tPos
        -- road2
        tOneInfo = tMapAllInfo.road2[i .. ""]
        tPos = ccp(tOneInfo.x, tOneInfo.y)
        self.pArrPoint2[#self.pArrPoint2 + 1] = tPos
    end

	-- debug状态下，显示所有信息
	-- do
	-- 	local pArrPt = CCPointArray:create(#self.pArrPoint1)
	-- 	for i, p in ipairs(self.pArrPoint1) do
	-- 		pArrPt:add(p)
	-- 	end
	-- 	local pLine1 = CCPolygonShape:create(pArrPt)
	-- 	pLine1:setLineColor(ccc4f(1, 0, 0, 1))
	-- 	pLine1:setLineWidth(3)
	-- 	pLine1:setClose(true)
	-- 	self.pMap:addChild(pLine1, 100)
	-- end
	-- do
	-- 	local pArrPt = CCPointArray:create(#self.pArrPointMid)
	-- 	for i, p in ipairs(self.pArrPointMid) do
	-- 		pArrPt:add(p)
	-- 	end
	-- 	local pLineMid = CCPolygonShape:create(pArrPt)
	-- 	pLineMid:setLineColor(ccc4f(0, 1, 0, 1))
	-- 	pLineMid:setLineWidth(3)
	-- 	pLineMid:setClose(true)
	-- 	self.pMap:addChild(pLineMid, 100)
	-- end
	-- do
	-- 	local pArrPt = CCPointArray:create(#self.pArrPoint2)
	-- 	for i, p in ipairs(self.pArrPoint2) do
	-- 		pArrPt:add(p)
	-- 	end
	-- 	local pLine2 = CCPolygonShape:create(pArrPt)
	-- 	pLine2:setLineColor(ccc4f(0, 0, 1, 1))
	-- 	pLine2:setLineWidth(3)
	-- 	pLine2:setClose(true)
	-- 	self.pMap:addChild(pLine2, 100)
	-- end

    -- 启动路线
    self.pArrStarts = {}

    for i=0,tMapAllInfo.start.count-1 do
        local tOneInfo = tMapAllInfo.start[i .. ""]
        local tPos = ccp(tOneInfo.x, tOneInfo.y)
        self.pArrStarts[#self.pArrStarts + 1] = tPos
    end

    -- 地图属性
    self.tMapInfo = clone(tMapAllInfo.map)

    -- 根据地图类型播放背景音乐

    
	local pPhysicFileStr = tMapAllInfo.physic
	local strPhyFile = CCFileUtils:sharedFileUtils():getWritablePath().."CurPhysicFile.plist"
	io.writefile(strPhyFile, pPhysicFileStr, "w+b")

	GB2ShapeCache:sharedGB2ShapeCache():addShapesWithFile(strPhyFile)
	-- 删除掉临时的文件
	os.remove(strPhyFile);

	local bodyDef1 = b2BodyDef:new_local()
	bodyDef1.type = b2_staticBody;
	local position = self.pMap:getPositionInCCPoint() -- 这个spriteMap是我们创建好的精灵
	bodyDef1.position.x = position.x / PTM_RATIO
	bodyDef1.position.y = position.y / PTM_RATIO
	-- bodyDef1.userData = self.pMap;
	local body_b2 = self.pWorld:CreateBody(bodyDef1) --这里是之前创建好的world

	-- 从缓冲区中找到数据，并将其设置进去
	local sc = GB2ShapeCache:sharedGB2ShapeCache()
	sc:addFixturesToBody(body_b2, "map"); -- 这个map是里头图片的名字，我用的是map.png
    self.body_list = {}


    -- 重新算一圈路的价值
    local nMapCir = self:GetCurMapMaxCircle()
    self.fOneCircleValue = 0;
    local nMaxPtCount = #self.pArrPointMid;
    for i, p in ipairs(self.pArrPointMid) do
        local nNext = (i%nMaxPtCount)+1;
        local tNextPos = self.pArrPointMid[nNext];
        self.tCircleOnePtValue[i] = ccpDistance(p, tNextPos);
        self.fOneCircleValue = self.fOneCircleValue + self.tCircleOnePtValue[i]
        self.tCirclePtValue[i] = clone(self.fOneCircleValue);
    end
    local fTotalValue = self.fOneCircleValue * nMapCir;
    -- 如果没有达到最低时间，则重新算速度
    if fTotalValue < MAX_CUR_RUN_SPEED*MIN_CAR_RUN_TIME then
    	MAX_CUR_RUN_SPEED = fTotalValue/MIN_CAR_RUN_TIME;
    end
    -- 将路程分成14分，在中间的12分出技能
    local fOneValue = fTotalValue /15.0;
    local nArrIdx = {}
    for i=1,12 do
        nArrIdx[i] = i;
    end

    -- 如果没有技能信息，则添加
    if self.tArrShowSkill == nil or table.nums(self.tArrShowSkill) == 0 then
        self.tArrShowSkill={};
        -- 所有技能的遍历处理
        for i,tOneSkill in ipairs(self.tFightInfo.tCarEvents) do
            local nIdx = GetRandom(1, #nArrIdx);
            local nCurIdx = nArrIdx[nIdx];
            table.remove(nArrIdx, nIdx);

            local fShowValue = tOneSkill.fRoadValue;
            if fShowValue == nil or fShowValue == 0 then
                 -- 如果免疫技能，则显示位置是负数，不能主动释放
                if tOneSkill.nType == CarSkillType.MIANYI_SKILL_ONE or 
                    tOneSkill.nType == CarSkillType.MIANYI_SKILL_ALL then
                    fShowValue = UNSET_SKILL_VALUE;
                else
                    fShowValue = fOneValue*nCurIdx + fOneValue* GetFloatRandom(0.15, 0.85);
                end
            end

            -- MISS的情况
            local tMiss = {}
            tMiss.bIsHad = false;
            tMiss.tList = {}
            tOneSkill.tMiss = tMiss;

            local tSkillInfo = {}
            tSkillInfo.nCarPos = tOneSkill.nCarPos;
            tSkillInfo.fRoadValue = fShowValue;
            tSkillInfo.tSkill = tOneSkill;
            tSkillInfo.bIsUse = false;
            tOneSkill.nEndUsePos = -1;
            self.tArrShowSkill[ tOneSkill.nCarPos ] = tSkillInfo;
        end
    end
    

    return true
end

-- 获取和设置保留数据
function LogicRaceMap:GetMapKeepInfo()
    local tKeepInfo = {}

    tKeepInfo.skillInfo = self.tArrShowSkill;

    return tKeepInfo;
end
function LogicRaceMap:SetMapKeepInfo(tKeepInfo)
    if tKeepInfo == nil then
        return;
    end

    self.tArrShowSkill = tKeepInfo.skillInfo;
end

-- 设置skill的数据
function LogicRaceMap:SetSkillMissInfo(tMissInfo)

	if tMissInfo == nil then
		return;
	end

    for i,v in ipairs(tMissInfo) do
    	local tOneSkill = self.tArrShowSkill[ v.idx ]
		if tOneSkill ~= nil then
			 local tMiss = {}
            tMiss.bIsHad = true;
            tMiss.tList = v.list
            tOneSkill.tSkill.tMiss = tMiss;
		end
    end
end

-- 暂停和恢复
function LogicRaceMap:PauseMap()
    self.bIsPause = true;
end
function LogicRaceMap:ResumeMap()
    self.bIsPause = false;
end

-- 开始显示入场仪式
function LogicRaceMap:StartToShowIntoMap()
    -- 暂停地图
    self:PauseMap()

    -- 开始获取重点的位置
    self.pMap:setScale(MAP_INTO_END_SCALE)
    -- 强制显示到对应的车子身上
    if self.tFightInfo.nMyPos == 1 then
        self:ForceToShowMapToPos( self.pArrStarts[2] )
    else
        self:ForceToShowMapToPos( self.pArrStarts[1] )
    end

    local tTargetPos = self.pMap:getPositionInCCPoint();
    -- 
    local tMapCon = self.pMap:getContentSize();

    -- 最大距离的移动情况
    self.pMap:setScale(MAP_FARTHERT_SCALES)

    local tStartPosTmp = ccp( tMapCon.height*(display.width/display.height)/2.0, tMapCon.height/2.0 )
    -- 强制显示到对应的车子身上
    self:ForceToShowMapToPos( tStartPosTmp )
    -- 开始的位置
    local tStartPos = self.pMap:getPositionInCCPoint();

    -- 获取当前车子的绝对位置
    local tEndPosTmp =  ccp(tMapCon.width-tStartPosTmp.x, tStartPosTmp.y)
    self:ForceToShowMapToPos( tEndPosTmp )
    local tEndPos = self.pMap:getPositionInCCPoint();

    -- 回复入场的倍数
    self.pMap:setScale(MAP_START_INTO_SCALE)

    self:ForceToShowMapToPos( tStartPosTmp )

    -- 如果允许点击跳过，则处理
    if TEACHMGR:IsCurInTeaching() and TEACHMGR:GetTeachEventRetainValue() == 1 then
        -- 镜头处理
        local tArrAct = CCArray:create()
        tArrAct:addObject( CCSpawn:createWithTwoActions(CCMoveTo:create(0.75, tStartPos),
                                                        CCScaleTo:create(0.75, MAP_FARTHERT_SCALES) ) )
        tArrAct:addObject( CCCallFunc:create(function()
            TEACHMGR:ShowSwichToNextStep();
        end) )

        self.pMap:runAction( CCSequence:create(tArrAct) )
    else
        -- 镜头处理
        local tArrAct = CCArray:create()
        tArrAct:addObject( CCSpawn:createWithTwoActions(CCMoveTo:create(0.75, tStartPos),
                                                        CCScaleTo:create(0.75, MAP_FARTHERT_SCALES) ) )
        tArrAct:addObject( CCDelayTime:create(0.5) )
        tArrAct:addObject( CCMoveTo:create(1.5, tEndPos) )
        tArrAct:addObject( CCDelayTime:create(0.5) )
        tArrAct:addObject( CCSpawn:createWithTwoActions(CCMoveTo:create(1, tTargetPos),
                                                        CCScaleTo:create(1, MAP_INTO_END_SCALE) ) )
        tArrAct:addObject( CCCallFunc:create(function()
            self:ReAddAllCarToShow(true)
        end) )
        -- 等待入场结束
        tArrAct:addObject( CCDelayTime:create(CAR_ENTER_TIME) )
        tArrAct:addObject( CCCallFunc:create(function()


            -- 回调通知，入场结束
            if self.pCBack.delegate and self.pCBack.delegate.MapCallBackEnterMapEnd then
                self.pCBack.delegate:MapCallBackEnterMapEnd();
            end
        end) )
        self.pMap:runAction( CCSequence:create(tArrAct) )
    end

    
end

-- 强制停止
function LogicRaceMap:ForceStopShowIntoMap()
    self.pMap:stopAllActions();

    self:ReAddAllCarToShow()

    if self.tFightInfo.nMyPos == 1 then
        self:ForceToShowCarByIdx(2)
    else
        self:ForceToShowCarByIdx(1)
    end
end

-- 添加要比赛的车，如果比赛的车
function LogicRaceMap:AddOneCar( tCarInfo , fSpeedRate, fASpeedRate, fTurnRate, bIsShowEnter, nCarIdx)
	if tCarInfo == nil then
		CTAssert(0, "创建比赛赛车失败")
		return
	end

	if self.pMap == nil then
		CTAssert(0, "还没有初始化地图")
		return
	end

	local nCurIdx = #self.pAllCars + 1

	if nCurIdx > #self.pArrStarts then
		CTAssert(0, "当前地图只支持" .. #self.pArrStarts .."辆车。")
		return
	end
    
    local nShowStartIdx = nCarIdx--nCurIdx
    -- if nShowStartIdx % 2 == 0 then
    --     nShowStartIdx = nCurIdx-1
    -- else
    --     nShowStartIdx = nCurIdx+1
    -- end
    local tTmp1 = self.pArrStarts[nShowStartIdx]
    local tCarPos = ccp(tTmp1.x, tTmp1.y)
    local tRot1 = GetP2PRotation(self.pArrPoint2[2], self.pArrPoint2[1])
	-- 创建车子
	local LogicRaceCar = requires(ROOT_PATH, "app/GameLg/Race/LogicRaceCar")
	local pSpCar = LogicRaceCar.new()
	pSpCar:InitCar(tRot1, tCarInfo, self.pArrPoint1, 
                            self.pArrPointMid, self.pArrPoint2)
    pSpCar:SetCarShowRateInfo(fSpeedRate, fASpeedRate, fTurnRate)

    tCarPos.x = tCarPos.x - pSpCar:getContentSize().width*(1-pSpCar:getAnchorPoint().x)*pSpCar:getScaleX()-12

    pSpCar:SetCarStartPosition( tCarPos )
    pSpCar:SetCurQualify(nCurIdx, false)
    pSpCar:SetStartCarPos(nCurIdx)
    pSpCar:SetMapDelegate(self)
    self.pMap:addChild(pSpCar, 5)
    -- 车子可以开始移动
    -- pSpCar:StartMoveCar(true)

    -- 车子的物理引擎
    local tCarBound = ctspbound(pSpCar)
    -- local pObject = self.pWorld:createBoxBody(100, tCarBound.size.width, tCarBound.size.height)
    -- pObject:bind(pSpCar)
    -- pObject:setPosition( self.pArrStarts[nCurIdx] )
    -- 
    -- pSpCar:SetCurPhysicObject(pObject)


     -- Create body and shape
    
    local bodyDef = b2BodyDef:new_local()
    bodyDef.type = b2_dynamicBody
    bodyDef.position:Set(tCarPos.x/PTM_RATIO, tCarPos.y/PTM_RATIO)
    bodyDef.userData = pSpCar
    local _body = self.pWorld:CreateBody(bodyDef)

    local circle = b2PolygonShape:new_local()
    -- circle.m_radius = 26.0/PTM_RATIO
    circle:SetAsBox(tCarBound.size.width/PTM_RATIO/2.0, tCarBound.size.height/PTM_RATIO/2.0)

    local shapeDef = b2FixtureDef:new_local()
    shapeDef.shape = circle
    shapeDef.density = 1.0
    shapeDef.friction = 0.0
    shapeDef.restitution = 0.0
    -- shapeDef.fixedRotation = true;
    _body:CreateFixture(shapeDef)
    -- 要修正转角
    _body:SetFixedRotation(true)
    -- 是否不允许
    -- _body:SetBullet(true)

    table.insert(self.body_list, _body)

    pSpCar:SetCurPhysicObject(_body)

    local fPhyRot = CC_DEGREES_TO_RADIANS(ChangeRotTo180Range(tRot1))
    _body:SetTransform( _body:GetPosition(), -fPhyRot);

    -- print("fdsafdsafdsa:", _body:IsFixedRotation(), _body:IsBullet())

    -- 初始化线路，前四个车子的路线都测试过。
    local nCurPos = nShowStartIdx % 3
    if nShowStartIdx == 1 or nShowStartIdx == 3 then
        pSpCar:SetCurRunRoad(CarMoveRoadType.Road1)
    elseif nShowStartIdx == 4 or nShowStartIdx == 2 then
        pSpCar:SetCurRunRoad(CarMoveRoadType.Mid)
    elseif  nCurPos == 1 then
        pSpCar:SetCurRunRoad(CarMoveRoadType.Road1)
    elseif nCurPos == 2 then
        pSpCar:SetCurRunRoad(CarMoveRoadType.Mid)
    else
        pSpCar:SetCurRunRoad(CarMoveRoadType.Road2)
    end
    -- 复制汽车
    self.pAllCars[nCurIdx] = pSpCar

    -- 显示移动进来
    if bIsShowEnter == true then
        pSpCar:setPosition( ccp( pSpCar:getPositionX()-360, pSpCar:getPositionY() ) )
        pSpCar:runAction( CCSequence:createWithTwoActions(CCMoveBy:create(CAR_ENTER_TIME, ccp(360, 0)),
                                                            CCPlace:create(tCarPos) ) )

        local tAct1 = CCArray:create()
        tAct1:addObject( CCDelayTime:create( GetFloatRandom(0.02, 0.06) ) )
        tAct1:addObject( CCMoveBy:create(0.03, ccp(0, 1.5)) )
        tAct1:addObject( CCMoveBy:create(0.06, ccp(0, -3)) )
        tAct1:addObject( CCMoveBy:create(0.03, ccp(0, 1.5)) )
        pSpCar:runAction( CCRepeatForever:create( CCSequence:create(tAct1) ) )
    end    
end

-- 获取计算范围值的处理
function LogicRaceMap:GetValueRateByRangeInfo( fMax, fMin, fCurMax, fCurMin, fCur )
    -- 获取的比例
    local fCurRate = 1.0

    -- 根据当前最大最小数字与规定的最大最小相比，获得使得不会低于规定最低数值的比例
    if fCurMin < fMin then
        fCurRate = fMin / fCurMin
    elseif fCurMax > fMax then
       fCurRate = fMax / fCurMax

       local fMinTmp = fCurMin * fCurRate
       if fMinTmp < fMin then
          fCurRate = fCurRate * (fMin / fMinTmp)
       end
    end
    -- 如果最大值没有超支，则直接返回当前值
    local fTmpCurMax = fCurMax*fCurRate
    if fTmpCurMax < fMax then
        return fCurRate
    end

    -- 超过部分另算比例
    local fTmpRate = (fMax-fMin)/(fTmpCurMax-fMin)

    -- 适合的当前值
    local fTmpCur = (fCurMin+(fCur-fCurMin)*fTmpRate)*fCurRate

    fCurRate = fTmpCur / fCur

    return fCurRate
end
-- 重新添加所有车子
function LogicRaceMap:ReAddAllCarToShow(bIsShowEnter)
    
    if bIsShowEnter == nil then
        bIsShowEnter = false;
    end

    -- 释放所有动画的frames
    SkillEffResMgr:removeAllInfo()

    -- 没有暂停
    -- self.bIsPause = false;
    -- 删除所有旧车子
    local nCarCount = #self.pAllCars
    for i=1,nCarCount do
        local pOneCar = self.pAllCars[i]
        pOneCar:removeFromParent()
        self.pAllCars[i] = nil
    end

    
    for i,v in ipairs(self.body_list) do
        self.pWorld:DestroyBody(v)
    end
    self.body_list = {}

    -- 添加车子
    local tRaceInfo = self.tFightInfo

    -- 按顺序保存车子信息
    local tArrAllCarInfo = {}
    -- 处理开始的索引
    local nSelfIdx = 1
    local nOtherIdx = 1
    -- 自己不排在第一
    if tRaceInfo.nMyPos ~= 0 then
        nSelfIdx = 0
    end
    -- 按顺序将车子的数据添加进来
    repeat
        local nCarIdx = #tArrAllCarInfo + 1;
        -- 自己的车
        if nSelfIdx > 0 and #(tRaceInfo.tMyCars) >= nSelfIdx then
            tArrAllCarInfo[nCarIdx] = tRaceInfo.tMyCars[nSelfIdx]
            tArrAllCarInfo[nCarIdx].bIsSelf = true;

            nCarIdx = #tArrAllCarInfo + 1;
        end
        -- 对手的车
        if nOtherIdx > 0 and #(tRaceInfo.tEnemyCars) >= nOtherIdx then
            tArrAllCarInfo[nCarIdx] = tRaceInfo.tEnemyCars[nOtherIdx]
            tArrAllCarInfo[nCarIdx].bIsSelf = false;
        end

        nSelfIdx = nSelfIdx+1
        nOtherIdx = nOtherIdx+1
        if nSelfIdx > #(tRaceInfo.tMyCars) and nOtherIdx > #(tRaceInfo.tEnemyCars) then
            break
        end
    until false

    -- 定义最大速度和最低速度
    local nCirCount = self:GetCurMapMaxCircle() * 1.0
    -- local fRate = ( nCirCount / (nCirCount+1.15) );
    local fMaxSpeed = MAX_CUR_RUN_SPEED
    local fMinSpeed = fMaxSpeed * ( nCirCount / (nCirCount+1) );
    -- fMinSpeed = fMinSpeed + (fMaxSpeed - fMinSpeed) * 0.15
    local fMaxASpeed = fMaxSpeed/2.5 -- 400
    local fMinASpeed = fMaxASpeed * ( nCirCount / (nCirCount+2.0) );
    local fMaxTurn  = 165 -- 145
    local fMinTurn  = fMaxTurn * ( nCirCount / (nCirCount+1.32) )

    -- 获取的得到最低最高速度
    local fMaxCarSpeed = 0
    local fMinCarSpeed = 1000000000
    local fMaxCarASpeed = 0
    local fMinCarASpeed = 1000000000
    local fMaxCarTurn = 0
    local fMinCarTurn = 1000000000

    -- 我的信息
    for nIdx,tOneCar in ipairs(tArrAllCarInfo) do
        -- 速度
        if fMaxCarSpeed < tOneCar.nSpeed  then
            fMaxCarSpeed = tOneCar.nSpeed
        end
        if fMinCarSpeed > tOneCar.nSpeed  then
            fMinCarSpeed = tOneCar.nSpeed
        end
        -- 加速度
        if fMaxCarASpeed < tOneCar.nASpeed  then
            fMaxCarASpeed = tOneCar.nASpeed
        end
        if fMinCarASpeed > tOneCar.nASpeed  then
            fMinCarASpeed = tOneCar.nASpeed
        end
        -- 加速度
        if fMaxCarTurn < tOneCar.nTurn  then
            fMaxCarTurn = tOneCar.nTurn
        end
        if fMinCarTurn > tOneCar.nTurn  then
            fMinCarTurn = tOneCar.nTurn
        end
    end

    local fSTTemp1 = 0.86
	local nSelfCarIdx = 1;
	local nOtherCarIdx = 2;
    -- 自己不排在第一
    if tRaceInfo.nMyPos ~= 0 then
		nSelfCarIdx = 2;
		nOtherCarIdx = 1;
    end
    -- 添加车子信息
    for nIdx,tOneCar in ipairs(tArrAllCarInfo) do
        -- 算出适合当前赛车的比例值
        local fSpeedRate = self:GetValueRateByRangeInfo(fMaxSpeed, fMinSpeed, fMaxCarSpeed, fMinCarSpeed, tOneCar.nSpeed)
        local fASpeedRate = self:GetValueRateByRangeInfo(fMaxASpeed, fMinASpeed, fMaxCarASpeed, fMinCarASpeed, tOneCar.nASpeed)
        -- local fTurnRate = self:GetValueRateByRangeInfo(fMaxTurn, fMinTurn, fMaxCarTurn, fMinCarTurn, tOneCar.nTurn)

        local fTurnValue = self:GetValueRateByRangeInfo(fMaxASpeed*fSTTemp1, fMinASpeed*fSTTemp1, fMaxCarTurn, fMinCarTurn, tOneCar.nTurn)*tOneCar.nTurn
        local fTurnRate = (((fTurnValue-fMinASpeed*fSTTemp1)/(fMaxASpeed*fSTTemp1-fMinASpeed*fSTTemp1))*(fMaxTurn-fMinTurn)+fMinTurn)/tOneCar.nTurn

        if tOneCar.bIsSelf == true then
        	self:AddOneCar(tOneCar, fSpeedRate, fASpeedRate, fTurnRate, bIsShowEnter, nSelfCarIdx)
        	nSelfCarIdx=nSelfCarIdx+2;
        else
        	self:AddOneCar(tOneCar, fSpeedRate, fASpeedRate, fTurnRate, bIsShowEnter, nOtherCarIdx)
        	nOtherCarIdx=nOtherCarIdx+2;
        end
    end
    -- 还没有结束
    self.bIsFirstEnd    = true;

    self.bIsAllEnd = false;

    -- 将所有车辆索引到当前车子中。
    self.pSkillMgr:AddShowCars(self.pAllCars)


    -- 群体免疫技能
    local tArrSelfMianYi = {}
    local tArrOtherMianYi = {}
    -- 群体有害技能
    local tArrSelfWorst = {}
    local tArrOtherWorst = {}

    -- 将所有技能都设置成没有播放，如果之前没有设置群体免疫技能播放情况，则开始设置
    for i,tOneSkill in pairs(self.tArrShowSkill) do
        tOneSkill.bIsUse = false;
        -- tSkillInfo.fRoadValue
        -- 受众类型
        local nTargetType = self.pSkillMgr:GetCarSkillTargetType(tOneSkill.tSkill.nType)
        -- tOneSkill.nEndUsePos = 0;
        -- 是否是自己的车子
        local bIsSelf = self:GetShowCarByPos( tOneSkill.nCarPos ):GetCurCarInfo().bIsSelf;
        -- 群体免疫可以立刻知道播放
        if tOneSkill.tSkill.nType == CarSkillType.MIANYI_SKILL_ALL then
            if tOneSkill.fRoadValue <= UNSET_SKILL_VALUE then
                if bIsSelf == true then
                    tArrSelfMianYi[#tArrSelfMianYi+1] = tOneSkill;
                else
                    tArrOtherMianYi[#tArrOtherMianYi+1] = tOneSkill;
                end
            else
                -- 群体免疫技能
                tArrSelfMianYi = {}
                tArrOtherMianYi = {}
                -- 群体有害技能
                tArrSelfWorst = {}
                tArrOtherWorst = {}
                break;
            end
        -- 有害技能
        elseif nTargetType == self.pSkillMgr:SkillShowType().ReduceEnemy or
                nTargetType == self.pSkillMgr:SkillShowType().GetEnemyToMate then
            if bIsSelf == true then
                tArrSelfWorst[#tArrSelfWorst+1] = tOneSkill;
            else
                tArrOtherWorst[#tArrOtherWorst+1] = tOneSkill;
            end
        end
    end

    -- 自己的全体免疫技能释放情况
    for i,tMianYi in ipairs(tArrSelfMianYi) do
        if #tArrOtherWorst <= 0 then
            break;
        end
        local nWorstIdx = GetRandom(1, #tArrOtherWorst)
        local tWorstSkill = tArrOtherWorst[nWorstIdx];
        -- 获取负数的位置
        tMianYi.fRoadValue = -tWorstSkill.nCarPos;

        -- 删除出数组
        table.remove(tArrOtherWorst, nWorstIdx)
    end

    -- 对方的全体免疫技能释放情况
    for i,tMianYi in ipairs(tArrOtherMianYi) do
        if #tArrSelfWorst <= 0 then
            break;
        end
        local nWorstIdx = GetRandom(1, #tArrSelfWorst)
        local tWorstSkill = tArrSelfWorst[nWorstIdx];
        -- 获取负数的位置
        tMianYi.fRoadValue = -tWorstSkill.nCarPos;

        -- 删除出数组
        table.remove(tArrSelfWorst, nWorstIdx)
    end

    -- 强制显示到对应的车子身上
    if self.tFightInfo.nMyPos == 1 then
        self:ForceToShowMapToPos( self.pArrStarts[2] )
    else
        self:ForceToShowMapToPos( self.pArrStarts[1] )
    end

    -- 回调通知，入场结束
    if bIsShowEnter == false and self.pCBack.delegate and self.pCBack.delegate.MapCallBackEnterMapEnd then
        self.pCBack.delegate:MapCallBackEnterMapEnd();
    end
end

-- 车子的开始和停止
function LogicRaceMap:ChangeAllCarsToStartAbled( bStart )
    self.bIsPause = false;

    for i,pOneCar in ipairs(self.pAllCars) do
        pOneCar:stopAllActions()
        pOneCar:StartMoveCar(bStart)
    end
    self:StartShowRacingClose()
end

-- 开场的开跑拉近镜头
function LogicRaceMap:StartShowRacingClose()
    -- 计时器
    local fScaleDel = (MAP_RACING_SCALE - self.pMap:getScale())/0.6;
    self:scheduleUpdate(function (dt)
        local fScaleX = self.pMap:getScale();
        fScaleX = fScaleX + fScaleDel*dt;
        if fScaleX > MAP_RACING_SCALE then
            fScaleX = MAP_RACING_SCALE;
            self:unscheduleUpdate();
        end
        self.pMap:setScale(fScaleX)
        self:ForceToShowCarByIdx(self.nShowCarIdx);
    end)
end

-- 切换下一辆车为视角
function LogicRaceMap:SwitchOtherCarToShow(nSelIdx)
    nSelIdx = nSelIdx or 1
	local nCarCount = #(self.tFightInfo.tMyCars) + #(self.tFightInfo.tEnemyCars)
	if nCarCount == 0 or nSelIdx > nCarCount or nSelIdx <= 0 then
		CTLOG("切换视角时，发现赛道上面没有对应车子: ".. nSelIdx .."  " .. nCarCount)
		return
	end

    -- 底下光环
    local pSpOldCar = self:GetCurShowCar();
    if pSpOldCar then
        pSpOldCar:ShowCurCarSelInfo(false)
    end

    -- 设置焦点
    self.nShowCarIdx = nSelIdx

    -- 名次变化相关
    local pSpOneCar = self:GetCurShowCar()
    if pSpOneCar then
        local nQualifyTmp = pSpOneCar:GetCurQualify();
        pSpOneCar:ShowCurQualifyEffect(nQualifyTmp)
        -- 底下光环
        pSpOneCar:ShowCurCarSelInfo(true)
    end
end

-- 获取地图属性信息
function LogicRaceMap:GetCurMapInfoList()
    return self.tMapInfo;
end

-- 获取当前移动的车子的信息
function LogicRaceMap:GetCurShowCar()
    return self.pAllCars[ self.nShowCarIdx ]
end

-- 获取当前移动的车子数据
function LogicRaceMap:GetShowCarByPos(nPos)
    local nCarCount = #(self.pAllCars)
    if nCarCount == 0 or nPos > nCarCount or nPos <= 0 then
        CTLOG("获取车子数据时，发现赛道上面没有对应车子: ".. nPos .."  " .. nCarCount)
        return nil;
    end
    return self.pAllCars[ nPos ]
end
-- 根据当前排名获取车辆
function LogicRaceMap:GetShowCarByQualify(nQualify)
    nQualify = toint(nQualify);
    for i,pSpCar in ipairs(self.pAllCars) do
        if pSpCar:GetCurQualify() == nQualify then
            return pSpCar;
        end
    end
    return nil;
end

-- 根据所以，获取当前车子所有的地图位置比例
function LogicRaceMap:GetCarPosRateByIndex( nIdx )
    local pSpCar = self.pAllCars[nIdx]
    if pSpCar == nil then
        CTAssert(0, "获取车子相对与地图的位置比时，发生了错误")
        return ccp(0, 0)
    end

    local tPosRate = ccp(0, 0)
    tPosRate.x = pSpCar:getPositionX() / self.pMap:getContentSize().width
    tPosRate.y = pSpCar:getPositionY() / self.pMap:getContentSize().height
    return tPosRate
end

-- 判断当前是否运行操作底部信息
function LogicRaceMap:GetIsCannotControl()
    -- 已经有一两车通过终点了，则不给操作了
    if self.bIsFirstEnd == false then
        return true;
    end
    -- 暂停中也不给操作
    if self.bIsPause == true then
        return true;
    end
    return false;
end
-- 获取当前车子的圈数
function LogicRaceMap:GetCurMapMaxCircle()
    return toint(self.tMapInfo.Circle);
end

function LogicRaceMap:FixMapPos(fPosX, fPosY )
    if self:getScale() ~= 1.0 then
        local tMapConSize = self.pMap:getContentSize()
        local tPos1 = self.pMap:convertToNodeSpace(ccp(0,0))
        local tPos2 = self.pMap:convertToNodeSpace( ctprccp(1,1) )

        local tCurPos = self.pMap:getPositionInCCPoint()
        local fDelX = (fPosX - tCurPos.x) / self.pMap:getScaleX()
        local fDelY = (fPosY - tCurPos.y) / self.pMap:getScaleY()
        tPos1.x = tPos1.x - fDelX
        tPos1.y = tPos1.y - fDelY
        tPos2.x = tPos2.x - fDelX
        tPos2.y = tPos2.y - fDelY
        if tPos1.x < 0 then
            fPosX = fPosX + tPos1.x* self.pMap:getScaleX();
            
        elseif tPos2.x > tMapConSize.width then
            fPosX = fPosX + (tPos2.x - tMapConSize.width) * self.pMap:getScaleX();
        else
            -- print("ddddd::", fPosX, tPos1.x, tPos2.x, tMapConSize.width)
        end

        if tPos1.y < 0 then
            fPosY = fPosY + tPos1.y* self.pMap:getScaleY();
        elseif tPos2.y > tMapConSize.height then
            fPosY = fPosY + (tPos2.y - tMapConSize.height)* self.pMap:getScaleY();
        end

        return fPosX, fPosY;
    end

    return fPosX, fPosY;
end

-- mark: ============== 循环处理 ================
function LogicRaceMap:LogicRaceMapLoop( dt )
    -- 如果是暂停，则不处理循环
    if self.bIsPause == true then
        -- local fPosX, fPosY = self.pMap:getPosition()
        -- fPosX, fPosY = self:FixMapPos(fPosX, fPosY);
        -- self.pMap:setPosition( ccp(fPosX, fPosY) )
        return;
    end

    -- 之前已经通过终点的车索引字典
    local tArrOldEndList = {}

    -- 判断车子跑完
    local bIsAllFinish = true

    -- 先处理当前跑动次数
	for i=1, #self.pAllCars do
    	local pSpCar = self.pAllCars[i]
        -- 获取之前已经结束的。
        if pSpCar:GetIsMoveEnd() then
            tArrOldEndList[ i ] = i;
        end

	    -- 其他的循环处理
	    pSpCar:CarLoop(dt)


        local nMapCir = self:GetCurMapMaxCircle()+1
        -- 只要有车没跑完就不算结束
        if pSpCar:GetCurCirCle() < nMapCir then
            bIsAllFinish = false
        end
    end

    -- 循环处理移动
    self.pWorld:Step(dt, 6, 4)
    if self.body_list ~= nil then
        for i, body in ipairs(self.body_list) do
            local sprite = GetBox2dBodyUserDataToNode(body)
            
            if sprite ~= nil then
                tolua.cast(sprite, "LogicRaceCar")
                local tPos1 = sprite:getPositionInCCPoint()
                sprite:setPosition(ccp(body:GetPosition().x * PTM_RATIO, body:GetPosition().y * PTM_RATIO))
                -- sprite:SetCurCarRot( ChangeRotTo360Range(-1*CC_RADIANS_TO_DEGREES(body:GetAngle()) ) )

                local tPos2 = sprite:getPositionInCCPoint()
                if tPos1.x ~= tPos2.x or tPos1.y ~= tPos2.y then
                    sprite:SetCurCarRot( GetP2PRotation(tPos2, tPos1) )
                end

            end
        end
    end
    self.pWorld:ClearForces()

    -- 将之前已经过了终点的和还没有过终点的分开处理
    local tArrHadPastList = {}
    local tNotPastList = {}

    local tOldPastList = {}


    -- 获取价值量
    for i=1, #self.pAllCars do
        local pSpCar = self.pAllCars[i]
	    -- -- 算出当前车子在当前路段位移的横向距离
	    -- local fCos = math.cos( CC_DEGREES_TO_RADIANS( -pSpCar:GetCurRoadRotation()+90) )
    	-- local fSin = math.sin( CC_DEGREES_TO_RADIANS( -pSpCar:GetCurRoadRotation()+90) )
    	-- local tLocalPos = pSpCar:GetCurRoadStartPt()
    	-- local fPosRelX = (pSpCar:getPositionX() - tLocalPos.x)*fCos + (pSpCar:getPositionY() - tLocalPos.y)*fSin

    	-- -- 获取当前车子所在位置的权重值
     --    local fCurLen = pSpCar:GetCurRoadLen()

        local fPosRelX, fCurLen = pSpCar:GetRelMidRoadValues()
	    local tOneLen = {}
	    tOneLen.fRoadMove = fPosRelX
        local fPosRate = fPosRelX/fCurLen;
        if fPosRate > 1 then
            fPosRate = 1;
        end
        local nRoadIdxTmp = pSpCar:GetCurRoadIdx()
        if nRoadIdxTmp == 0 then
            nRoadIdxTmp = 1;
        end
        local fValueTmp1 = self.tCirclePtValue[nRoadIdxTmp-1] or 0;
        local fValueTmp2 = (self.tCircleOnePtValue[nRoadIdxTmp] or 0)*fPosRate;
	    tOneLen.value = (pSpCar:GetCurCirCle()-1)*self.fOneCircleValue + fValueTmp1 + fValueTmp2;
	    tOneLen.nIdx = i
	    

        -- 判断是否拥有一样的数值

        -- 如果之前没有到达终点
        if tArrOldEndList[i] == nil then
            if pSpCar:GetIsMoveEnd() then
                tOldPastList[#tOldPastList+1] = tOneLen;
            else
                local nInsertIdx = #tNotPastList + 1;
                for m,tOneInfo in ipairs(tNotPastList) do
                    if tOneLen.value > tOneInfo.value then
                        nInsertIdx = m;
                        break;
                    end
                end
                table.insert(tNotPastList, nInsertIdx, tOneLen);
            end
            
        -- 之前已经到达终点
        else
            local nInsertIdx = #tArrHadPastList + 1;
            for m,tOneInfo in ipairs(tArrHadPastList) do
                if tOneLen.value > tOneInfo.value then
                    nInsertIdx = m;
                    break;
                end
            end
            table.insert(tArrHadPastList, nInsertIdx, tOneLen);
        end
    end

    for i=#tOldPastList,1,-1 do
        table.insert(tNotPastList, 1, tOldPastList[i] );
    end


    -- 按排位排好所有车
    local tAllCarMoveLen = {}
    -- 先将已经跑完的排号，再将没有跑完的接上
    for i,v in ipairs(tArrHadPastList) do
       tAllCarMoveLen[#tAllCarMoveLen+1] = v;
    end
    for i,v in ipairs(tNotPastList) do
       tAllCarMoveLen[#tAllCarMoveLen+1] = v;
    end

    -- -- 根据选择排序法，算出权重排行榜，最大的排前面
    -- for i=1,(#tAllCarMoveLen) do
    --     local nCarStartIdx = tAllCarMoveLen[i].nIdx;
    --     -- 交换过来之后，直接设置该车子的排位
    --     local pSpOneCar = self.pAllCars[ nCarStartIdx ]
    -- end

    --- 焦点车辆的处理，在技能之前，通过终点之后不修正镜头了
    if self.bIsFirstEnd == true then
        self:ForceToShowCarByIdx(self.nShowCarIdx);
    else
        -- self.pMap:setPosition( self.pEndMapPos )
    end

    -- 名次变化相关
    local nQualifyTmp = self:GetCurShowCar():GetCurQualify();
    
    -- 根据选择排序法，算出权重排行榜，最大的排前面
    for i=1,(#tAllCarMoveLen) do
        local nCarStartIdx = tAllCarMoveLen[i].nIdx;
    	-- 交换过来之后，直接设置该车子的排位
    	local pSpOneCar = self.pAllCars[ nCarStartIdx ]

        local nOldQualify = pSpOneCar:GetCurQualify()
        if nOldQualify ~= i and (nQualifyTmp == nOldQualify or i == nQualifyTmp) then
            pSpOneCar:ShowCurQualifyEffect(i)
        end
        pSpOneCar:SetCurQualify(i)

        -- 判断这辆车是否有技能
        local tShowSkill = self.tArrShowSkill[nCarStartIdx]
        local fNewRoadValue = tAllCarMoveLen[i].value
        if tShowSkill ~= nil then
            local fOldValue = pSpOneCar:GetCurRoadMoveValue();
            pSpOneCar:SetCurRoadMoveValue( fNewRoadValue );
            -- 当车辆移动到指定距离时，开始触发技能
            if tShowSkill.fRoadValue > 0 and tShowSkill.fRoadValue > fOldValue and tShowSkill.fRoadValue <= fNewRoadValue and tShowSkill.bIsUse == false then
                tShowSkill.bIsUse = true;
                -- 开始播放技能
                self.pSkillMgr:StartToShowSkill(tShowSkill.tSkill, nCarStartIdx);
            end
        else
            pSpOneCar:SetCurRoadMoveValue( fNewRoadValue );
        end


        if pSpOneCar:GetIsMoveEndAndCanDealInfo() then
            pSpOneCar:SetCannotMoveEndAndCanDealInfo()
            -- 回调通知一辆车通过终点

            -- 如果是在梦中的战斗，则慢显示一下
            if TEACHMGR:IsInDreamWar() == true then
                self:performWithDelay(function()
                    if self.pCBack.delegate and self.pCBack.delegate.MapCallBackOneCarAcrossOneEnd then
                        self.pCBack.delegate:MapCallBackOneCarAcrossOneEnd(pSpOneCar);
                    end
                end, 0.3)
            else
                if self.pCBack.delegate and self.pCBack.delegate.MapCallBackOneCarAcrossOneEnd then
                    self.pCBack.delegate:MapCallBackOneCarAcrossOneEnd(pSpOneCar);
                end
            end
            
        end
    end


    -- 如果是全部结束了，则进入结束界面
    if bIsAllFinish == true and self.bIsAllEnd == false then
        self.bIsAllEnd = true;

        self:performWithDelay(function()
           self.bIsPause = true;
            -- 是否回调
            if self.pCBack.delegate and self.pCBack.delegate.MapCallBackRaceEnd then
                self.pCBack.delegate:MapCallBackRaceEnd(self.tArrShowSkill);
            end
        end, 1.2)
        return
    end
end

-- 将对应车子的聚焦
function LogicRaceMap:ForceToShowCarByIdx( nCarPos  )
    -- 坐标位置的处理
    if nCarPos > #self.pAllCars then
        CTAssert(0, "焦点的车子不存在")
        return
    end

    local pSpLocalCar = self.pAllCars[ nCarPos ]
    
    self:ForceToShowMapToPos( pSpLocalCar:getPositionInCCPoint()  )
end

-- 将地图强制移动到地图上对应的位置
function LogicRaceMap:ForceToShowMapToPos( tPos )
    local fX = tPos.x * self.pMap:getScaleX()
    local fY = tPos.y * self.pMap:getScaleY()

    -- 修改出发的位置
    local tMapConSize = self.pMap:getContentSize()
    local tMapSize = tMapConSize
    tMapSize.width = tMapSize.width * self.pMap:getScaleX()
    tMapSize.height = tMapSize.height * self.pMap:getScaleY()
    local fPosX = display.cx-fX
    if fPosX > 0 then
        fPosX = 0
    elseif fPosX <= -(tMapSize.width-display.width) then
        fPosX = -(tMapSize.width-display.width)
    end

    local fPosY = display.cy - fY
    if fPosY > 0 then
        fPosY = 0
    elseif fPosY <= -(tMapSize.height-display.height) then
        fPosY = -(tMapSize.height-display.height)
    end    

    self.pMap:setPosition(fPosX, fPosY)
end

-- 当起始位置为 nCenterCarPos 的车在中心时， tArrCars 是车索引数组，判断有多少个车子所以在范围内
function LogicRaceMap:GetCarsInOneScreen( nCenterCarPos, tArrCars)
    -- 坐标位置的处理
    if nCenterCarPos > #self.pAllCars then
        CTAssert(0, "判断车子是否同屏时，索引错误：", nCenterCarPos, nOtherCarPos)
        return {}, false;
    end
    -- 如果已经没有数组了，则false
    if table.nums(tArrCars) <= 0 then
        return {}, false;
    end

    -- 中心车子的位置
    local pSpLocalCar = self:GetShowCarByPos( nCenterCarPos )
    local tMidSpPos = pSpLocalCar:getParent():convertToWorldSpace( pSpLocalCar:getPositionInCCPoint() )
    -- 地图左下角和右上角
    local tMapPos1 = self.pMap:convertToWorldSpace( ccp(0,0) )
    local tMapPos2 = self.pMap:convertToWorldSpace( ctspccp(self.pMap, 1, 1) )

    -- 中心车子，距离中心位置的横向偏差
    local fMidWidth = ctprwidth(0.5)
    local fVLen = 0;
    if tMidSpPos.x - tMapPos1.x < fMidWidth then
        fVLen = fMidWidth -  (tMidSpPos.x - tMapPos1.x);
    elseif tMapPos2.x - tMidSpPos.x < fMidWidth then
        fVLen = (tMapPos2.x - tMidSpPos.x) - fMidWidth;
    end
    -- 中心车子，距离中心位置的竖向偏差
    local fMidHeight = ctprheight(0.5)
    local fHLen = 0;
    if tMidSpPos.y - tMapPos1.y < fMidHeight then
        fHLen = fMidHeight -  (tMidSpPos.y - tMapPos1.y);
    elseif tMapPos2.y - tMidSpPos.y < fMidHeight then
        fHLen = (tMapPos2.y - tMidSpPos.y) - fMidHeight;
    end

    local tArrIdx = {}
    local bIsHad  = false;

    for k,v in pairs(tArrCars) do
        if v ~= nCenterCarPos and self:IsTwoCarInOneScreen(tMidSpPos, v, fVLen, fHLen) == true then
            tArrIdx[ #tArrIdx + 1] = k;
            bIsHad = true;
        end
    end

    return tArrIdx, bIsHad;
end

-- 当 tMidSpPos 在中心时，nOtherCarPos 位置的车是否完全显示在同一个屏幕中
function LogicRaceMap:IsTwoCarInOneScreen( tMidSpPos, nOtherCarPos, fVLen, fHLen)
    -- 坐标位置的处理
    if nOtherCarPos > #self.pAllCars then
        CTAssert(0, "判断车子是否同屏时，索引错误1：", nOtherCarPos)
        return false;
    end

    local pSpOtherCar = self:GetShowCarByPos( nOtherCarPos )
    local tOtherSpPos = pSpOtherCar:getParent():convertToWorldSpace( pSpOtherCar:getPositionInCCPoint() )

    -- 如果横向在范围内
    if math.abs( tMidSpPos.x + fVLen - tOtherSpPos.x ) < (ctprwidth(0.5) - ctipwidth(38) ) then
        local fHeight = tMidSpPos.y + fHLen - tOtherSpPos.y;
        local fHalfHeight = ctprheight(0.5)
        -- 如果竖向在范围内
        if fHeight >= (-fHalfHeight+50) and fHeight <= (fHalfHeight-110) then
            return true;
        end
    end

    return false;
end

-- mark: ============== 车子的回调信息 ================
-- 回调重新设置路的类型，nStartPos是初始位置,nRoadIdx是现在开始的路面信息
function LogicRaceMap:CallBackJudgeChangeCarRoadType( pSpCar, nStartPos, nRoadIdx)
    local nCurPos = pSpCar:GetCurQualify();
    -- -- 如果只移动比较短的距离，则不抢道
    if pSpCar:GetCurRoadMoveValue() < 600 then
        return false;
    end
    -- 内道
    local pSpPrevCar = self:GetShowCarByQualify(nCurPos-1);
    local pSpPrevCar2 = self:GetShowCarByQualify(nCurPos-2);
    local pSpNextCar = self:GetShowCarByQualify(nCurPos+1);

   
    

    -- 跑中路
    local nRunRoadType = CarMoveRoadType.Mid;
    -- 如果没有前面的车，则跑内道
    if pSpPrevCar == nil then
        nRunRoadType = CarMoveRoadType.Road1;
    -- 第二名跑2道
    elseif pSpPrevCar2 == nil then
        nRunRoadType = CarMoveRoadType.Mid;
    else
        local nPrevRoadType = pSpPrevCar:GetCurRunRoad()
        local nPrevRoadType2 = pSpPrevCar2:GetCurRunRoad()

        local nCurRoadType = pSpCar:GetCurRunRoad();
        local nSelRoadType = nCurRoadType;
        -- 和上一辆车路线一样
        if nPrevRoadType == nCurRoadType then
            if nPrevRoadType == CarMoveRoadType.Road1 then
                nSelRoadType = CarMoveRoadType.Mid;
            -- 如果都在中路
            elseif nPrevRoadType == CarMoveRoadType.Mid then
                if nPrevRoadType2 == CarMoveRoadType.Road1 then
                    nSelRoadType = CarMoveRoadType.Road2;
                else
                    nSelRoadType = CarMoveRoadType.Road1;
                end
            elseif nPrevRoadType == CarMoveRoadType.Road2 then
                nSelRoadType = CarMoveRoadType.Mid;
            end
        -- 
        else
            -- 先选一条和前两辆不一样的路线
            local tArrRoadType = {};
            for i=CarMoveRoadType.Road1,CarMoveRoadType.Road2 do
                if i ~= nPrevRoadType and i ~= nPrevRoadType2 then
                    tArrRoadType[ #tArrRoadType+1 ] = i;
                end
            end
            nSelRoadType = tArrRoadType[1];

            -- 如果选的路线偏差两条路线，则看情况处理
            if math.abs(nSelRoadType - nCurRoadType) > 1  then
                -- 如果前一辆车不是中间的，则跑中间
                if nPrevRoadType ~= CarMoveRoadType.Mid then
                    nSelRoadType = CarMoveRoadType.Mid;
                -- 如果前一辆车在中间，则当前车不换路
                else
                    nSelRoadType = nCurRoadType;
                end
            end
        end

        nRunRoadType = nSelRoadType
    end

    if pSpPrevCar ~= nil then
        local fCurV = pSpCar:GetRelMidRoadValues();
        -- 获取距离差
        local fPrev = pSpPrevCar:GetRelMidRoadValues()-fCurV;
        local fNext = 10000;
        if pSpNextCar ~= nil then
            fNext = fCurV - pSpNextCar:GetRelMidRoadValues();
        end

        if fPrev > 480 and fNext > 120 then
            local nCurRoadType = pSpCar:GetCurRunRoad();
            -- 如果都在中路
            if nCurRoadType == CarMoveRoadType.Mid then
                nRunRoadType = CarMoveRoadType.Road1;
            -- 如果在外道
            elseif nCurRoadType == CarMoveRoadType.Road2 then
                nRunRoadType = CarMoveRoadType.Mid;
            end
        end
    end

    -- local nRunRoadType = CarMoveRoadType.Mid;
    -- -- 如果没有前面的车，则跑内道
    -- if pSpPrevCar == nil then
    --     nRunRoadType = CarMoveRoadType.Road1;
    -- -- 第二名跑2道
    -- elseif nCurPos == 2 then
    --     nRunRoadType = CarMoveRoadType.Mid;
    -- else
    --     local nPrevRoadType = pSpPrevCar:GetCurRunRoad()
    --     local nCurRoadType = pSpCar:GetCurRunRoad();
    --     -- 不和上一辆车同道
    --     local tArrRoadType = {};
    --     for i=CarMoveRoadType.Road1,CarMoveRoadType.Road2 do
    --         if i ~= nPrevRoadType then
    --             tArrRoadType[ #tArrRoadType+1 ] = i;
    --         end
    --     end
    --     -- 随机选择一道路
    --     local nSelRoadType = tArrRoadType[ GetRandom(1, 2) ];
    --     -- 如果随机的路和当前车的路偏差太大
    --     if math.abs(nSelRoadType - nCurRoadType) > 1 then
    --         -- 如果前一辆车不是中间的，则跑中间
    --         if nPrevRoadType ~= CarMoveRoadType.Mid then
    --             nSelRoadType = CarMoveRoadType.Mid;
    --         -- 如果前一辆车在中间，则当前车不换路
    --         else
    --             nSelRoadType = nCurRoadType;
    --         end
    --     end

    --     nRunRoadType = nSelRoadType
    -- end

     -- 如果要改道时，位置空间太急时，不改道
    if pSpPrevCar ~= nil then
        if math.abs( pSpPrevCar:GetCurRoadMoveValue()-pSpCar:GetCurRoadMoveValue() ) < 130 then
            -- 如果这样短的话，不能处理
            if nRunRoadType == pSpPrevCar:GetCurRunRoad() then
                return false;
            end
        end
    end

    -- 如果当前车要移动到
    if pSpNextCar then
        if math.abs(pSpCar:GetCurRoadMoveValue()-pSpNextCar:GetCurRoadMoveValue()) < 130 then
            -- 如果这样短的话，不能处理
            if nRunRoadType == pSpNextCar:GetCurRunRoad() then
                return false;
            end
        end
    end

    pSpCar:SetCurRunRoad(nRunRoadType, false);
    return false
end

-- 判断是否移动圈数结束
function LogicRaceMap:CallBackJudgeMoveCircleEnd( pSpCar, nCircleIdx )
    local nMapCir = self:GetCurMapMaxCircle()
    -- 圈数结束
    if (nMapCir+1) == nCircleIdx then
        -- 如果之前还没有移动结束
        if pSpCar:GetIsMoveEnd() == false then
            -- 如果是第一辆车突破终点，则给特写镜头
            if self.bIsFirstEnd == true then
                -- 停止移动
                self.bIsPause = true;
                local nCarPos = pSpCar:GetStartCarPos();

                local fOldScale = self.pMap:getScale()
                -- 找出放大后的倍数
                self.pMap:setScale( MAP_FARTHERT_SCALES )

                self:ForceToShowCarByIdx( nCarPos );

                local tMoveEndPos = self.pMap:getPositionInCCPoint();

                self.pMap:setScale(fOldScale)

                -- 强制显示到对应的车子身上
                self:ForceToShowCarByIdx( nCarPos );
                -- 获取当前车子的绝对位置
                local pSpLocalCar = self.pAllCars[ nCarPos ]
                local tCarPos = pSpLocalCar:getParent():convertToWorldSpace( pSpLocalCar:getPositionInCCPoint() )

                local tAll = ctprccp(1,1);
                -- 设置锚点
                self:setAnchorPoint( ccp(tCarPos.x/tAll.x, tCarPos.y/tAll.y) )
                -- 结束的地图位置
                self.pEndMapPos = self.pMap:getPositionInCCPoint();

                -- 播放过终点的声音
                SOUNDMGR:PlayEffRaceFirstCarAcrossEnd()

                local tArrAct = CCArray:create()
                -- tArrAct:addObject( CCScaleTo:create(0.2, 3.5) )
                tArrAct:addObject( CCDelayTime:create(1) )
                tArrAct:addObject( CCScaleTo:create(0.1, MAP_FIRST_END_SCALE) )
                -- 不是梦中的剧情，则有恢复原来路线
                if TEACHMGR:IsInDreamWar() == false then
                    tArrAct:addObject( CCCallFunc:create(function()
                        self.bIsPause = false;


                        local tMapAct = CCArray:create()
                        tMapAct:addObject( CCMoveTo:create(1, tMoveEndPos) )
                        tMapAct:addObject( CCScaleTo:create(1, MAP_FARTHERT_SCALES) )
                        self.pMap:runAction( CCSpawn:create(tMapAct) )

                    end) )
                end

                
                self:runAction( CCSequence:create(tArrAct) )
            end
            self.bIsFirstEnd    = false;
        end
        return true;
    end
    return false;
end


-- mark: =============== 技能的回调信息 ===============
-- 技能显示位置
function LogicRaceMap:SkillMgrShowCar(nSkillCarPos)
    if self.bIsFirstEnd == false then
        return;
    end
    -- 强制显示到对应的车子身上
    self:ForceToShowCarByIdx(nSkillCarPos);

    local fPosX1, fPosY1 = self.pMap:getPosition()
    local fPosX, fPosY = self:FixMapPos(fPosX1, fPosY1)

    self.pMap:setPosition(ccp(fPosX, fPosY))
end
-- 开始真正的播放技能
function LogicRaceMap:SkillMgrStartToShowSkill(nSkillCarPos)
    -- 开始播放技能
    self.bIsPause = true;

    local fScaleTime = 0.1
    local tDelayAct = CCDelayTime:create(1);

    self:unscheduleUpdate()
    self.pMap:setScale(MAP_START_SKILL_SCALE)

    -- 计算倍数错误
    self:setScale(1.0/MAP_SKILL_INTO_SCALE)
    -- 强制显示到对应的车子身上
    self:ForceToShowCarByIdx(nSkillCarPos);


    -- 获取当前车子的绝对位置
    local pSpLocalCar = self.pAllCars[ nSkillCarPos ]
    local tCarPos = pSpLocalCar:getParent():convertToWorldSpace( pSpLocalCar:getPositionInCCPoint() )

    local tAll = ctprccp(1,1);
    -- 设置锚点
    self:setAnchorPoint( ccp(tCarPos.x/tAll.x, tCarPos.y/tAll.y) )

    local fPosX1, fPosY1 = self.pMap:getPosition()
    local fPosX, fPosY = self:FixMapPos(fPosX1, fPosY1)

    local tBackAct = nil;
    if self.nShowCarIdx == nSkillCarPos then
        self.pMap:runAction( CCSequence:createWithTwoActions(tDelayAct:copy():autorelease(),
                                                             CCMoveTo:create(fScaleTime, ccp(fPosX, fPosY))) )

        local tWPos1 = self.pMap:getParent():convertToWorldSpace(ccp(fPosX1, fPosY1))
        local tWPos2 = self.pMap:getParent():convertToWorldSpace(ccp(fPosX, fPosY))
        local tMove1 = ccp( (tWPos2.x-tWPos1.x)/self:getScaleX(), (tWPos2.y-tWPos1.y)/self:getScaleY() )
        tBackAct = CCSequence:createWithTwoActions(tDelayAct:copy():autorelease(),
                                                   CCMoveBy:create(fScaleTime,  tMove1) )
    else
        self.pMap:setPosition(ccp(fPosX, fPosY))
    end

    self:setScale(1)
    -- 播放技能
    SOUNDMGR:PlayEffRaceUseSkill()

    -- local tArrAct = CCArray:create()
    -- tArrAct:addObject( CCScaleTo:create(0.15, 2.0) )
    -- tArrAct:addObject( CCDelayTime:create(1) )
    -- tArrAct:addObject( CCScaleTo:create(0.0, 1.0) )
    -- self:runAction( CCSequence:create(tArrAct) )

    local tArrAct = CCArray:create()
    -- tArrAct:addObject( CCScaleTo:create(0.15, 1.0) )
    tArrAct:addObject( tDelayAct )
    tArrAct:addObject( CCScaleTo:create(fScaleTime, 1.0/MAP_SKILL_INTO_SCALE) )
    self:runAction( CCSequence:create(tArrAct) )

    return tBackAct
end

-- 判断是否需要显示技能过程
function LogicRaceMap:SkillMgrIsCanShowSkillProgress()
    return (self.bIsFirstEnd)
end

-- 技能效果显示上来,tArrSkillInfo是技能效果数组，
-- 每个成员： ArrIdx,  ArrEffSp,  ArrAct,  ArrWord, ArrWordAct, ActTime 成员分别表示车子索引，效果Sp，动作，文字，文字动作,动作总共时间
-- tSkill 是当前的技能属性
-- fPlaySpeed 是当前播放速度
-- tArrDatas 是要往回传的数据
-- 返回的是，播放这些效果所用的总时间
function LogicRaceMap:SkillMgrEffectOneCar( tArrSkillInfo, tSkill, tArrEnemy, tArrSelf, fPlaySpeed, tArrDatas )
    if tArrSkillInfo == nil then
        return 0;
    end
    -- 移动到对应车子的回调
    local function CallFuncCallbackToMove(pSpTheCar)
        local fX = pSpTheCar:getPositionX() * self.pMap:getScaleX()
        local fY = pSpTheCar:getPositionY() * self.pMap:getScaleY()

        -- 修改出发的位置
        local tMapSize = self.pMap:getContentSize()
        tMapSize.width = tMapSize.width * self.pMap:getScaleX()
        tMapSize.height = tMapSize.height * self.pMap:getScaleY()
        local fPosX = display.cx-fX
        if fPosX > 0 then
            fPosX = 0
        elseif fPosX <= -(tMapSize.width-display.width) then
            fPosX = -(tMapSize.width-display.width)
        end

        local fPosY = display.cy - fY
        if fPosY > 0 then
            fPosY = 0
        elseif fPosY <= -(tMapSize.height-display.height) then
            fPosY = -(tMapSize.height-display.height)
        end
        fPosX, fPosY = self:FixMapPos(fPosX, fPosY);
        -- 地图移动过去
        self.pMap:runAction( CCMoveTo:create(0.15, ccp(fPosX, fPosY)) )
    end

    -- 使用的总时间
    local fTotalTime = 0.0

    -- 遍历所有技能效果显示
    for i,tOneSkill in ipairs(tArrSkillInfo) do
        -- 效果结束
        local function CallFuncCallBackToShowOnEffect( pSpTheCar )
            self.pSkillMgr:AddEffectToOneRaceCar(pSpTheCar, tSkill, tArrDatas)
        end

        -- 取出所有属性
        local tArrIdx = clone(tOneSkill.ArrIdx);
        local tArrSp = tOneSkill.ArrEffSp;
        local tArrAct = tOneSkill.ArrAct;
        local tArrCarAct = tOneSkill.ArrCarAct;
        local tArrWord = tOneSkill.ArrWord;
        local tArrWordAct = tOneSkill.ArrWordAct
        local fActTotalTime = tOneSkill.ActTime;
        -- 给每辆车施加效果
        repeat
            -- 获取运到的索引
            local nOneIdx = -1
            for k,v in pairs(tArrIdx) do
                nOneIdx = k;
                break;
            end
            -- 如果索引已经使用完，则不处理了
            if nOneIdx < 0 then
                break;
            end
            -- 移动过去的时间
            -- fTotalTime = fTotalTime + 0.1;
            local fMaxOneActTime = 0;
            -- 还存在的索引数组
            local tSameScreenCarIdx = {}
            -- 是否都一起播放
            if tOneSkill.bIsPlayTogether == true then
                -- 不看是否在同一屏幕，直接全部显示
                tSameScreenCarIdx = clone(tArrIdx)
                for i=1,#tArrIdx do
                    tSameScreenCarIdx[i] = i;
                end
            else
                -- 找出同屏索引
                tSameScreenCarIdx = self:GetCarsInOneScreen(tArrIdx[nOneIdx], tArrIdx)
                table.insert(tSameScreenCarIdx, 1, nOneIdx)
            end
            
            -- 遍历所有当前屏幕中招的车子
            for m,nTmpIdx in ipairs(tSameScreenCarIdx) do
                -- 取出来车子索引
                local nOneCarIdx = tArrIdx[nTmpIdx];
                -- 删除当前车子所在数组索引
                tArrIdx[nTmpIdx] = nil; 

                -- 获取车子
                local pSpCar = self:GetShowCarByPos(nOneCarIdx)

                -- 找出和这个车子同一个屏幕的车子索引
                local pSpEff = tArrSp[nTmpIdx];
                if pSpEff then
                    pSpEff:setVisible(false);
                    -- 将效果添加显示
                    pSpCar:addChild( pSpEff )
                end
                

                -- 显示动画
                if m == 1 then
                    local pMoveDelay = CCDelayTime:create(fTotalTime);
                    -- 信息
                    pSpCar:runAction( CCSequence:createWithTwoActions(pMoveDelay, 
                                                CCCallFuncN:create(CallFuncCallbackToMove) ) )
                    -- 添加移动屏幕的时间
                    fTotalTime = fTotalTime + 0.17;
                end
                -- 技能效果的动画
                if pSpEff then
                    local pArrAct = CCArray:create();
                    pArrAct:addObject( CCDelayTime:create(fTotalTime) )
                    pArrAct:addObject( CCShow:create() )
                    pArrAct:addObject( tArrAct[nTmpIdx] )
                    pSpEff:runAction( CCSequence:create(pArrAct) )
                end
                
                local fAddTime = fActTotalTime[nTmpIdx]
                if fAddTime == nil then
                    fAddTime = 0;
                end
                if fMaxOneActTime < fAddTime then
                    fMaxOneActTime = fAddTime;
                end
                -- 如果没有动画，则执行原本流程
                if tArrCarAct[nTmpIdx] == nil then
                    pSpCar:runAction( CCSequence:createWithTwoActions(CCDelayTime:create(fTotalTime+fAddTime),
                                                        CCCallFuncN:create(CallFuncCallBackToShowOnEffect)) )
                -- 如果有动画，则执行需要的动画
                else
                    pSpCar:runAction( tArrCarAct[nTmpIdx] )
                end
                

                -- 如果有效果文字
                local pLbWord = tArrWord[nTmpIdx];
                if pLbWord ~= nil then
                    for n,pOneWord in ipairs(pLbWord) do
                        pSpCar:getParent():addChild(pOneWord, 5)
                        pOneWord:setPosition( ccpAdd(pSpCar:getPositionInCCPoint(), pOneWord:getPositionInCCPoint()) )
                        pOneWord:runAction(CCSequence:createWithTwoActions(CCDelayTime:create(fTotalTime+(0.4*(n-1))), tArrWordAct[nTmpIdx][n]))
                    end
                end
                -- 每个中技能的车子信息
                fTotalTime = fTotalTime + 0.12 / fPlaySpeed;

                -- 如果当前的单体免疫技能没有播放，则开始设置是否立刻播放当前技能
                for i,tTmpSkill in pairs(self.tArrShowSkill) do
                    -- 受众类型
                    local nTargetType = self.pSkillMgr:GetCarSkillTargetType(tSkill.nType)
                    -- 如果是当前车子有免疫功能
                    if tTmpSkill.tSkill.nType == CarSkillType.MIANYI_SKILL_ONE and
                        tTmpSkill.nCarPos == nOneCarIdx and
                        (nTargetType == self.pSkillMgr:SkillShowType().ReduceEnemy or
                        nTargetType == self.pSkillMgr:SkillShowType().GetEnemyToMate) then
                        if tTmpSkill.fRoadValue <= UNSET_SKILL_VALUE then
                            tTmpSkill.fRoadValue = -tSkill.nCarPos;
                        end
                    end
                end
            end
            -- 动画的总时间
            fTotalTime = fTotalTime + fMaxOneActTime;
        until false
    end

    return fTotalTime;
end

-- 当前技能是否被免疫掉
function LogicRaceMap:SkillMgrCurSkillIsBeMianYi(tSkill)
    -- 单体的免疫 
    local tSimpleMianYi = {}

    -- 如果是当前技能播放结束后要播放的技能，则开始播放
    for i,tTmpSkill in pairs(self.tArrShowSkill) do
        -- 群体免疫可以立刻知道播放
        local nStartPos = toint(-tTmpSkill.fRoadValue);
        if nStartPos == tSkill.nCarPos then
            -- 单体免疫要特殊处理，如果这回合队友有群体免疫了。则延后单体免疫处理
            if tTmpSkill.tSkill.nType == CarSkillType.MIANYI_SKILL_ONE then
                tSimpleMianYi[#tSimpleMianYi+1] = tTmpSkill;

            -- 如果没有使用过，则开始使用免疫
            elseif tTmpSkill.bIsUse ~= true then
                -- 如果出了群体免疫技能，则将所有未播放的技能重新播放
                if tTmpSkill.tSkill.nType == CarSkillType.MIANYI_SKILL_ALL then
                    return true;
                end
            end
        end
    end
    -- 单体免疫技能
    for i,tTmpSkill in ipairs(tSimpleMianYi) do
        repeat
            if tTmpSkill.bIsUse == true then
                break;
            end

            return true;
        until true
    end

    return false;
end

-- 播放技能结束
function LogicRaceMap:SkillMgrShowSkillEnd(tSkill, tArrLeeks, tArrAdds)
    -- 开始播放技能
    self.bIsPause = false;
    -- 回复跑中的倍数
    self.pMap:setScale(MAP_RACING_SCALE)

    self:setScale(1)

    -- 单体的免疫 
    local tSimpleMianYi = {}
    local bIsCallSimple = true;
    local bIsUseMianYi = false

    -- 如果是当前技能播放结束后要播放的技能，则开始播放
    for i,tTmpSkill in pairs(self.tArrShowSkill) do
        -- 群体免疫可以立刻知道播放
        local nStartPos = toint(-tTmpSkill.fRoadValue);
        if nStartPos == tSkill.nCarPos then
            -- 单体免疫要特殊处理，如果这回合队友有群体免疫了。则延后单体免疫处理
            if tTmpSkill.tSkill.nType == CarSkillType.MIANYI_SKILL_ONE then
                tSimpleMianYi[#tSimpleMianYi+1] = tTmpSkill;

            -- 如果没有使用过，则开始使用免疫
            elseif tTmpSkill.bIsUse ~= true then
                tTmpSkill.bIsUse = true;
                -- 开始播放技能
                self.pSkillMgr:StartToShowSkill(tTmpSkill.tSkill, tTmpSkill.nCarPos);
                -- 如果出了群体免疫技能，则将所有未播放的技能重新播放
                if tTmpSkill.tSkill.nType == CarSkillType.MIANYI_SKILL_ALL then
                    bIsCallSimple = false;
                    -- 开始使用免疫
                    bIsUseMianYi = true;
                end
            end
        end
    end
    -- 单体免疫技能
    for i,tTmpSkill in ipairs(tSimpleMianYi) do
        repeat
            if tTmpSkill.bIsUse == true then
                break;
            end

            if bIsCallSimple == true then
                tTmpSkill.bIsUse = true;
                -- 开始播放技能
                self.pSkillMgr:StartToShowSkill(tTmpSkill.tSkill, tTmpSkill.nCarPos);
                -- 开始使用免疫
                bIsUseMianYi = true;
            else
                tTmpSkill.fRoadValue = UNSET_SKILL_VALUE
            end
        until true
    end
end

return LogicRaceMap