--
-- Author: Link
-- Date: 2014-04-27 15:18:17
--

local TeachMgrLayer = class("TeachMgrLayer", function()
	return display.newLayer();
end)

local SHOW_CLIP_LAYER = false;

-- 教学类型的定义
TeachConType = {
	None 				= -1,	-- 不在教学状态
	Event 				= 0,	-- 事件处理
	Dialog 				= 1,	-- 弹出对话框
	TeachRect 			= 2,	-- 点击区域
	TeachButton			= 3,	-- 点击按钮
	TeachShowText		= 4,	-- 显示区域，且显示文字
	FirstTeachButton	= 5,	-- 第一次启动做的动作
	FirstTeachNotDo 	= 6,	-- 第一次启动不做的动作
	JustTouchRect 		= 7,	-- 只是显示点击区域，不用等待底下操作反馈
	TouchMove 			= 8,	-- 滑动屏幕
}

-- 教学的Event类型
local TeachEventTeyp = {
	None 					= 0,	-- 没有事件
	ForceIntoStory			= 1,	-- 强制进入最新剧情
	ForceUnlockStory1		= 2,	-- 第1次强制解锁剧情到BOSS关
	ForceUnlockStory2		= 3,	-- 第2次强制解锁剧情到BOSS关
	NoticeToClickPerfect 	= 4,	-- 是游戏中允许perferc加速
	NoticeToClickBad 		= 5,	-- 是游戏中允许bad加速
	BlankEvent 				= 6,	-- 是空事件，用于上一个记录行为是1的事件，用于记录，不会自动跳向下一个教学
	SwitchFirstBuildingCao 	= 7,	-- 是切换到第一个经济建筑槽位
	SwitchHeChengBuilding 	= 8,	-- 是切换到合成的建筑槽位
	ShowLevelRewardNotice	= 9,	-- 显示等级奖励
	TempCanAllTeach 		= 10,	-- 临时允许全屏操作，不会自动跳向下一个教学
	StopTempAllTeach 		= 11,	-- 停止临时运行全屏操作
	BlankEventByAutoSkip 	= 12,	-- 是空事件，用于上一个记录行为是1的事件，用于记录，会自动跳向下一个教学
	RestartRacingCD			= 13,	-- 比赛中重新开始倒计时
	StartRacingPerfect		= 14,	-- 完美开场加速
	SkillEnterMap 			= 15, 	-- 跳过开场动画
	ClearRankListInfo 		= 16,	-- 可以显示排位，且清除所有排位信息
	EnterDreamFight			= 17,	-- 进入梦中对决
	ForceBackToBase 		= 18,	-- 强制从比赛中回到基地界面
	SwitchFirstBuilding 	= 19,	-- 切换到第一个车建筑物
}

local TeachStageType = {
	FirstTeach 	= 1,
	EquipCar 	= 2,
	
	Building 	= 100,
	Endless 	= 101,
	Loot 		= 102,
	HeChange 	= 103,
	SwitchMap 	= 104,
	JingYing 	= 105,
	Rank 		= 106,
	TaoBao 		= 107,
}

-- 初始化 JC_QuDi.png
function TeachMgrLayer:ctor()
	-- 当前的回调
	self.pCBack = {}
	setmetatable(self.pCBack, {__mode = "v"})
	-- 切片处理
	self.pLyClip = nil;
	-- 当前是否处在第一次教学中
	self.bIsInDreamWar	= nil;
	-- 显示是否在教学阶段
	self.bIsTeaching	= false;
	-- 当前是否可以显示排位
	self.bIsShowRank 	= false;
	-- 当前教学等级
	self.nTeachStartLv	= 52;
	-- 当前的类型
	self.nTeachType		= TeachConType.None;

	-- 教学步骤
	self.nTeachStage	= 1;
	self.nTeachBigStep	= 1;
	self.nTeachCurStep 	= 1;
	-- 事件的保留值
	self.nTeachEventValue	= -1;
	-- 是否临时允许全屏触控
	self.bIsCanTempAllTouch = false;

	-- 当前第一次进入教学
	self.bIsFirstTeach	= false;
	self.bIsTouchBegan 	= false;

	self.nCurEvent 		= TeachEventTeyp.None

	-- 教学顺序
	-- self.tStageShowList = {
	-- 	"1","2","3","4","5","6",
	-- }
	-- 开场动画
	self.pSpStartSwf = nil;
	-- 教学的优先级
	self.tStageOrder = {
		TeachStageType.FirstTeach,
		TeachStageType.EquipCar,
		
		TeachStageType.JingYing,
		TeachStageType.SwitchMap,

		TeachStageType.Building,
		TeachStageType.HeChange,
		TeachStageType.Rank,
		TeachStageType.Endless,
		TeachStageType.Loot,
		TeachStageType.TaoBao,
	}
	-- 当前要进行的教学数组
	self.nArrCurTeachList = {
		-- [1, 2, 4],--第一个数字是id，第二个数据是步骤,第三个是优先级
	}
	-- 当前截取的区域
	self.tCurRect = CCRectMake(0, 0, 0, 0);

	-- 剪头
	self.pSpWords = nil;
	self.pSpHand  = nil;

	-- 教学的信息
	self.tTeachInfo = nil;

	-- self:InsertOneTeachID( TeachStageType.SwitchMap, 1 )
	-- self:InsertOneTeachID( TeachStageType.Loot, 1 )
	-- self:InsertOneTeachID( TeachStageType.Rank, 1 )
end

-- 设置大厅的逻辑管理
function TeachMgrLayer:SetLobbyLogicDeleate( pLobby )
	self.pCBack.pLobby = pLobby;
end

-- 根据教学的Stage和BigStep来开始教学
function TeachMgrLayer:StartTeach(nStage, nStep)
	-- 创建
	self:CreateTechInfo()
	self.nTeachStage	= nStage;
	self.nTeachBigStep	= nStep;
	-- 
	self.nTeachCurStep 	= 1;
	-- 当前第一次进入教学
	self.bIsFirstTeach	= true;
	-- 开始当前教学步骤
	self:ShowCurTeachStep()
end

-- 判断开始剧情
function TeachMgrLayer:TeachLoop( dt )
	-- 不在教学中，才会判断是否进入下一个教学
	if GAME_STATUS.GetStatus(1) > STATUS_MAIN.LOGIN and self:IsCurInTeaching() ~= true then
		if #self.nArrCurTeachList > 0 then
			local tTeachID = self.nArrCurTeachList[1]
			-- 教学结束了再删除
			GameViewMgr:shareMgr():RemoveAllViewes();
			self.nTeachStartLv = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nLv;
			self:StartTeach(tTeachID[1], tTeachID[2])
		end
	end
end

-- 根据用户信息判断是否开始教学
function TeachMgrLayer:JudgeUserInfoToIntoTeach()
	-- 第一个关
	self:JudgeInsertOneTeachID( TeachStageType.FirstTeach )

	-- 跟等级关联的教学类型的
	local nUserLv = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nLv
	-- 建筑
	if DATAMGR:GetUserInfoMgr():GetBuildingUnlockLv() <= nUserLv then
		-- 第一个建筑没有建好
		local tTmpList = DATAMGR:GetUserInfoMgr():GetAllUserBuildingList();
		if tTmpList[1] == nil or tTmpList[1].nBuildingID == NOT_EQUIPED then
			self:JudgeInsertOneTeachID( TeachStageType.Building )
		end
	end
	-- 试练
	if DATAMGR:GetUserInfoMgr():GetEndlessUnlockLv() <= nUserLv then
		self:JudgeInsertOneTeachID( TeachStageType.Endless )
	end
	-- 掠夺
	if DATAMGR:GetUserInfoMgr():GetLootFundationUnlockLv() <= nUserLv then
		self:JudgeInsertOneTeachID( TeachStageType.Loot )
	end
	-- 合成
	if DATAMGR:GetUserInfoMgr():GetHeChengFundationUnlockLv() <= nUserLv then
		self:JudgeInsertOneTeachID( TeachStageType.HeChange )
	end
	-- 排位
	if DATAMGR:GetUserInfoMgr():GetRankFundationUnlockLv() <= nUserLv then
		self:JudgeInsertOneTeachID( TeachStageType.Rank )
	end
	-- 淘宝屋
	if DATAMGR:GetUserInfoMgr():GetTaoBaoFundationUnlockLv() <= nUserLv then
		self:JudgeInsertOneTeachID( TeachStageType.TaoBao )
	end

	-- 第一次解锁地图的时候
	if DATAMGR:GetUserInfoMgr():GetUnlockMapCnt() == 2 then
		if DATAMGR:GetUserInfoMgr():GetCurPlayerSelMapID() == 3001 then
			self:JudgeInsertOneTeachID( TeachStageType.SwitchMap )
		end
	end

	-- 剧情任务
	local tTaskProgress = DATAMGR:GetTaskInfoMgr():GetTaskCurProgress()

	-- 刚跳到第1章第二节的时候
	if tTaskProgress.nBigIdx == 1 and tTaskProgress.nCurMidIdx == 2 then
		local tCurTeam = DATAMGR:GetUserInfoMgr():GetCurTeamList().tArrCarList;
		-- 第二辆车没有上阵
		if #tCurTeam == 2 and tCurTeam[2].nCarIdx == NOT_EQUIPED then
			self:JudgeInsertOneTeachID( TeachStageType.EquipCar )
		end
		
	end

	-- 跳到第二章的时候
	if tTaskProgress.nBigIdx >= 2 then
		self:JudgeInsertOneTeachID( TeachStageType.JingYing )
	end

	if #self.nArrCurTeachList > 0 then
		return true;
	end
	return false;
end

-- 获取当前教学的优先级，数字越低，优先级越高
function TeachMgrLayer:GetTeachStageOrder(nStageID)
	for i,v in ipairs(self.tStageOrder) do
		if v == nStageID then
			return i;
		end
	end
	return 999;
end

-- 判断是否开始插入教学
function TeachMgrLayer:JudgeInsertOneTeachID(nStageID)
	local tTeachInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().tTeachInfos
	local nOneTeachID = tTeachInfo[ nStageID ];
	if nOneTeachID == 0 or nOneTeachID == nil then
		self:InsertOneTeachID( nStageID, 1 )
	elseif nOneTeachID > 0 then
		self:InsertOneTeachID( nStageID, nOneTeachID )
	end
end

-- 插入教学索引，优先级数字越小，越排前面
function TeachMgrLayer:InsertOneTeachID(nStageID, nStep)
	-- 获取优先级，优先级不存在，则不处理
	local nCurOrdr = self:GetTeachStageOrder(nStageID)
	if nCurOrdr == 999 then
		return;
	end

	local nIdx = #self.nArrCurTeachList + 1;
	for m,tOneInfo in ipairs(self.nArrCurTeachList) do
		if tOneInfo[3] > nCurOrdr then
			nIdx = m;
			break;
		-- 相同优先级表示是相同的教学，跳过不处理
		elseif tOneInfo[3] == nCurOrdr then
			return;
		end
	end

	local tTeachInfo = {}

	tTeachInfo[ 1 ] = nStageID
	tTeachInfo[ 2 ] = nStep;
	tTeachInfo[ 3 ] = nCurOrdr;
	table.insert(self.nArrCurTeachList, nIdx, tTeachInfo)
end

-- 加载教学信息
function TeachMgrLayer:CreateTechInfo()
	-- 如果教学数据不存在，读取教学数据
	if self.tTeachInfo == nil then
		-- 教学信息
		local strTeachInfo = DATAMGR:LoadOneDataFileAndJudgeIsChange("Images/StaticDatas/Teach.json")
		self.tTeachInfo = json.decode(strTeachInfo)
		print("教学文件读取:", self.tTeachInfo)
	end
	-- 如果是没有创建镂空
	if self.pLyClip == nil then
		-- 截取的图片
		local pSpMask = display.newScale9Sprite("Images/JC_QuDi.png", 300, 300, CCSizeMake(300, 200));
		pSpMask:setScale( ctipwidth(1) )
		pSpMask:setAnchorPoint(ccp(0, 0))

		-- 截取的区域
		self.pLyClip = CCClippingNode:create(pSpMask)
		self.pLyClip:setInverted(true);
		self.pLyClip:setAlphaThreshold(0.00);
		self:addChild(self.pLyClip);
		-- 灰色层次
		local pLyTmp = CCLayerColor:create(ccc4(0, 0, 0, 160));
		self.pLyClip:addChild(pLyTmp)
		-- 默认先不显示
		self.pLyClip:setVisible(false);

		
	end

	if self.pSpHand == nil then
		-- 手的动画
		self.pSpHand = CCSprite:create("Images/shou.png")
		self.pSpHand:setAnchorPoint( ccp(0.192, 0.831) )
		self.pSpHand:setVisible(false)
		self.pSpHand:setScale( ctipwidth(0.78) )
		self:addChild(self.pSpHand, 2)
	end
end

-- 删除所有当前教学所有数据
function TeachMgrLayer:RemoveTeachInfo()
	self.tTeachInfo = nil

	self.bIsInDreamWar = nil;

	-- self.nArrCurTeachList = {}

	-- 
	if self.pLyClip ~= nil then
		self.pLyClip:removeFromParent();
		self.pLyClip = nil;
	end

	if self.pSpHand then
		self.pSpHand:removeFromParent();
		self.pSpHand = nil;
	end

	if self.pSpWords ~= nil then
		self.pSpWords:removeFromParent();
		self.pSpWords = nil;
	end

	if self.pSpStartSwf ~= nil then
		self.pSpStartSwf:removeFromParent();
		self.pSpStartSwf = nil;

		-- 清除对应的技能效果frame
		local strPlist = Public_GetResPathByKey("Images/kj-kaijidonghua.plist")
			local strXml
			local tDeiveType = GetCurrentDeviceType();
		-- 如果是ipad
		if tDeiveType == CTDeviceTypeIpad or tDeiveType == CTDeviceTypeIpadHD then
			strXml = Public_GetResPathByKey("Images/kj-kaijidonghua768x1024.xml")
		-- 如果是ipone5HD
		elseif tDeiveType == CTDeviceTypeIphone5HD then
			strXml = Public_GetResPathByKey("Images/kj-kaijidonghua640x1136.xml")
		else
			strXml = Public_GetResPathByKey("Images/kj-kaijidonghua640x960.xml")
		end
		nickXml.NickTexCacheMgr:shareCache():removeFramesByPlistFileName(strPlist);
		nickXml.FlaXmlCache:sharedFlaXmlCache():removeFlaXmlWithName(strXml);
	end

	-- 当前的类型
	self.nTeachType = TeachConType.None;
end

-- 是否可以显示排行榜
function TeachMgrLayer:IsCanShowRank()
	if self.bIsShowRank == true then
		return true;
	end

	-- 功能未解锁
	local nCurLv = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().nLv;
	local nFoundationUnlockLv = DATAMGR:GetUserInfoMgr():GetRankFundationUnlockLv();
	if nCurLv < nFoundationUnlockLv then
		return false;
	end

	-- 本地记录下当前步骤
	local tTeachInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().tTeachInfos
	local nRankInfo = tTeachInfo[ TeachStageType.Rank ];
	-- 未经过教学，不可以显示
	if nRankInfo and nRankInfo >= 0 then
		return false;
	end

	return true;
end

-- 是否在梦中的对决
function TeachMgrLayer:IsInDreamWar()
	if self.bIsInDreamWar == true then
		return true;
	end
	return false;
end

-- 获取是否是
function TeachMgrLayer:IsCanAllTouch()
	return self.bIsCanTempAllTouch
end

-- 获取当前是否还在教学中。
function TeachMgrLayer:IsCurInTeaching()
	if self.nTeachType == TeachConType.None then
		return false;
	end
	return true;
end

-- 获取当前是否在滑动教学中
function TeachMgrLayer:IsCurInMovingTeaching()
	if self.nTeachType == TeachConType.TouchMove then
		return true;
	end
	return false;
end

-- 获取当前是否在等待下一步操作的教学中
function TeachMgrLayer:IsCurInBlankEventTeaching()
	if self.nTeachType == TeachConType.Event and self.nCurEvent == TeachEventTeyp.BlankEvent then
		return true;
	end
	return false;
end

-- 获取教学中保留的值
function TeachMgrLayer:GetTeachEventRetainValue()
	-- print("保留值是:", self.nTeachEventValue)
	return self.nTeachEventValue
end

-- 获取当前大步骤
function TeachMgrLayer:GetTeachStage()
	return self.nTeachStage;
end

-- 

-- 显示单手点击
function TeachMgrLayer:StartShowClickAction(tOnePos)
	-- local pEndPos = tOnePos;
	-- local pStartPos = ccp(tOnePos.x+ctipwidth(120), tOnePos.y)
	self.pSpHand:setVisible(true)
	self.pSpHand:setPosition(tOnePos)

	local pArrAct = CCArray:create();
	-- pArrAct:addObject( CCShow:create() )
	-- pArrAct:addObject( CCPlace:create(pStartPos) )
	-- pArrAct:addObject( CCMoveTo:create(0.5, pEndPos) )
	-- pArrAct:addObject(CCDelayTime:create(0.15))
	pArrAct:addObject( CCCallFunc:create(function()
		-- 换图
		local pOneHandTex = CCTextureCache:sharedTextureCache():addImage("Images/shou2.png")
		self.pSpHand:setTexture(pOneHandTex)
		self.pSpHand:setPosition( ccpAdd(self.pSpHand:getPositionInCCPoint(), ccp(5,-8) ) )
		self.pSpHand:setScale(ctipwidth(0.64))
	end))
	pArrAct:addObject(CCDelayTime:create(0.2))
	pArrAct:addObject( CCCallFunc:create(function()
		-- 换图
		local pOneHandTex = CCTextureCache:sharedTextureCache():addImage("Images/shou.png")
		self.pSpHand:setTexture(pOneHandTex)
		self.pSpHand:setPosition(tOnePos)
		self.pSpHand:setScale(ctipwidth(0.78))

		for i=1,3 do
			if i == 1 then
				local pSpWen = CCSprite:create("Images/quan.png")
				pSpWen:setAnchorPoint( ccp(0.495, 0.425) )
				pSpWen:setPosition( ctspccp(self.pSpHand, self.pSpHand:getAnchorPoint().x, self.pSpHand:getAnchorPoint().y) )
				self.pSpHand:addChild(pSpWen, -1)

				local pArrAct1 = CCArray:create();
				pArrAct1:addObject( CCFadeTo:create(0.5, 0) )
				pArrAct1:addObject( CCRemoveSelf:create(true) )
				pSpWen:runAction( CCRepeatForever:create( CCSequence:create(pArrAct1) ) )
			else
				local pSpWen = CCSprite:create("Images/quan2.png")
				pSpWen:setScale(0.0)
				pSpWen:setAnchorPoint( ccp(0.495, 0.42) )
				pSpWen:setPosition( ctspccp(self.pSpHand, self.pSpHand:getAnchorPoint().x, self.pSpHand:getAnchorPoint().y) )
				self.pSpHand:addChild(pSpWen, -1)

				local pArrAct1 = CCArray:create();
				pArrAct1:addObject(CCDelayTime:create(0.22*(i-2)))
				pArrAct1:addObject( CCScaleTo:create(0.44, 1.2) )
				pArrAct1:addObject( CCRemoveSelf:create(true) )
				pSpWen:runAction( CCRepeatForever:create( CCSequence:create(pArrAct1) ) )
			end
		end
	end))
	-- pArrAct:addObject(CCDelayTime:create(1.3))
	-- pArrAct:addObject( CCHide:create() )
	pArrAct:addObject(CCDelayTime:create(1.2))
	-- 手部的动画
	self.pSpHand:runAction( CCRepeatForever:create( CCSequence:create(pArrAct) ) )
end

-- 显示滑动手指
function TeachMgrLayer:StartShowMoveHand(tStartPos, tEndPos, fTime)
	self.pSpHand:setVisible(true)
	self.pSpHand:setPosition(tStartPos)

	if fTime == nil or fTime <= 0 then
		fTime = ccpDistance(tStartPos, tEndPos) / ctipwidth(300);
	end

	local pArrAct = CCArray:create();
	pArrAct:addObject(CCDelayTime:create(0.2))
	pArrAct:addObject( CCCallFunc:create(function()
		-- 换图
		local pOneHandTex = CCTextureCache:sharedTextureCache():addImage("Images/shou2.png")
		self.pSpHand:setTexture(pOneHandTex)
		self.pSpHand:setPosition( ccpAdd(self.pSpHand:getPositionInCCPoint(), ccp(5,-8) ) )
		self.pSpHand:setScale(ctipwidth(0.64))
	end))
	pArrAct:addObject( CCDelayTime:create(0.2) )
	pArrAct:addObject( CCMoveTo:create(fTime, tEndPos) )
	pArrAct:addObject( CCDelayTime:create(0.2) )
	pArrAct:addObject( CCHide:create() )
	pArrAct:addObject( CCDelayTime:create(1.2) )
	pArrAct:addObject( CCCallFunc:create(function()
		-- 换图
		local pOneHandTex = CCTextureCache:sharedTextureCache():addImage("Images/shou.png")
		self.pSpHand:setTexture(pOneHandTex)
		self.pSpHand:setPosition(tStartPos)
		self.pSpHand:setScale(ctipwidth(0.78))
	end))
	pArrAct:addObject( CCShow:create() )

	-- 手部的动画
	self.pSpHand:runAction( CCRepeatForever:create( CCSequence:create(pArrAct) ) )
end

-- 获取key名字
function TeachMgrLayer:GetStageShowKey(nStageIdx)
	local strKey --= self.tStageShowList[ self.nTeachStage ]
	if strKey == nil then
		strKey = ""..nStageIdx;
	end
	return strKey;
end

-- 显示当前的教学步骤
function TeachMgrLayer:ShowCurTeachStep()
-- -- 类型的定义
-- local TeachConType = {
-- 	Dialog 			= 1,	-- 弹出对话框
-- 	TeachRect 		= 2,	-- 点击区域
-- 	TeachButton		= 3,	-- 点击按钮
-- 	TeachShowText	= 4,	-- 显示区域，且显示文字
-- JustTouchRect
-- }
	-- 如果有注释文字，则删除掉
	if self.pSpWords ~= nil then
		self.pSpWords:removeFromParent();
		self.pSpWords = nil;
	end
	-- 获取当前操作的数据
	local strKey = self:GetStageShowKey( self.nTeachStage )
	local tBigStep = self.tTeachInfo.steps[ strKey ].list[self.nTeachBigStep];
	local tControlList = tBigStep.control;
	local tCurControl = tControlList[ self.nTeachCurStep ];
	local nConType = tCurControl[ 1 ];

	print("教学步骤:", self.nTeachStage, self.nTeachBigStep, self.nTeachCurStep, nConType)
	-- 不同操作类型，进行不同的操作
	-- 对话框
	if nConType == TeachConType.Dialog then
		-- 运行显示截取的时候，才能修改显示
		self.pLyClip:setVisible(false);
		-- 
		self.pSpHand:setVisible(false)
		
		DIALOGMGR:ShowTeachDialog(nil, tBigStep.text[ tCurControl[2] ], nil, nil)

	-- 事件
	elseif nConType == TeachConType.Event then
		local nEventID = tCurControl[2];
		local nUseType = tCurControl[3];
		self.pSpHand:setVisible(false)
		-- 运行显示截取的时候，才能修改显示
		self.pLyClip:setVisible(false);

		self.nCurEvent 		= nEventID

		-- 不是要求必须第一次进来处理的事件，或者第一次进来，则处理当前事件
		local bToNext = true;
		-- print("fdafdsa:", self.bIsFirstTeach, nEventID)
		-- if nUseType ~= 1 or self.bIsFirstTeach == true then
		bToNext = self:TeachDealEvent(nEventID)
		-- end

		-- 保存当前类型
		self.nTeachType = nConType;
		-- 当前第一次进入教学
		self.bIsFirstTeach	= false;
		-- 切换到下一个命令
		if bToNext == true then
			self.nCurEvent 		= TeachEventTeyp.None
			self:ShowSwichToNextStep();
		end
		return;
	else
		if SHOW_CLIP_LAYER == true then
			self.pLyClip:setVisible(true);
		else
			self.pLyClip:setVisible(false);
		end
		self.pSpHand:setVisible(true)
		self.pSpHand:stopAllActions()
		self.pSpHand:removeAllChildren();
		
		-- 获取触控属性
		local tTouchInfo = self.tTeachInfo.controlList[""..tCurControl[2]];

		-- 获取区域信息，且是否要换算位置
		local tRectIdx = 1;
		local bIsOrig = false;
		local bIsChange = true;
		local fRectScale = 1.0;
		if SHARE_FUNC.nFrameType == CTDeviceTypeIphone5HD or SHARE_FUNC.nFrameType == CTDeviceTypeItouch5HD then
			tRectIdx = 2;
		elseif SHARE_FUNC.nFrameType == CTDeviceTypeIphoneHD or SHARE_FUNC.nFrameType == CTDeviceTypeItouchHD or
				SHARE_FUNC.nFrameType == CTDeviceTypeIphone or SHARE_FUNC.nFrameType == CTDeviceTypeItouch then
			tRectIdx = 3;
			-- 如果是低清
			if SHARE_FUNC.nFrameType == CTDeviceTypeIphone or SHARE_FUNC.nFrameType == CTDeviceTypeItouch then
				fRectScale = 0.5;
			end
		else
			bIsOrig = true;
		end
		if tRectIdx ~= 1 then
			local tNext = tTouchInfo[1][tRectIdx]
			if tNext == nil or #tNext == 0 then
				tRectIdx = 1;
			else
				bIsChange = false;
			end
		end


		-- 标签
		local tRectInfo = tTouchInfo[1][tRectIdx];
		local nRectType = tRectInfo[1]
		for i=1,#tRectInfo do
			if i > 1 and i < 6  then
				tRectInfo[i] = tRectInfo[i] * fRectScale;
			end
		end

		-- 如果是要第一次进入的，则不跳过
		if nConType == TeachConType.FirstTeachButton and self.bIsFirstTeach == false then
			self:ShowSwichToNextStep();
			return;
		end

		-- 如果是第一次进入，则跳过
		if nConType == TeachConType.FirstTeachNotDo and self.bIsFirstTeach == true then
			self:ShowSwichToNextStep();
			return;
		end

		-- 显示的倍数
		local fShowScale = tRectInfo[6];
		if fShowScale == nil or bIsOrig == true then
			fShowScale = 1;
		end

		local pSpMark
		-- 显示截取的区域
		if SHOW_CLIP_LAYER == true then
			pSpMark = self.pLyClip:getStencil();
		else
			-- 如果只是点击区域
			if nConType == TeachConType.TeachRect then
				pSpMark = self.pLyClip:getStencil();
				self.pLyClip:setVisible(true);
			else
				pSpMark = CCNode:create();
			end
		end
		-- 尺寸大小
		pSpMark:setContentSize( CCSizeMake( tRectInfo[4] , tRectInfo[5] ) )
		-- 根据不同情况处理不同位置
		if bIsChange == true then
			-- 当前层上对齐
			if nRectType == 1 then
				local fPosX = ctipwidth( tRectInfo[2] );
				local fPosY = ctprheight(1) - ctipwidth( 1024 - tRectInfo[3] );
				pSpMark:setPosition( ccp( fPosX, fPosY ) )

			-- 当前层，中对齐
			elseif nRectType == 2 then
				local fPosX = ctipwidth( tRectInfo[2] ) - (fShowScale-1)*tRectInfo[4]/2.0;
				local fPosY = ctprheight(0.5) - ctipwidth( 512 - tRectInfo[3] )*fShowScale;
				pSpMark:setContentSize( CCSizeMake( tRectInfo[4]*fShowScale, tRectInfo[5]*fShowScale ) )
				pSpMark:setPosition( ccp( fPosX, fPosY ) );

			-- 下对齐
			else
				pSpMark:setPosition( ccp( ctipwidth(tRectInfo[2]), ctipwidth(tRectInfo[3]) ) )
			end
			pSpMark:setScale( ctipwidth(1) )
		else
			pSpMark:setPosition( ccp( tRectInfo[2], tRectInfo[3] ) )
			pSpMark:setScale( 1 )
		end


		-- 如果只是点击区域
		if nConType == TeachConType.TeachRect then
			-- 注释的文字信息
			local tTextInfo = tTouchInfo[3];
			local fShowScale = ctipwidth(1);
			-- 文字底框
			self.pSpWords = CCScale9Sprite:createWithInsets(CCRectMake(60, 30, 400, 85), "Images/xzfwq-diban2.png")
			self.pSpWords:setContentSize( CCSizeMake(tTextInfo[4], tTextInfo[5]) )
			self.pSpWords:setScale(fShowScale)
			self.pSpWords:setAnchorPoint( ccp(0, 0) )
			-- 当前层上对齐
			if tTextInfo[1] == 1 then
				local fPosX = ctipwidth( tTextInfo[2] );
				local fPosY = ctprheight(1) - ctipwidth( 1024 - tTextInfo[3] );
				self.pSpWords:setPosition( ccp( fPosX, fPosY ) )

			-- 当前层，中对齐
			elseif tTextInfo[1] == 2 then
				local fPosX = ctipwidth( tTextInfo[2] ) ;
				local fPosY = ctprheight(0.5) - ctipwidth( 512 - tTextInfo[3] )*fShowScale;
				self.pSpWords:setPosition( ccp( fPosX, fPosY ) );

			-- 下对齐
			else
				-- pSpMark:setPosition( ccp( ctipwidth(tTextInfo[2]), ctipwidth(tTextInfo[3]) ) )
				self.pSpWords:setPosition( ccp(tTextInfo[2]*fShowScale, tTextInfo[3]*fShowScale) )
			end
			self:addChild(self.pSpWords, 10);

			-- 注释的文字
			local pLbWord = CCLabelTTF:create(tTextInfo[7], LABEL_BOLD_FONT_NAME, tTextInfo[6],
								CCSizeMake(tTextInfo[4]-10, tTextInfo[5]-10),
								kCCTextAlignmentCenter, kCCVerticalTextAlignmentCenter)
			pLbWord:setPosition( ctspccp(self.pSpWords, 0.5, 0.5) )
			pLbWord:setColor( ccc3(0, 0, 0) );
			self.pSpWords:addChild(pLbWord)
		end
		

		-- 当前显示区域
		self.tCurRect = ctspbound(pSpMark);

		-- print("教学：：", SHARE_FUNC.nFrameType, tRectIdx, self.tCurRect:getMinX(), self.tCurRect:getMinY())

		-- 箭头类型
		local nArrowType = tTouchInfo[2][1];

		-- 停止所有动画
		self.pSpHand:stopAllActions();
		self.pSpHand:removeAllChildren();
		self.pSpHand:setPosition( ccp(self.tCurRect:getMidX(), self.tCurRect:getMidY()) )

		-- 如果是滑动教学
		if nConType == TeachConType.TouchMove then
			local tStartMovePos = ccp(self.tCurRect:getMinX()+self.tCurRect.size.width*tTouchInfo[3][1],
										self.tCurRect:getMinY()+self.tCurRect.size.height*tTouchInfo[3][2]);

			local tEndMovePos = ccp(self.tCurRect:getMinX()+self.tCurRect.size.width*tTouchInfo[3][3],
									self.tCurRect:getMinY()+self.tCurRect.size.height*tTouchInfo[3][4]);
			self:StartShowMoveHand(tStartMovePos, tEndMovePos, tTouchInfo[3][5])
		-- 其他的是点击教学
		else
			self:StartShowClickAction( self.pSpHand:getPositionInCCPoint() )
		end
	end
	-- 保存当前类型
	self.nTeachType = nConType;

	-- 上传类型
	local nUpType = tCurControl[ 3 ];
	-- 上传当前步骤
	if nUpType == 2 then
		self:SendTeachProgress(self.nTeachStage, self.nTeachBigStep)
	end

	-- 当前第一次进入教学
	if nConType ~= TeachConType.FirstTeachButton and nConType ~= TeachConType.FirstTeachNotDo and nConType ~= TeachConType.Dialog then
		self.bIsFirstTeach	= false;
	end
end

-- 教学处理事件
function TeachMgrLayer:TeachDealEvent( nEvent )
	print("事件开始处理！！", nEvent)
-- -- 教学的Event类型
-- local TeachEventTeyp = {
	-- ForceIntoStory			= 1,	-- 强制进入最新剧情
	-- ForceUnlockStory1		= 2,	-- 第1次强制解锁剧情到BOSS关
	-- ForceUnlockStory2		= 3,	-- 第2次强制解锁剧情到BOSS关
	-- NoticeToClickPerfect 	= 4,	-- 是游戏中允许perferc加速
	-- NoticeToClickBad 		= 5,	-- 是游戏中允许bad加速
	-- BlankEvent 				= 6,	-- 是空事件，用于上一个记录行为是1的事件，用于记录
	-- SwitchFirstBuildingCao 	= 7,	-- 是切换到第一个建筑槽位
	-- SwitchHeChengBuilding 	= 8,	-- 是切换到合成的建筑槽位
	-- TempCanAllTeach 		= 10,	-- 临时允许全屏操作，不会自动跳向下一个教学
	-- StopTempAllTeach 		= 11,	-- 停止临时运行全屏操作
	-- BlankEventByAutoSkip 	= 12,	-- 是空事件，用于上一个记录行为是1的事件，用于记录，会自动跳向下一个教学
	-- EnterDreamFight			= 17,	-- 进入梦中对决
	-- ForceBackToBase 		= 18,	-- 强制从比赛中回到基地界面
-- }
	-- 第一次解锁到最底关卡
	if nEvent == TeachEventTeyp.ForceUnlockStory1 then
		DATAMGR:GetTaskInfoMgr():ForceToTask(1, 1, 5);
		-- if self.pCBack.pLobby then
		-- 	local pFuBen = self.pCBack.pLobby:GetFuBenLogic();
		-- 	pFuBen:ShowStoryView(1,1, 2)
		-- end
	-- 第2次解锁
	elseif nEvent == TeachEventTeyp.ForceUnlockStory2 then
		DATAMGR:GetTaskInfoMgr():ForceToTask(1, 2, 5);
		if self.pCBack.pLobby then
			local pFuBen = self.pCBack.pLobby:GetFuBenLogic();
			pFuBen:ShowStoryView(1,2, 2)
		end
	-- 跳到第一个建筑槽位
	elseif nEvent == TeachEventTeyp.SwitchFirstBuildingCao then
		if self.pCBack.pLobby then
			local pBuilding = self.pCBack.pLobby:GetBaseLogic():GetBuildingLogic();
			pBuilding:ForceShowBuildingCaoByIdx(0)
		end
	-- 跳到第二个建筑物
	elseif nEvent == TeachEventTeyp.SwitchHeChengBuilding then
		if self.pCBack.pLobby then
			local pBuilding = self.pCBack.pLobby:GetBaseLogic():GetBuildingLogic();
			pBuilding:ForceShowBuildingByIdx(1)
		end
	-- 跳到第一个建筑物
	elseif nEvent == TeachEventTeyp.SwitchFirstBuilding then
		if self.pCBack.pLobby then
			local pBuilding = self.pCBack.pLobby:GetBaseLogic():GetBuildingLogic();
			pBuilding:ForceShowBuildingByIdx(0)
		end
	-- 显示等级奖励
	elseif nEvent == TeachEventTeyp.ShowLevelRewardNotice then
		if (self.nTeachStartLv%5)==0 then
			local strKey = self:GetStageShowKey( self.nTeachStage )
			local tBigStep = self.tTeachInfo.steps[ strKey ].list[self.nTeachBigStep];
			-- local nLevel = tBigStep.num[1];
			DIALOGMGR:ShowDialog(DLGTYPE.LevelUpReward, self.nTeachStartLv);
		end
		
	-- 空事件
	elseif nEvent == TeachEventTeyp.BlankEvent then
		return false;
	-- 设置时间类型为1，比如点击perfect加速
	elseif nEvent == TeachEventTeyp.NoticeToClickPerfect then
		self.nTeachEventValue = 1;
	-- 设置时间类型为0，比如不运行点击加速
	elseif nEvent == TeachEventTeyp.NoticeToClickBad then
		self.nTeachEventValue = 0;
	-- 临时允许全屏操作，不会自动跳向下一个教学
	elseif nEvent == TeachEventTeyp.TempCanAllTeach then
		self.bIsCanTempAllTouch = true;
		return false;
	-- 停止临时运行全屏操作
	elseif nEvent == TeachEventTeyp.StopTempAllTeach then
		self.bIsCanTempAllTouch = false;
	-- 是空事件，用于上一个记录行为是1的事件，用于记录，会自动跳向下一个教学
	elseif nEvent == TeachEventTeyp.BlankEventByAutoSkip then
		
	-- 开始比赛的倒计时
	elseif nEvent == TeachEventTeyp.RestartRacingCD then
		local pRacingLogic = SHARE_FUNC:GetMainLogic():GetRaceLogic();
		if pRacingLogic ~= nil then
			pRacingLogic:ShowCountDownToStart();
		end

	-- 完美开场起跑
	elseif nEvent == TeachEventTeyp.StartRacingPerfect then
		local pRacingLogic = SHARE_FUNC:GetMainLogic():GetRaceLogic();
		if pRacingLogic ~= nil then
			pRacingLogic:TeachStartToRunPerfect();
		end

	-- 跳过开场动画
	elseif nEvent == TeachEventTeyp.SkillEnterMap then
		local pRacingLogic = SHARE_FUNC:GetMainLogic():GetRaceLogic();
		if pRacingLogic ~= nil then
			pRacingLogic:ShowFirstIntoMap(true)
		end

	-- 清楚所有排行列表
	elseif nEvent == TeachEventTeyp.ClearRankListInfo then
		local tSpecial = DATAMGR:GetUserInfoMgr():GetUserSpecialInfo();
		tSpecial.tRankPlayerList = {}
		-- 可以显示排行榜
		self.bIsShowRank 	= true;

	-- 进入梦中对决
	elseif nEvent == TeachEventTeyp.EnterDreamFight then
		self.bIsInDreamWar = true;
local strJson = [==[
{
	"tIntoInfo": {
		"bIsEnemy": false, 
		"nType": 1, 
		"nUid": 10000
	}, 
	"tFight": {
		"tSelfInfo": {
		    "strName": "申雪瑤", 
		    "nLv": 6,
		    "nUid": 10118
		}, 
		"tOtherInfo": {
		    "strName": "諸可燕", 
		    "nLv": 1,
		    "nUid": 10000
		}, 
		"nMapID": 3013, 
		"tCarEvents": [
            {
		    	"nType": 11, 
		    	"fAddition": 25,
		    	"fAddition1": 0,
                "fTime": 3, 
                "nCarPos": 2, 
                "fRoadValue": 2600,
                "nSubTo":1,
                "nAddTo":8
            },
            {
		    	"nType": 10, 
		    	"fAddition": 20,
		    	"fAddition1": 0,
                "fTime": 2.5, 
                "nCarPos": 10, 
                "fRoadValue": 9600,
                "nSubTo":1,
                "nAddTo":14
            },
            {
		    	"nType": 3, 
		    	"fAddition": 0,
		    	"fAddition1": 0,
                "fTime": 2.5, 
                "nCarPos": 1, 
                "fRoadValue": 5000,
                "nSubTo":1,
                "nAddTo":0
            },
            {
		    	"nType": 5, 
		    	"fAddition": 0,
		    	"fAddition1": 0,
                "fTime": 2.5, 
                "nCarPos": 9, 
                "fRoadValue": -2,
                "nSubTo":0,
                "nAddTo":8
            }
		], 
		"nMyPos": 1, 
		"tEnemyCars": [
            {
                "a": 2500, 
                "cLv": 0, 
                "id": 1091, 
                "lv": 80, 
                "s": 2500, 
                "t": 2500,
                "sLv": 6
            }, 
            {
                "a": 2000, 
                "cLv": 0, 
                "id": 1092, 
                "lv": 80, 
                "s": 2000, 
                "t": 2000,
                "sLv": 6
            }, 
            {
                "a": 2000, 
                "cLv": 0, 
                "id": 1093, 
                "lv": 80, 
                "s": 2000, 
                "t": 2000,
                "sLv": 6
            }, 
            {
                "a": 2000, 
                "cLv": 0, 
                "id": 1094, 
                "lv": 80, 
                "s": 2000, 
                "t": 2000,
                "sLv": 6
            }, 
            {
                "a": 2000, 
                "cLv": 0, 
                "id": 1095, 
                "lv": 80, 
                "s": 2000, 
                "t": 2000,
                "sLv": 6
            }
		], 
		"nFightID": 2, 
		"nFightType": 4, 
		"tMyCars": [
            {
                "a": 3200, 
                "cLv": 0, 
                "id": 1102, 
                "lv": 80, 
                "s": 3000, 
                "t": 3200,
                "sLv": 6
            }, 
            {
                "a": 2900, 
                "cLv": 0, 
                "id": 1103, 
                "lv": 80, 
                "s": 2900, 
                "t": 2900,
                "sLv": 6
            }, 
            {
                "a": 2800, 
                "cLv": 0, 
                "id": 1100, 
                "lv": 80, 
                "s": 2800, 
                "t": 2800,
                "sLv": 6
            }, 
            {
                "a": 2700, 
                "cLv": 0, 
                "id": 1099, 
                "lv": 80, 
                "s": 2700, 
                "t": 2700,
                "sLv": 6
            }, 
            {
                "a": 2600, 
                "cLv": 0, 
                "id": 1098, 
                "lv": 80, 
                "s": 2600, 
                "t": 2600,
                "sLv": 6
            }
		]
	}
}
		]==]

      	-- 解析
    	local tRaceInfo = json.decode(strJson)

		for i,v in ipairs(tRaceInfo.tFight.tEnemyCars) do
			if v.id ~= nil then
				v.nCarID = v.id
				v.nLv = v.lv
				v.nASpeed = v.a
				v.nTurn = v.t
				v.nSpeed = v.s
				v.nMergeLv = v.cLv
				v.strStar = DATAMGR:GetItemInfoMgr():GetLevelStarNameByType(v.sLv)

				v.id = nil
				v.id = nil
				v.a = nil
				v.t = nil
				v.s = nil
				v.cLv = nil
				v.sLv = nil
			end
		end

		for i,v in ipairs(tRaceInfo.tFight.tMyCars) do
			if v.id ~= nil then
				v.nCarID = v.id
				v.nLv = v.lv
				v.nASpeed = v.a
				v.nTurn = v.t
				v.nSpeed = v.s
				v.nMergeLv = v.cLv
				v.strStar = DATAMGR:GetItemInfoMgr():GetLevelStarNameByType(v.sLv)

				v.id = nil
				v.id = nil
				v.a = nil
				v.t = nil
				v.s = nil
				v.cLv = nil
				v.sLv = nil
			end
		end

		-- 如果是在开场阶段
		local strPlist
		local strXml
		local fDelayTime = 0
		if self.pSpStartSwf == nil then
			strPlist = Public_GetResPathByKey("Images/kj-kaijidonghua.plist")
		 	nickXml.NickTexCacheMgr:shareCache():AddOnePlistFile(strPlist, 0.656);
		 	local tPos 
		 	local tDeiveType = GetCurrentDeviceType();
	        -- 如果是ipad
	        if tDeiveType == CTDeviceTypeIpad or tDeiveType == CTDeviceTypeIpadHD then
				strXml = Public_GetResPathByKey("Images/kj-kaijidonghua768x1024.xml")
				tPos = ccp(320, 320)
	        -- 如果是ipone5HD
	        elseif tDeiveType == CTDeviceTypeIphone5HD then
				strXml = Public_GetResPathByKey("Images/kj-kaijidonghua640x1136.xml")
				tPos = ccp(320, 508)
	        else
				strXml = Public_GetResPathByKey("Images/kj-kaijidonghua640x960.xml")
				tPos = ccp(320, 332)
	        end
			local pSprite = nickXml.NickXmlSprite:createWithStart(strXml, 0);
			pSprite:setPosition(tPos);

			local pLyBg = CCLayerColor:create(ccc4(0, 0, 0, 255))
			pLyBg:addChild(pSprite)

			self:addChild(pLyBg, 1);
			pSprite:startPlay();

			self.pSpStartSwf = pLyBg;

			fDelayTime = (1/24.0)*pSprite:getFrameCnt()
		end
		-- 进入比赛
		self:performWithDelay(function()
			self.pSpStartSwf:removeFromParent()
			self.pSpStartSwf = nil;
			nickXml.NickTexCacheMgr:shareCache():removeFramesByPlistFileName(strPlist);
			nickXml.FlaXmlCache:sharedFlaXmlCache():removeFlaXmlWithName(strXml);

			SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.StartShowLocalRace, tRaceInfo)
		end, fDelayTime)
		return false
		
	-- 强行回到基地界面
	elseif nEvent == TeachEventTeyp.ForceBackToBase then
		self.bIsInDreamWar = false;
		-- 强制回到基地
		SHARE_FUNC:GetMainLogic():ForceShowBaseAndTopList();
	end
	return true;
end

-- 教学按钮点击
function TeachMgrLayer:TeachDialogClick( nIdx )
	-- 如果还有下一个教学，则显示下一个教学
	if self:IsCurInTeaching() == false then
		return;
	end
	self:ShowSwichToNextStep();
end

-- 切换并显示到下一步骤
function TeachMgrLayer:ShowSwichToNextStep()

	-- print(debug.traceback())
	-- 没有处理
	if self.nTeachType == TeachConType.None then
		return;
	end
	-- 判断是否通知下一个
	self:JudgeToNoticeToNextStep()
	-- 如果还有下一个教学，则显示下一个教学
	if self:ChangeToNextTeachStep() == false then
		self:ShowCurTeachStep();
	end
end
-- 隐藏当前教学信息
function TeachMgrLayer:SetCurTeachVisabled(bIsVisiable, bIsJudgeNext)

	-- 
	if self.pLyClip ~= nil then
		self.pLyClip:setVisible(bIsVisiable)
	end

	if self.pSpHand ~= nil then
		self.pSpHand:setVisible(bIsVisiable)
		if bIsVisiable == false then
			self.pSpHand:stopAllActions()
		else
			self:StartShowClickAction( ccp( self.tCurRect:getMidX(), self.tCurRect:getMidY() ) )
		end
	end

	if self.pSpWords ~= nil then
		self.pSpWords:setVisible(bIsVisiable)
	end

	if bIsJudgeNext == nil or bIsJudgeNext == true then
		-- 判断是否并发送下一步的记录
		self:JudgeToNoticeToNextStep();
	end
end
-- 判断当前是否标记到下一步
function TeachMgrLayer:JudgeToNoticeToNextStep()
	-- 如果不在教学中，则不处理
	if self:IsCurInTeaching() == false then
		return;
	end
	-- 获取当前操作的数据
	local strKey = self:GetStageShowKey( self.nTeachStage )
	local tStage = self.tTeachInfo.steps[ strKey ].list;
	local tOneStep = tStage[ self.nTeachBigStep ];
	local tControlList = tOneStep.control;
	local nUpType = tControlList[self.nTeachCurStep][ 3 ];

	-- 获取信息
	if nUpType == 1 then
		local nNextStage, nNextStep = self:GetNextStepTeachId(self.nTeachStage, self.nTeachBigStep);
		self:SendTeachProgress(nNextStage, nNextStep);
	end	
end

-- 下一个教学的切换
function TeachMgrLayer:ChangeToNextTeachStep()
	self.nTeachCurStep = self.nTeachCurStep+1;
	-- 获取当前操作的数据
	local strKey = self:GetStageShowKey( self.nTeachStage )
	local tStage = self.tTeachInfo.steps[ strKey ].list;
	local tOneStep = tStage[ self.nTeachBigStep ];
	local tControlList = tOneStep.control;
	local nConCount = #tControlList;
	-- 是否已经结束
	local bIsEnd = 1;
	-- 如果超过总数
	if self.nTeachCurStep > nConCount then
		self.nTeachCurStep = 1;
		-- 如果是继续
		self.nTeachStage, self.nTeachBigStep, bIsEnd = self:GetNextStepTeachId(self.nTeachStage, self.nTeachBigStep);
	end
	-- 结束
	if bIsEnd == 0 or bIsEnd == 2 then
		-- 删除所有信息
		table.remove(self.nArrCurTeachList,1)
		self:RemoveTeachInfo();
		if bIsEnd == 0 then
			self:SendTeachProgress(self.nTeachStage, self.nTeachBigStep);
		else
			-- 本地记录下当前步骤
			local tTeachInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().tTeachInfos
			tTeachInfo[ self.nTeachStage ] = self.nTeachBigStep;
		end
		return true;
	end
	return false;
end

-- 获取下一个step的id
function TeachMgrLayer:GetNextStepTeachId(nStageID, nStepID)
	-- 获取当前操作的数据
	local strKey = self:GetStageShowKey( nStageID )
	local tStage = self.tTeachInfo.steps[ strKey ].list;
	local tOneStep = tStage[ nStepID ];
	-- 是否已经结束
	local bIsEnd = 1;
	-- 如果超过总数
	-- 如果是继续
	if tOneStep.continue == 1 then
		nStepID = nStepID+1;
		-- 如果也是当前教学大步骤的最后一个步骤，则进入下一个大步骤
		if  nStepID > #tStage then
			nStepID = -1;
			-- nStageID = nStageID + 1;
			-- 如果下一个大步骤不存在，则结束
			-- local strNextKey = self:GetStageShowKey( nStageID )
			-- local tNextStage = self.tTeachInfo.steps[ strNextKey ];
			-- if tNextStage == nil then
				bIsEnd = 0;
			-- end
		end
	elseif tOneStep.continue == 2 then
		bIsEnd = 2;
		nStepID = -1;
	else
		bIsEnd = 0;
		nStepID = -1;
	end

	return nStageID, nStepID, bIsEnd
end

-- 教学的发送数据
function TeachMgrLayer:SendTeachProgress(nStageID, nStep)
	-- 本地记录下当前步骤
	local tTeachInfo = DATAMGR:GetUserInfoMgr():GetUserBaseInfo().tTeachInfos
	tTeachInfo[ nStageID ] = nStep;
	-- 发送教学步骤
	local tLuaData = {}
	tLuaData.teachStageId = nStageID
	tLuaData.teachStepId = nStep

	print("发送教学进度:", nStageID, nStep)
	-- 发送数据
	SHARE_FUNC.NoticeMainLogicSendMsg(CTONCMD.IntoTeachNewStep, tLuaData)
end

-- mark: ========================= 教学的触控 =================================
function TeachMgrLayer:TeachTouchBegan(tLocation, nTouchCount)
	self.bIsTouchBegan 	= false;
	-- 没有触控到
	if self.nTeachType == TeachConType.None then
		return true;
	end

	-- if self.pLyClip:isVisible() == true then
	-- 	self.bIsTouchBegan = self.tCurRect:containsPoint( tLocation );
	-- end

	-- 是否临时允许全屏触控
	if self.bIsCanTempAllTouch == true then
		self.bIsTouchBegan = true;
		return true;
	end

	-- 如果不是对话框或事件，或者是教程里面可以判断的点击时间，则要判断点击范围
	if self.nTeachType ~= TeachConType.Dialog and 
		self.nTeachType ~= TeachConType.Event and 
		self.nTeachType ~= TeachConType.TeachRect and
		self.nTeachType ~= TeachConType.TeachShowText and
		self.nTeachType ~= TeachConType.JustTouchRect then

		-- 显示时才可以操作
		if self.nTeachType == TeachConType.TouchMove then
			self.bIsTouchBegan = self.tCurRect:containsPoint( tLocation );
		else
			if self.pSpHand:isVisible() then
				self.bIsTouchBegan = self.tCurRect:containsPoint( tLocation );
			end
		end
		
	end

	return self.bIsTouchBegan;
end

function TeachMgrLayer:TeachTouchMoved(tLocation, tPrevPos, tBeganPos)
	-- 没有触控到
	if self.nTeachType == TeachConType.None then
		return true;
	end

	return self.bIsTouchBegan;
end

function TeachMgrLayer:TeachTouchEnded(tLocation, tPrevPos, tBeganPos)
	-- 没有触控到
	if self.nTeachType == TeachConType.None then
		return true;
	end
	if self.nTeachType == TeachConType.TeachRect or self.nTeachType == TeachConType.JustTouchRect then
		if self.tCurRect:containsPoint( tLocation ) == true then
			self:ShowSwichToNextStep();
			return false;
		end
	end
	local bIsTouch1 = self.bIsTouchBegan;
	return bIsTouch1
end

-- 教学全局都在
TEACHMGR= TeachMgrLayer.new();
TEACHMGR:retain()
